<?php

/**
 * The admin-specific functionality of the plugin.
 *
 * @link       http://preventdirectaccess.com/
 * @since      1.0.0
 *
 * @package    Pda_Contact_Forms
 * @subpackage Pda_Contact_Forms/admin
 */

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    Pda_Contact_Forms
 * @subpackage Pda_Contact_Forms/admin
 * @author     BWPS <hello@preventdirectaccess.com>
 */
class Pda_Contact_Forms_Admin {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string $plugin_name The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string $version The current version of this plugin.
	 */
	private $version;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 *
	 * @param      string $plugin_name The name of this plugin.
	 * @param      string $version The version of this plugin.
	 */
	public function __construct( $plugin_name, $version ) {

		$this->plugin_name = $plugin_name;
		$this->version     = $version;

	}

	/**
	 * Register the stylesheets for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_styles() {

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in Pda_Contact_Forms_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The Pda_Contact_Forms_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */

		wp_enqueue_style( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'css/pda-contact-forms-admin.css', array(), $this->version, 'all' );

	}

	/**
	 * Register the JavaScript for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_scripts() {

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in Pda_Contact_Forms_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The Pda_Contact_Forms_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */

		wp_enqueue_script( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'js/pda-contact-forms-admin.js', array( 'jquery' ), $this->version, false );

		wp_enqueue_script( 'pda_contact_forms_js' . $this->plugin_name, plugin_dir_url( __FILE__ ) . './js/pda-contact-forms-admin.js', array( 'jquery' ) );
		wp_localize_script( 'pda_contact_forms_js' . $this->plugin_name, 'pda_contact_forms_data', array( 'ajax_url' => admin_url( 'admin-ajax.php' ) ) );

	}

	function protect_file_by_ninja_forms_files_upload( $form_data ) {
		if ( is_plugin_active( 'ninja-forms-uploads/file-uploads.php' ) ) {
			$settings = get_option( 'pda_gold_contact_forms' );
			if ( $settings ) {
				$options = unserialize( $settings );
				if ( array_key_exists( 'ninja_forms_file_uploads', $options ) ) {
					$ninja_forms_file_uploads = $options['ninja_forms_file_uploads'];
				}
				if ( isset( $ninja_forms_file_uploads ) && $ninja_forms_file_uploads == 'true' ) {
					foreach ( $form_data['fields'] as $data ) {
						if ( $data['upload_types'] == 'pdf, doc, docx' ) {
							foreach ( $data['value'] as $key => $value ) {
								global $wpdb;
								$results = $wpdb->get_var( "SELECT data FROM {$wpdb->prefix}ninja_forms_uploads WHERE id = $key" );
								$data    = unserialize( $results );
								if ( class_exists( 'PDA_v3_Gold_Repository' ) ) {
									$repo_v3 = new PDA_v3_Gold_Repository();
									$repo_v3->protect_prevent_files( $data['attachment_id'] );
								}
							}
						}
					}
				}
			}
		}
	}

	function get_post_id_file_in_forms( $submission_id, $form_id ) {
		$data_form               = get_post_meta( $submission_id );
		$ccf_submission_data_map = $data_form['ccf_submission_data_map'];
		$arr_file                = unserialize( $ccf_submission_data_map[0] );
		$type                    = array_filter( $arr_file, function ( $f ) {
			return $f['type'] == "file";
		} );
		$key                     = array_keys( $type );
		$ccf_submission_data     = $data_form['ccf_submission_data'];
		$arr_file_data           = unserialize( $ccf_submission_data[0] );
		$data                    = [];
		foreach ( $key as $k => $value ) {
			array_push( $data, $arr_file_data[ $value ] );
		}

		$settings = get_option( 'pda_gold_contact_forms' );
		if ( $settings ) {
			$option = unserialize( $settings );
			if ( array_key_exists( 'custom_contact_forms', $option ) ) {
				$custom_contact_forms = $option['custom_contact_forms'];
			}
		}
		if ( isset( $custom_contact_forms ) && $custom_contact_forms == 'true' ) {
			//protect file pda gold
			if ( class_exists( 'Pda_Gold_Admin' ) ) {
				$repo = new Pda_Gold_Admin;
				foreach ( $data as $data_file ) {
					$repo->insert_prevent_direct_access( $data_file['id'], true, null );
				}
			}
			//protect file pda v3
			if ( class_exists( 'PDA_v3_Gold_Repository' ) ) {
				$repo_v3 = new PDA_v3_Gold_Repository();
				foreach ( $data as $data_file_v3 ) {
					$repo_v3->protect_prevent_files( $data_file_v3['id'] );
				}
			}
		}
	}

	function check_plugin_pda_gold_activate() {
		$message = "";
		if ( Yme_Plugin_Utils::is_plugin_activated( 'pda_gold' ) === 0 ) {
			$message = YME_MESSAGES::$PDA['PDA_NEVER_PURCHASED'];
		}

		if ( Yme_Plugin_Utils::is_plugin_activated( 'pda_gold' ) === 1 && Yme_Plugin_Utils::is_plugin_activated( 'pda_v3' ) === 1 ) {
			$message = YME_MESSAGES::$PDA['PDA_NEVER_ACTIVATE'];
		}

		if ( Yme_Plugin_Utils::is_plugin_activated( 'pda_gold' ) === 2 ) {
			$message = YME_MESSAGES::$PDA['PDA_UPDATE_VERSION'];
		}

		if ( ! empty( $message ) ) {
			?>
            <div class="notice notice-error is-dismissible">
                <p><b><?php echo esc_html( $this->plugin_name ); ?>: </b><?php echo $message; ?></p>
            </div>
			<?php
		}
	}

	function render_contact_forms() {
		$settings = get_option( 'pda_gold_contact_forms' );
		if ( $settings ) {
			$option = unserialize( $settings );
			if ( is_array( $option ) ) {
				if ( array_key_exists( 'custom_contact_forms', $option ) ) {
					$custom_contact_forms = $option['custom_contact_forms'];
				}
				if ( array_key_exists( 'contact_form_7', $option ) ) {
					$contact_form_7 = $option['contact_form_7'];
				}
				if ( array_key_exists( 'ninja_forms_file_uploads', $option ) ) {
					$ninja_forms_file_uploads = $option['ninja_forms_file_uploads'];
				}
				if ( array_key_exists( 'pda_advanced_custom_fields', $option ) ) {
					$pda_advanced_custom_fields = $option['pda_advanced_custom_fields'];
				}
			}
		}
		$is_showed_submit_btn = $this->have_any_contact_form();

		if ( ! $this->have_any_contact_form() ) {
			require_once PDA_CONTACT_FORMS_PLUGIN_PATH . '/includes/views/view-pda-contact-form-no-contact-form-plugins.php';
		} else {
			?>
            <div class="main_container">
                <form method="post" id="pda_contacts_forms">
                    <input type="hidden" nonce="<?php echo wp_create_nonce( 'pda_contact_forms_nonce' ); ?>"
                           id="nonce_contact_forms"/>
                    <table class="pda_v3_settings_table pda_member_table" cellpadding="4">
                        <tr>
                            <td colspan="2">
                                <h3><?php esc_html_e( 'PLUGIN INTEGRATION', 'prevent-direct-access-gold' ) ?></h3>
								<?php esc_html_e( 'Automatically protect new file uploads through the following plugins', 'prevent-direct-access-gold' ) ?>
                            </td>
                        </tr>
						<?php if ( is_plugin_active( 'custom-contact-forms/custom-contact-forms.php' ) ) { ?>
                            <tr>
                                <td>
									<?php if ( isset( $custom_contact_forms ) && $custom_contact_forms == 'true' ) { ?>
                                        <label class="pda_switch" for="contact_forms">
                                            <input type="checkbox" id="contact_forms" name="contact_forms" checked/>
                                            <span class="pda-slider round"></span>
                                        </label>
									<?php } else { ?>
                                        <label class="pda_switch" for="contact_forms">
                                            <input type="checkbox" id="contact_forms" name="contact_forms"/>
                                            <span class="pda-slider round"></span>
                                        </label>
									<?php } ?>
                                </td>
                                <td>
                                    <p>
                                        <label class="pda_member_style_label"><?php esc_html_e( 'Custom Contact Forms', 'prevent-direct-access-gold' ) ?></label>
                                    </p>
                                </td>
                            </tr>
						<?php } ?>

						<?php if ( is_plugin_active( 'contact-form-7/wp-contact-form-7.php' ) ) { ?>
                            <tr>
                                <td>
									<?php if ( isset( $contact_form_7 ) && $contact_form_7 == 'true' ) { ?>
                                        <label class="pda_switch" for="contact_form_7">
                                            <input type="checkbox" id="contact_form_7" name="contact_form_7"
                                                   checked/>
                                            <span class="pda-slider round"></span>
                                        </label>
									<?php } else { ?>
                                        <label class="pda_switch" for="contact_form_7">
                                            <input type="checkbox" id="contact_form_7" name="contact_form_7"/>
                                            <span class="pda-slider round"></span>
                                        </label>
									<?php } ?>
                                </td>
                                <td>
                                    <p>
                                        <label class="pda_member_style_label"><?php esc_html_e( 'Contact Form 7', 'prevent-direct-access-gold' ) ?></label>
                                    </p>
                                </td>
                            </tr>
						<?php } ?>

						<?php if ( is_plugin_active( 'ninja-forms-uploads/file-uploads.php' ) ) { ?>
                            <tr>
                                <td>
									<?php if ( isset( $ninja_forms_file_uploads ) && $ninja_forms_file_uploads == 'true' ) { ?>
                                        <label class="pda_switch" for="ninja_forms_file_uploads">
                                            <input type="checkbox" id="ninja_forms_file_uploads"
                                                   name="ninja_forms_file_uploads" checked/>
                                            <span class="pda-slider round"></span>
                                        </label>
									<?php } else { ?>
                                        <label class="pda_switch" for="ninja_forms_file_uploads">
                                            <input type="checkbox" id="ninja_forms_file_uploads"
                                                   name="ninja_forms_file_uploads"/>
                                            <span class="pda-slider round"></span>
                                        </label>
									<?php } ?>
                                </td>
                                <td>
                                    <p>
                                        <label class="pda_member_style_label"><?php esc_html_e( 'Ninja Forms - File Uploads', 'prevent-direct-access-gold' ) ?></label>
                                    </p>
                                </td>
                            </tr>
						<?php } ?>

						<?php if ( is_plugin_active( 'advanced-custom-fields-pro/acf.php' ) || ( is_plugin_active( 'advanced-custom-fields/acf.php' ) && is_plugin_active( 'acf-repeater/acf-repeater.php' ) ) ) { ?>
                            <tr>
                                <td>
									<?php if ( isset( $pda_advanced_custom_fields ) && $pda_advanced_custom_fields == 'true' ) { ?>
                                        <label class="pda_switch" for="pda_advanced_custom_fields">
                                            <input type="checkbox" id="pda_advanced_custom_fields"
                                                   name="pda_advanced_custom_fields" checked/>
                                            <span class="pda-slider round"></span>
                                        </label>
									<?php } else { ?>
                                        <label class="pda_switch" for="pda_advanced_custom_fields">
                                            <input type="checkbox" id="pda_advanced_custom_fields"
                                                   name="pda_advanced_custom_fields"/>
                                            <span class="pda-slider round"></span>
                                        </label>
									<?php } ?>
                                </td>
                                <td>
                                    <p>
                                        <label class="pda_member_style_label"><?php esc_html_e( 'Advanced Custom Fields - Repeater File & Image Fields', 'prevent-direct-access-gold' ) ?></label>
                                    </p>
                                </td>
                            </tr>
						<?php } ?>
                    </table>
					<?php
					if ( $is_showed_submit_btn ) {
						submit_button();
					}
					?>
                </form>
            </div>
			<?php
		}
	}

	function have_any_contact_form() {
		$plugins = [
			'custom-contact-forms/custom-contact-forms.php',
			'contact-form-7/wp-contact-form-7.php',
			'ninja-forms-uploads/file-uploads.php',
			'acf-repeater/acf-repeater.php',
			'advanced-custom-fields-pro/acf.php'
		];
		foreach ( $plugins as $plugin ) {
			if ( is_plugin_active( $plugin ) ) {
				return true;
			}
		}

	}

	function contact_forms_update_settings() {
		$nonce = $_REQUEST['security_check'];
		if ( ! wp_verify_nonce( $nonce, 'pda_contact_forms_nonce' ) ) {
			wp_die( 'invalid_nonce' );
		}
		$settings      = $_REQUEST['settings'];
		$contact_forms = serialize( $settings );
		update_option( 'pda_gold_contact_forms', $contact_forms );
		wp_send_json( true );
		wp_die();
	}

	function protect_file_by_contact_form_7() {
		if ( class_exists( 'WPCF7_Submission' ) ) {
			$form = WPCF7_Submission::get_instance();
			if ( $form ) {
				$files = $form->uploaded_files();
				foreach ( $files as $file_key => $file ) {
					$filename    = basename( $file );
					$upload_file = wp_upload_bits( $filename, null, file_get_contents( $file ) );
					if ( ! $upload_file['error'] ) {
						$wp_filetype   = wp_check_filetype( $filename, null );
						$attachment    = array(
							'post_mime_type' => $wp_filetype['type'],
//                            'post_parent' => $parent_post_id,
							'post_title'     => preg_replace( '/\.[^.]+$/', '', $filename ),
							'post_content'   => '',
							'post_status'    => 'inherit'
						);
						$attachment_id = wp_insert_attachment( $attachment, $upload_file['file'], 0 );
						if ( ! is_wp_error( $attachment_id ) ) {
							require_once( ABSPATH . "wp-admin" . '/includes/image.php' );
							$attachment_data = wp_generate_attachment_metadata( $attachment_id, $upload_file['file'] );
							wp_update_attachment_metadata( $attachment_id, $attachment_data );

							$settings = get_option( 'pda_gold_contact_forms' );
							if ( $settings ) {
								$option = unserialize( $settings );
								if ( array_key_exists( 'contact_form_7', $option ) ) {
									$contact_form_7 = $option['contact_form_7'];
								}
							}
							if ( isset( $contact_form_7 ) && $contact_form_7 == 'true' ) {
								//protect file pda gold
								if ( class_exists( 'Pda_Gold_Admin' ) ) {
									$repo = new Pda_Gold_Admin;
									$repo->insert_prevent_direct_access( $attachment_id, true, null );
								}
								//protect file pda v3
								if ( class_exists( 'PDA_v3_Gold_Repository' ) ) {
									$repo_v3 = new PDA_v3_Gold_Repository();
									$repo_v3->protect_prevent_files( $attachment_id );
								}
							}
						}
					}
				}

			}
		}
	}

	public $pda_results = array();

	function pda_array_flatten( $array ) {
		if ( ! is_array( $array ) ) {
			return false;
		}
		foreach ( $array as $key => $value ) {
			if ( is_array( $value ) ) {
				if ( ! empty( $value['id'] ) ) {
					array_push( $this->pda_results, $value['id'] );
				}
				$this->pda_array_flatten( $value );
			}
		}

		return $this->pda_results;
	}

	function pda_contact_forms_handle_save_post( $post_id ) {
		if ( ( Yme_Plugin_Utils::is_plugin_activated( 'pda_gold' ) === - 1 || Yme_Plugin_Utils::is_plugin_activated( 'pda_v3' ) === - 1 ) && ( is_plugin_active( 'advanced-custom-fields-pro/acf.php' ) || is_plugin_active( 'acf-repeater/acf-repeater.php' ) ) ) {
			$settings = get_option( 'pda_gold_contact_forms' );
			if ( $settings ) {
				$options = unserialize( $settings );
				if ( array_key_exists( 'pda_advanced_custom_fields', $options ) ) {
					$pda_advanced_custom_fields = $options['pda_advanced_custom_fields'];
				}
				if ( isset( $pda_advanced_custom_fields ) && $pda_advanced_custom_fields == 'true' ) {
					if ( get_post_meta( $post_id ) ) {
					    if ( false === get_fields( $post_id ) ) {
					        return;
                        }

						$fields  = array_filter( get_fields( $post_id ), function ( $field ) {
							if ( isset( $field[0] ) ) {
								return $field;
							}
						} );
						$results = $this->pda_array_flatten( $fields );
						if ( is_array( $results ) ) {
							foreach ( $results as $post_id ) {
								$repo_v3 = new PDA_v3_Gold_Repository();
								if ( ! $repo_v3->is_protected_file( $post_id ) ) {
									$service = new PDA_Private_Link_Services;
									$service->protect_file( $post_id );
								}
							}
						}
					}
				}
			}
		}
	}

	/**
	 * Add custom settings for repeater field
	 *
	 * @param $field
	 */
	public function render_field_settings( $field ) {
		acf_render_field_setting( $field, array(
			'label'        => __( 'Auto-protect New File Uploads', 'prevent-direct-access-gold' ),
			'instructions' => __( 'Automatically protect all file uploads through this repeater field', 'prevent-direct-access-gold' ),
			'name'         => PCF_Constant::ACF_FIELD_FILE_PROTECTION,
			'type'         => 'true_false',
			'ui'           => 1,
		) );

		acf_render_field_setting( $field, array(
			'label'        => __( 'Only Show Protected Files', 'prevent-direct-access-gold' ),
			'instructions' => __( 'Only show protected files on “Add Files” or “Add Images” popup' ),
			'name'         => PCF_Constant::ACF_FIELD_ONLY_SHOW_PROTECTED_FILE,
			'type'         => 'true_false',
			'ui'           => 1,
		) );
	}

	/**
     * Filter unprotected files
     *
	 * @param array $query
	 *
	 * @return array
	 */
	public function pcf_filter_unprotected_files( $query = array() ) {
		if ( ! isset( $_POST['query'] ) ) {
			return $query;
		}

		$query_obj = $_POST['query'];

		if ( ! isset( $query_obj['_acfuploader'] ) ) {
			return $query;
		}

		$acf_field_id = $query_obj['_acfuploader'];

		$pda_only_show_protected_file = $this->get_parent_setting( $acf_field_id, 'pda_only_show_protected_file' );

		if ( is_null( $pda_only_show_protected_file ) || $pda_only_show_protected_file === 0 ) {
			return $query;
		}

		if ( ! class_exists( 'PDA_v3_Gold_Repository' ) ) {
			return $query;
		}

		$attachments_query = new WP_Query( $query );

		if ( ! $attachments_query->have_posts() ) {
			return $query;
		}

		$protected_files   = array();
		$unprotected_files = array();

		if ( $attachments_query->have_posts() ) {
			$repo = new PDA_v3_Gold_Repository();

			while ( $attachments_query->have_posts() ) : $attachments_query->the_post();
				$id = get_the_ID();
				if ( $repo->is_protected_file( $id ) ) {
					$protected_files[] = $id;
				} else {
					$unprotected_files[] = $id;
				}
			endwhile;
		}

		wp_reset_query();

		$query['post__in']     = $protected_files;
		$query['post__not_in'] = $unprotected_files;

		error_log( 'Protected files: ' . wp_json_encode( $protected_files ) );

		return $query;
	}

	/**
     * Handle upload files
     *
	 * @param $attachment_id
	 *
	 * @return mixed
	 */
	public function pcf_handle_upload( $attachment_id ) {

		if ( ! class_exists( 'PDA_Private_Link_Services' ) ) {
			return $attachment_id;
		}

		if ( ! isset( $_POST['_acfuploader'] ) ) {
			return $attachment_id;
		}

		$acf_field_id                 = $_POST['_acfuploader'];
		$pda_only_show_protected_file = $this->get_parent_setting( $acf_field_id, 'pda_file_protection' );

		if ( is_null( $pda_only_show_protected_file ) || $pda_only_show_protected_file === 0 ) {
			return $attachment_id;
		}

		PDA_Private_Link_Services::protect_file( $attachment_id );

		return $attachment_id;
	}

	/**
	 * Get parent setting
	 *
	 * @param int $acf_field_id ACF Field ID
	 * @param string $setting The setting's name
	 *
	 * @return bool|null
	 */
	private function get_parent_setting( $acf_field_id, $setting ) {
		$field_settings = get_field_object( $acf_field_id );

		if ( ! isset( $field_settings ) ) {
			return null;
		}

		$parent          = $field_settings['parent'];
		$parent_settings = acf_maybe_get_field( $parent, false, false );


		if ( ! isset( $parent_settings[ $setting ] ) ) {
			return null;
		}

		return $parent_settings[ $setting ];
	}
}
