(function( $ ) {
	'use strict';

	/**
	 * All of the code for your admin-facing JavaScript source
	 * should reside in this file.
	 *
	 * Note: It has been assumed you will write jQuery code here, so the
	 * $ function reference has been prepared for usage within the scope
	 * of this function.
	 *
	 * This enables you to define handlers, for when the DOM is ready:
	 *
	 * $(function() {
	 *
	 * });
	 *
	 * When the window is loaded:
	 *
	 * $( window ).load(function() {
	 *
	 * });
	 *
	 * ...and/or other possibilities.
	 *
	 * Ideally, it is not considered best practise to attach more than a
	 * single DOM-ready or window-load handler for a particular page.
	 * Although scripts in the WordPress core, Plugins and Themes may be
	 * practising this, we should strive to set a better example in our own work.
	 */

    $(function() {
        $('#pda_contacts_forms').submit(function(evt) {
            evt.preventDefault();
            evt.stopImmediatePropagation();
            _updateSettingsContactForms({
                custom_contact_forms: $('#contact_forms').prop('checked'),
                contact_form_7: $('#contact_form_7').prop('checked'),
                ninja_forms_file_uploads: $('#ninja_forms_file_uploads').prop('checked'),
                pda_advanced_custom_fields: $('#pda_advanced_custom_fields').prop('checked'),
            }, function (error) {
                if(error) {
                    console.error(error);
                }
                // location.reload(true);
            });
        });
    });

    function _updateSettingsContactForms(settings, cb) {
        var _data = {
            action: 'contact_forms_update_settings',
            settings: settings,
            security_check: $("#nonce_contact_forms").attr('nonce'),
        }
        $.ajax({
            url: pda_contact_forms_data.ajax_url,
            type: 'POST',
            data: _data,
            success: function(data) {
                if (data == 'invalid_nonce') {
                    alert('No! No! No! Verify Nonce Fails!');
                } else if(data) {
                    toastr.success('Settings has been updated successfully!', 'PDA Contact Forms')
                } else {
                    console.log("Failed", data);
                }
                cb();
            },
            error: function(error) {
                cb(error);
            },
            timeout: 5000
        });
    }

})( jQuery );
