<?php

/**
 * The plugin bootstrap file
 *
 * This file is read by WordPress to generate the plugin information in the plugin
 * admin area. This file also includes all of the dependencies used by the plugin,
 * registers the activation and deactivation functions, and defines a function
 * that starts the plugin.
 *
 * @link              http://preventdirectaccess.com/
 * @since             1.2.1
 * @package           Pda_Contact_Forms
 *
 * @wordpress-plugin
 * Plugin Name:       PDA Contact Forms
 * Plugin URI:        https://preventdirectaccess.com/extensions/contact-forms-integration/?utm_source=user-website&utm_medium=plugin_link&utm_campaign=pda-contact-forms
 * Description:       Automatically protect all file uploads through Advanced Custom Fields - Repeater File & Image Fields as well as the most popular WordPress contact form plugins.
 * Version:           1.2.1
 * Author:            BWPS
 * Author URI:        https://preventdirectaccess.com/extensions/contact-forms-integration/?utm_source=user-website&utm_medium=plugin_link&utm_campaign=pda-contact-forms
 * License:           GPL-2.0+
 * License URI:       http://www.gnu.org/licenses/gpl-2.0.txt
 * Text Domain:       pda-contact-forms
 * Domain Path:       /languages
 */

// If this file is called directly, abort.
if ( ! defined( 'WPINC' ) ) {
	die;
}

/**
 * Currently plugin version.
 * Start at version 1.0.0 and use SemVer - https://semver.org
 * Rename this for your plugin and update it as you release new versions.
 */
define( 'PDA_CONTACT_FORMS_VERSION', '1.2.1' );

define( 'PDA_CONTACT_FORMS_PLUGIN_PATH',  plugin_dir_path( __FILE__ ) );

if ( ! defined( 'PDA_CONTACT_FORM_PLUGIN_FOLDER' ) ) {
	define( 'PDA_CONTACT_FORM_PLUGIN_FOLDER', 'pda-contact-forms/pda-contact-forms.php' );
}

/**
 * The code that runs during plugin activation.
 * This action is documented in includes/class-pda-contact-forms-activator.php
 */
function activate_pda_contact_forms() {
	require_once plugin_dir_path( __FILE__ ) . 'includes/class-pda-contact-forms-activator.php';
	require_once plugin_dir_path( __FILE__ ) . 'includes/class-pda-contact-forms-utils.php';
	Pda_Contact_Forms_Activator::activate(plugin_basename( __FILE__ ));
}

/**
 * The code that runs during plugin deactivation.
 * This action is documented in includes/class-pda-contact-forms-deactivator.php
 */
function deactivate_pda_contact_forms() {
	require_once plugin_dir_path( __FILE__ ) . 'includes/class-pda-contact-forms-deactivator.php';
	Pda_Contact_Forms_Deactivator::deactivate();
}

register_activation_hook( __FILE__, 'activate_pda_contact_forms' );
register_deactivation_hook( __FILE__, 'deactivate_pda_contact_forms' );

/**
 * The core plugin class that is used to define internationalization,
 * admin-specific hooks, and public-facing site hooks.
 */
require plugin_dir_path( __FILE__ ) . 'includes/class-pda-contact-forms.php';

/**
 * Remove PDA Contact Form plugin in list plugins update of WP.
 *
 * @param object $value Data update cache.
 *
 * @return object $value Data update cache.
 */
function contact_form_filter_plugin_updates( $value ) {
	if ( ! isset( $value->response[ PDA_CONTACT_FORM_PLUGIN_FOLDER ] ) ) {
		return $value;
	}
	unset( $value->response[ PDA_CONTACT_FORM_PLUGIN_FOLDER ] );

	return $value;
}

/**
 * Check condition to hide "Check for updates" button.
 *
 * @return bool
 * Return false: If all main plugins is deactivate
 * Else: return true
 */
function contact_form_main_plugins_are_deactivated() {
	$main_plugins = array( 'pda_gold', 'pda_v3' );
	foreach ( $main_plugins as $main_plugin ) {
		if ( - 1 === Yme_Plugin_Utils::is_plugin_activated( $main_plugin ) ) {
			return false;
		}
	}

	return true;
}

/**
 * Begins execution of the plugin.
 *
 * Since everything within the plugin is registered via hooks,
 * then kicking off the plugin from this point in the file does
 * not affect the page life cycle.
 *
 * @since    1.0.0
 */
function run_pda_contact_forms() {
	$plugin = new Pda_Contact_Forms();
	$plugin->run();

	if ( ! contact_form_main_plugins_are_deactivated() ) {
		$configs = require PDA_CONTACT_FORMS_PLUGIN_PATH . '/includes/class-pda-contact-forms-configs.php';
		$url     = $configs->update_url;
		if ( method_exists( 'Puc_v4p8_Factory', 'buildUpdateChecker' ) ) {
			Puc_v4p8_Factory::buildUpdateChecker(
				$url,
				__FILE__,
				'pda-contact-forms'
			);
		} elseif ( method_exists( 'Puc_v4_Factory', 'buildUpdateChecker' ) ) {
			Puc_v4_Factory::buildUpdateChecker(
				$url,
				__FILE__,
				'pda-contact-forms'
			);
		} else {
			add_filter( 'site_transient_update_plugins', 'contact_form_filter_plugin_updates' );
		}
	} else {
		add_filter( 'site_transient_update_plugins', 'contact_form_filter_plugin_updates' );
	}

}

add_action( 'plugins_loaded', 'run_pda_contact_forms', 20 );
