<?php

/**
 * The admin-specific functionality of the plugin.
 *
 * @link       www.buildwps.com/prevent-direct-access-premium-plugin/?utm_source=user-website&utm_medium=pluginpage&utm_campaign=plugin-author-link
 * @since      1.0.0
 *
 * @package    Pda_Wp_Magic_Link
 * @subpackage Pda_Wp_Magic_Link/admin
 */

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    Pda_Wp_Magic_Link
 * @subpackage Pda_Wp_Magic_Link/admin
 * @author     buildwps <hello@ymese.com>
 */
class Pda_Wp_Magic_Link_Admin {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string $plugin_name The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string $version The current version of this plugin.
	 */
	private $version;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @param string $plugin_name The name of this plugin.
	 * @param string $version The version of this plugin.
	 *
	 * @since    1.0.0
	 *
	 */
	public function __construct( $plugin_name, $version ) {

		$this->plugin_name = $plugin_name;
		$this->version     = $version;

	}

	/**
	 * Register the stylesheets for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_styles() {

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in Pda_Wp_Magic_Link_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The Pda_Wp_Magic_Link_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */

		wp_enqueue_style( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'css/pda-wp-magic-link-admin.css', array(), $this->version, 'all' );

	}

	/**
	 * Register the JavaScript for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_scripts() {

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in Pda_Wp_Magic_Link_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The Pda_Wp_Magic_Link_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */

		if ( function_exists( 'get_current_screen' ) ) {
			$screen = get_current_screen();
			if ( 'toplevel_page_pda-gold' === $screen->id || 'toplevel_page_ppp_gold_options' === $screen->id ) {
				if ( isset( $_GET['tab'] ) && 'magic_link' === $_GET['tab'] ) {
					wp_enqueue_script( 'pda_magic_link_js' . $this->plugin_name, plugin_dir_url( __FILE__ ) . './js/pda-wp-magic-link-admin.js', array( 'jquery' ), $this->version, false );
					wp_localize_script( 'pda_magic_link_js' . $this->plugin_name, 'pda_magic_link_data', array( 'ajax_url' => admin_url( 'admin-ajax.php' ) ) );
					wp_enqueue_script( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'js/pda-wp-magic-link-admin.js', array( 'jquery' ), $this->version, false );
				}
			}
		}

	}

	/**
	 * Show admin notice that reminds user to install prevent direct access gold version
	 */
	public function pda_wml_admin_notice() {
		if ( ! Pda_Wp_Magic_Link_Utils::is_main_plugin_activate() ) {
			?>
			<div class="notice notice-error is-dismissible">
				<p>
					<b><?php echo esc_html( $this->plugin_name ); ?>: </b>
					<?php echo "Please install and activate <a target=\"_blank\" rel=\"noopener noreferrer\" href=\"https://preventdirectaccess.com/pricing/\">Prevent Direct Access Gold</a> or <a target=\"_blank\" rel=\"noopener noreferrer\" href=\"https://preventdirectaccess.com/protect-wordpress-pages-posts/\">Protect Pages & Posts Gold</a> plugin"; ?>
				</p>
			</div>
			<?php
		}
	}

	public function add_magiclink_button() {
		if ( ! current_user_can( 'edit_posts' ) && ! current_user_can( 'edit_pages' ) ) {
			return;
		} else if ( get_user_option( 'rich_editing' ) == 'true' ) {
			add_filter( 'mce_external_plugins', array( $this, 'magiclink_mce_plugin' ) );
			add_filter( 'mce_buttons', array( $this, 'register_magiclink_button' ) );
		}
	}

	public function magiclink_mce_plugin( $plugin_array ) {
		$plugin_array['magiclink'] = plugins_url( 'admin/js/', dirname( __FILE__ ) ) . 'magiclink-mce-button.js';

		return $plugin_array;
	}

	public function register_magiclink_button( $buttons ) {
		array_push( $buttons, "|", 'magiclink' );

		return $buttons;
	}

	public function magiclink_shortcode( $attrs, $content = null ) {

		extract( shortcode_atts( array(
			'id'   => '',
			'word' => '',
			'url'  => ''
		), $attrs ) );
		$link = '';

		//PDA Gold is activated
		if ( class_exists( 'PDA_Gold_Repository' ) ) {
			$repo = new PDA_Gold_Repository();
			if ( isset( $id ) ) {
				$default_file = $repo->get_default_advance_file_by_host_id( $id );
				if ( isset( $default_file ) && $default_file->is_prevented === "1" ) {
					$prefix_url = get_option( 'pda_prefix_url' );
					if ( ! is_multisite() ) {
						$link = site_url() . "/{$prefix_url}/" . $default_file->url;
					} else {
						$link = site_url() . "/{$prefix_url}/site/" . get_current_blog_id() . '/' . $default_file->url;
					}
				} else {
					$link = $url;
				}
			}
		} else if ( class_exists( 'PDA_S3_Repository' ) ) {
			$repo = new PDA_S3_Repository;
			if ( isset( $id ) ) {
				$result = $repo->get_default_private_link( $id );
				if ( ! is_null( $result ) ) {
					$link = $result->signed_url;
				} else {
					$link = $repo->insert_default_private_link( $id );
				}
			}
		} else {
			$link = $url;
		}

		$word = empty( $word ) ? $link : $word;

		//PDA Gold S3 is activated
		$magic_link = <<<_end_
        <!-- Begin magic link -->
        <a href="{$link}" target="_blank" ref="noopener noreferrer">$word</a>
        <!-- End magic link -->
_end_;

		return $magic_link;
	}

	public function filter_protected_file( $content ) {
		if ( yme_plugin_utils::is_plugin_activated( 'pap_gold' ) === - 1 || Yme_Plugin_Utils::is_plugin_activated( 'pap_gold_old' ) === - 1 ) {
			$content = $this->handle_protected_file_ppp( $content );
		}
		if ( Yme_Plugin_Utils::is_plugin_activated( 'pda-s3' ) === - 1 ) {
			$content = $this->handle_protected_file_s3( $content );
		}
		if ( Yme_Plugin_Utils::is_plugin_activated( 'pda_gold' ) === - 1 || Yme_Plugin_Utils::is_plugin_activated( 'pda_v3' ) === - 1 ) {
//            $content = $this->handle_protected_file_pda_gold( $content );
			if ( ! magic_link_get_settings( PDA_Magic_Link_Constants::PDA_REPLACE_ALL_PAGE_POST ) ) {
				$content = $this->handle_protected_file_pdav3( $content );
			} else {
				$selected_posts_id = magic_link_get_setting_values( PDA_Magic_Link_Constants::PDA_REPLACE_ONLY_PAGE_POST );
				if ( in_array( get_the_ID(), $selected_posts_id ) ) {
					$content = $this->handle_protected_file_pdav3( $content );
				}
			}
		}

		return $content;
	}

	private function handle_protected_file_ppp( $content ) {
		$elements = array();
		preg_match_all( '/<a(.*)href=\"([^\"]*)\"(.*)>(.*)<\/a>/iU', $content, $elements );
		$links = array_unique( $elements[2] );
		foreach ( $links as $link ) {
			$post_id      = url_to_postid( $link );
			$private_link = $this->get_default_private_link_page_post( $post_id );
			if ( ! empty( $private_link ) ) {
				$content = str_replace( $link, $private_link, $content );
			}
		}

		return $content;
	}

	function handle_before_render_feature_image( $html ) {
		if ( Yme_Plugin_Utils::is_plugin_activated( 'pda_gold' ) === - 1 || Yme_Plugin_Utils::is_plugin_activated( 'pda_v3' ) === - 1 ) {
			if ( ! magic_link_get_settings( PDA_Magic_Link_Constants::PDA_REPLACE_ALL_PAGE_POST ) ) {
				$html = $this->check_condition_before_render_thumbnail( $html );
			} else {
				$selected_posts_id = magic_link_get_setting_values( PDA_Magic_Link_Constants::PDA_REPLACE_ONLY_PAGE_POST );
				if ( in_array( get_the_ID(), $selected_posts_id ) ) {
					$html = $this->check_condition_before_render_thumbnail( $html );
				}
			}
		}

		return $html;
	}

	function check_condition_before_render_thumbnail( $html ) {
		$img = array();
		preg_match_all( '/<img\s[^>]*?src\s*=\s*[\'\"]([^\'\"]*?)[\'\"][^>]*?>/iU', $html, $img );
		$links_file = array_unique( $img[1] );

		return $this->check_condition_before_return_content( $links_file, $html );
	}

	private function handle_protected_file_pdav3( $content ) {
		$img = array();
		preg_match_all( '/<img\s[^>]*?src\s*=\s*[\'\"]([^\'\"]*?)[\'\"][^>]*?>/iU', $content, $img );
		$links_file = array_unique( $img[1] );

		$elements = array();
		preg_match_all( '/<a(.*)href=\"([^\"]*)\"(.*)>(.*)<\/a>/iU', $content, $elements );
		$links_href = array_unique( $elements[2] );

		$iframe = array();
		preg_match_all( '/<iframe\s[^>]*?src\s*=\s*[\'\"]([^\'\"]*?)[\'\"][^>]*?>/iU', $content, $iframe );
		$src = array_unique( $iframe[1] );

		$links = array_merge( $links_file, $links_href, $src );

		return $this->check_condition_before_return_content( $links, $content );
	}

	function check_condition_before_return_content( $links, $content ) {
		$current_user_role = wp_get_current_user()->roles;
		$pda_file_access   = magic_link_get_setting_values( 'file_access_permission' );

		switch ( $pda_file_access ) {
			case "custom_roles":
				$user_Login      = wp_get_current_user()->roles;
				$whitelist_roles = magic_link_get_setting_values( 'whitelist_roles' );
				if ( ! empty( array_intersect( $user_Login, $whitelist_roles ) ) ) {
					return $this->handle_content_for_pda_v3( $links, $content );
				}
				break;
			case "admin_users":
				if ( in_array( "administrator", $current_user_role ) ) {
					return $this->handle_content_for_pda_v3( $links, $content );
				}
				break;
			case "author":
				global $post;
				$author_id = $post->post_author;
				if ( get_current_user_id() == $author_id ) {
					return $this->handle_content_for_pda_v3( $links, $content );
				}
				break;
			case "logged_users":
				if ( is_user_logged_in() ) {
					return $this->handle_content_for_pda_v3( $links, $content );
				}
				break;
			case "anyone":
				return $this->handle_content_for_pda_v3( $links, $content );
			case "blank":
				break;
		}

		return $content;
	}

	function handle_content_for_pda_v3( $links_file, $content ) {
		$pda_prefix_url = null;
		if ( class_exists( 'Pda_Gold_Functions' ) ) {
			$settings       = new Pda_Gold_Functions();
			$pda_prefix_url = $settings->prefix_roles_name( "pda_prefix_url" );
		} else {
			if ( get_option( "pda_prefix_url" ) ) {
				$pda_prefix_url = get_option( "pda_prefix_url" );
			}
		}

		if ( $pda_prefix_url == null ) {
			if ( class_exists( 'PDA_v3_Constants' ) ) {
				$pda_prefix_url = PDA_v3_Constants::$default_private_link_prefix;
			}
		}
		$wp_upload_dir = wp_upload_dir();
		$baseurl       = $wp_upload_dir['baseurl'];
		foreach ( $links_file as $file ) {
			$attachment = $this->get_attachment_from_url( $baseurl, $file );
			$size       = '';
			if ( empty( $attachment ) ) {
				list( $size, $url_file ) = pml_remove_size_from_url( $file );
				$attachment = $this->get_attachment_from_url( $baseurl, $url_file );
			}

			if ( ! empty( $attachment ) ) {
				$post_id      = $attachment[0]->ID;
				$private_link = $this->get_default_private_link_pdav3( $post_id );

				if ( ! empty( $size ) ) {
					$private_link = $private_link . '?size=' . str_replace( '-', '', $size );
				}

				$pda_gold_helpers = new Pda_Gold_Functions;
				if ( Yme_Plugin_Utils::is_plugin_activated( 'pdas3' ) !== - 1 || ! $pda_gold_helpers->check_file_synced_s3( $post_id ) ) {
					$content = str_replace( $file, home_url() . "/" . $pda_prefix_url . "/" . $private_link, $content );
				}
			} else {
				$new_file = str_replace( '/_pda', '', $file );
				$content  = str_replace( $file, $new_file, $content );
			}
		}

		return $content;
	}

	private function handle_protected_file_s3( $content ) {
		$img = array();
		preg_match_all( '/<img\s[^>]*?src\s*=\s*[\'\"]([^\'\"]*?)[\'\"][^>]*?>/iU', $content, $img );
		$links_file = array_unique( $img[1] );

		$wp_upload_dir = wp_upload_dir();
		$baseurl       = $wp_upload_dir['baseurl'];
		foreach ( $links_file as $file ) {
			preg_match( '(-\d+x\d+)', $file, $matches );
			if ( empty( $matches ) ) {
				$url_file = $file;
			} else {
				$url_file = str_replace( $matches[0], "", $file );
			}
			$meta_value = str_replace( $baseurl . '/', "", $url_file );
			$attachment = get_posts( array(
				'post_type'  => 'attachment',
				'meta_key'   => '_wp_attached_file',
				'meta_value' => $meta_value
			) );
			if ( ! empty( $attachment ) ) {
				$post_id = $attachment[0]->ID;
//            $post_id = Pda_Wp_Magic_Link_Repo::get_attachment_id_from_url($url_file);
				if ( $post_id != 0 ) {
					$private_link = $this->get_default_private_link_s3( $post_id );
					if ( ! empty( $private_link ) ) {
						$massaged_file = $this->filePatternWithFileSize( $file );
						$content       = preg_replace( $massaged_file, $private_link, $content );
					}
				}
			}

		}

		return $content;
	}

	private function filePatternWithFileSize( $url ) {
		$file_type    = $this->get_file_type( $url );
		$pattern      = "/\.$file_type$/";
		$massaged_url = str_replace( "/", "\/", preg_replace( $pattern, "(-\d+x\d+){0,1}.$file_type", $url ) );

		return "/$massaged_url/";
	}

	private function get_file_type( $url ) {
		$t = explode( '.', $url );
		if ( sizeof( $t ) === 1 ) {
			return "";
		}
		$file_type = end( $t );

		return $file_type;
	}

	private function get_link_tag_content( $content ) {
		$results = array();
		preg_match_all( '/<img\s[^>]*?src\s*=\s*[\'\"]([^\'\"]*?)[\'\"][^>]*?>/iU', $content, $results );

		return $results;
	}

	private function get_default_private_link_page_post( $post_id ) {
		$this->delete_private_link_expired();
		$link = "";
		if ( class_exists( 'PPP_Repository' ) ) {
			$repo = new PPP_Repository();
			if ( ! $repo->is_page_post_protected( $post_id ) ) {
				return $link;
			} else {
				$private_link = $repo->get_default_private_link( $post_id );
				if ( ! is_null( $private_link ) ) {
					$link = $private_link->url;
				} else {
					$link = $repo->insert_defaul_private_link( $post_id );
				}
			}

			if ( method_exists( 'Prevent_Page_Pup_Function_Gold', 'massagePrivateUrl' ) ) {
				$ppp_gold_function = new Prevent_Page_Pup_Function_Gold();
				$link              = $ppp_gold_function->massagePrivateUrl( $link, $post_id );
			} else {
				$prefix = Prevent_Page_BWPSConstant_Gold::get_private_url( $post_id );
				if ( ! is_multisite() ) {
					$link = magic_link_get_home_url_with_ssl() . "{$prefix}/" . $link;
				} else {
					$link = magic_link_get_home_url_with_ssl() . "{$prefix}/site/" . get_current_blog_id() . '/' . $link;
				}
			}

			return $link;
		}
	}

	private function get_default_private_link_pda_gold( $post_id ) {
		$this->delete_private_link_expired();
		$link = "";
		if ( class_exists( 'PDA_Gold_Repository' ) ) {
			$repo     = new PDA_Gold_Repository;
			$pda_info = $repo->get_pda_gold_info( $post_id );
			if ( is_null( $pda_info ) ) {
				return $link;
			} else {
				$private_link = $repo->get_default_private_link( $post_id );
				if ( ! is_null( $private_link ) ) {
					$link = $private_link->url;
				} else {
					$link = $repo->insert_defaul_private_link( $post_id );
				}
			}

			return $link;
		}

		return $link;
	}

	public function get_default_private_link_pdav3( $post_id ) {
		$this->delete_private_link_expired();
		$link = "";
		if ( class_exists( 'PDA_v3_Gold_Repository' ) ) {
			$repo              = new PDA_v3_Gold_Repository;
			$file_is_protected = $repo->is_protected_file( $post_id );
			if ( ! $file_is_protected ) {
				return $link;
			} else {
				$private_link = $repo->get_default_private_link( $post_id );
				if ( ! is_null( $private_link ) ) {
					$link = $private_link->url;
				} else {
					$link = $repo->insert_default_private_link( $post_id, magic_link_get_setting_values( 'mg_time_die_private_link' ) );
				}
			}

			return $link;
		}

		return $link;
	}

	function query_get_default_private_link( $value, $table_name, $post_id ) {
		$value = "SELECT * FROM $table_name where is_prevented = 1 and is_default = 1 and (limit_downloads is NULL OR limit_downloads > hits_count) and (expired_date is NULL OR expired_date > UNIX_TIMESTAMP()) and post_id = $post_id";

		return $value;
	}

	function insert_default_private_link_for_pda( $value, $post_id, $url ) {
//		$download_limit = pda_magic_link_get_setting_type_is_string( PDA_Magic_Link_Constants::PDA_MAGIC_LINK_DOWNLOAD_LIMIT );
//		if ( empty( $download_limit ) ) {
//			$download_limit = 7;
//		}
//		$value['limit_downloads'] = $download_limit;

		return $value;

	}

	private function get_default_private_link_s3( $post_id ) {
		$this->delete_private_link_expired();
		$link = "";
		if ( class_exists( 'PDA_S3_Repository' ) ) {
			$repo    = new PDA_S3_Repository;
			$s3_info = $repo->get_s3_info( $post_id );
			if ( is_null( $s3_info ) ) {
				return $link;
			} else if ( ! is_null( $s3_info ) && $s3_info->is_public === "1" ) {
				return $link;
			}
			$private_link = $repo->get_default_private_link( $post_id );
			if ( is_null( $private_link ) ) {
				$link = $repo->insert_default_private_link( $post_id );
			} else {
				$link = $private_link->signed_url;
			}

			return $link;
		}
	}

	public function custom_widget_text_content( $html ) {
		//TODO: need to improve
//        $results = $this->get_link_tag_content($html);
//        $tmp = array_unique($results[1]);
//	    $massagedLinks = array_map(function($link) {
//            error_log(serialize($link));
//            $file_type = $this->get_file_type($link);
//	        $link = $this->remove_crop_numbers($link, $file_type);
//	        return $link;
//        }, $tmp);
//	    foreach ($massagedLinks as $link) {
//            $file_type = $this->get_file_type($link);
//            $pattern = "(-\d+x\d+){0,1}.$file_type";
//            $original_url = '/' . str_replace('/', '\/',str_replace('.' . $file_type, $pattern, $link) ) . '/i';
//            $post_id = Pda_Wp_Magic_Link_Repo::get_attachment_id_from_url($link);
//            $private_link = $this->get_default_private_link($post_id);
//            if( !empty( $private_link ) ) {
//                $html = preg_replace($original_url, $private_link, $html);
//            }
//        }
		return $html;
	}

	function pda_render_magic_link_settings() { ?>
		<div class="main_container">
			<form id="magic_link_setting_form">
				<input type="hidden" nonce="<?php echo wp_create_nonce( "magic_link_ajax_nonce" ); ?>"
				       id="magic_link_general_nonce"/>
				<table class="pda_magic_link_settings_table" cellpadding="4">
					<?php
					include PDA_MAGIC_LINK_BASE_DIR . 'includes/views/pda/view-pda-wp-magic-link-expiry-time.php';
					//					include PDA_MAGIC_LINK_BASE_DIR . 'includes/views/pda/view-pda-wp-magic-link-limit-click.php';
					include PDA_MAGIC_LINK_BASE_DIR . 'includes/views/pda/view-pda-wp-magic-link-fap.php';
					include PDA_MAGIC_LINK_BASE_DIR . 'includes/views/pda/view-pda-wp-magic-link-search-and-replace.php';
					?>
				</table>
				<?php submit_button(); ?>
			</form>
		</div>
		<?php
	}

	function ppp_render_magic_link_settings() { ?>
		<div class="main_container">
			<form id="ppp_magic_link_setting_form">
				<input type="hidden" nonce="<?php echo wp_create_nonce( "magic_link_ajax_nonce" ); ?>"
				       id="magic_link_general_nonce"/>
				<table class="pda_magic_link_settings_table ppp_magic_link_settings" cellpadding="4">
					<?php
					include PDA_MAGIC_LINK_BASE_DIR . 'includes/views/ppp/view-pda-wp-magic-link-expiry-time.php';
					include PDA_MAGIC_LINK_BASE_DIR . 'includes/views/ppp/view-pda-wp-magic-link-delete-expiry-link.php';
					?>
				</table>
				<?php submit_button(); ?>
			</form>
		</div>
		<?php
	}

	function update_option_setting_magic() {
		$nonce = $_REQUEST['security_check'];
		if ( ! wp_verify_nonce( $nonce, 'magic_link_ajax_nonce' ) ) {
			wp_die( 'invalid_nonce' );
		}
		$keys             = [];
		$request_settings = $_REQUEST['settings'];
		foreach ( $request_settings as $key => $value ) {
			array_push( $keys, $key );
		}
		$options_mg_db = unserialize( get_option( "pda_settings_magic_link" ) );
		foreach ( $keys as $k ) {
			$options_mg_db[ $k ] = $request_settings[ $k ];
		}
		$pda_mg = serialize( $options_mg_db );
		update_option( 'pda_settings_magic_link', $pda_mg );
		wp_send_json( true );
		wp_die();
	}

	function delete_private_link_expired() {
		$repo            = new Pda_Wp_Magic_Link_Repo();
		$is_activate_ppp = Yme_Plugin_Utils::is_plugin_activated( 'pap_gold_old' ) === - 1 || Yme_Plugin_Utils::is_plugin_activated( 'pap_gold' ) === - 1 ? true : false;

		// Delete default private link expired P&P
		if ( $is_activate_ppp ) {
			$ppp_table_name = $this->get_table_name( 'pur_private_links_gold' );
			$repo->delete_default_private_link( "expired_date", "hits_count", "limit_downloads", $ppp_table_name );
		}

		// Delete default private link expired PDA Gold
		if ( class_exists( 'PDA_Gold_Repository' ) || class_exists( 'PDA_v3_Gold_Repository' ) ) {
			$pda_table_name = $this->get_table_name( 'prevent_direct_access' );
			$repo->delete_default_private_link( "expired_date", "hits_count", "limit_downloads", $pda_table_name );
		}
	}

	function check_user_access_private_link( $advance_file ) {
		$link_id           = $advance_file->ID;
		$all_roles         = explode( ";", $advance_file->roles );
		$current_user_role = wp_get_current_user()->roles;
		if ( is_super_admin( wp_get_current_user()->ID ) ) {
			array_push( $current_user_role, 'administrator' );
		}

		if ( ! empty( array_intersect( $current_user_role, $all_roles ) ) ) {
			$repo      = new Pda_Wp_Magic_Link_Repo();
			$data_link = $repo->get_data_by_private_link_id( $link_id, wp_get_current_user()->ID );
			if ( $data_link === null ) {
				$repo->insert_data( $link_id );

				return true;
			} else {
				$private_link_download = $repo->get_data_by_private_link_id( $link_id, wp_get_current_user()->ID );
				if ( $advance_file->expired_date !== null ) {
					$expire_day            = date( 'Y-m-d', $advance_file->expired_date );
					$create_day            = date_format( date_create( $advance_file->time ), 'Y-m-d' );
					$date1                 = new DateTime( $expire_day );
					$date2                 = new DateTime( $create_day );
					$result                = $date1->diff( $date2 );
					$days                  = $result->days;
					$expired_day_timestamp = strtotime( '+' . $days . ' days', $private_link_download->expired_date );
					$curr_date             = new DateTime();
					$is_valid_expired_date = $private_link_download->expired_date === null || $curr_date->getTimestamp() < $expired_day_timestamp;
				} else {
					$is_valid_expired_date = true;
				}

				$is_valid_access_download = $advance_file->limit_downloads === null || (int) $private_link_download->downloads < (int) $advance_file->limit_downloads;

				if ( $is_valid_access_download && $is_valid_expired_date ) {
					$repo->update_data( $link_id );

					return true;
				}
			}
		}

		return false;
	}

	public function handle_s3_type_for_user_private_link( $attachment_id, $type ) {
		if ( function_exists( 'pda_s3_check_file_synced_s3' ) && pda_s3_check_file_synced_s3( $attachment_id ) ) {
			if ( PDA_v3_Constants::PDA_PRIVATE_LINK_S3_USER === $type || PDA_v3_Constants::PDA_PRIVATE_LINK_USER === $type ) {
				if ( class_exists( 'PDA_S3_UTILS' ) ) {
					$s3_utils   = new PDA_S3_UTILS;
					$signed_url = $s3_utils->create_presigned_url( $attachment_id, '5 seconds' );
					error_log( 'Singled URL: ' . $signed_url );
					header( "HTTP/1.1 301 Moved Permanently" );
					header( "Location: $signed_url" );
					exit();
				}
			}
		}
	}

	function render_tooltip( $key ) {
		$content = tooltip_content( $key ); ?>
		<span title="<?php echo $content ?>" class="dashicons dashicons-warning pda-v3-gold-tooltip"></span>
		<?php
	}

	/**
	 * Replace original link by default private link in srcset
	 *
	 * @param $sources
	 * @param $size_array
	 * @param $image_src
	 * @param $image_meta
	 * @param $attachment_id
	 *
	 * @return array
	 */
	function handle_calculate_image_srcset( $sources, $size_array, $image_src, $image_meta, $attachment_id ) {
		if ( is_admin() ) {
			return $sources;
		}
		if ( ! class_exists( 'PDA_v3_Gold_Repository' ) ) {
			return $sources;
		}
		$repo = new PDA_v3_Gold_Repository();
		if ( $repo->is_protected_file( $attachment_id ) ) {
			if ( check_condition_create_default_private_link() ) {
				$original_url = wp_get_attachment_url( $attachment_id );
				$sources      = array_map( function ( $source ) use ( $attachment_id, $original_url ) {
					$is_calculated_size = wp_basename( $original_url ) !== wp_basename( $source['url'] );
					$source['url']      = replace_original_link_by_default_private_link( $attachment_id, $source['url'], $is_calculated_size );

					return $source;
				}, $sources );
			}
		}

		return $sources;
	}

	/**
	 * Function handle image meta for srcset
	 *
	 * @param $image_meta
	 * @param $size_array
	 * @param $image_src
	 * @param $attachment_id
	 *
	 * @return image meta
	 */
	public function handle_calculate_image_srcset_meta( $image_meta, $size_array, $image_src, $attachment_id ) {
		if ( is_admin() ) {
			return $image_meta;
		}
		if ( ! class_exists( 'PDA_v3_Gold_Repository' ) ) {
			return $image_meta;
		}
		$repo_v3 = new PDA_v3_Gold_Repository();
		if ( ! $repo_v3->is_protected_file( $attachment_id ) ) {
			return $image_meta;
		}
		if ( empty( $image_meta['file'] ) ) {
			return $image_meta;
		}
		if ( ! check_condition_create_default_private_link( $attachment_id ) ) {
			return $image_meta;
		}
		if ( false !== strpos( $image_src, $image_meta['file'] ) ) {
			return $image_meta;
		}
		$image_basename     = wp_basename( $image_meta['file'] );
		$image_meta['file'] = rawurlencode( $image_basename );
		if ( ! empty( $image_meta['sizes'] ) ) {
			$image_meta['sizes'] = array_map( function ( $size ) {
				$size['file'] = rawurlencode( $size['file'] );

				return $size;
			}, $image_meta['sizes'] );
		}

		return $image_meta;
	}

	/**
	 * Get table name
	 *
	 * @param string $table_name Table name.
	 *
	 * @return string
	 */
	private function get_table_name( $table_name ) {
		global $wpdb;
		$this->wpdb = &$wpdb;

		return $wpdb->prefix . $table_name;
	}

	/**
	 * @param $baseurl
	 * @param $url_file
	 *
	 * @return mixed
	 */
	private function get_attachment_from_url( $baseurl, $url_file ) {
		$meta_value     = str_replace( $baseurl . '/', "", $url_file );
		$arr_meta_value = explode( "/", $meta_value );
		if ( ! in_array( "_pda", $arr_meta_value ) ) {
			array_unshift( $arr_meta_value, "_pda" );
		}
		$meta_value = implode( "/", $arr_meta_value );

		return get_posts( array(
			'post_type'  => 'attachment',
			'meta_key'   => '_wp_attached_file',
			'meta_value' => $meta_value
		) );
	}

	/**
	 * Handle all logic need to run in admin page.
	 */
	public function handle_admin_init() {
		install_pda_download_table();
	}

}
