(function( $ ) {
	'use strict';

	/**
	 * All of the code for your admin-facing JavaScript source
	 * should reside in this file.
	 *
	 * Note: It has been assumed you will write jQuery code here, so the
	 * $ function reference has been prepared for usage within the scope
	 * of this function.
	 *
	 * This enables you to define handlers, for when the DOM is ready:
	 *
	 * $(function() {
	 *
	 * });
	 *
	 * When the window is loaded:
	 *
	 * $( window ).load(function() {
	 *
	 * });
	 *
	 * ...and/or other possibilities.
	 *
	 * Ideally, it is not considered best practise to attach more than a
	 * single DOM-ready or window-load handler for a particular page.
	 * Although scripts in the WordPress core, Plugins and Themes may be
	 * practising this, we should strive to set a better example in our own work.
	 */

    $(function() {
        $(".pda-select2").select2({
            'width': '100%',
        });
        $("#pda_replace_all_page_post").change(function() {
            if ($("#pda_replace_all_page_post").prop('checked') === true) {
                $("#pda-access-only-page-post").show();
                $("#pda_only_replace_page_post_select2").prop('required', true);
            } else {
                $("#pda-access-only-page-post").hide();
                $("#pda_only_replace_page_post_select2").prop('required', false);
            }
        });
        $("#pda_replace_all_page_post").trigger("change");

        $('#magic_link_setting_form').submit(function(evt) {
            evt.preventDefault();
            evt.stopImmediatePropagation();

            var time_die = $('#mg_time_die_private_link').val();
            var units_times = $('#mg_units_times_die_private_link').val();
            if (typeof(time_die) == "undefined") {
                time_die = '7';
                units_times = 'days';
            } else {
                if (time_die.length == 0 || parseInt(time_die) <= 0) {
                    time_die = '7';
                    units_times = 'days';
                }
            }

            _updateSettingsMagicLink({
                mg_time_die_private_link: time_die + " " + units_times,
                whitelist_roles: $("#pda_role_select2").val(),
                file_access_permission: $("#file_access_permission").val(),
                pda_replace_default_private_link_all_page_post: $("#pda_replace_all_page_post").prop('checked'),
                pda_replace_default_private_link_only_page_post: $("#pda_only_replace_page_post_select2").val(),
                // pda_magic_link_download_limit: $("#pda_magic_link_download_limit").val(),
            }, function (error) {
                if(error) {
                    console.error(error);
                }
            });

        });

        $('#ppp_magic_link_setting_form').submit(function(evt) {
            evt.preventDefault();
            evt.stopImmediatePropagation();

            var time_die = $('#ppp_time_die_private_link').val();
            var units_times = $('#ppp_units_times_die_private_link').val();
            if (typeof(time_die) == "undefined") {
                time_die = '7';
                units_times = 'days';
            } else {
                if (time_die.length == 0 || parseInt(time_die) <= 0) {
                    time_die = '7';
                    units_times = 'days';
                }
            }
            _updateSettingsMagicLink({
                ppp_time_die_private_link: time_die + " " + units_times,
            }, function (error) {
                if(error) {
                    console.error(error);
                }
            });

        });
    });

    function _updateSettingsMagicLink(settings, cb) {
        var _data = {
            action: 'update_option_setting_magic',
            settings: settings,
            security_check: $("#magic_link_general_nonce").attr('nonce'),
        }
        $.ajax({
            url: pda_magic_link_data.ajax_url,
            type: 'POST',
            data: _data,
            success: function(data) {
                if (data == 'invalid_nonce') {
                    alert('No! No! No! Verify Nonce Fails!');
                } else if(data) {
                    toastr.success('Settings has been updated successfully!', 'PDA Private Magic Links')
                } else {
                    console.log("Failed", data);
                }
                cb();
            },
            error: function(error) {
                cb(error);
            },
            timeout: 5000
        });
    }

})( jQuery );
