<?php

function tooltip_content( $key ) {
	$contents = array(
		'default-expiry-time-for-private-magic-links'    => 'How long private magic links should be valid before expiration?',
		'auto-delete-all-expired-magic-links'            => 'Deprecated in upcoming version. Default: true.',
		'file-access-permission-for-private-magic-links' => 'Who can see or access these private magic links?',
		'apply-private-magic-links-for-page'             => 'Only turn on magic links for these pages. Highly recommended to enable for performance purposes. Default: all pages.',
	);

	return $contents[ $key ];
}

/**
 * @param $nameSettings
 *
 * @return bool
 */
function magic_link_get_settings( $nameSettings ) {
	$settings = get_option( "pda_settings_magic_link" );

	return maglic_link_check_option_settings( $settings, $nameSettings );
}

/**
 * @param $settings
 * @param $nameSettings
 *
 * @return bool
 */
function maglic_link_check_option_settings( $settings, $nameSettings ) {
	if ( $settings ) {
		$options = unserialize( $settings );
		if ( array_key_exists( $nameSettings, $options ) && ( $options[ $nameSettings ] == "true" || $options[ $nameSettings ] == "1" ) ) {
			return true;
		}
	}

	return false;
}

/**
 * @param $name
 *
 * @return array|string
 */
function magic_link_get_setting_values( $name ) {
	$settings = get_option( "pda_settings_magic_link" );
	if ( $settings ) {
		$options = @unserialize( $settings );
		if ( false === $options || empty( $options ) ) {
			return array();
		}
		if ( array_key_exists( $name, $options ) && ! empty( $options[ $name ] ) ) {
			return $options[ $name ];
		}
	}

	return array();
}

/**
 * Get setting type is string
 *
 * @param $name_settings
 *
 * @return string
 */
function pda_magic_link_get_setting_type_is_string( $name_settings ) {
	$settings = get_option( 'pda_settings_magic_link', false );
	if ( $settings ) {
		$options = @unserialize( $settings );
		if ( false === $options || empty( $options ) ) {
			return '';
		}
		if ( array_key_exists( $name_settings, $options ) && $options[ $name_settings ] !== '' ) {
			return $options[ $name_settings ];
		}
	}

	return '';
}

/**
 *
 */
function install_pda_download_table() {
	$db = new PDA_Magic_Link_DB();
	$db->install();
}

function check_version_for_force_default_delete_private_link( $version ) {
	$current_version = PML_VERSION;
	$current_version = (int) str_replace( '.', '', $current_version );
	if ( $current_version <= $version ) {
		return true;
	}

	return false;
}

function check_condition_create_default_private_link() {
	if ( Yme_Plugin_Utils::is_plugin_activated( 'pda_gold' ) === - 1 || Yme_Plugin_Utils::is_plugin_activated( 'pda_v3' ) === - 1 ) {
		if ( ! magic_link_get_settings( PDA_Magic_Link_Constants::PDA_REPLACE_ALL_PAGE_POST ) ) {
			return check_fap_magic_link();
		} else {
			$selected_posts_id = magic_link_get_setting_values( PDA_Magic_Link_Constants::PDA_REPLACE_ONLY_PAGE_POST );
			if ( in_array( get_the_ID(), $selected_posts_id ) ) {
				return check_fap_magic_link();
			}
		}
	}

	return false;
}

function check_fap_magic_link() {
	$current_user_role = wp_get_current_user()->roles;
	$pda_file_access   = magic_link_get_setting_values( 'file_access_permission' );

	switch ( $pda_file_access ) {
		case "custom_roles":
			$user_Login      = wp_get_current_user()->roles;
			$whitelist_roles = magic_link_get_setting_values( 'whitelist_roles' );
			if ( ! empty( array_intersect( $user_Login, $whitelist_roles ) ) ) {
				return true;
			}
			break;
		case "admin_users":
			if ( in_array( "administrator", $current_user_role ) ) {
				return true;
			}
			break;
		case "author":
			global $post;
			$author_id = $post->post_author;
			if ( get_current_user_id() == $author_id ) {
				return true;
			}
			break;
		case "logged_users":
			if ( is_user_logged_in() ) {
				return true;
			}
			break;
		case "anyone":
			return true;
		case "blank":
			return false;
	}

	return false;
}

function replace_original_link_by_default_private_link( $attachment_id, $original_link, $is_calculated_size = true ) {
	$settings         = new Pda_Gold_Functions();
	$pda_prefix_url   = $settings->prefix_roles_name( PDA_v3_Constants::PDA_PREFIX_URL );
	$admin_magic_link = new Pda_Wp_Magic_Link_Admin( '', '' );
	$private_link     = $admin_magic_link->get_default_private_link_pdav3( $attachment_id );

	if ( $is_calculated_size ) {
		list ( $size ) = pml_remove_size_from_url( $original_link );
		if ( ! empty( $size ) ) {
			$private_link = $private_link . '?size=' . str_replace( '-', '', $size );
		}
	}

	return pml_get_home_url_with_ssl() . $pda_prefix_url . '/' . $private_link;
}

function pml_get_home_url_with_ssl() {
	return is_ssl() ? home_url( '/', 'https' ) : home_url( '/' );
}

/**
 * Remove size from URL.
 *
 * @param $url_file
 *
 * @return array
 */
function pml_remove_size_from_url( $url_file ) {
	$default_results = array( '', $url_file );

	if ( ! pml_is_image_type( $url_file ) ) {
		return $default_results;
	}

	if ( class_exists( 'Pda_v3_Gold_Helper' ) && method_exists( 'Pda_v3_Gold_Helper', 'get_image_size_of_link' ) ) {
		$gold_helper = new Pda_v3_Gold_Helper();

		return $gold_helper->get_image_size_of_link( $url_file );
	}

	// TODO: The above snippet also handle the logic as get_image_size_of_link function belonged to class Pda_v3_Gold_Helper. We will remove this when all users updated to lasted version. This will help the current user fix the file size problems without upgrading the lasted Gold.
	$file_size_regex = '(-\d+x\d+\.\w+)'; // -x-y.jpg with x, y is positive number.
	preg_match_all( $file_size_regex, $url_file, $matches, PREG_PATTERN_ORDER );

	$found = end( $matches[0] );

	if ( empty( $found ) ) {
		return $default_results;
	}

	$tmp  = explode( '.', $found );
	$size = $tmp[0];
	$ext  = $tmp[1];

	$url_file = str_replace( $found, ".$ext", $url_file );

	return array( $size, $url_file );
}

function pml_is_image_type( $file_name ) {
	preg_match( '/\.(gif|jpg|jpe?g|tiff|png|bmp|webp)$/i', $file_name, $matches );

	return ! empty( $matches );
}

/**
 * Check condition to hide "Check for updates" button.
 *
 * @return bool
 * Return false: If all main plugins is deactivate
 * Else: return true
 */
function magic_link_main_plugins_are_deactivated() {
	$main_plugins = array( 'pda_gold', 'pap_gold', 'pda_v3', 'pap_gold_old' );
	foreach ( $main_plugins as $main_plugin ) {
		if ( - 1 === Yme_Plugin_Utils::is_plugin_activated( $main_plugin ) ) {
			return false;
		}
	}

	return true;
}

/**
 * Check current page is plugin page.
 *
 * @return bool
 */
function magic_link_is_plugin_page() {
	global $pagenow;
	if ( ! is_multisite() ) {
		return 'plugins.php' === $pagenow;
	}

	return is_network_admin();
}

/**
 * Get home URL following SSL
 *
 * @return string
 */
function magic_link_get_home_url_with_ssl() {
	return is_ssl() ? home_url( '/', 'https' ) : home_url( '/' );
}
