<?php

/**
 *
 * This file is read by WordPress to generate the plugin information in the plugin
 * admin area. This file also includes all of the dependencies used by the plugin,
 * registers the activation and deactivation functions, and defines a function
 * that starts the plugin.
 *
 * @link              www.buildwps.com/prevent-direct-access-premium-plugin/?utm_source=user-website&utm_medium=pluginpage&utm_campaign=plugin-author-link
 * @since             1.0.12.7
 * @package           Pda_Wp_Magic_Link
 *
 * @wordpress-plugin
 * Plugin Name:       PDA Private Magic Links
 * Plugin URI:        https://preventdirectaccess.com/extensions/private-magic-links/?utm_source=user-website&utm_medium=plugin_link&utm_campaign=pda-magic-link
 * Description:       Auto expire and dynamically update all private links of your protected files on your content without having to do any manual work.
 * Version:           1.0.12.7
 * Author:            BWPS
 * Author URI:        https://preventdirectaccess.com/extensions/private-magic-links/?utm_source=user-website&utm_medium=plugin_link&utm_campaign=pda-magic-link
 * License:           GPL-2.0+
 * License URI:       http://www.gnu.org/licenses/gpl-2.0.txt
 * Text Domain:       pda-wp-magic-link
 * Domain Path:       /languages
 */

// If this file is called directly, abort.
if ( ! defined( 'WPINC' ) ) {
	die;
}
define( 'PDA_MAGIC_LINK_BASE_DIR', plugin_dir_path( __FILE__ ) );
/**
 * Currently pligin version.
 * Start at version 1.0.0 and use SemVer - https://semver.org
 * Rename this for your plugin and update it as you release new versions.
 */
define( 'PML_VERSION', '1.0.12.7' );

if ( ! defined( 'PDA_MAGIC_LINK_PLUGIN_FOLDER' ) ) {
	define( 'PDA_MAGIC_LINK_PLUGIN_FOLDER', 'pda-magic-link/pda-wp-magic-link.php' );
}
/**
 * The code that runs during plugin activation.
 * This action is documented in includes/class-pda-wp-magic-link-activator.php
 */
function activate_pda_wp_magic_link() {
	require_once plugin_dir_path( __FILE__ ) . 'includes/class-pda-wp-magic-link-activator.php';
	require_once plugin_dir_path( __FILE__ ) . 'includes/class-pda-wp-magic-link-utils.php';
	Pda_Wp_Magic_Link_Activator::activate( plugin_basename( __FILE__ ) );
}

/**
 * The code that runs during plugin deactivation.
 * This action is documented in includes/class-pda-wp-magic-link-deactivator.php
 */
function deactivate_pda_wp_magic_link() {
	require_once plugin_dir_path( __FILE__ ) . 'includes/class-pda-wp-magic-link-deactivator.php';
	Pda_Wp_Magic_Link_Deactivator::deactivate();
}

register_activation_hook( __FILE__, 'activate_pda_wp_magic_link' );
register_deactivation_hook( __FILE__, 'deactivate_pda_wp_magic_link' );

/**
 * The core plugin class that is used to define internationalization,
 * admin-specific hooks, and public-facing site hooks.
 */
require plugin_dir_path( __FILE__ ) . 'includes/class-pda-wp-magic-link.php';

/**
 * Remove PDA Magic Link plugin in list plugins update of WP.
 *
 * @param object $value Data update cache.
 *
 * @return object $value Data update cache.
 */
function magic_link_filter_plugin_updates( $value ) {
	if ( ! isset( $value->response[ PDA_MAGIC_LINK_PLUGIN_FOLDER ] ) ) {
		return $value;
	}
	unset( $value->response[ PDA_MAGIC_LINK_PLUGIN_FOLDER ] );

	return $value;
}

/**
 * Begins execution of the plugin.
 *
 * Since everything within the plugin is registered via hooks,
 * then kicking off the plugin from this point in the file does
 * not affect the page life cycle.
 *
 * @since    1.0.0
 */
function run_pda_wp_magic_link() {
	$plugin = new Pda_Wp_Magic_Link();
	$plugin->run();

	if ( ! magic_link_main_plugins_are_deactivated() ) {
		$configs = require PDA_MAGIC_LINK_BASE_DIR . '/includes/class-pda-wp-magic-link-configs.php';
		$url     = $configs->update_url;
		if ( method_exists( 'Puc_v4p8_Factory', 'buildUpdateChecker' ) ) {
			Puc_v4p8_Factory::buildUpdateChecker(
				$url,
				__FILE__,
				'pda-magic-link'
			);
		} elseif ( method_exists( 'Puc_v4_Factory', 'buildUpdateChecker' ) ) {
			Puc_v4_Factory::buildUpdateChecker(
				$url,
				__FILE__,
				'pda-magic-link'
			);
		} else {
			add_filter( 'site_transient_update_plugins', 'magic_link_filter_plugin_updates' );
		}
	} else {
		add_filter( 'site_transient_update_plugins', 'magic_link_filter_plugin_updates' );
	}
}

add_action( 'plugins_loaded', 'run_pda_wp_magic_link', 20 );
