<?php

/**
 * The admin-specific functionality of the plugin.
 *
 * @link       preventdirectaccess.com
 * @since      1.0.0
 *
 * @package    Pda_Membership_Integration
 * @subpackage Pda_Membership_Integration/admin
 */

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    Pda_Membership_Integration
 * @subpackage Pda_Membership_Integration/admin
 * @author     BWPS <hello@preventdirectaccess.com>
 */
class Pda_Membership_Integration_Admin {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string $plugin_name The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string $version The current version of this plugin.
	 */
	private $version;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @param string $plugin_name The name of this plugin.
	 * @param string $version The version of this plugin.
	 *
	 * @since    1.0.0
	 *
	 */
	public function __construct( $plugin_name, $version ) {

		$this->plugin_name = $plugin_name;
		$this->version     = $version;

	}

	/**
	 * Register the stylesheets for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_styles() {

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in Pda_Membership_Integration_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The Pda_Membership_Integration_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */
		// Enqueue css for page media
		if ( "upload" === get_current_screen()->id ) {
			wp_enqueue_style( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'css/pda-membership-integration-admin.css', array(), $this->version, 'all' );
			wp_enqueue_style( $this->plugin_name . '-membership-component', plugin_dir_url( __FILE__ ) . 'js/dist/style/membership-form.css', array(), $this->version, 'all' );
		}

	}

	/**
	 * Register the JavaScript for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_scripts() {

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in Pda_Membership_Integration_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The Pda_Membership_Integration_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */

		if ( "upload" === get_current_screen()->id ) {
			wp_enqueue_script( 'membership_integration_upload_js' . $this->plugin_name, plugin_dir_url( __FILE__ ) . './js/pda-membership-integration-admin.js', array( 'jquery' ), $this->version );
			wp_localize_script( 'membership_integration_upload_js' . $this->plugin_name, 'membership_integration_data', array( 'all_members' => $this->get_all_members() ) );
			wp_enqueue_script( $this->plugin_name . '-membership-form', plugin_dir_url( __FILE__ ) . 'js/dist/membership-form.js', array( 'prevent-direct-access-gold-v3-bundle-js' ), $this->version, false );
		}

	}

	public function integration_memberships_load_js_css() {
		wp_enqueue_script( 'membership_integration_setting_js' . $this->plugin_name, plugin_dir_url( __FILE__ ) . './js/pda-membership-integration-setting.js', array( 'jquery' ), $this->version );
		wp_localize_script( 'membership_integration_setting_js' . $this->plugin_name, 'membership_integration_data', array( 'ajax_url' => admin_url( 'admin-ajax.php' ) ) );

		wp_register_style( 'membership_integration_css_' . $this->plugin_name, plugin_dir_url( __FILE__ ) . './css/pda-membership-integration-admin-setting.css', array(), $this->version );
		wp_enqueue_style( 'membership_integration_css_' . $this->plugin_name );
	}

	public function pda_membership_integration_rest_api_init_cb() {
		register_rest_route( 'pda-membership-integration/v1', '/get-all-members', array(
			'methods'  => 'GET',
			'callback' => array( $this, 'get_all_members' ),
		) );

		register_rest_route( 'pda-membership-integration/v1', '/data-memberships', array(
			'methods'  => 'GET',
			'callback' => array( $this, 'pda_memberships_get_data_for_popup' ),
		) );

	}

	function pda_memberships_get_data_for_popup() {
		global $userAccessManager;

		return array(
			'user_access_manager'         => is_object( $userAccessManager ),
			'memberships_2'               => class_exists( 'MS_Model_Membership' ),
			'paid_memberships_pro'        => function_exists( 'pmpro_hasMembershipLevel' ),
			'woo_memberships'             => function_exists( 'wc_memberships_is_user_member' ),
			'woo_subscriptions'           => class_exists( 'WC_Subscriptions_Admin' ),
			'ar_member'                   => is_plugin_active( 'armember/armember.php' ),
			'restrict_content_pro'        => is_plugin_active( 'restrict-content-pro/restrict-content-pro.php' ),
			'simple_wordpress_membership' => is_plugin_active( 'simple-membership/simple-wp-membership.php' ),
		);
	}

	function get_all_members() {
		global $userAccessManager;
		$user_access_manager = [];
		if ( is_object( $userAccessManager ) ) {
			$users = $this->get_uam_user_groups();
			foreach ( $users as $user ) {
				array_push( $user_access_manager, $user['name'] );
			}
		}

		$memberships_2 = [];
		if ( class_exists( 'MS_Model_Membership' ) ) {
			$member_2    = MS_Factory::load( 'MS_Model_Membership', [] );
			$memberships = $member_2->get_membership_names();
			foreach ( $memberships as $member ) {
				array_push( $memberships_2, $member );
			}
		}

		$paid_membership_pro = [];
		if ( function_exists( 'pmpro_hasMembershipLevel' ) ) {
			$membership_pro = $this->get_data_memberships_pro();
			foreach ( $membership_pro as $member ) {
				array_push( $paid_membership_pro, $member->name );
			}
		}

		$woo_memberships             = get_all_membership_plans_for_woo_memberships();
		$woo_subscriptions           = get_all_id_items_in_woo_subscriptions();
		$ar_member                   = get_all_membership_plans_for_ar_member();
		$restrict_content_pro        = get_all_memberships_level_of_restrict_content_pro();
		$simple_wordpress_membership = pda_mem_get_all_memberships_level_for_simple_memberships();

		return [
			"user_access_manager"         => $user_access_manager,
			"memberships_2"               => $memberships_2,
			"paid_memberships_pro"        => $paid_membership_pro,
			"woo_memberships"             => $woo_memberships,
			"woo_subscriptions"           => $woo_subscriptions,
			"ar_member"                   => $ar_member,
			"restrict_content_pro"        => $restrict_content_pro,
			"simple_wordpress_membership" => $simple_wordpress_membership,
		];
	}

	/**
	 * Check have Membership support.
	 *
	 * @param bool $is_bulk_action is bulk action.
	 *
	 * @return bool
	 */
	public function do_not_have_any_membership_plugins( $is_bulk_action = false ) {
		global $userAccessManager;

		$has_support = ! is_object( $userAccessManager )
		               && ! function_exists( 'pmpro_hasMembershipLevel' )
		               && ! function_exists( 'wc_memberships_is_user_member' )
		               && ! class_exists( "WC_Subscriptions_Admin" )
		               && ! is_plugin_active( 'armember/armember.php' )
		               && ! is_plugin_active( 'simple-membership/simple-wp-membership.php' );

		if ( $is_bulk_action ) {
			return $has_support;
		}

		return $has_support && ! is_plugin_active( 'restrict-content-pro/restrict-content-pro.php' ) && ! class_exists( 'MS_Model_Membership' );
	}

	public function check_plugin_pda_and_pap_activate() {
		if (
			Yme_Plugin_Utils::is_plugin_activated( 'pda_gold' ) !== - 1 &&
			Yme_Plugin_Utils::is_plugin_activated( 'pda_v3' ) !== - 1 &&
			Yme_Plugin_Utils::is_plugin_activated( 'pap_gold' ) !== - 1 &&
			Yme_Plugin_Utils::is_plugin_activated( 'pap_gold_old' ) !== - 1
		) {
			$message = "Please install and activate <a target=\"_blank\" rel=\"noopener noreferrer\" href=\"https://preventdirectaccess.com/pricing/\">Prevent Direct Access Gold</a> or <a target=\"_blank\" rel=\"noopener noreferrer\" href=\"https://preventdirectaccess.com/protect-wordpress-pages-posts/\">Protect Pages & Posts Gold</a> plugin";
			?>
			<div class="notice notice-error is-dismissible">
				<p><b><?php _e( $this->plugin_name ); ?>: </b><?php _e( $message, 'prevent-direct-access-gold' ); ?></p>
			</div>
			<?php
		}

		/**
		 * Notices for plugin update
		 */
		$updates = $this->is_yme_plugin_has_updates();
		global $pagenow;
		if ( ! empty( $updates ) ) {
			foreach ( $updates as $update ) {
				if ( 'Prevent Direct Access Gold' === $update->Name ) {
					$position_notice = array( 'upload.php', 'plugins.php' );
					if ( in_array( $pagenow, $position_notice ) ) {
						?>
						<div class="notice notice-warning is-dismissible">
							<p>
								<b><?php _e( $this->plugin_name ); ?>: </b> Please
								update<b> <?php _e( $update->Name ); ?></b>
								to its latest version (<?php esc_html_e( $update->update->new_version ); ?>) for our
								plugins to work properly.
							</p>
						</div>
						<?php
					}
				} else if ( 'plugins.php' === $pagenow && ( is_plugin_active( 'armember/armember.php' ) || is_plugin_active( 'restrict-content-pro/restrict-content-pro.php' ) ) ) {
					?>
					<div class="notice notice-warning is-dismissible">
						<p>
							<b><?php _e( $this->plugin_name ); ?>: </b> Please
							update<b> <?php _e( $update->Name ); ?></b>
							to its latest version (<?php esc_html_e( $update->update->new_version ); ?>) for our plugins
							to work properly.
						</p>
					</div>
					<?php
				}
			}
		}
	}

	public function get_uam_user_groups() {
		global $userAccessManager;

		$userGroupHandler = $userAccessManager->getUserGroupHandler();
		$user_groups      = $userGroupHandler->getUserGroups();
		$groups           = array_map( function ( $group ) {
			return array(
				'name' => $group->getName(),
				'desc' => $group->getDescription()
			);
		}, $user_groups );

		return $groups;
	}

	public function render_memberships_integration_settings() {
		$settings = get_option( Pda_Membership_Constants::PDA_MEMBERSHIP_OPTIONS );
		$options  = [];
		if ( $settings ) {
			if ( false !== @unserialize( $settings ) ) {
				$options = @unserialize( $settings );
			}
		}
		global $userAccessManager;

		if ( $this->do_not_have_any_membership_plugins() ) {
			$notification = new Pda_Membership_Notification();
			echo $notification->render_notification();
		} else {
			?>
			<div class="main_container">
				<form id="membership_integration_form">
					<input type="hidden" nonce="<?php echo wp_create_nonce( 'pda_ajax_nonce_membership' ); ?>"
					       id="nonce_membership"/>
					<table class="pda_v3_settings_table pda_membership_table" cellpadding="4">
						<tr>
							<td colspan="2">
								<h3><?php esc_html_e( 'PLUGIN INTEGRATION', 'prevent-direct-access-gold' ) ?></h3>
								<?php esc_html_e( 'Allow the following roles & memberships to access your protected media files', 'prevent-direct-access-gold' ) ?>
							</td>
						</tr>

						<?php if ( is_object( $userAccessManager ) ) {
							include PDA_PLUGIN_DIR_PATH . '/includes/views/view-pda-membership-integration-uam.php';
						}
						if ( class_exists( 'MS_Model_Membership' ) ) {
							include PDA_PLUGIN_DIR_PATH . '/includes/views/view-pda-membership-integration-membership-2.php';
						}
						if ( function_exists( 'pmpro_hasMembershipLevel' ) ) {
							include PDA_PLUGIN_DIR_PATH . '/includes/views/view-pda-membership-integration-paid-membership-pro.php';
						}
						if ( function_exists( 'wc_memberships_is_user_member' ) ) {
							include PDA_PLUGIN_DIR_PATH . '/includes/views/view-pda-membership-integration-woo-membership.php';
						}
						if ( class_exists( "WC_Subscriptions_Admin" ) ) {
							include PDA_PLUGIN_DIR_PATH . '/includes/views/view-pda-membership-integration-woo-subscriptions.php';
						}
						if ( is_plugin_active( 'armember/armember.php' ) ) {
							include PDA_PLUGIN_DIR_PATH . '/includes/views/view-pda-membership-integration-ar-member.php';
						}
						if ( is_plugin_active( 'restrict-content-pro/restrict-content-pro.php' ) ) {
							include PDA_PLUGIN_DIR_PATH . '/includes/views/view-pda-membership-integration-restrict-content-pro.php';
						}
						if ( is_plugin_active( 'simple-membership/simple-wp-membership.php' ) ) {
							include PDA_PLUGIN_DIR_PATH . '/includes/views/view-pda-membership-integration-simple-membership.php';
						}
						?>

					</table>
					<input type="submit" value="<?php _e( 'Save Changes' ); ?>"
					       class="button button-primary pda_membership_button_save">
				</form>
			</div>
			<?php
		}
	}

	function get_all_items_in_woo_subcriptions() {
		$products        = wc_get_products( [
			'product_type' => 'subscription'
		] );
		$arr_product_ids = [];
		foreach ( $products as $product ) {
			array_push( $arr_product_ids, $product->get_id() );
		}

		return $arr_product_ids;
	}

	public function so_wp_ajax_update_membership_integration() {
		$nonce = $_REQUEST['security_check'];
		if ( ! wp_verify_nonce( $nonce, 'pda_ajax_nonce_membership' ) ) {
			error_log( 'not verify nonce', 0 );
			wp_die( 'invalid_nonce' );
		}
		$settings   = $_REQUEST['settings'];
		$membership = serialize( $settings );
		update_option( Pda_Membership_Constants::PDA_MEMBERSHIP_OPTIONS, $membership, 'no' );
		wp_send_json( true );
		wp_die();
	}

	function is_in_user_access_manager() {
		global $userAccessManager;
		if ( is_object( $userAccessManager ) && pda_membership_get_setting_boolen( Pda_Membership_Constants::PDA_MEMBERSHIP_APPLY_UAM ) ) {
			$user_access_manage = pda_membership_get_setting_array( 'membership_integration' );
			foreach ( $user_access_manage as $value ) {
				if ( Pda_Membership_Integration_Admin::uamIsUserInGroup( $value ) ) {
					return true;
				}
			}
		}

		return false;
	}

	function is_in_membership_2() {
		if ( class_exists( 'MS_Model_Membership' ) && pda_membership_get_setting_boolen( Pda_Membership_Constants::PDA_MEMBERSHIP_APPLY_MEMBERSHIP_2 ) ) {
			$membership_2 = pda_membership_get_setting_array( 'membership_2' );
			$membership   = Pda_Membership_Integration_Admin::memberships_2_Group();
			if ( ! empty( array_intersect( $membership_2, $membership ) ) ) {
				return true;
			}
		}

		return false;
	}

	function is_in_paid_memberships_pro() {
		if ( function_exists( 'pmpro_hasMembershipLevel' ) && pda_membership_get_setting_boolen( Pda_Membership_Constants::PDA_MEMBERSHIP_APPLY_PAID_MEMBERSHIP_PRO ) ) {
			$paid_membership_pro = pda_membership_get_setting_array( 'paidmembershippro' );
			$memberships         = Pda_Membership_Integration_Admin::get_memberships_pro_group();
			if ( ! empty( array_intersect( $paid_membership_pro, $memberships ) ) ) {
				return true;
			}
		}

		return false;
	}

	function is_in_woo_memberships() {
		if ( function_exists( 'wc_memberships_is_user_member' ) && pda_membership_get_setting_boolen( Pda_Membership_Constants::PDA_MEMBERSHIP_APPLY_WOO_MEMBERSHIP ) ) {
			$woo_memberships = pda_membership_get_setting_array( 'woo_memberships' );
			foreach ( $woo_memberships as $value ) {
				if ( wc_memberships_is_user_active_member( get_current_user_id(), $value ) ) {
					return true;
				}
			}
		}

		return false;
	}

	function is_in_woo_subscriptions() {
		if ( class_exists( "WC_Subscriptions_Admin" ) && pda_membership_get_setting_boolen( Pda_Membership_Constants::PDA_MEMBERSHIP_APPLY_WOO_SUBSCRIPTIONS ) ) {
			$woo_subscriptions = pda_membership_get_setting_array( 'woo_subcriptions' );
			foreach ( $woo_subscriptions as $product_id ) {
				if ( wcs_user_has_subscription( get_current_user_id(), $product_id, 'active' ) ) {
					return true;
				}
			}
		}

		return false;
	}

	function is_in_ar_member() {
		if ( is_plugin_active( 'armember/armember.php' ) && pda_membership_get_setting_boolen( Pda_Membership_Constants::PDA_MEMBERSHIP_APPLY_AR_MEMBER ) ) {
			$list_id_access = pda_membership_get_setting_array( 'ar_member' );
			if ( ! empty( $list_id_access ) ) {
				$user_plan_id = get_user_meta( get_current_user_id(), 'arm_user_plan_ids', true );
				if ( $user_plan_id ) {
					if ( ! empty( array_intersect( $list_id_access, $user_plan_id ) ) ) {
						return true;
					}
				}
			}
		}

		return false;
	}

	function is_in_restrict_content_pro() {
		if ( is_plugin_active( 'restrict-content-pro/restrict-content-pro.php' ) && pda_membership_get_setting_boolen( Pda_Membership_Constants::PDA_MEMBERSHIP_APPLY_RESTRICT_CONTENT_PRO ) ) {
			$pda_res_type = pda_membership_get_setting_string( Pda_Membership_Constants::PDA_MEMBERSHIP_RES_TYPE );
			if ( 'pda_access_level' === $pda_res_type ) {
				$pda_res_access_level = pda_membership_get_setting_string( Pda_Membership_Constants::PDA_MEMBERSHIP_RES_ACCESS_LEVEL );

				return rcp_user_has_access( get_current_user_id(), $pda_res_access_level );
			} elseif ( 'pda_memberships_level' === $pda_res_type ) {
				$list_id_access = pda_membership_get_setting_array( 'restrict_content_pro' );
				if ( ! empty( $list_id_access ) ) {
					$customer = rcp_get_customer_by_user_id( get_current_user_id() );
					if ( $customer ) {
						$membership = rcp_get_customer_single_membership( $customer->get_id() );
						if ( $membership ) {
							if ( in_array( $membership->get_object_id(), $list_id_access ) ) {
								return true;
							}
						}
					}
				}
			}
		}

		return false;
	}

	function is_in_simple_membership() {
		if ( ! is_plugin_active( 'simple-membership/simple-wp-membership.php' ) || ! method_exists( 'SwpmMembershipLevelUtils', 'get_all_membership_levels_in_array' ) || ! method_exists( 'SwpmMemberUtils', 'get_user_by_user_name' ) || ! pda_membership_get_setting_boolen( Pda_Membership_Constants::PDA_MEMBERSHIP_APPLY_SIMPLE_WORDPRESS_MEMBERSHIP ) ) {
			return false;
		}

		$level_ids_access = pda_membership_get_setting_array( Pda_Membership_Constants::PDA_MEMBERSHIP_INTEGRATION_SIMPLE_WORDPRESS_MEMBERSHIP );
		if ( empty( $level_ids_access ) ) {
			return false;
		}

		$member_info = SwpmMemberUtils::get_user_by_user_name( wp_get_current_user()->user_login );
		if ( is_null( $member_info ) ) {
			return false;
		}

		foreach ( $level_ids_access as $level_id ) {
			$members = SwpmMemberUtils::get_all_members_of_a_level( $level_id );
			if ( ! $members ) {
				continue;
			}

			$member_ids = array_column( $members, 'member_id' );
			if ( in_array( $member_info->member_id, $member_ids ) ) {
				return true;
			}
		}

		return false;
	}

	function check_memberships_fap() {
		return $this->is_in_user_access_manager() ||
		       $this->is_in_membership_2() ||
		       $this->is_in_paid_memberships_pro() ||
		       $this->is_in_woo_memberships() ||
		       $this->is_in_woo_subscriptions() ||
		       $this->is_in_ar_member() ||
		       $this->is_in_restrict_content_pro() ||
		       $this->is_in_simple_membership();
	}

	function pda_mem_check_fap_for_uam( $meta ) {
		global $userAccessManager;
		if ( ! is_object( $userAccessManager ) ) {
			return false;
		}

		foreach ( explode( ';', $meta['user_access_manager'] ) as $value ) {
			if ( Pda_Membership_Integration_Admin::uamIsUserInGroup( $value ) ) {
				return true;
			}
		}

		return false;
	}

	function pda_mem_check_fap_mem_2( $meta ) {
		if ( ! class_exists( 'MS_Model_Membership' ) ) {
			return false;
		}

		$membership = Pda_Membership_Integration_Admin::memberships_2_Group();
		if ( ! empty( array_intersect( explode( ';', $meta['memberships_2'] ), $membership ) ) ) {
			return true;
		}

		return false;
	}

	function pda_mem_check_fap_paid_mem_pro( $meta ) {
		if ( ! function_exists( 'pmpro_hasMembershipLevel' ) ) {
			return false;
		}

		$memberships = Pda_Membership_Integration_Admin::get_memberships_pro_group();
		if ( ! empty( array_intersect( explode( ';', $meta['paid_memberships_pro'] ), $memberships ) ) ) {
			return true;
		}

		return false;
	}

	function pda_mem_check_fap_woo_mem( $meta ) {
		if ( ! function_exists( 'wc_memberships_is_user_member' ) ) {
			return false;
		}

		foreach ( explode( ';', $meta['woo_memberships'] ) as $value ) {
			if ( wc_memberships_is_user_active_member( get_current_user_id(), $value ) ) {
				return true;
			}
		}

		return false;
	}

	function pda_mem_check_fap_woo_sub( $meta ) {
		if ( ! class_exists( "WC_Subscriptions_Admin" ) ) {
			return false;
		}

		foreach ( explode( ';', $meta['woo_subscriptions'] ) as $product_id ) {
			if ( wcs_user_has_subscription( get_current_user_id(), $product_id, 'active' ) ) {
				return true;
			}
		}

		return false;
	}

	function pda_mem_check_fap_ar_mem( $meta ) {
		if ( ! is_plugin_active( 'armember/armember.php' ) ) {
			return false;
		}

		if ( empty( $meta['ar_member'] ) ) {
			return false;
		}

		$user_plan_id = get_user_meta( get_current_user_id(), 'arm_user_plan_ids', true );
		if ( ! $user_plan_id ) {
			return false;
		}

		$plan_id = explode( ';', $meta['ar_member'] );
		if ( ! empty( array_intersect( $plan_id, $user_plan_id ) ) ) {
			return true;
		}

		return false;
	}

	function pda_mem_check_fap_restrict_content_pro( $meta ) {
		if ( ! is_plugin_active( 'restrict-content-pro/restrict-content-pro.php' ) || ! function_exists( 'rcp_get_customer_by_user_id' ) || ! function_exists( 'rcp_get_customer_single_membership' ) ) {
			return false;
		}

		$customer = rcp_get_customer_by_user_id( get_current_user_id() );
		if ( ! $customer ) {
			return false;
		}

		$membership = rcp_get_customer_single_membership( $customer->get_id() );
		if ( ! $membership ) {
			return false;
		}

		$restrict_content_pro = $meta['restrict_content_pro'];
		if ( empty( $restrict_content_pro ) ) {
			return false;
		}

		$res_type = $restrict_content_pro['type'];
		if ( 'pda-access-level' === $res_type ) {
			$res_id = $restrict_content_pro['id'];
			if ( empty( $res_id[0] ) && "0" !== $res_id[0] ) {
				return false;
			}

			if ( rcp_user_has_access( get_current_user_id(), $res_id[0] ) ) {
				return true;
			}
		} else if ( 'pda-membership-level' === $res_type ) {
			$res_id = $restrict_content_pro['id'];
			if ( in_array( $membership->get_object_id(), $res_id ) ) {
				return true;
			}
		}

		return false;
	}

	function pda_mem_check_fap_simple_mem( $meta ) {
		if ( ! is_plugin_active( 'simple-membership/simple-wp-membership.php' ) || ! method_exists( 'SwpmMembershipLevelUtils', 'get_all_membership_levels_in_array' ) || ! method_exists( 'SwpmMemberUtils', 'get_user_by_user_name' ) ) {
			return false;
		}

		$member_info = SwpmMemberUtils::get_user_by_user_name( wp_get_current_user()->user_login );
		if ( is_null( $member_info ) ) {
			return false;
		}

		$level_id_selected = explode( ';', $meta['simple_wordpress_membership'] );
		foreach ( $level_id_selected as $level_id ) {
			$members = SwpmMemberUtils::get_all_members_of_a_level( $level_id );
			if ( ! $members ) {
				continue;
			}

			$member_ids = array_column( $members, 'member_id' );
			if ( in_array( $member_info->member_id, $member_ids ) ) {
				return true;
			}
		}

		return false;
	}

	function pda_check_access_for_membership_integration( $attachment_id ) {
		$meta = get_post_meta( $attachment_id, PDA_v3_Constants::$pda_meta_key_memberships_integration, true );
		if ( 'no-memberships' === $meta ) {
			return false;
		}

		if ( 'memberships-default' === $meta || empty( $meta ) ) {
			return $this->check_memberships_fap();
		}

		$fap_uam        = $this->pda_mem_check_fap_for_uam( $meta );
		$fap_mem_2      = $this->pda_mem_check_fap_mem_2( $meta );
		$fap_paid_pro   = $this->pda_mem_check_fap_paid_mem_pro( $meta );
		$fap_woo_mem    = $this->pda_mem_check_fap_woo_mem( $meta );
		$fap_woo_sub    = $this->pda_mem_check_fap_woo_sub( $meta );
		$fap_ar_mem     = $this->pda_mem_check_fap_ar_mem( $meta );
		$fap_res_pro    = $this->pda_mem_check_fap_restrict_content_pro( $meta );
		$fap_simple_mem = $this->pda_mem_check_fap_simple_mem( $meta );

		return $fap_uam || $fap_mem_2 || $fap_paid_pro || $fap_woo_mem || $fap_woo_sub || $fap_ar_mem || $fap_res_pro || $fap_simple_mem;
	}

	//get membershippro of user
	private function get_memberships_pro_group( $user_member_pro = '' ) {
		if ( empty ( $user_member_pro ) ) {
			$user_member_pro = get_current_user_id();
		}

		$results = [];

		if ( ! function_exists( 'pmpro_hasMembershipLevel' ) ) {
			return $results;
		}

		$get_membership_level = pmpro_getMembershipLevelForUser( $user_member_pro );
		if ( is_object( $get_membership_level ) ) {
			array_push( $results, $get_membership_level->name );
		}

		return $results;
	}

	// get name paid memberships pro
	public function get_data_memberships_pro() {
		global $wpdb;
		$table_name = $wpdb->prefix . "pmpro_membership_levels";
		$data       = $wpdb->get_results( "SELECT name FROM $table_name " );

		return $data;
	}

	private function memberships_2_Group( $userMember = '' ) {
		if ( empty ( $userMember ) ) {
			$userMember = get_current_user_id();
		}
		$arr = [];
		if ( class_exists( 'MS_Model_Membership' ) ) {
			$member      = MS_Factory::load( 'MS_Model_Member', $userMember );
			$memberships = $member->get_membership_ids();
			foreach ( $memberships as $key => $membership_id ) {
				$membership = MS_Factory::load( 'MS_Model_Membership', $membership_id );
				array_push( $arr, $membership->name );
			}
		}

		return $arr;
	}

	private function uamIsUserInGroup( $groupName = '', $userId = '' ) {
		if ( empty( $userId ) ) {
			$userId = get_current_user_id();
		}

		global $userAccessManager;

		if ( isset( $userAccessManager ) ) {
			$userGroupHandler  = $userAccessManager->getUserGroupHandler();
			$userGroupsForUser = $userGroupHandler->getUserGroupsForObject(
				\UserAccessManager\Object\ObjectHandler::GENERAL_USER_OBJECT_TYPE,
				$userId
			);

			foreach ( $userGroupsForUser as $element ) {
				if ( $element->getName() == $groupName ) {
					return true;
				}
			}
		} else {
			return false;
		}
	}

	public function membership_integration_custom_bulk_actions( $actions ) {
		global $userAccessManager;
		if ( Yme_Plugin_Utils::is_plugin_activated( 'pda_gold' ) !== - 1 && Yme_Plugin_Utils::is_plugin_activated( 'pda_v3' ) !== - 1 ) {
			return $actions;
		}

		if ( ! $this->do_not_have_any_membership_plugins( true ) && pda_membersip_check_role_protection() ) {
			$actions['assign_membership'] = __( 'Assign Memberships', 'assign_membership' );
		}

		return $actions;
	}

	public function membership_integration_bulk_action_handler( $redirect_to, $doaction, $post_ids ) {
		if ( $doaction !== 'assign_membership' || empty( $post_ids ) || empty( $_REQUEST["membership"] ) ) {
			return $redirect_to;
		}

		$pda_repo     = new PDA_v3_Gold_Repository;
		$members_data = array();
		$memberships  = array(
			'user_access_manager',
			'memberships_2',
			'paid_memberships_pro',
			'woo_memberships',
			'woo_subscriptions',
			'ar_member',
			'restrict_content_pro',
			'simple_wordpress_membership'
		);

		foreach ( $post_ids as $id ) {
			if ( ! $pda_repo->is_protected_file( $id ) ) {
				continue;
			}

			$old_member = get_post_meta( $id, PDA_v3_Constants::$pda_meta_key_memberships_integration, true );
			foreach ( $memberships as $member ) {
				$new_value               = str_replace( ",", ";", $_REQUEST["value_memberships"] );
				$old_value               = empty( $old_member[ $member ] ) ? "" : $old_member[ $member ];
				$members_data[ $member ] = $_REQUEST["membership"] == $member ? $new_value : $old_value;
			}

			update_post_meta( $id, PDA_v3_Constants::$pda_meta_key_memberships_integration, $members_data );
		}

		return $redirect_to;
	}

	public function handle_footer_scripts() {
		if ( "upload" === get_current_screen()->id ) {
			ob_start();
			require_once PDA_PLUGIN_DIR_PATH . 'includes/class-pda-membership-integration-bulk-actions.php';
			$bulkaction = new Pda_Membership_Integration_Bulk_Action();
			$bulkaction->template();
			$output = ob_get_clean();

			if ( ! $output ) {
				return;
			}

			echo '<script type="text/html" class="pda-membership-template" id="pda-membership-template-upload">';
			echo $output;
			echo '</script>';
		}
	}

	/**
	 * Migrate data for memberships
	 */
	public function pda_memberships_migrate_data() {
		// Remove meta box of Simple WordPress Membership plugin in attachment page
		// Comment for release
		// pda_membership_remove_metabox_of_simple_membership_in_attachment_page();

		migrate_data_when_update_plugin();
		migrate_data_for_woo_memberships();
		$upgrader = new Pda_MI_Upgrader();
		$upgrader->process();
	}

	/**
	 * Check plugin create by Ymese team having the new updates.
	 *
	 * @return array
	 */
	private function is_yme_plugin_has_updates() {
		$updates = get_plugin_updates();
		$results = array();
		foreach ( $updates as $plugin ) {
			if ( 'Prevent Direct Access Gold' === $plugin->Name ) {
				if ( version_compare( $plugin->Version, '3.1.0' ) < 0 ) {
					array_push( $results, $plugin );
				}
			}
			if ( 'PDA Access Restriction' === $plugin->Name ) {
				if ( version_compare( $plugin->Version, '1.0.4.4' ) < 0 ) {
					array_push( $results, $plugin );
				}
			}
		}

		return $results;
	}

	/**
	 * get category allow membership plan in ARMember
	 *
	 * @param $terms_id array category id
	 *
	 * @return array
	 */
	function get_all_category_allow_plan( $terms_id ) {
		global $arm_access_rules;
		$data    = $arm_access_rules->arm_prepare_rule_data(
			array(
				'type'       => 'taxonomy',
				'slug'       => 'category',
				'plan'       => '',
				'protection' => '1',
			)
		);
		$results = array();
		foreach ( $data as $value ) {
			if ( in_array( $value['id'], $terms_id ) ) {
				foreach ( $value['plans'] as $plan ) {
					array_push( $results, $plan );
				}
			}
		}

		return $results;
	}

	public function pda_memberships_handle_after_pda_gold_set_fap( $data ) {
		pda_memberships_set_fap( $data );
	}

	/**
	 * @param $options
	 *
	 * @return array
	 */
	public function puc_add_token_to_headers( $options ) {
		$options['headers'] = array(
			'Accept'        => 'application/json',
			'Authorization' => 'helloymese',
		);

		return $options;
	}

}
