<?php

if ( ! function_exists( 'is_plugin_active' ) ) {
	include_once( ABSPATH . 'wp-admin/includes/plugin.php' );
}

/**
 * @param $setting_name
 *
 * @return bool
 */
function pda_membership_get_setting_boolen( $name_setting ) {
	$settings = get_option( Pda_Membership_Constants::PDA_MEMBERSHIP_OPTIONS, false );

	return pda_membership_check_option_settings( $settings, $name_setting );
}

/**
 * @param $name_settings
 *
 * @return array
 */
function pda_membership_get_setting_array( $name_settings ) {
	$settings = get_option( Pda_Membership_Constants::PDA_MEMBERSHIP_OPTIONS, false );
	if ( $settings ) {
		$options = @unserialize( $settings );
		if ( false === $options || empty( $options ) ) {
			return array();
		}
		if ( array_key_exists( $name_settings, $options ) && ! empty( $options[ $name_settings ] ) ) {
			return $options[ $name_settings ];
		}
	}

	return array();
}

/**
 * @param $name_settings
 *
 * @return string
 */
function pda_membership_get_setting_string( $name_settings ) {
	$settings = get_option( Pda_Membership_Constants::PDA_MEMBERSHIP_OPTIONS, false );
	if ( $settings ) {
		$options = @unserialize( $settings );
		if ( false === $options || empty( $options ) ) {
			return '';
		}
		if ( array_key_exists( $name_settings, $options ) && ! empty( $options[ $name_settings ] ) ) {
			return $options[ $name_settings ];
		}
	}

	return '';
}

/**
 * Check option setting of membership was turn on.
 *
 * @param boolean $settings Setting.
 * @param string $name_settings name.
 *
 * @return bool
 */
function pda_membership_check_option_settings( $settings, $name_settings ) {
	if ( $settings ) {
		$options = @unserialize( $settings );
		if ( false === $options || empty( $options ) ) {
			return array();
		}
		if ( array_key_exists( $name_settings, $options ) && $options[ $name_settings ] === 'true' ) {
			return true;
		}
	}

	return false;
}


/**
 * Check role protection on setting page.
 *
 * @return bool
 */
function pda_membersip_check_role_protection() {
	if ( class_exists( 'Pda_Gold_Functions' ) ) {
		$gold_function = new Pda_Gold_Functions();
		if ( method_exists( $gold_function, 'pda_check_role_protection' ) ) {
			return $gold_function->pda_check_role_protection();
		}
	}

	return false;
}

/**
 * Migrate data when update plugin
 */
function migrate_data_when_update_plugin() {
	if ( ! get_option( Pda_Membership_Constants::PDA_MEMBERSHIP_OPTION_CHECK_MIGRATE_DATA ) ) {
		if ( - 1 === Yme_Plugin_Utils::is_plugin_activated( 'pda_gold' ) || - 1 === Yme_Plugin_Utils::is_plugin_activated( 'pda_v3' ) ) {
			if ( - 1 === Yme_Plugin_Utils::is_plugin_activated( 'ip_block' ) ) {
				global $wpdb;
				$table_name = $wpdb->prefix . "pda_ip_block";
				$results    = $wpdb->get_results( "SELECT * FROM $table_name" );
				foreach ( $results as $result ) {
					$user_roles = unserialize( $result->user_roles );
					if ( 'memberships' === $user_roles['type'] ) {
						update_data_for_ip_block_table( $wpdb, $table_name, $result->ID, $user_roles );
					} elseif ( 'default' === $user_roles['type'] ) {
						update_post_meta( $result->post_id, PDA_v3_Constants::$pda_meta_key_memberships_integration, 'memberships-default' );
					} else {
						update_post_meta( $result->post_id, PDA_v3_Constants::$pda_meta_key_memberships_integration, 'no-memberships' );
					}
				}
			} else {
				$data = pda_memberships_get_post_meta_by_meta_key( PDA_v3_Constants::$pda_meta_key_user_roles );
				foreach ( $data as $value ) {
					if ( 'memberships' === $value->meta_value ) {
						update_post_meta( $value->post_id, PDA_v3_Constants::$pda_meta_key_user_roles, 'admin-user' );
					} elseif ( 'default' === $value->meta_value ) {
						update_post_meta( $value->post_id, PDA_v3_Constants::$pda_meta_key_memberships_integration, 'memberships-default' );
					} else {
						update_post_meta( $value->post_id, PDA_v3_Constants::$pda_meta_key_memberships_integration, 'no-memberships' );
					}
				}
			}
		}

		update_option( Pda_Membership_Constants::PDA_MEMBERSHIP_OPTION_CHECK_MIGRATE_DATA, true );
	}
}

/**
 * Migrate data for WooCommerce Memberships
 */
function migrate_data_for_woo_memberships() {
	if ( ! get_option( Pda_Membership_Constants::PDA_MEMBERSHIP_OPTION_CHECK_MIGRATE_DATA_WOO_MEMBERSHIPS ) ) {
		migrate_data_on_popup();
		migrate_data_in_setting();
		update_option( Pda_Membership_Constants::PDA_MEMBERSHIP_OPTION_CHECK_MIGRATE_DATA_WOO_MEMBERSHIPS, true );
	}
}

/**
 * Migrate data on popup
 */
function migrate_data_on_popup() {
	if ( - 1 === Yme_Plugin_Utils::is_plugin_activated( 'pda_gold' ) || - 1 === Yme_Plugin_Utils::is_plugin_activated( 'pda_v3' ) ) {
		$all_plan = get_all_membership_plans_for_woo_memberships();
		$old_data = pda_memberships_get_post_meta_by_meta_key( PDA_v3_Constants::$pda_meta_key_memberships_integration );
		foreach ( $old_data as $data ) {
			if ( 'no-memberships' !== $data->meta_value && 'memberships-default' !== $data->meta_value ) {
				$values  = unserialize( $data->meta_value );
				$results = array();
				if ( ! empty( $values['woo_memberships'] ) ) {
					foreach ( explode( ';', $values['woo_memberships'] ) as $member_name ) {
						foreach ( $all_plan as $plan ) {
							if ( $member_name === $plan['name'] ) {
								array_push( $results, $plan['id'] );
							}
						}
					}
					$woo_memberships           = implode( ';', $results );
					$values['woo_memberships'] = $woo_memberships;
					update_post_meta( $data->post_id, PDA_v3_Constants::$pda_meta_key_memberships_integration, $values );
				}
			}
		}
	}
}

/**
 * Migrate data in setting
 */
function migrate_data_in_setting() {
	$options = get_data_in_setting();
	if ( ! empty( $options ) ) {
		$woo_memberships = array_key_exists( 'woo_memberships', $options ) ? $options['woo_memberships'] : '';
		if ( ! empty( $woo_memberships ) ) {
			$all_plan = get_all_membership_plans_for_woo_memberships();
			$results  = array();
			foreach ( $woo_memberships as $member ) {
				foreach ( $all_plan as $plan ) {
					if ( $member === $plan['name'] ) {
						array_push( $results, $plan['id'] );
					}
				}
			}
			$options['woo_memberships'] = $results;
			update_option( Pda_Membership_Constants::PDA_MEMBERSHIP_OPTIONS, serialize( $options ), 'no' );
		}
	}
}

function get_data_in_setting() {
	$settings = get_option( Pda_Membership_Constants::PDA_MEMBERSHIP_OPTIONS );
	if ( $settings ) {
		return unserialize( $settings );
	}

	return array();
}

/**
 * Update user roles by id
 *
 * @param $wpdb parameter global
 * @param $table_name table of ip block plugin
 * @param $id ID file
 * @param $user_roles user roles data
 */
function update_data_for_ip_block_table( $wpdb, $table_name, $id, $user_roles ) {
	$user_roles['type'] = 'admin-user';
	$wpdb->update(
		$table_name,
		array(
			'user_roles' => serialize( $user_roles ),
		),
		array(
			'ID' => $id,
		)
	);
}

/**
 * Get post meta by meta key
 *
 * @param $meta_key meta key
 *
 * @return mixed
 */
function pda_memberships_get_post_meta_by_meta_key( $meta_key ) {
	global $wpdb;

	return $value = $wpdb->get_results( $wpdb->prepare( "SELECT * FROM $wpdb->postmeta WHERE meta_key = %s", $meta_key ) );
}

/**
 * Get all membership plan of WooCommerce Memberships plugin
 *
 * @return array
 */
function get_all_membership_plans_for_woo_memberships() {
	if ( ! function_exists( 'wc_memberships_is_user_member' ) ) {
		return array();
	}

	$all_plans        = wc_memberships_get_membership_plans();
	$plan_name_and_id = array_map( function ( $plan ) {
		return array(
			'id'   => $plan->slug,
			'name' => $plan->name
		);
	}, $all_plans );

	return array_values( $plan_name_and_id );
}

/**
 * Get all id and items of WooCommerce Subscriptions plugin
 *
 * @return array
 */
function get_all_id_items_in_woo_subscriptions() {
	if ( ! class_exists( "WC_Subscriptions_Admin" ) ) {
		return array();
	}
	$products = wc_get_products(
		array(
			'product_type' => 'subscription'
		)
	);

	return array_map( function ( $product ) {
		return array(
			'id'   => (string) $product->get_id(),
			'name' => $product->get_title()
		);
	}, $products );
}

/**
 * Get all membership plan of AR Member plugin
 *
 * @return mixed
 */
function get_all_membership_plans_for_ar_member() {
	if ( ! is_plugin_active( 'armember/armember.php' ) ) {
		return array();
	}

	global $arm_subscription_plans;
	$all_plans = $arm_subscription_plans->arm_get_all_subscription_plans( 'arm_subscription_plan_id, arm_subscription_plan_name' );

	$plan_name_and_id = array_map( function ( $plan ) {
		return array(
			'id'   => $plan['arm_subscription_plan_id'],
			'name' => $plan['arm_subscription_plan_name']
		);
	}, $all_plans );

	return array_values( $plan_name_and_id );
}

/**
 * Get all memberships level of Restrict Content Plugin
 *
 * @return array
 */
function get_all_memberships_level_of_restrict_content_pro() {
	if ( ! is_plugin_active( 'restrict-content-pro/restrict-content-pro.php' ) || ! function_exists( 'rcp_get_subscription_levels' ) ) {
		return array();
	}

	$memberships_lever = rcp_get_subscription_levels();

	return array_map( function ( $membership ) {
		return array(
			'id'   => $membership->id,
			'name' => $membership->name
		);
	}, $memberships_lever );
}

function pda_mem_get_all_memberships_level_for_simple_memberships() {
	if ( ! is_plugin_active( 'simple-membership/simple-wp-membership.php' ) || ! class_exists( 'SwpmMembershipLevelUtils' ) || ! method_exists( 'SwpmMembershipLevelUtils', 'get_all_membership_levels_in_array' ) ) {
		return array();
	}

	$simple_memberships_level = SwpmMembershipLevelUtils::get_all_membership_levels_in_array();

	return array_map( function ( $key, $value ) {
		return array(
			'id'   => (string) $key,
			'name' => $value
		);
	}, array_keys( $simple_memberships_level ), $simple_memberships_level );
}

function pda_memberships_set_fap( $data ) {
	if ( - 1 === Yme_Plugin_Utils::is_plugin_activated( 'pda_gold' ) || - 1 === Yme_Plugin_Utils::is_plugin_activated( 'pda_v3' ) ) {
		if ( isset( $data["memberships_fap"] ) && ! empty( $data["memberships_fap"] ) ) {
			if ( $data["memberships_fap"] === 'memberships-default' || $data["memberships_fap"] === 'no-memberships' ) {
				update_post_meta( $data["post_id"], PDA_v3_Constants::$pda_meta_key_memberships_integration, $data["memberships_fap"] );
			} else {
				$data_member_keys = array(
					"user_access_manager",
					"memberships_2",
					"paid_memberships_pro",
					"woo_memberships",
					"woo_subscriptions",
					"ar_member",
					"restrict_content_pro",
					"simple_wordpress_membership",
				);
				$data_members     = array();
				foreach ( $data_member_keys as $key ) {
					if ( isset( $data[ $key ] ) ) {
						$data_members[ $key ] = $data[ $key ];
					}
				}
				if ( ! empty( $data_members ) ) {
					update_post_meta( $data["post_id"], PDA_v3_Constants::$pda_meta_key_memberships_integration, $data_members );
				}
			}
		}
	}
}

function pda_mi_remove_plugin_options() {
	$options = [
		Pda_Membership_Constants::PDA_MI_UPGRADER_VERSION,
		PDA_MI_UPGRADER_VERSION
	];

	foreach ( $options as $opt ) {
		delete_option( $opt );
	}
}

function pda_membership_remove_metabox_of_simple_membership_in_attachment_page() {
	if ( ! is_plugin_active( 'simple-membership/simple-wp-membership.php' ) ) {
		return;
	}

	remove_meta_box(
		'swpm_sectionid',
		'attachment',
		'advanced'
	);
}
