<?php
/**
 * Created by PhpStorm.
 * User: gaupoit
 * Date: 4/29/19
 * Time: 12:01
 */

if ( defined( ABSPATH ) ) {
	exit;
}

if ( ! class_exists( 'Pda_MI_Upgrader' ) ) {
	/**
	 * Class Pda_MI_Upgrader
	 *
	 * Upgrader class for migrating or updating data
	 */
	class Pda_MI_Upgrader {
		private $update_version = '1.0';

		public function __construct() {
			$this->update_version = get_option( Pda_Membership_Constants::PDA_MI_UPGRADER_VERSION, false ) === false ? '1.0' : get_option( Pda_Membership_Constants::PDA_MI_UPGRADER_VERSION );
		}

		/**
		 * Upgrading processing
		 */
		public function process() {
			if ( - 1 === Yme_Plugin_Utils::is_plugin_activated( 'pda_gold' ) || - 1 === Yme_Plugin_Utils::is_plugin_activated( 'pda_v3' ) ) {
				$this->process_version_1_0();
			}
		}

		/**
		 * Processing the first version for restrict_content_pro data structure
		 */
		private function process_version_1_0() {
			if ( '1.0' === $this->update_version ) {
				// Get all posts having post meta's key is pda_membership_integration
				$attachments = get_posts( array(
					'post_type' => 'attachment',
					'meta_key'  => PDA_v3_Constants::$pda_meta_key_memberships_integration
				) );
				foreach ( $attachments as $post ) {
					$meta_value = get_post_meta( $post->ID, PDA_v3_Constants::$pda_meta_key_memberships_integration, true );

					if ( isset( $meta_value['restrict_content_pro'] ) ) {
						$value = $meta_value['restrict_content_pro'];
						error_log( 'Old value: ' . wp_json_encode( $value ) );
						if ( is_string( $value ) && '' !== $value ) {
							$new_value                          = array(
								'id'   => explode( ';', $value ),
								'type' => 'pda-membership-level',
							);
							$meta_value['restrict_content_pro'] = $new_value;
							$success                            = update_post_meta( $post->ID, PDA_v3_Constants::$pda_meta_key_memberships_integration, $meta_value );
							error_log( "Data migration for the post: $post->ID and the result is " . wp_json_encode( $success ) );
						}
					}
				}

				$this->migrate_data_settings_for_restrict_content_pro();

				update_option( Pda_Membership_Constants::PDA_MI_UPGRADER_VERSION, '1.1' );
			}
		}

		/**
		 * Migrate data in settings for restrict content pro
		 */
		private function migrate_data_settings_for_restrict_content_pro() {
			$settings = get_option( Pda_Membership_Constants::PDA_MEMBERSHIP_OPTIONS, false );
			if ( $settings ) {
				$options = @unserialize( $settings );
				if ( false !== $options || ! empty( $options ) ) {
					if ( array_key_exists( Pda_Membership_Constants::PDA_MEMBERSHIP_APPLY_RESTRICT_CONTENT_PRO, $options ) && 'true' === $options[ Pda_Membership_Constants::PDA_MEMBERSHIP_APPLY_RESTRICT_CONTENT_PRO ] ) {
						if ( array_key_exists( 'restrict_content_pro', $options ) && ! empty( $options['restrict_content_pro'] ) ) {
							$options[ Pda_Membership_Constants::PDA_MEMBERSHIP_RES_TYPE ] = 'pda_memberships_level';
							update_option( Pda_Membership_Constants::PDA_MEMBERSHIP_OPTIONS, serialize( $options ), 'no' );
						}
					}
				}

			}
		}

	}
}
