<?php
/**
 * The plugin bootstrap file
 *
 * This file is read by WordPress to generate the plugin information in the plugin
 * admin area. This file also includes all of the dependencies used by the plugin,
 * registers the activation and deactivation functions, and defines a function
 * that starts the plugin.
 *
 * @link              https://preventdirectaccess.com/extensions/?utm_source=user-website&utm_medium=pluginsite_link&utm_campaign=pda-memberships
 * @since             1.1.7
 * @package           Pda_Membership_Integration
 *
 * @wordpress-plugin
 * Plugin Name:       PDA Membership Integration
 * Plugin URI:        https://preventdirectaccess.com/extensions/?utm_source=user-website&utm_medium=pluginsite_link&utm_campaign=pda-memberships
 * Description:       Integrate the top most popular membership plugins with Prevent Direct Access Gold for direct access to your protected WordPress files.
 * Version:           1.1.7
 * Author:            BWPS
 * Author URI:        https://preventdirectaccess.com/extensions/?utm_source=user-website&utm_medium=pluginsite_link&utm_campaign=pda-memberships
 * License:           GPL-2.0+
 * License URI:       http://www.gnu.org/licenses/gpl-2.0.txt
 * Text Domain:       pda-membership
 * Domain Path:       /languages
 */

// If this file is called directly, abort.
if ( ! defined( 'WPINC' ) ) {
	die;
}

/**
 * Currently plugin version.
 * Start at version 1.0.0 and use SemVer - https://semver.org
 * Rename this for your plugin and update it as you release new versions.
 */
define( 'PDA_MEMBERSHIP_NAME_VERSION', '1.1.7' );
define( 'PDA_PLUGIN_DIR_PATH', plugin_dir_path( __FILE__ ) );

/**
 * The code that runs during plugin activation.
 * This action is documented in includes/class-pda-membership-integration-activator.php
 */
function activate_pda_membership_integration() {
	require_once plugin_dir_path( __FILE__ ) . 'includes/class-pda-membership-integration-activator.php';
	Pda_Membership_Integration_Activator::activate( plugin_basename( __FILE__ ) );
}

/**
 * The code that runs during plugin deactivation.
 * This action is documented in includes/class-pda-membership-integration-deactivator.php
 */
function deactivate_pda_membership_integration() {
	require_once plugin_dir_path( __FILE__ ) . 'includes/class-pda-membership-integration-deactivator.php';
	Pda_Membership_Integration_Deactivator::deactivate();
}


function uninstall_pda_membership_integration() {
	pda_mi_remove_plugin_options();
}

register_activation_hook( __FILE__, 'activate_pda_membership_integration' );
register_deactivation_hook( __FILE__, 'deactivate_pda_membership_integration' );

/**
 * The core plugin class that is used to define internationalization,
 * admin-specific hooks, and public-facing site hooks.
 */
require plugin_dir_path( __FILE__ ) . 'includes/class-pda-membership-integration.php';

/**
 * Begins execution of the plugin.
 *
 * Since everything within the plugin is registered via hooks,
 * then kicking off the plugin from this point in the file does
 * not affect the page life cycle.
 *
 * @since    1.0.0
 */
function run_pda_membership_integration() {

	$plugin = new Pda_Membership_Integration();
	$plugin->run();

	register_uninstall_hook( __FILE__, 'uninstall_pda_membership_integration' );

	$configs = require PDA_PLUGIN_DIR_PATH . '/includes/class-pda-membership-integration-configs.php';
	$url     = '${UPDATE_URL}' === $configs->update_url
		? 'https://j59dqwbnzk.execute-api.ap-southeast-1.amazonaws.com/prod/hello?action=get_metadata&slug=pda-membership-integration'
		: $configs->update_url;
	Puc_v4_Factory::buildUpdateChecker(
		$url,
		__FILE__,
		'pda-membership-integration'
	);

}
run_pda_membership_integration();
