<?php

/**
 * The admin-specific functionality of the plugin.
 *
 * @link       https://preventdirectaccess.com/extensions/?utm_source=user-website&utm_medium=pluginsite_link&utm_campaign=pda_gold
 * @since      1.0.0
 *
 * @package    pda_s3
 * @subpackage pda_s3/admin
 */

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    pda_s3
 * @subpackage pda_s3/admin
 * @author     BWPS <hello@preventdirectaccess.com>
 *
 */
require_once dirname( __FILE__ ) . '/../includes/constants.php';
require_once( ABSPATH . 'wp-admin/includes/screen.php' );

use PDAS3\Providers\Aws_Helper;
use PDAS3\AsyncTasks\PDA_S3_Clear_CF_Cache;

class pda_s3_Admin {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string $plugin_name The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string $version The current version of this plugin.
	 */
	private $version;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 *
	 * @param      string $plugin_name The name of this plugin.
	 * @param      string $version The version of this plugin.
	 */
	public function __construct( $plugin_name, $version ) {

		$this->plugin_name = $plugin_name;
		$this->version     = $version;

	}

	/**
	 * Register the stylesheets for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_styles() {

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in pda_s3_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The pda_s3_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */


		if ( ! empty( get_current_screen() ) && "prevent-direct-access-gold_page_amazon-s3" === get_current_screen()->id ) {
			wp_enqueue_style( $this->plugin_name . '-toastr', plugin_dir_url( __FILE__ ) . 'css/lib/toastr.min.css', array(), $this->version, 'all' );
			wp_enqueue_style( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'css/pda-s3-admin.css', array(), $this->version, 'all' );
			wp_enqueue_style( 'select2', plugin_dir_url( __FILE__ ) . 'css/lib/select2.min.css', array(), $this->version, 'all' );

		}

	}

	/**
	 * Register the JavaScript for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_scripts() {

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in pda_s3_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The pda_s3_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */

		if ( ! empty( get_current_screen() ) && "prevent-direct-access-gold_page_amazon-s3" === get_current_screen()->id ) {
			wp_enqueue_script( 'select2' . $this->plugin_name, plugin_dir_url( __FILE__ ) . './js/lib/select2.min.js', array( 'jquery' ), '4.0.3', true );
			wp_enqueue_script( 'toastr' . $this->plugin_name, plugin_dir_url( __FILE__ ) . './js/lib/toastr.min.js', array( 'jquery' ), $this->version, true );

			wp_enqueue_script( 'pda_s3_setting_general_js' . $this->plugin_name, plugin_dir_url( __FILE__ ) . './js/pda-s3-admin.js', array(), $this->version );
			wp_localize_script( 'pda_s3_setting_general_js' . $this->plugin_name, 'pda_s3_setting_data', array( 'ajax_url' => admin_url( 'admin-ajax.php' ) ) );

			wp_enqueue_script( $this->plugin_name . 'pdas3_js', plugin_dir_url( __FILE__ ) . 'js/pda-s3-aws-admin.js', array( 'jquery' ), $this->version, false );
			wp_localize_script( $this->plugin_name . 'pdas3_js', 'pda_s3_data', array(
					'ajaxurl'   => admin_url( 'admin-ajax.php' ),
					'cache_api' => get_home_url_with_ssl() . 'wp-json/wps3/v1/assets'
				)
			);
		}
	}

	/**
	 * Register API
	 */
	public function wps3_rest_api_init_cb() {
		$api = new PDA_S3_API();
		$api->register_rest_routes();
	}

	/**
	 * Render S3 link
	 *
	 * @param string $content The content.
	 *
	 * @return mixed|null|string|string[]
	 */
	public function render_s3_link( $content ) {
		if ( empty( $content ) ) {
			return $content;
		}
		if ( Yme_Plugin_Utils::is_plugin_activated( 'pda_v3' ) === - 1 || Yme_Plugin_Utils::is_plugin_activated( 'pda_gold' ) === - 1 ) {
			$id = get_the_ID();
			if ( false === $id ) {
				return $content;
			}
			if ( 'true' === pda_s3_get_setting( PdaS3Constants::AUTO_REPLACE_PROTECTED_FILE ) ) {
				$post_types_selected = pda_s3_get_setting( PdaS3Constants::REPLACED_POSTS_TYPES );
				$selected_items      = pda_s3_get_setting( PdaS3Constants::REPLACED_PAGES_POSTS );
				if ( is_array( $selected_items ) ) {
					$ids = array_map( 'intval', $selected_items );
				} else {
					$ids = array();
				}
				$utils = new PDA_S3_UTILS();
				if ( is_array( $post_types_selected ) && in_array( get_post_type( get_the_ID() ), $post_types_selected ) ) {
					$content = $utils->replace_protected_files_by_s3_links( $content );
				} else if ( in_array( $id, $ids, true ) ) {
					$content = $utils->replace_protected_files_by_s3_links( $content );
				}
			}
		}

		return $content;
	}

	function update_option_setting_pdas3() {
		$nonce = $_REQUEST['security_check'];
		if ( ! wp_verify_nonce( $nonce, 'pdas3_ajax_nonce' ) ) {
			wp_die( 'invalid_nonce' );
		}

		$settings = $_REQUEST['settings'];

		$keys = [];

		foreach ( $settings as $key => $value ) {
			array_push( $keys, $key );
		}

		$options_s3_db = unserialize( get_option( pda_s3_BWPSConstant::PDAS3_SETTINGS_NAME ) );

		foreach ( $keys as $k ) {
			$options_s3_db[ $k ] = $settings[ $k ];
		}
		$pda_s3 = serialize( $options_s3_db );
		update_option( pda_s3_BWPSConstant::PDAS3_SETTINGS_NAME, $pda_s3, 'no' );
		wp_send_json( true );
		wp_die();
	}

	/**
	 * Delete file from S3 when deleting attachment's file
	 *
	 * @param $post_id
	 *
	 * @throws Exception
	 */
	public function handle_delete_attachment( $post_id ) {
		if ( ! empty( get_post_meta( $post_id, PdaS3Constants::S3_LINK_META_NAME, true ) ) ) {
			$utils = new PDA_S3_UTILS;
			try {
				$utils->delete_s3_file( $post_id );
			} catch ( Aws\S3\Exception\S3Exception $exception ) {
				PDA_S3_UTILS::Logger()->error( $exception->getMessage() );
			}
		}
	}

	function add_submenu_pda() {
		add_submenu_page( 'pda-gold', __( 'Amazon S3', 'pda-settings-page' ), __( 'Amazon S3', 'pda-settings-page' ), 'manage_options', 'amazon-s3', array(
			$this,
			'pda_s3_render_ui'
		) );
	}

	function pda_s3_render_ui() {
		?>
		<div class="wrap">
			<div id="icon-themes" class="icon32"></div>
			<h2><?php _e( 'Prevent Direct Access Gold: Amazon S3 Integration' ); ?></h2>
			<?php
			$settings_utils = new PDA_S3_Settings_Util();
			$default_tab    = $settings_utils->validS3Setting() ? 'settings_s3' : 'aws_s3';
			$activate_tab   = isset( $_GET['tab'] ) ? $_GET['tab'] : $default_tab;
			$this->render_tabs( $activate_tab );
			$this->render_content( $activate_tab );
			?>
		</div>
		<?php
	}

	private function render_tabs( $active_tab ) {
		$settings_utils = new PDA_S3_Settings_Util();
		?>
		<h2 class="nav-tab-wrapper">
			<?php if ( $settings_utils->validS3Setting() ) { ?>
				<a href="?page=amazon-s3&tab=settings_s3"
				   class="nav-tab <?php echo $active_tab == 'settings_s3' ? 'nav-tab-active' : ''; ?>"><?php _e( 'Settings', 'pdas3-setting' ); ?></a>
			<?php } ?>
			<a href="?page=amazon-s3&tab=aws_s3"
			   class="nav-tab <?php echo $active_tab == 'aws_s3' ? 'nav-tab-active' : ''; ?>"><?php _e( 'Storage', 'pdas3-setting' ); ?></a>
		</h2>
		<?php
	}

	private function render_content( $active_tab ) {
		switch ( $active_tab ) {
			case 'settings_s3':
				$this->render_settings_tab();
				break;
			case 'aws_s3':
				$this->render_aws_s3_tab();
				break;
		}
	}

	function render_aws_s3_tab() {
		$settings_utils = new PDA_S3_Settings_Util();

		$regions = [
			'us-east-1',
			'us-east-2',
			'us-west-1',
			'us-west-2',
			'ca-central-1',
			'eu-central-1',
			'eu-west-1',
			'eu-west-2',
			'eu-west-3',
			'ap-northeast-1',
			'ap-northeast-2',
			'ap-northeast-3',
			'ap-southeast-1',
			'ap-southeast-2',
			'ap-south-1',
			'sa-east-1',
			'cn-north-1',
			'cn-northwest-1'
		];

		$region     = $settings_utils->get_setting_values( 'region' );
		$aws_key    = $settings_utils->get_setting_values( 'aws_key' );
		$aws_secret = $settings_utils->get_setting_values( 'aws_secret' );
		$bucket     = $settings_utils->get_setting_values( 'bucket' );

		include PDA_S3_DIRNAME . '/includes/views/view-pda-s3-configuration.php';
	}

	function pda_s3_integration_update_general_settings() {
		$nonce = $_REQUEST['security_check'];
		if ( ! wp_verify_nonce( $nonce, 'pdas3_ajax_nonce' ) ) {
			error_log( 'not verify nonce', 0 );
			wp_die( 'invalid_nonce' );
		}
		$settings = $_REQUEST['settings'];

		$aws_helper = new Aws_Helper();
		$isOk = $aws_helper->testConfiguration( array(
			'aws_key'    => $settings["aws_key"],
			'aws_secret' => $settings["aws_secret"],
			'bucket'     => $settings["bucket"],
			'region'     => $settings["region"],
		) );

		if ( ! $isOk ) {
			wp_send_json( $isOk );
			wp_die();
		}

		$keys          = [];
		$options_s3_db = unserialize( get_option( pda_s3_BWPSConstant::PDAS3_SETTINGS_NAME ) );
		if ( $options_s3_db ) {
			if ( array_key_exists( "domain", $options_s3_db ) ) {
				unset( $options_s3_db['domain'] );
			}
		}

		foreach ( $settings as $key => $value ) {
			array_push( $keys, $key );
		}
		foreach ( $keys as $k ) {
			$options_s3_db[ $k ] = $settings[ $k ];
		}
		$data = $options_s3_db;

		$pda_s3 = serialize( $data );

		update_option( pda_s3_BWPSConstant::PDAS3_SETTINGS_NAME, $pda_s3, 'no' );
		wp_send_json( $settings );
		wp_die();
	}

	function render_settings_tab() {
		$time_die_url = get_time_die_for_url( 'mg_time_die_private_link' );
		$time         = $time_die_url['time'];
		$units        = $time_die_url['units'];
		$max          = $time_die_url['max'];

		$cdn                          = pda_s3_get_setting( 'cdn' ) === false ? '' : pda_s3_get_setting( 'cdn' );
		$remove_local_file            = pda_s3_get_setting( 'remove_local_file' ) === false ? '' : pda_s3_get_setting( 'remove_local_file' );
		$asset_cdn                    = pda_s3_get_setting( 'asset_cdn' ) === false ? '' : pda_s3_get_setting( 'asset_cdn' );
		$auto_sync_to_s3              = pda_s3_get_setting( 'auto_sync_to_s3' ) === false ? '' : pda_s3_get_setting( 'auto_sync_to_s3' );
		$auto_sync_to_s3_after_upload = pda_s3_get_setting( 'auto_sync_to_s3_after_upload' ) === false ? '' : pda_s3_get_setting( 'auto_sync_to_s3_after_upload' );
		?>
		<div class="main_container main_container_s3">
			<form id="pda_s3_setting_form">
				<input type="hidden" nonce="<?php echo wp_create_nonce( "pdas3_ajax_nonce" ); ?>"
				       id="pdas3_general_nonce"/>
				<table class="s3_settings_table" cellpadding="4">
					<tr>
						<td colspan="2">
							<h3><?php echo esc_html__( 'AMAZON S3 INTEGRATION', 'prevent-direct-access-gold' ) ?></h3>
						</td>
					</tr>
					<tr>
						<td>
							<?php if ( $auto_sync_to_s3_after_upload == 'true' ) { ?>
								<label class="pda_switch" for="pda_s3_auto_sync_to_s3_after_upload">
									<input type="checkbox" id="pda_s3_auto_sync_to_s3_after_upload"
									       name="pda_s3_auto_sync_to_s3_after_upload" checked/>
									<span class="pda-slider round"></span>
								</label>
								<div class="pda_error" id="pda_l_error"></div>
							<?php } else { ?>
								<label class="pda_switch" for="pda_s3_auto_sync_to_s3_after_upload">
									<input type="checkbox" id="pda_s3_auto_sync_to_s3_after_upload"
									       name="pda_s3_auto_sync_to_s3_after_upload"/>
									<span class="pda-slider round"></span>
								</label>
								<div class="pda_error" id="pda_l_error"/>
							<?php } ?>
						</td>
						<td>
							<p>
								<label><?php echo esc_html__( 'Sync Files After Upload', 'prevent-direct-access-gold' ) ?></label>
								<?php echo esc_html__( 'Automatically sync files to Amazon S3 after upload', 'prevent-direct-access-gold' ) ?>
							</p>
						</td>
					</tr>
					<tr>
						<td>
							<?php if ( $auto_sync_to_s3 == 'true' ) { ?>
								<label class="pda_switch" for="pda_s3_auto_sync_to_s3">
									<input type="checkbox" id="pda_s3_auto_sync_to_s3" name="pda_s3_auto_sync_to_s3"
									       checked/>
									<span class="pda-slider round"></span>
								</label>
								<div class="pda_error" id="pda_l_error"></div>
							<?php } else { ?>
								<label class="pda_switch" for="pda_s3_auto_sync_to_s3">
									<input type="checkbox" id="pda_s3_auto_sync_to_s3" name="pda_s3_auto_sync_to_s3"/>
									<span class="pda-slider round"></span>
								</label>
								<div class="pda_error" id="pda_l_error"/>
							<?php } ?>
						</td>
						<td>
							<p>
								<label><?php echo esc_html__( 'Sync Files Once Protected', 'prevent-direct-access-gold' ) ?></label>
								<?php echo esc_html__( 'Automatically sync files to Amazon S3 once protected by Prevent Direct Access Gold', 'prevent-direct-access-gold' ) ?>
							</p>
						</td>
					</tr>
					<tr>
						<td>
							<?php if ( $remove_local_file == 'true' ) { ?>
								<label class="pda_switch" for="pda_s3_remove_local_file">
									<input type="checkbox" id="pda_s3_remove_local_file" name="pda_s3_remove_local_file"
									       checked/>
									<span class="pda-slider round"></span>
								</label>
								<div class="pda_error" id="pda_l_error"></div>
							<?php } else { ?>
								<label class="pda_switch" for="pda_s3_remove_local_file">
									<input type="checkbox" id="pda_s3_remove_local_file"
									       name="pda_s3_remove_local_file"/>
									<span class="pda-slider round"></span>
								</label>
								<div class="pda_error" id="pda_l_error"/>
							<?php } ?>
						</td>
						<td>
							<p>
								<label><?php echo esc_html__( 'Remove Files From Server', 'prevent-direct-access-gold' ) ?></label>
								<?php echo esc_html__( 'Automatically remove file uploads in the server after synced to Amazon S3', 'prevent-direct-access-gold' ) ?>
							</p>
						</td>
					</tr>
					<tr>
						<td class="feature-input"><span class="feature-input"></span></td>
						<td>
							<p>
								<label><?php echo esc_html__( 'Expiration Time For Signed URLs', 'prevent-direct-access-gold' ) ?></label>
								<?php echo esc_html__( 'Set default expiration time for Amazon S3/CloudFront signed URLs used by WooCommerce & "Private Download Links"', 'prevent-direct-access-gold' ) ?>
							</p>
							<?php if ( $time != '' ) { ?>
								<input required value="<?php echo $time; ?>" class="width-100" type="number"
								       id="mg_time_die_private_link" max="<?php echo $max; ?>" min="1"/>
							<?php } else { ?>
								<input value="1" class="width-100" type="number" id="mg_time_die_private_link"
								       max="<?php echo $max; ?>" min="1"/>
							<?php } ?>

							<select id="mg_units_times_die_private_link">
								<option value="days" <?php if ( $units == "days" ) {
									echo "selected";
								} ?> ><?php echo esc_html__( 'Days', 'prevent-direct-access-gold' ) ?>
								</option>
								<option value="hours" <?php if ( $units == "hours" ) {
									echo "selected";
								} ?>><?php echo esc_html__( 'Hours', 'prevent-direct-access-gold' ) ?>
								</option>
								<option value="minutes" <?php if ( $units == "minutes" ) {
									echo "selected";
								} ?> ><?php echo esc_html__( 'Minutes', 'prevent-direct-access-gold' ) ?>
								</option>
								<option value="seconds" <?php if ( $units == "seconds" ) {
									echo "selected";
								} ?> ><?php echo esc_html__( 'Seconds', 'prevent-direct-access-gold' ) ?>
								</option>
							</select>
						</td>
					</tr>
					<tr>
						<td colspan="2">
							<hr>
						</td>
					</tr>
					<tr>
						<td colspan="2">
							<h3><?php echo esc_html__( 'CLOUDFRONT INTEGRATION', 'prevent-direct-access-gold' ) ?></h3>
						</td>
					</tr>
					<tr>
						<td class="feature-input"><span class="feature-input"></span></td>
						<td>
							<p>
								<label><?php echo esc_html__( 'Custom Domain (CNAME) For Media Files', 'prevent-direct-access-gold' ) ?></label>
								<?php echo _e( 'Replace the default S3 domain with your CloudFront or custom domain for all your media files. Recommended. <a href="https://preventdirectaccess.com/docs/amazon-s3-cloudfront-integration/#domain-media-files">More info</a>', 'prevent-direct-access-gold' ) ?>
							</p>
							<input placeholder="Enter CDN domain name" type="text" id="pda_s3_cname" name="pda_s3_cname"
							       value="<?php echo esc_attr( $cdn ) ?>"/>
					</tr>
					<!-- Comment by thinhnp --->
					<!-- Reason: https://app.asana.com/0/862774702554010/866735432496203  --->
					<!--                    <tr>-->
					<!--                        <td></td>-->
					<!--                        <td>-->
					<!--                            <p>-->
					<?php //echo esc_html__( "Synchronize assets (js, css files) to $bucket_name bucket", 'pda-s3' ) ?><!--</p>-->
					<!--                            <input type="button" id="pda-s3-apply-cache" class="button" value="Sync"/>-->
					<!--                        </td>-->
					<!--                    </tr>-->
					<tr>
						<td class="feature-input"><span class="feature-input"></span></td>
						<td>
							<p>
								<label><?php echo esc_html__( 'Custom Domain (CNAME) For Asset Files', 'prevent-direct-access-gold' ) ?></label>
								<?php echo _e( 'Serve your site’s assets (CSS, JS, fonts, etc) from your CloudFront or custom domain. Recommended. <a href="https://preventdirectaccess.com/docs/amazon-s3-cloudfront-integration/#domain-asset-files">More info</a>', 'prevent-direct-access-gold' ) ?>

							</p>
							<input placeholder="Enter CDN domain name" type="text" id="pda_s3_asset_cname"
							       name="pda_s3_asset_cname"
							       value="<?php echo esc_attr( $asset_cdn ) ?>"/>
					</tr>
					<tr>
						<td colspan="2">
							<hr>
						</td>
					</tr>
					<tr>
						<td colspan="2">
							<h3><?php echo esc_html__( 'CONTENT FILES', 'prevent-direct-access-gold' ) ?></h3></td>
					</tr>
					<?php include PDA_S3_DIRNAME . '/includes/views/view-pda-s3-replace-content-option.php' ?>
					<tr>
						<td>
						</td>
						<td>
							<?php
							submit_button();
							?>
						</td>
					</tr>
				</table>
			</form>
		</div>
		<?php
	}

	/**
	 * Auto sync file to S3 handler
	 *
	 * @param int $id Attachment's ID
	 *
	 * @return bool|mixed
	 * @throws \PDAS3\Providers\Exception Exception
	 */
	public function auto_sync_file_to_s3( $id ) {
		$auto_sync_to_s3 = pda_s3_get_setting( 'auto_sync_to_s3' );
		$is_synced       = pda_s3_check_file_synced_s3( $id );
		if ( 'true' === $auto_sync_to_s3 && ! $is_synced ) {
			$utils = new PDA_S3_UTILS();
			try {
				$utils->sync_file_to_s3( $id );
			} catch ( \Exception $exception ) {
				PDA_S3_UTILS::Logger()->error( $exception->getMessage() );
			}
		}
	}

	/**
	 * Filter to handle woo file download
	 *
	 * @param string $file_path File path
	 *
	 * @return string $file_path
	 */
	public function handle_woo_file_download( $file_path, $instance, $download_id ) {

		/**
		 * Priority:
		 *  Check to force to signed URL for s3 first
		 *  Check with sync and protected.
		 */
		$time_die   = pda_get_signed_url_timeout();
		$aws_helper = new Aws_Helper();
		$signed_url = $aws_helper->create_read_presigned_url( $file_path, $time_die );
		if ( ! empty( $signed_url ) ) {
			return $signed_url;
		}

		/**
		 * Check with protected URL or Synced URL.
		 */
		$pda_s3_utils = new PDA_S3_UTILS();
		$new_path = $pda_s3_utils->get_signed_url_when_downloading( $file_path );
		if ( ! empty( $new_path ) ) {
			return $new_path;
		}

		return $file_path;
	}

	/**
	 * Add a custom bulk action to sync file to s3
	 *
	 * @param mixed $actions Bulk actions
	 *
	 * @return mixed
	 */
	public function pda_s3_add_bulk_action( $actions ) {
		$s3_util = new PDA_S3_Settings_Util();
		if ( $s3_util->validS3Setting() && pda_s3_check_role_protection() ) {
			$actions[ PdaS3Constants::SYNC_BULK_ACTION_NAME ]    = __( 'Sync to S3', 'prevent-direct-access' );
			$actions[ PdaS3Constants::UNSYNC_BULK_ACTION_NAME ]  = __( 'Unsync from S3', 'prevent-direct-access' );
			$actions[ PdaS3Constants::OFFLOAD_BULK_ACTION_NAME ] = __( 'Offload to S3', 'prevent-direct-access' );

		}

		return $actions;
	}

	/**
	 * Handle pda s3 custom bulk action
	 *
	 * @param string redirect_to Redirect's URL.
	 * @param string $doaction Current bulk action's name
	 * @param $post_ids Post's IDs
	 *
	 * @return string
	 * @throws Exception exception.
	 */
	public function pda_s3_bulk_action_handler( $redirect_to, $doaction, $post_ids ) {
		if ( $doaction !== PdaS3Constants::SYNC_BULK_ACTION_NAME && $doaction !== PdaS3Constants::UNSYNC_BULK_ACTION_NAME && $doaction !== PdaS3Constants::OFFLOAD_BULK_ACTION_NAME ) {
			return $redirect_to;
		}

		if ( isset( $post_ids ) ) {
			foreach ( $post_ids as $post_id ) {
				//TODO: Support images
				$utils = new PDA_S3_UTILS;
				switch ( $doaction ) {
					case PdaS3Constants::SYNC_BULK_ACTION_NAME:
						$is_synced = pda_s3_check_file_synced_s3( $post_id );
						if ( ! $is_synced ) {
							try {
								$utils->sync_file_to_s3( $post_id );
							} catch ( Exception $exception ) {
								PDA_S3_UTILS::Logger()->error( $exception->getMessage() );
							}
						}
						break;
					case PdaS3Constants::UNSYNC_BULK_ACTION_NAME:
						$is_synced = pda_s3_check_file_synced_s3( $post_id );
						if ( $is_synced ) {
							try {
								$utils->delete_s3_file( $post_id );
							} catch ( Exception $exception ) {
								PDA_S3_UTILS::Logger()->error( $exception->getMessage() );
							}
						}
						break;
					case PdaS3Constants::OFFLOAD_BULK_ACTION_NAME:
						handle_for_bulk_action_offload( $post_id, $utils );
						break;
				}
			}
		}

		return $redirect_to;
	}

	/**
	 * Callback function to handle wp_get_attachment_url filter
	 *
	 * @param string $url Original URL.
	 * @param $post_id Post ID
	 *
	 * @return mixed The s3 link or the original link
	 */
	public function pda_s3_wp_get_attachment_url( $url, $post_id ) {

		$s3_link = pda_s3_get_s3_link( $post_id );

		//Not synced return url
		if ( empty( $s3_link ) ) {
			return $url;
		}

		if ( ! class_exists( 'PDA_v3_Gold_Repository' ) ) {
			return $url;
		}

		$repo              = new PDA_v3_Gold_Repository();
		$is_protected_file = $repo->is_protected_file( $post_id );
		$local_file        = get_attached_file( $post_id );
		$not_remove        = is_file( $local_file );
		//If synced + protected + not remove local file => still keep the same url
		if ( $is_protected_file && $not_remove ) {
			return $url;
		}

		if ( $is_protected_file ) {
			return $s3_link;
		}

		return apply_filters( PdaS3Constants::HOOK_GET_ATTACHMENT_URL, $s3_link, $post_id );
	}

	public function resource_replace_css( $src ) {
		return $this->replace_asset_links( $src, 'css' );
	}

	public function resource_replace_js( $src ) {
		return $this->replace_asset_links( $src, 'js' );
	}

	private function replace_asset_links( $src, $type ) {
		if ( is_admin() ) {
			return $src;
		}
		// Commented by thinhnp
		// Reason: https://app.asana.com/0/862774702554010/866735432496203
//		$cache = get_option( 'pda_s3_src_files', false );
//		if ( $cache === false ) {
//			$cache = [];
//		} else {
//			$cache = json_decode( $cache, true );
//		}
//
//		$option_name   = 'css' === $type ? PdaS3Constants::REFRESH_CACHE_CSS : PdaS3Constants::REFRESH_CACHE_JS;
//		$refresh_cache = get_option( $option_name );
//		if ( '1' === $refresh_cache ) {
//			if ( in_array( $src, $cache ) ) {
//				return $src;
//			}
//			array_push( $cache, $src );
//			update_option( PdaS3Constants::ASSET_CACHING_FILE, wp_json_encode( $cache ) );
//
//			return $src;
//		}
		$cdn = pda_s3_get_setting( 'asset_cdn' );
		if ( empty( trim( $cdn ) ) || false === $cdn ) {
			return $src;
		}

		$src = str_replace( $_SERVER['HTTP_HOST'], $cdn, $src );

		return $src;
	}

	/**
	 * Handler for hook after protecting the attachment file
	 *
	 * @param int $post_id Post ID.
	 */
	public function handle_after_protect_file( $post_id ) {
		//If protected and auto synced then no need to move files around.
		if ( ! $this->handle_pda_auto_sync_after_protect_file_when_upload( $post_id ) ) {
			//Synced and need to move around
			$this->handle_file_protection( $post_id, false );
		}
	}

	/**
	 * Handler for hook after un-protecting the attachment file
	 *
	 * @param int $post_id Post ID.
	 */
	public function handle_after_un_protect_file( $post_id ) {
		$this->handle_file_protection( $post_id, true );
	}

	/**
	 * Helper function to handle file protection
	 *
	 * @param int $post_id
	 * @param $is_public
	 */
	private function handle_file_protection( $post_id, $is_public ) {
		if ( pda_s3_check_file_synced_s3( $post_id ) ) {
			error_log( 'Hook after protecting the file' );
			$aws_helper = new Aws_Helper();
			$util       = new PDA_S3_UTILS();
			$file_paths = $util->get_file_paths( $post_id, false );
			foreach ( $file_paths as $size => $path ) {
				try {
					$before_upload_file_path = str_replace( '/_pda', '', $path );
					$s3_link                 = $aws_helper->move_object( $before_upload_file_path, $is_public, $post_id );
					if ( 'original' === $size ) {
						update_post_meta( $post_id, PdaS3Constants::S3_LINK_META_NAME, $s3_link );
					}
				} catch ( Exception $ex ) {
					error_log( $ex->getMessage() );
				}
			}
		}
	}

	function add_button_handle_sync_or_unsync( $actions, $post, $detached ) {
		$s3_util    = new PDA_S3_Settings_Util();
		$data['id'] = $post->ID;
		if ( $s3_util->validS3Setting() && pda_s3_check_role_protection() ) {
			wp_enqueue_script( 'pda_s3_handle_sync_file_js' . $this->plugin_name, plugin_dir_url( __FILE__ ) . './js/pda-s3-handle-sync-file.js', array(), $this->version );
			wp_localize_script( 'pda_s3_handle_sync_file_js' . $this->plugin_name, 'pda_s3_handle_sync_file', array(
				'ajax_url' => admin_url( 'admin-ajax.php' ),
			) );

			wp_enqueue_style( 'pda_s3_handle_sync_file_css' . $this->plugin_name, plugin_dir_url( __FILE__ ) . 'css/pda-s3-handle-sync-file.css', array(), $this->version, 'all' );

			$input_nonce = '<input type=' . '"hidden" value="' . wp_create_nonce( pda_s3_BWPSConstant::S3_SYNC_NONCE ) . '" id="pda_s3_nonce_sync_file_'. $post->ID .'"/>';
			$is_synced           = pda_s3_check_file_synced_s3( $post->ID );
			$text                = $is_synced ? 'Unsync' : 'Sync';
			$title               = $is_synced ? 'Unsync this file from S3' : 'Sync this file to S3';
			$actions['pda_sync'] = '<a id="pda-s3-sync-file_' . $post->ID . '" class="pda-s3-sync-file" title="' . $title . '" aria-label="PDA S3 ' . $post->post_title . '">' . $text . '</a>' . $input_nonce;
		}

		return $actions;
	}

	/**
	 * Handle sync file
	 */
	function pda_s3_handle_sync_file() {
        $nonce = $_REQUEST['security_check'];
        error_log( $nonce);
		if ( ! wp_verify_nonce( $nonce, pda_s3_BWPSConstant::S3_SYNC_NONCE ) ) {
			wp_die( 'Invalid nonce' );
		}

		$post_id   = $_REQUEST['post_id'];
		$is_synced = pda_s3_check_file_synced_s3( $post_id );
		$utils     = new PDA_S3_UTILS();
		if ( $is_synced ) {
			try {
				$utils->delete_s3_file( $post_id );
			} catch ( Exception $exception ) {
				wp_send_json( false );
				wp_die();
			}
			wp_send_json( 'unsync' );
			wp_die();
		} else {
			try {
				$utils->sync_file_to_s3( $post_id );
			} catch ( Exception $exception ) {
				wp_send_json( false );
				wp_die();
			}
			$is_deleted_and_sync = check_file_is_deleted_and_is_sync( $post_id );
			if ( $is_deleted_and_sync ) {
				wp_send_json( 'sync-deleted' );
			} else {
				wp_send_json( 'sync' );
			}
			wp_die();
		}
	}

	/**
	 * Handle file after protecting when uploading file
	 *
	 * @param int $attachment_id Attachment's ID
	 */
	public function handle_pda_after_protect_file_when_upload( $attachment_id ) {
		$is_auto_synced = pda_s3_get_setting( 'auto_sync_to_s3' );
		if ( 'true' === $is_auto_synced ) {
			$utils = new PDA_S3_UTILS;
			try {
				$utils->sync_file_to_s3( $attachment_id );
			} catch ( \Exception $exception ) {
				PDA_S3_UTILS::Logger()->error( $exception->getMessage() );
			}
		}
	}

	/**
	 * Handle file after protecting when uploading file
	 *
	 * @param int $attachment_id Attachment's ID
	 */
	public function handle_pda_auto_sync_after_protect_file_when_upload( $attachment_id ) {
		$is_auto_synced_enabled = pda_s3_get_setting( 'auto_sync_to_s3' ) === 'true';
		$is_synced              = pda_s3_check_file_synced_s3( $attachment_id );
		if ( ! $is_synced && $is_auto_synced_enabled ) {
			$utils = new PDA_S3_UTILS;
			try {
				error_log( 'Auto-protect happened!' );
				$utils->sync_file_to_s3( $attachment_id );

				return true;
			} catch ( \Exception $exception ) {
				PDA_S3_UTILS::Logger()->error( $exception->getMessage() );
			}
		}

		return false;
	}

	/**
	 * Notification
	 */
	public function pda_s3_admin_notices() {
		$s3_util = new PDA_S3_Settings_Util();
		if ( ! $s3_util->validS3Setting() ) {
			$url = site_url() . '/wp-admin/admin.php?page=amazon-s3&tab=aws_s3';
			?>
			<div class="notice notice-error is-dismissible">
				<p><b><?php echo esc_html( $this->plugin_name ); ?>
						: </b><?php echo 'Please configure your <a href="' . $url . '">Amazon S3 access key and bucket</a>.'; ?>
				</p>
			</div>
			<?php
		}

		if ( Yme_Plugin_Utils::is_plugin_activated( 'pda_gold' ) !== - 1 && Yme_Plugin_Utils::is_plugin_activated( 'pda_v3' ) !== - 1 ) {
			$message = YME_MESSAGES::$PDA['PDA_NEVER_ACTIVATE'];
		}

		if ( ! empty( $message ) ) {
			?>
			<div class="notice notice-error is-dismissible">
				<p><b><?php echo esc_html( $this->plugin_name ); ?>: </b><?php _e( $message, 'pda-s3' ); ?></p>
			</div>
			<?php
		}
	}

	/**
	 * Check has bucket
	 * @return bool
	 */
	function handle_pda_check_s3_has_bucket() {
		$s3_util = new PDA_S3_Settings_Util();
		if ( $s3_util->validS3Setting() ) {
			return true;
		}

		return false;
	}


	/**
	 * handle show file status under pda column
	 *
	 * @param $attachment_id
	 */
	public function handle_show_file_status_in_pda_column( $attachment_id ) {
		wp_enqueue_style( 'pda_s3_file_status' . $this->plugin_name, plugin_dir_url( __FILE__ ) . 'css/pda-s3-file-status.css', array(), $this->version, 'all' );
		$s3_util         = new PDA_S3_UTILS();
		$is_deleted      = $s3_util->check_file_is_deleted( $attachment_id );
		$is_synced_to_s3 = pda_s3_check_file_synced_s3( $attachment_id );
		if ( $is_synced_to_s3 ) {
			$text  = $is_deleted ? PdaS3Constants::PDA_S3_FILE_STATUS_OFFLOADED : PdaS3Constants::PDA_S3_FILE_STATUS_SYNCED;
			$class = $is_deleted ? PdaS3Constants::PDA_S3_CLASS_FOR_FILE_STATUS_OFFLOADED : PdaS3Constants::PDA_S3_CLASS_FOR_FILE_STATUS_SYNCED;
			$title = $is_deleted ? PdaS3Constants::PDA_S3_TITLE_FOR_FILE_STATUS_OFFLOADED : PdaS3Constants::PDA_S3_TITLE_FOR_FILE_STATUS_SYNCED;
			?>
			<span id="pda-s3-sync-file-status_<?php echo esc_attr( $attachment_id ); ?>"
			      class="protection-status <?php echo esc_attr( $class ); ?>"
			      title="<?php echo esc_attr( $title ); ?>">
                    <i class="fa fa-check-circle" aria-hidden="true"></i>
				<?php echo esc_html( $text ); ?>
            </span>
			<?php
		}
	}

	/**
	 * Hook to apply CDN
	 *
	 * @param string $s3_link S3 link.
	 * @param int $post_id Post ID.
	 *
	 * @return string $s3_link;
	 */
	public function pda_s3_apply_cdn( $s3_link, $post_id ) {
		$cdn = pda_s3_get_setting( 'cdn' );

		if ( false === $cdn || empty( $cdn ) ) {
			return $s3_link;
		}
		$parsed_url = parse_url( $s3_link );
		$root       = pda_s3_get_root_folder( $post_id, false );
		if ( false !== $parsed_url ) {
			$replace = trailingslashit( $parsed_url['host'] ) . $root;
			if ( false === strpos( $s3_link, $replace ) ) {
				$settings = new PDA_S3_Settings_Util;
				$config   = $settings->getAwsConfigs();
				$replace  = trailingslashit( $parsed_url['host'] ) . trailingslashit( $config['bucket'] ) . $root;
				if ( false === strpos( $s3_link, $replace ) ) {
					return $s3_link;
				}
			}
			$s3_link = str_replace( $replace, $cdn, $s3_link );
		}

		return $s3_link;
	}

	function auto_sync_after_upload( $metadata, $attachment_id ) {
		$auto_sync_to_s3_after_upload = pda_s3_get_setting( 'auto_sync_to_s3_after_upload' );
		if ( 'true' === $auto_sync_to_s3_after_upload ) {
			$utils = new PDA_S3_UTILS();
			try {
				if ( pda_s3_check_file_synced_s3( $attachment_id ) ) {
					return $metadata;
				}
				//only sync the original file
				$utils->sync_file_to_s3( $attachment_id, $metadata );
			} catch ( \Exception $exception ) {
				PDA_S3_UTILS::Logger()->error( $exception->getMessage() );
			}
		}

		return $metadata;
	}

	public function handle_get_image_tag( $html, $id, $alt, $title, $align, $size ) {
		if ( ! pda_s3_check_file_synced_s3( $id ) ) {
			return $html;
		}

		if ( ! is_string( $html ) ) {
			return $html;
		}

		preg_match( '@\ssrc=[\'\"]([^\'\"]*)[\'\"]@', $html, $matches );

		if ( ! isset( $matches[1] ) ) {
			// Can't establish img src
			return $html;
		}

		$img_src = $matches[1];

		return $html;
	}

	/**
	 * Handle get attachment image src
	 *
	 * @param $image
	 * @param $attachment_id
	 * @param $size
	 * @param $icon
	 *
	 * @return mixed
	 */
	public function handle_get_attachment_image_src( $image, $attachment_id, $size, $icon ) {
		if ( ! class_exists( 'PDA_v3_Gold_Repository' ) ) {
			return $image;
		}
		$repo         = new PDA_v3_Gold_Repository();
		$is_protected = $repo->is_protected_file( $attachment_id );

		if ( ! $is_protected || ! pda_s3_check_file_synced_s3( $attachment_id ) ) {
			return $image;
		}

		if ( wp_attachment_is_image( $attachment_id ) && isset( $image[0] ) ) {
			$util = new PDA_S3_UTILS();
			$size = pda_s3_convert_dimensions_to_size_name( $attachment_id, $size );
			if ( is_null( $size ) ) {
				$size = 'original';
			}
			$time_die = pda_get_signed_url_timeout();
			$url      = $util->create_presigned_url_for_image_sizes( $image[0], $attachment_id, $size, $time_die );
			$url      = pda_s3_encode_filename_in_path( $url );
			$image[0] = $url;
		}

		return $image;
	}

	/**
	 * Handle image get intermediate size
	 *
	 * @param $data
	 * @param $post_id
	 * @param $size
	 *
	 * @return mixed
	 */
	public function handle_image_get_intermediate_size( $data, $post_id, $size ) {
		if ( ! class_exists( 'PDA_v3_Gold_Repository' ) ) {
			return $data;
		}

		$repo         = new PDA_v3_Gold_Repository();
		$is_protected = $repo->is_protected_file( $post_id );

		if ( $is_protected && pda_s3_check_file_synced_s3( $post_id ) ) {
			if ( wp_attachment_is_image( $post_id ) && isset( $data['url'] ) ) {
				$util        = new PDA_S3_UTILS();
				$size        = pda_s3_convert_dimensions_to_size_name( $post_id, $size );
				$time_die    = pda_get_signed_url_timeout();
				$url         = $util->create_presigned_url_for_image_sizes( $data['url'], $post_id, $size, $time_die );
				$url         = pda_s3_encode_filename_in_path( $url );
				$data['url'] = $url;
			}
		}

		return $data;
	}

	/**
	 * Handle calculate image srcset
	 *
	 * @param $sources
	 * @param $size_array
	 * @param $image_src
	 * @param $image_meta
	 * @param $attachment_id
	 *
	 * @return mixed
	 */
	public function handle_calculate_image_srcset( $sources, $size_array, $image_src, $image_meta, $attachment_id ) {
		// The logic
		// If the file is synced and protected => signed URL
		// If the file is synced and un-protect => S3 URL
		if ( ! is_array( $sources ) ) {
			// Sources corrupt
			return $sources;
		}

		if ( ! pda_s3_check_file_synced_s3( $attachment_id ) ) {
			return $sources;
		}

		$is_protected = pda_s3_is_protected_file( $attachment_id );

		foreach ( $sources as $width => $source ) {
			$filename = wp_basename( $source['url'] );
			$size     = pda_s3_find_image_size_from_width( $image_meta['sizes'], $width, $filename );
			$util     = new PDA_S3_UTILS();
			if ( $is_protected ) {
				$is_valid_perm = pda_s3_is_valid_pages_and_permission( $attachment_id );
				if ( ! $is_valid_perm ) {
					return $sources;
				}
				$time_die = pda_get_signed_url_timeout();
				if ( is_null( $size ) ) {
					$url = $util->create_presigned_url( $attachment_id, $time_die );
				} else {
					$url = $util->create_presigned_url_for_image_sizes( $source['url'], $attachment_id, $size, $time_die );
				}
			} else {
				$url = $util->create_s3_url_for_image_size( $source['url'], $attachment_id );
			}
			$url                      = pda_s3_encode_filename_in_path( $url );
			$sources[ $width ]['url'] = $url;
		}

		return $sources;
	}

	public function handle_calculate_image_srcset_meta( $image_meta, $size_array, $image_src, $attachment_id ) {
		if ( empty( $image_meta['file'] ) ) {
			// Corrupt `_wp_attachment_metadata`
			return $image_meta;
		}
		if ( ! pda_s3_is_valid_perm_to_show_signed_url( $attachment_id ) ) {
			return $image_meta;
		}
		if ( false !== strpos( $image_src, $image_meta['file'] ) ) {
			// Path matches URL, no need to change
			return $image_meta;
		}
		if ( ! class_exists( 'PDA_v3_Gold_Repository' ) ) {
			return $image_meta;
		}
		$image_basename     = wp_basename( $image_meta['file'] );
		$image_meta['file'] = rawurlencode( $image_basename );
		if ( ! empty( $image_meta['sizes'] ) ) {
			$image_meta['sizes'] = array_map( function ( $size ) {
				$size['file'] = rawurlencode( $size['file'] );

				return $size;
			}, $image_meta['sizes'] );
		}

		return $image_meta;
	}

	public function handle_wp_update_attachment_metadata( $data, $attachment_id ) {
		if ( ! pda_s3_check_file_synced_s3( $attachment_id ) ) {
			return $data;
		}

		try {
			$utils = new PDA_S3_UTILS();
			$utils->sync_file_to_s3( $attachment_id, $data );
			pda_s3_clean_garbage( $attachment_id );
		} catch ( Exception $ex ) {
			PDA_S3_UTILS::Logger()->error( $ex->getMessage() );
		} finally {
			return $data;
		}

	}

	public function handle_load_image_to_edit_path( $filepath, $attachment_id, $size ) {
		if ( is_file( $filepath ) ) {
			return $filepath;
		}

		if ( 'full' !== $size ) {
			return $filepath;
		}

		if ( ! pda_s3_check_file_synced_s3( $attachment_id ) ) {
			return $filepath;
		}

		$utils         = new PDA_S3_UTILS();
		$paths         = $utils->get_file_paths( $attachment_id, false );
		$original_path = $paths['original'];
		try {
			$ok = $utils->copy_file_from_bucket( $original_path, $attachment_id );
			if ( $ok ) {
				pda_s3_add_to_garbage( $attachment_id, $original_path );

				return $original_path;
			}
		} catch ( Exception $ex ) {
			PDA_S3_UTILS::Logger()->error( $ex->getMessage() );
		}

		return $filepath;
	}


	/**
	 * Handle wp_unique_filename filter
	 *
	 * @param string $filename File name
	 * @param string $ext File extension
	 * @param string $dir Directory
	 * @param $unique_filename_callback
	 *
	 * @return string $filename
	 */
	public function handle_wp_unique_filename( $filename, $ext, $dir, $unique_filename_callback ) {
		$aws_utils = new PDA_S3_UTILS();
		$filename  = $aws_utils->handle_unique_name_in_bucket( $filename, $ext, $dir );

		return $filename;
	}

	/**
	 * Handle EWW optimization
	 *
	 * @param $filepath
	 * @param $ext
	 * @param $is_original
	 *
	 * @return mixed
	 */
	public function handle_eww_optimization( $filepath, $ext, $is_original ) {
		$post_id = get_attachment_id_by_file_path( $filepath );
		if ( false === $post_id ) {
			return $filepath;
		}
		if ( pda_s3_check_file_synced_s3( $post_id ) ) {
			$aws_utils = new PDA_S3_UTILS();
			try {
				$s3_url = $aws_utils->sync_to_s3_by_file_path( $filepath );
			} catch ( Exception $ex ) {
				PDA_S3_UTILS::Logger()->error( $ex->getMessage() );
			}
		}

		return $filepath;
	}

	public function handle_redirect_s3_signed_url( $value, $attachment_id ) {
		if ( pda_s3_check_file_synced_s3( $attachment_id ) ) {
			$time_die_signe_url = pda_get_signed_url_timeout();
			$util               = new PDA_S3_UTILS();
			$s3_signed_url      = $util->create_presigned_url( $attachment_id, $time_die_signe_url );

			return $s3_signed_url;
		}

		return false;
	}

	/**
	 * Handle after delete the object
	 *
	 * @param $result
	 * @param $post_id
	 * @param $key
	 * @param $is_moving
	 *
	 * @return mixed
	 */
	public function handle_after_delete_s3_file( $result, $post_id, $key, $is_moving ) {
	    if ( should_clear_cache( $post_id, $is_moving ) ) {
	        //replace _pda and massage the URL
            $util = new Aws_Helper();
            //Remove prefix _pda or pda-media
		    $key = str_replace( PdaS3Constants::PDA_S3_PROTECTED_ROOT_FOLDER, '', $key );
		    $key = str_replace( PdaS3Constants::PDA_S3_PROTECTED_PUBLIC_FOLDER, '', $key );
            $path_to_clear_cache = $util->s3_encode_url_path( $key );
		    $task                = new PDA_S3_Clear_CF_Cache();
		    $task->push_to_queue( array(
			    'path' => $path_to_clear_cache
		    ) );
		    $task->save()->dispatch();
        }
	    return $result;
    }

    function handle_plugins_activated( $plugin, $network_activation ){
	    $name_plugin = explode("/",$plugin)[0];
	    if ( $name_plugin === 'pda-s3' ) {
            PDA_S3_UTILS::offload_backup_files();
        }
    }

	/**
	 * Handle elementor frontend content
	 *
	 * @param string $content The content
	 *
	 * @return string
	 */
	public function handle_elementor_frontend_content( $content ) {
		return $this->render_s3_link( $content );
	}

	/**
	 * Handle after move files
	 * @hook PDA Gold
	 *
	 * @param WP_Error $move_result   Result after move files.
	 * @param integer  $attachment_id Attachment ID.
	 *
	 * @return NULL|WP_Error|boolean
	 */
	public function pda_after_move_files( $move_result, $attachment_id ) {
		if ( ! is_wp_error( $move_result ) ) {
			return $move_result;
		}

		if ( check_file_is_deleted_and_is_sync( $attachment_id ) ) {
			return true;
		}

		return $move_result;
	}
}
