(function( $ ) {
	'use strict';

	/**
	 * All of the code for your admin-facing JavaScript source
	 * should reside in this file.
	 *
	 * Note: It has been assumed you will write jQuery code here, so the
	 * $ function reference has been prepared for usage within the scope
	 * of this function.
	 *
	 * This enables you to define handlers, for when the DOM is ready:
	 *
	 * $(function() {
	 *
	 * });
	 *
	 * When the window is loaded:
	 *
	 * $( window ).load(function() {
	 *
	 * });
	 *
	 * ...and/or other possibilities.
	 *
	 * Ideally, it is not considered best practise to attach more than a
	 * single DOM-ready or window-load handler for a particular page.
	 * Although scripts in the WordPress core, Plugins and Themes may be
	 * practising this, we should strive to set a better example in our own work.
	 */
    $(function() {

        $('#pda_s3_general_form').submit(function(evt) {
            evt.preventDefault();
            evt.stopImmediatePropagation();
            _updateSettingsGeneral({
                aws_key: $("#aws_key").val(),
                aws_secret: $("#aws_secret").val(),
                bucket: $("#bucket").val(),
                region: $("#awsregion").val(),
                is_aws_default: $("#is_aws_default").prop('checked'),
            }, function(error) {
                if(error) {
                    console.error(error);
                } else {
                    location.reload();
                }
            });
        });

        setRequiredForForm();

        $("#pda_replaced_pages_select2").change(function(){
            if ( $("#pda_replaced_post_type_select2").val() === null && $("#pda_replaced_pages_select2").val() === null) {
                $("#pda_replaced_pages_select2").prop('required', true);
                $("#pda_replaced_post_type_select2").prop('required', true);
            } else {
                $("#pda_replaced_pages_select2").prop('required', false);
                $("#pda_replaced_post_type_select2").prop('required', false);
            }
        });

        function _updateSettingsGeneral(settings, cb){
            var _data = {
                action: 'pda_s3_integration_update_general_settings',
                settings: settings,
                security_check: $("#pdas3_general_nonce").attr('nonce'),
            }
            $('#summit').val('Submiting');
            $("#submit").prop("disabled", true);
            $.ajax({
                url: pda_s3_setting_data.ajax_url,
                type: 'POST',
                data: _data,
                success: function(data) {
                    $("#submit").prop("disabled", false);
                    if (data == 'invalid_nonce') {
                        alert('No! No! No! Verify Nonce Fails!');
                    } else if(data) {
                        toastr.success('Your settings have been updated successfully!', 'Prevent Direct Access S3')
                        cb();
                    } else {
                        toastr.error('Opps! There is a problem with your aws configuration. Please double check again!', 'Prevent Direct Access S3');
                        console.log("Failed", data);
                    }
                },

                error: function(error) {
                    $("#submit").prop("disabled", false);
                    console.log("Errors", error);
                    cb(error);
                },
                timeout: 5000
            });
        }

        function setRequiredForForm() {
            $("#pda_replaced_pages_select2").select2();
            $("#pda_replaced_post_type_select2").select2();
            $("#pda_auto_replace_protected_file").change(function() {
                if($(this).prop('checked')) {
                    $(".pda-pages-posts-replace").show('slow');
                    $("#pda-s3-singed-url-timeout").show('slow');
                    if ( $("#pda_replaced_post_type_select2").val() === null && $("#pda_replaced_pages_select2").val() === null) {
                        $("#pda_replaced_pages_select2").prop('required', true);
                        $("#pda_replaced_post_type_select2").prop('required', true);
                    } else {
                        $("#pda_replaced_pages_select2").prop('required', false);
                        $("#pda_replaced_post_type_select2").prop('required', false);
                    }
                } else {
                    $(".pda-pages-posts-replace").hide('slow');
                    $("#pda-s3-singed-url-timeout").hide('slow');
                    $("#pda_replaced_pages_select2").prop('required', false);
                    $("#pda_replaced_post_type_select2").prop('required', false);
                }
            });
            $("#pda_auto_replace_protected_file").trigger('change');

            $("#pda_replaced_post_type_select2").change(function(){
                if ( $("#pda_replaced_post_type_select2").val() === null && $("#pda_replaced_pages_select2").val() === null) {
                    $("#pda_replaced_pages_select2").prop('required', true);
                    $("#pda_replaced_post_type_select2").prop('required', true);
                } else {
                    $("#pda_replaced_pages_select2").prop('required', false);
                    $("#pda_replaced_post_type_select2").prop('required', false);
                }
            });
        }
    });

})( jQuery );
