<?php
/**
 * Created by PhpStorm.
 * User: gaupoit
 * Date: 10/19/18
 * Time: 10:38
 */

namespace PDAS3\AsyncTasks;

use WP_Background_Process;
use PDA_S3_UTILS;
use PDA_S3_Repository;

if ( ! class_exists( 'PDA_S3_Copy_Files_From_Bucket' ) ) {
    /**
     * Async class for S3 functions
     *
     * Class PDA_S3_Sync_To_S3
     */
    class PDA_S3_Copy_Files_From_Bucket extends WP_Background_Process {

        /**
         * @var string action's name
         */
        protected $action = 'pda_s3_copy_files_from_bucket';

        /**
         * Action for the task
         *
         * @var string
         */
        protected function task( $item) {
            $repo = new PDA_S3_Repository();
            $aws = new PDA_S3_UTILS();
            $ids = $repo->get_post_id_by_meta_key();
            $files_offloaded = array();
            foreach ($ids as $post) {
                $file_paths = $aws->get_file_paths( $post->post_id, false );
                foreach ( $file_paths as $file_path ) {
                    if (!is_file($file_path)) {
                        try {
                            $aws->copy_file_from_bucket($file_path, $post->post_id);
                        } catch (Exception $exception) {
                            PDA_S3_UTILS::Logger()->error($exception->getMessage());
                        }
                    }
                }
                if ( ! empty($file_paths) ) {
                    array_push($files_offloaded, array(
                        'id' => $post->post_id,
                        'protection_status' => pda_s3_is_protected_file($post->post_id)
                    ));
                }
            }
            update_option(\PdaS3Constants::PDA_S3_OFFLOADED_OPTION, $files_offloaded);
            return false;
        }

        /**
         * Complete
         *
         * Override if applicable, but ensure that the below actions are
         * performed, or, call parent::complete().
         */
        protected function complete() {
            parent::complete();

            // Show notice to user or perform some other arbitrary task...
        }

    }
}
