<?php

/**
 * Fired during plugin activation
 *
 * @link       https://preventdirectaccess.com/extensions/?utm_source=user-website&utm_medium=pluginsite_link&utm_campaign=pda_gold
 * @since      1.0.0
 *
 * @package    pda_s3
 * @subpackage pda_s3/includes
 */

/**
 * Fired during plugin activation.
 *
 * This class defines all code necessary to run during the plugin's activation.
 *
 * @since      1.0.0
 * @package    pda_s3
 * @subpackage pda_s3/includes
 * @author     BWPS <hello@preventdirectaccess.com>
 */
class pda_s3_Activator {

	/**
	 * Short Description. (use period)
	 *
	 * Long Description.
	 *
	 * @since    1.0.0
	 */
	public static function activate( $plugin_basename ) {
		if ( Yme_Plugin_Utils::is_plugin_activated( 'pda_gold' ) !== - 1 && Yme_Plugin_Utils::is_plugin_activated( 'pda_v3' ) !== - 1 ) {
			deactivate_plugins( $plugin_basename );
			wp_die( YME_MESSAGES::$PDA['PDA_NEVER_PURCHASED'] );
		}

		$isValidPurcharsed = PDA_S3_UTILS::IsValidPurchased();

		if ( ! $isValidPurcharsed ) {
			deactivate_plugins( $plugin_basename );
			wp_die( YME_MESSAGES::$PDA['PDA_ADDON_STOLEN'] );
		}

		if ( PDA_S3_UTILS::is_ssu_lower_or_equal_than_version( '1.0.0' ) ) {
			deactivate_plugins( $plugin_basename );
			wp_die(
				wp_kses(
					sprintf(
						// translators: %s: Plugin name.
						__( 'Please update <a target="_blank" href="https://wordpress.org/plugins/wp-s3-smart-upload/">WordPress Amazon S3 - Wasabi Smart File Uploads</a> to the latest version for %s plugin to work properly.', 's3-smart-upload' ),
						PDA_S3_PLUGIN_NAME
					),
					array(
						'a' => array(
							'href'   => array(),
							'target' => array(),
						),
					)
				)
			);
		}
		PDA_S3_UTILS::addDefaultOptions();
	}

}
