<?php
/**
 * Created by PhpStorm.
 * User: gaupoit
 * Date: 6/11/18
 * Time: 10:44
 */

if ( ! defined( 'ABSPATH' ) ) {
	die( 'You do not have sufficient permissions to access this file.' );
}

use PDAS3\Providers\Aws_Helper;

if ( ! class_exists( 'PDA_S3_API' ) ) {
	/**
	 * Class PDA_S3_API
	 */
	class PDA_S3_API {
		/**
		 * AWS Helper
		 *
		 * @var Aws_Helper
		 */
		private $aws_helper;
		/**
		 * PDA_S3_API constructor.
		 */
		public function __construct() {
			$this->aws_helper = new Aws_Helper();
		}
		/**
		 * Register rest routes
		 */
		public function register_rest_routes() {
			register_rest_route( 'wps3/v1', '/s3/(?P<id>\d+)', array(
				'methods'  => 'POST',
				'callback' => array( $this, 'sync_s3' ),
			) );
			register_rest_route( 'wps3/v1', '/s3/delete/(?P<id>\d+)/', array(
				'methods'  => 'POST',
				'callback' => array( $this, 'unsync_s3' ),
			) );
			
			register_rest_route( 'wps3/v1', '/debug/', array(
				'methods'  => 'GET',
				'callback' => array( $this, 'debug' ),
			) );

// Comment by: thinhnp
//Reason: https://app.asana.com/0/862774702554010/866735432496203
//			register_rest_route( 'wps3/v1', '/assets', array(
//				'methods'  => 'POST',
//				'callback' => array( $this, 'pull_assets' ),
//			) );
			
			register_rest_route( 'wps3/v1', '/debug', array(
				'methods'  => 'GET',
				'callback' => array( $this, 'debug' ),
			) );

            register_rest_route( 'wps3/v1', '/check-file-is-deleted-and-sync/(?P<id>\d+)', array(
                'methods'  => 'POST',
                'callback' => array( $this, 'check_file_is_deleted_and_sync' ),
            ) );
		}

		public function check_file_is_deleted_and_sync( $data ) {
            return check_file_is_deleted_and_is_sync( $data['id'] );
        }

		/**
		 * String file to S3 bucket API.
		 *
		 * @param array $data Data.
		 *
		 * @return bool|mixed|WP_Error
		 * @throws \PDAS3\Providers\Exception S3 Exception.
		 */
		public function sync_s3( $data ) {
			$utils = new PDA_S3_UTILS();
			$data = $utils->sync_file_to_s3( $data['id'] );
			if ( empty( $data ) ) {
                return new WP_Error( PdaS3Constants::S3_EXCEPTION, __( 'Failed to sync the file. Please try again or check the configuration.', 'pda-s3' ), array( 'status' => 400 ) );
            }
			return $data;
		}
		/**
		 * Un-sync s3 file.
		 *
		 * @param array $data Data.
		 *
		 * @return mixed
		 */
		public function unsync_s3( $data ) {
			$utils = new PDA_S3_UTILS();
			try {
				return $utils->delete_s3_file( $data['id'] );
			} catch ( Exception $exception ) {
				PDA_S3_UTILS::Logger()->error( $exception->getMessage() );
				error_log( 'DEBUG: ' . wp_json_encode( $exception->getMessage() ) );
				return new WP_Error( PdaS3Constants::S3_EXCEPTION, __( 'Failed to unsync the file. Please try again or check the configuration.', 'pda-s3' ), array( 'status' => 400 ) );
			}
		}
		
		/**
		 * Pull assets and push to s3
		 *
		 * @return mixed
		 */
// Comment by: thinhnp
// Reason: https://app.asana.com/0/862774702554010/866735432496203
//		public function pull_assets() {
//			$src_files      = json_decode( get_option( 'pda_s3_src_files' ), true );
//			$aws_helper     = new Aws_Helper();
//			$synced_file    = array_map( function ( $src ) use ( $aws_helper ) {
//				$file = str_replace( site_url(), untrailingslashit( ABSPATH ), pda_s3_remove_query_string( $src ) );
//				if ( ! is_file( $file ) ) {
//					return null;
//				}
//				$s3_link   = $aws_helper->putFile( $file, true );
//				$is_sycned = empty( $s3_link ) ? false : true;
//				if ( empty( $is_sycned ) ) {
//					return null;
//				}
//
//				return $src;
//			}, $src_files );
//			$massaged_files = array_filter( $synced_file, function ( $file ) {
//				return ! is_null( $file );
//			} );
//			update_option( PdaS3Constants::ASSET_CACHING_FILE, wp_json_encode( $massaged_files ) );
//			update_option( PdaS3Constants::REFRESH_CACHE_CSS, false );
//			update_option( PdaS3Constants::REFRESH_CACHE_JS, false );
//
//			return $massaged_files;
//		}
		
		public function debug() {
			$utils = new PDA_S3_UTILS();
			return $utils->get_file_paths( 1434 );
		}
	}
}
