<?php
/**
 * Created by PhpStorm.
 * User: gaupoit
 * Date: 4/12/18
 * Time: 09:19
 */

namespace PDAS3\Providers;

use BWPS\PDAS3\Aws3\Aws\Common\Exception\MultipartUploadException;
use BWPS\PDAS3\Aws3\Aws\S3\MultipartUploader;
use BWPS\PDAS3\Aws3\Aws\S3\S3Client;
use BWPS\PDAS3\Aws3\Aws\CloudFront\CloudFrontClient;

if ( ! class_exists( 'Aws_Helper' ) ) {

	class Aws_Helper {

		protected $s3;

		protected $options;

		public function __construct() {
			require_once plugin_dir_path( dirname( __FILE__ ) ) . 'vendor/autoload.php';
			require_once plugin_dir_path( dirname( __FILE__ ) ) . 'vendor/Aws3/aws-autoloader.php';
			$settings      = new \PDA_S3_Settings_Util;
			$this->options = $settings->getAwsConfigs();

			if ( ! empty( $this->options ) ) {
				$this->s3 = new S3Client( [
					'version'     => 'latest',
					'region'      => $this->options['region'],
					'credentials' => [
						'key'    => $this->options['aws_key'],
						'secret' => $this->options['aws_secret']
					]
				] );
			}
		}

		public function getFiles() {
			if ( is_null( $this->s3 ) ) {
				return false;
			}

			return $this->s3->listObjects( array(
				'Bucket' => 'pur-free-asia',
			) );
		}

		/**
		 * Get object to client
		 *
		 * @param $file_path
		 * @param $save_as
		 * @param $post_id
		 *
		 * @return bool
		 */
		public function copy_object_to_client( $file_path, $save_as, $post_id ) {
			if ( is_null( $this->s3 ) ) {
				return false;
			}
			$root = trailingslashit( pda_s3_get_root_folder( $post_id ) );
			$key  = $this->getFileKey( $file_path );
			$args = array(
				'Bucket' => $this->options['bucket'],
				'Key'    => $root . $key,
				'SaveAs' => $save_as,
			);

			return $this->s3->getObject( $args );
		}

		/**
		 * Put file to S3
		 *
		 * @param string $file_path File path.
		 * @param boolean $is_public ACL Mode.
		 *
		 * @return mixed
		 * @throws \Exception S3 exception.
		 */
		public function put_file( $file_path, $is_public, $root ) {
			if ( is_null( $this->s3 ) ) {
				return new WP_Error( 'wrong_aws_configration_', __( 'Please double check your aws configuration at the plugin setting', 'pda-s3' ), array( 'status' => 400 ) );
			}
			$source_file = $file_path;
			$key         = str_replace( '/_pda', '', trailingslashit( $root ) . $this->getFileKey( $file_path ) );
//			if ( $this->is_file_existed( $key ) ) {
//				$key_file = $this->resolveDuplicateFile( $file_path );
//				$key = str_replace( '/_pda', '', trailingslashit( $root ) . $this->getFileKey( $key_file ) );
//			}
			$acl = $this->getAcl( $is_public );
			try {
				$result = $this->s3->putObject( array(
					'Bucket'     => $this->options['bucket'],
					'Key'        => $key,
					'SourceFile' => $source_file,
					'region'     => $this->options['region'],
					'ACL'        => $acl,
					'Metadata'   => array(
						'X-Robots-Tag' => 'none',
					),
				) );

				return $result->get( 'ObjectURL' );
			} catch ( Aws\S3\Exception\S3Exception $exception ) {
				throw $exception;
			} catch ( \InvalidArgumentException $exception ) {
				throw  $exception;
			} catch ( \Exception $exception ) {
				throw $exception;
			}
		}

		/**
		 * @param $filePath
		 *
		 * @return string
		 * empty string if user have never did the aws configuration
		 */
		public function createPresignedUrl( $filePath, $time_die, $attachment_id ) {

			if ( is_null( $this->s3 ) ) {
				return "";
			}

			$root         = pda_s3_get_root_folder( $attachment_id );
			$key          = trailingslashit( $root ) . $this->getFileKey( $filePath );
			$cmd          = $this->s3->getCommand( 'GetObject', [
				'Bucket' => $this->options['bucket'],
				'Key'    => $key
			] );
			$request      = $this->s3->createPresignedRequest( $cmd, $time_die );
			$presignedUrl = (string) $request->getUri();
			// TODO: support CDN later
			return $presignedUrl;
//			$cdn = pda_s3_get_setting( 'cdn' );
//			if ( empty( $cdn ) ) {
//				return $presignedUrl;
//			}
//
//			$host = parse_url( $presignedUrl, PHP_URL_HOST );
//
//			if ( false === $host ) {
//				return $presignedUrl;
//			}
//
//			return str_replace( $host, $cdn, $presignedUrl );

		}

		/**
		 * Upate object ACL
		 *
		 * @param string $filePath The file's path
		 * @param bool $isPublic Public mode
		 * @param int $post_id Post ID
		 *
		 * @return string
		 * @throws \Exception
		 */
		public function updateObjACL( $filePath, $isPublic, $post_id ) {

			if ( is_null( $this->s3 ) ) {
				return "";
			}
			$root = pda_s3_get_root_folder( $post_id );
			$key  = trailingslashit( $root ) . $this->getFileKey( $filePath );
			$acl  = $isPublic == "1" ? 'public-read' : 'private';
			try {
				$result = $this->s3->putObjectAcl( array(
					'Bucket' => $this->options['bucket'],
					'Key'    => $key,
					'ACL'    => $acl,
				) );
			} catch ( Aws\S3\S3Exception $e ) {
				throw $e;
			}

			return $result;
		}

		public function getFileKey( $filePath ) {
			$subdir    = str_replace( ABSPATH, '', $filePath );
			$subdir    = trailingslashit( substr( $subdir, 0, strrpos( $subdir, '/' ) ) );
			$file_name = basename( $filePath );
			$key       = "$subdir$file_name";

			return $key;
		}

		public function is_file_existed( $key ) {
			if ( is_null( $this->s3 ) ) {
				return false;
			}
			$result = $this->s3->doesObjectExist(
				$this->options['bucket'],
				$key
			);

			return $result;
		}

		public function resolveDuplicateFile( $filePath ) {

			$info      = pathinfo( $filePath );
			$extension = $info['extension'];
			$date      = new \DateTime;
			$post_fix  = $date->getTimestamp();

			return str_replace( ".$extension", "-$post_fix.$extension", $filePath );

		}

		private function getAcl( $is_public ) {
			return $is_public ? 'public-read' : 'private';
		}

		/**
		 * Delete S3 file
		 *
		 * @param string $filePath File path.
		 * @param int $post_id ID.
		 * @param string $root Root folder
		 * @param bool $is_moving
		 *
		 * @return mixed
		 * @throws \Exception
		 */
		public function delete_s3_file( $filePath, $post_id, $root = '', $is_moving = false ) {
			if ( is_null( $this->s3 ) ) {
				return new WP_Error( 'wrong_aws_configration_', __( 'Please double check your aws configuration at the plugin setting', 'pda-s3' ), array( 'status' => 400 ) );
			}
			$root = empty( $root ) ? pda_s3_get_root_folder( $post_id ) : $root;
			$key  = trailingslashit( $root ) . $this->getFileKey( $filePath );
			try {

				$result = $this->s3->deleteObject( array(
					'Bucket' => $this->options['bucket'],
					'Key'    => $key
				) );

				return apply_filters( \PdaS3Constants::HOOK_AFTER_DELETE_S3_FILE, $result, $post_id, $key, $is_moving );

			} catch ( Aws\S3\Exception\S3Exception $exception ) {
				throw $exception;
			} catch ( \InvalidArgumentException $exception ) {
				throw $exception;
			} catch ( \Exception $exception ) {
				throw $exception;
			}
		}

		public function getExpiredTime() {
			$settings = get_option( "pda_settings_magic_link" );
			$time_die = \PdaS3Constants::PDAS3_DEFAULT_EXPIRED_TIME;
			if ( $settings ) {
				$options = unserialize( $settings );
				if ( is_array( $options ) ) {
					if ( array_key_exists( 'mg_time_die_private_link', $options ) ) {
						$time_die = $options['mg_time_die_private_link'];
					}
				}
			}

			return $time_die;

		}

		public function updateBucketPolicy( $whiteListIps ) {
			$bucket          = $this->options['bucket'];
			$isBucketExisted = $this->s3->doesBucketExist( $bucket );
			if ( $isBucketExisted ) {
				return $this->s3->putBucketPolicy( array(
					'Bucket' => $bucket,
					'Policy' => json_encode( array(
						'Statement' => array(
							array(
								'Sid'       => 'pdas3-policy-ipallow',
								'Action'    => array(
									's3:*'
								),
								'Effect'    => 'Allow',
								"Principal" => "*",
								'Resource'  => array(
									"arn:aws:s3:::{$bucket}/pda/*"
								),
								'Condition' => array(
									"IpAddress" => array(
										"aws:SourceIp" => $whiteListIps
									)
								)

							)
						)
					) )
				) );
			}
		}

		/**
		 * Move object
		 *
		 * @param string $file_path Source
		 * @param bool $isPublic Public mode
		 * @param int $post_id Post ID
		 *
		 * @throws \Exception
		 *
		 * @return mixed
		 */
		public function move_object( $file_path, $isPublic, $post_id ) {
			$key           = $this->getFileKey( $file_path );
			$previous_root = pda_s3_get_previous_root_folder_by_mode( $post_id );
			$source        = $this->s3_encode_url_path( trailingslashit( $previous_root ) . $key );
			$root = pda_s3_get_root_folder( $post_id );
			$dest =  trailingslashit( $root ) . $key;
			$acl = $isPublic == "1" ? 'public-read' : 'private';
			try {
				$result = $this->s3->copyObject( [
					'Bucket'     => $this->options['bucket'],
					'CopySource' => trailingslashit( $this->options['bucket'] ) . $source,
					'Key'        => $dest,
					'ACL'        => $acl
				] );
				$this->delete_s3_file( $file_path, $post_id, $previous_root, true );
				return $result['ObjectURL'];
			} catch ( \Exception $ex ) {
				throw $ex;
			}
		}

		public function testConfiguration( $options ) {
			try {
				$s3 = new S3Client( [
					'version'     => 'latest',
					'region'      => $options['region'],
					'credentials' => [
						'key'    => $options['aws_key'],
						'secret' => $options['aws_secret']
					]
				] );

				return $s3->doesBucketExist( $options['bucket'] );
			} catch ( \Exception $ex ) {
				\PDA_S3_UTILS::Logger()->error( $ex->getMessage() );

				return new \WP_Error( 'wrong_aws_configration_', __( 'Please double check your aws configuration at the plugin setting', 'pda-s3' ), array( 'status' => 400 ) );
			}
		}

		/**
		 * Create invalidation for CloudFront
		 *
		 * @param $distribution_id
		 * @param $path
		 *
		 * @return WP_Error
		 */
		public function create_invalidation( $distribution_id, $path ) {
			try {
				$client = new CloudFrontClient( [
					'version'     => '2017-10-30',
					'region'      => $this->options['region'],
					'credentials' => [
						'key'    => $this->options['aws_key'],
						'secret' => $this->options['aws_secret']
					]
				] );
				$result = $client->createInvalidation( [
					'DistributionId'    => $distribution_id,
					'InvalidationBatch' => [
						'CallerReference' => 'pda-s3-caller-ref-' . uniqid(),
						'Paths'           => [
							'Items'    => [ $path ],
							'Quantity' => 1,
						],
					]
				] );
				error_log( 'Result: ' . wp_json_encode( $result ) );

				return $result;
			} catch ( \Exception $ex ) {
				\PDA_S3_UTILS::Logger()->error( $ex->getMessage() );
				error_log( 'Result: ' . $ex->getMessage() );

				return new \WP_Error( 'wrong_aws_configration_', __( 'Please double check your aws configuration at the plugin setting', 'pda-s3' ), array( 'status' => 400 ) );
			}
		}

		public function get_distribution_id( $domain_name ) {
			try {
				$client = new CloudFrontClient( [
					'version'     => '2017-10-30',
					'region'      => $this->options['region'],
					'credentials' => [
						'key'    => $this->options['aws_key'],
						'secret' => $this->options['aws_secret']
					]
				] );
				$dists  = $client->listDistributions();
				$items  = $dists['DistributionList']['Items'];
				$key    = array_search( $domain_name, array_column( $items, 'DomainName' ) );
				if ( false === $key ) {
					if ( ! function_exists( 'array_column' ) ) {
						return $key;
					}
					$quantities      = array_column( array_column( $items, 'Aliases' ), 'Quantity' );
					$cdn_items_index = [];
					//getting the item having CDN
					foreach ( $quantities as $key => $value ) {
						if ( $value > 0 ) {
							array_push( $cdn_items_index, $key );
						}
					}
					//Find the CDN in items
					foreach ( $cdn_items_index as $key => $value ) {
						$alias = $items[ $value ]['Aliases']['Items'];
						if ( false !== array_search( $domain_name, $alias ) ) {
							return $items[ $value ]['Id'];
						}
					}

					return $key;
				}

				return $items[ $key ]['Id'];
			} catch ( \Exception $ex ) {
				\PDA_S3_UTILS::Logger()->error( $ex->getMessage() );
				error_log( 'Result: ' . $ex->getMessage() );

				return new \WP_Error( 'wrong_aws_configration_', __( 'Please double check your aws configuration at the plugin setting', 'pda-s3' ), array( 'status' => 400 ) );
			}
		}

		/**
		 * Upload large file to S3 Bucket.
		 *
		 * @param string $file_path File's path
		 * @param bool $is_public ACL mode
		 *
		 * @return bool
		 */
		public function uploadLargeFile( $file_path, $is_public ) {
			if ( is_null( $this->s3 ) ) {
				return false;
			}
			$key      = $this->getFileKey( $file_path );
			$acl      = $this->getAcl( $is_public );
			$uploader = new MultipartUploader( $this->s3, $file_path, [
				'bucket'   => $this->options['bucket'],
				'key'      => $key,
				'region'   => $this->options['region'],
				'ACL'      => $acl,
				'Metadata' => array(
					'X-Robots-Tag' => 'none',
				),
			] );
			try {
				$result = $uploader->upload();
				error_log( "Upload complete: {$result['ObjectURL']}" );

				return $result['ObjectURL'];
			} catch ( MultipartUploadException $e ) {
				error_log( $e->getMessage() );
			}
		}

		public function s3_encode_url_path( $path ) {
			return implode( '/', array_map( 'urlencode', explode( '/', $path ) ) );
		}


		/**
		 * Create the presigned URL.
		 *
		 * @param string $download_url Download URL.
		 * @param string $time_die     Deadline time.
		 *
		 * @return string|false empty string if user have never did the aws configuration
		 */
		public function create_read_presigned_url( $download_url, $time_die ) {
			if ( is_null( $this->s3 ) || ! $download_url) {
				return false;
			}

			if ( false === strpos( parse_url( $download_url, PHP_URL_HOST ), 'amazonaws.com' ) ) {
				return false;
			}

			if ( false === strpos( $download_url, $this->options['bucket'] ) ) {
				return false;
			}

			$file_name = ltrim( parse_url( $download_url, PHP_URL_PATH ), '/' );

			$cmd = $this->s3->getCommand(
				'GetObject',
				[
					'Bucket' => $this->options['bucket'],
					'Key'    => urldecode( $file_name ),
				]
			);

			$request = $this->s3->createPresignedRequest( $cmd, $time_die );
			return strval( $request->getUri() );
		}
	}

}

