<?php
/**
 * Created by PhpStorm.
 * User: gaupoit
 * Date: 9/7/18
 * Time: 14:30
 */

/**
 * Check whether the file is synced to s3
 *
 * @param int $post_id Attachment's ID
 *
 * @return bool synced
 */
function pda_s3_check_file_synced_s3( $post_id ) {
	$metadata = get_post_meta( $post_id, PdaS3Constants::S3_LINK_META_NAME, true );

	return false !== $metadata && ! empty( $metadata );
}


/**
 * Function to check whether the file is synced to s3 by s3 link.
 *
 * @param string $s3_link S3 Link.
 *
 * @return mixed
 */
function pda_s3_get_post_id_by_s3_link( $s3_link ) {
	$repo    = new PDA_S3_Repository();
	$post_id = $repo->get_post_id_by_meta_value( PdaS3Constants::S3_LINK_META_NAME, $s3_link );
	if ( false === $post_id ) {
		$url_path = parse_url( $s3_link, PHP_URL_PATH );
		if ( strpos( $url_path, PdaS3Constants::PDA_S3_PROTECTED_ROOT_FOLDER ) === 1 ) {
			$s3_link = str_replace( PdaS3Constants::PDA_S3_PROTECTED_ROOT_FOLDER, PdaS3Constants::PDA_S3_PROTECTED_PUBLIC_FOLDER, $s3_link );
		} else if ( strpos( $url_path, PdaS3Constants::PDA_S3_PROTECTED_PUBLIC_FOLDER ) === 1 ) {
			$s3_link = str_replace( PdaS3Constants::PDA_S3_PROTECTED_PUBLIC_FOLDER, PdaS3Constants::PDA_S3_PROTECTED_ROOT_FOLDER, $s3_link );
		}
		$post_id = $repo->get_post_id_by_meta_value( PdaS3Constants::S3_LINK_META_NAME, $s3_link );
	}

	return $post_id;
}

/**
 * Get post by protected link
 *
 * @param string $link Protected link.
 *
 * @return mixed
 */
function pda_s3_get_post_id_by_original_link( $link ) {
	$wp_upload_dir  = wp_upload_dir();
	$baseurl        = $wp_upload_dir['baseurl'];
	$meta_value     = str_replace( $baseurl . '/', '', $link );
	$arr_meta_value = explode( '/', $meta_value );
	$repo           = new PDA_S3_Repository();
	if ( ! in_array( '_pda', $arr_meta_value, true ) ) {
		array_unshift( $arr_meta_value, '_pda' );
	}
	$meta_value = implode( '/', $arr_meta_value );
	$post_id    = $repo->get_post_id_by_meta_value( '_wp_attached_file', $meta_value );
	if ( false !== $post_id ) {
		return $post_id;
	}
	$meta_value = str_replace( '_pda/', '', $meta_value );

	return $repo->get_post_id_by_meta_value( '_wp_attached_file', $meta_value );
}

/**
 * Get post ID by CDN link
 *
 * @param string $link Original link.
 *
 * @return bool|int
 */
function pda_s3_get_post_id_by_cdn_link( $link ) {
	$cdn = pda_s3_get_setting( 'cdn' );
	if ( false === $cdn ) {
		return false;
	}

	if ( empty( $link ) ) {
		return false;
	}

	if ( strpos( $cdn, $link ) < 0 ) {
		return false;
	}
	$wp_upload_dir = wp_upload_dir();
	$baseurl       = $wp_upload_dir['baseurl'];
	$upload_path   = wp_parse_url( $baseurl, PHP_URL_PATH );
	$path          = wp_parse_url( $link, PHP_URL_PATH );
	$path          = ltrim( str_replace( $upload_path, '', $path ), '/' );
	$repo          = new PDA_S3_Repository();
	$post_id       = $repo->get_post_id_by_meta_value( '_wp_attached_file', $path );
	if ( false !== $post_id ) {
		return $post_id;
	}
	$arr_meta_value = explode( '/', $path );
	if ( ! in_array( '_pda', $arr_meta_value, true ) ) {
		array_unshift( $arr_meta_value, '_pda' );
	}
	$meta_value = implode( '/', $arr_meta_value );

	return $repo->get_post_id_by_meta_value( '_wp_attached_file', $meta_value );
}

/**
 * Get s3 link for synced attachment file
 *
 * @param int $post_id Attachment's ID
 *
 * @return mixed Synced s3 link
 */
function pda_s3_get_s3_link( $post_id ) {
	return get_post_meta( $post_id, PdaS3Constants::S3_LINK_META_NAME, true );
}

/**
 * Trailing slash ensuring no leading slashes
 *
 * @param $string
 */
function pda_s3_trailingslash( $string ) {
	return ltrim( trailingslashit( $string ), '/\\' );
}

/**
 * Helper function to get setting's value.
 *
 * @param $setting_name
 *
 * @return bool
 */
function pda_s3_get_setting( $setting_name ) {
	$setting = get_option( PdaS3Constants::PDAS3_SETTINGS_NAME, false );
	if ( false === $setting ) {
		return false;
	}
	$setting = unserialize( $setting );
	if ( ! isset( $setting[ $setting_name ] ) ) {
		return false;
	}

	return $setting[ $setting_name ];
}

/**
 * Function to remove query string from url
 *
 * @param string $url The URL.
 *
 * @return mixed
 */
function pda_s3_remove_query_string( $url ) {
	$parts = explode( '?', $url );

	return reset( $parts );
}

/**
 * Get home url with ssl
 *
 * @return mixed
 */
function get_home_url_with_ssl() {
	return is_ssl() ? home_url( '/', 'https' ) : home_url( '/' );
}

/**
 * Remove protected prefix
 *
 * @param string $file File path
 *
 * @return mixed
 */
function remove_protected_prefix( $file ) {
	return str_replace( '/_pda', '', $file );
}

/**
 * @param $option_name
 *
 * @return array
 */
function get_time_die_for_url( $option_name ) {
	$settings = get_option( pda_s3_BWPSConstant::PDAS3_SETTINGS_NAME );
	$time     = 1;
	$units    = 'minutes';
	$max      = 7;
	if ( $settings ) {
		$options = unserialize( $settings );
		if ( is_array( $options ) ) {
			if ( array_key_exists( $option_name, $options ) ) {
				$time_die = $options[ $option_name ];
				if ( $time_die != '' ) {
					$arr_time_die = explode( ' ', $time_die );
					$time         = $arr_time_die[0];
					$units        = $arr_time_die[1];
					if ( $units == "seconds" ) {
						$max = 604800;
					}
					if ( $units == "days" ) {
						$max = 7;
					}
					if ( $units == "hours" ) {
						$max = 168;
					}
					if ( $units == "minutes" ) {
						$max = 10080;
					}
				}
			}
		}
	}

	return array(
		'time'  => $time,
		'units' => $units,
		'max'   => $max
	);

}

/**
 * Check if the file is protected.
 *
 * @param int $post_id Post ID.
 *
 * @return bool
 */
function pda_s3_is_protected_file( $post_id ) {
	if ( ! class_exists( 'PDA_v3_Gold_Repository' ) ) {
		return false;
	}
	$repo = new PDA_v3_Gold_Repository();

	return $repo->is_protected_file( $post_id );
}

/**
 * Check whether the file is offloaded
 *
 * @param int $post_id Post ID
 *
 * @return bool
 */
function check_file_is_deleted_and_is_sync( $post_id ) {
	$s3_util = new PDA_S3_UTILS();

	return $s3_util->check_file_is_deleted( $post_id ) && pda_s3_check_file_synced_s3( $post_id );
}

/**
 * Get root folder
 *
 * @param int $attachment_id Attachment's ID
 * @param bool $include_domain Including domain mode
 *
 * @return string
 */
function pda_s3_get_root_folder( $attachment_id, $include_domain = true ) {
	$domain = $include_domain ? '/' . $_SERVER['HTTP_HOST'] : '';

	if ( ! class_exists( 'PDA_v3_Gold_Repository' ) ) {
		return PdaS3Constants::PDA_S3_PROTECTED_PUBLIC_FOLDER . $domain;
	}
	$repo = new PDA_v3_Gold_Repository();

	return $repo->is_protected_file( $attachment_id ) ? PdaS3Constants::PDA_S3_PROTECTED_ROOT_FOLDER . $domain : PdaS3Constants::PDA_S3_PROTECTED_PUBLIC_FOLDER . $domain;
}

/**
 * Get the previous root folder
 *
 * @param int $attachment_id Post ID
 *
 * @return string Root folder
 */
function pda_s3_get_previous_root_folder_by_mode( $attachment_id ) {
	if ( ! class_exists( 'PDA_v3_Gold_Repository' ) ) {
		return PdaS3Constants::PDA_S3_PROTECTED_PUBLIC_FOLDER . "/" . $_SERVER['HTTP_HOST'];
	}
	$repo = new PDA_v3_Gold_Repository();

	return ! $repo->is_protected_file( $attachment_id ) ? PdaS3Constants::PDA_S3_PROTECTED_ROOT_FOLDER . "/" . $_SERVER['HTTP_HOST'] : PdaS3Constants::PDA_S3_PROTECTED_PUBLIC_FOLDER . "/" . $_SERVER['HTTP_HOST'];
}

function pda_s3_remove_size_from_file_name( $url, $remove_extension = false ) {
	$url = preg_replace( '/^(\S+)-[0-9]{1,4}x[0-9]{1,4}(\.[a-zA-Z0-9\.]{2,})?/', '$1$2', $url );

	if ( $remove_extension ) {
		$ext = pathinfo( $url, PATHINFO_EXTENSION );
		$url = str_replace( ".$ext", '', $url );
	}

	return $url;
}

function pda_s3_convert_dimensions_to_size_name( $attachment_id, $dimensions ) {
	$w                     = ( isset( $dimensions[0] ) && $dimensions[0] > 0 ) ? $dimensions[0] : 1;
	$h                     = ( isset( $dimensions[1] ) && $dimensions[1] > 0 ) ? $dimensions[1] : 1;
	$original_aspect_ratio = $w / $h;
	$meta                  = wp_get_attachment_metadata( $attachment_id );

	if ( ! isset( $meta['sizes'] ) || empty( $meta['sizes'] ) ) {
		return null;
	}

	$sizes = $meta['sizes'];
	uasort( $sizes, function ( $a, $b ) {
		// Order by image area
		return ( $a['width'] * $a['height'] ) - ( $b['width'] * $b['height'] );
	} );

	$nearest_matches = array();

	foreach ( $sizes as $size => $value ) {
		if ( $w > $value['width'] || $h > $value['height'] ) {
			continue;
		}
		$aspect_ratio = $value['width'] / $value['height'];

		if ( $aspect_ratio === $original_aspect_ratio ) {
			return $size;
		}

		$nearest_matches[] = $size;
	}

	// Return nearest match
	if ( ! empty( $nearest_matches ) ) {
		return $nearest_matches[0];
	}

	return null;
}

function pda_s3_encode_filename_in_path( $file ) {
	$url = parse_url( $file );

	if ( ! isset( $url['path'] ) ) {
		// Can't determine path, return original
		return $file;
	}

	if ( isset( $url['query'] ) ) {
		// Manually strip query string, as passing $url['path'] to basename results in corrupt � characters
		$file_name = wp_basename( str_replace( '?' . $url['query'], '', $file ) );
	} else {
		$file_name = wp_basename( $file );
	}

	if ( false !== strpos( $file_name, '%' ) ) {
		// File name already encoded, return original
		return $file;
	}

	$encoded_file_name = rawurlencode( $file_name );

	if ( $file_name === $encoded_file_name ) {
		// File name doesn't need encoding, return original
		return $file;
	}

	return str_replace( $file_name, $encoded_file_name, $file );
}

/**
 * File image size from width
 *
 * @param $sizes
 * @param $width
 * @param $filename
 *
 * @return int|null|string
 */
function pda_s3_find_image_size_from_width( $sizes, $width, $filename ) {
	foreach ( $sizes as $name => $size ) {
		if ( $width === absint( $size['width'] ) && $size['file'] === $filename ) {
			return $name;
		}
	}

	return null;
}

function pda_s3_is_valid_pages_and_permission( $attachment_id ) {
	if ( is_admin() ) {
		return true;
	}

	$id = get_the_ID();
	if ( false === $id ) {
		return false;
	}

	if ( 'true' !== pda_s3_get_setting( PdaS3Constants::AUTO_REPLACE_PROTECTED_FILE ) ) {
		return false;
	}

	$post_types_selected = pda_s3_get_setting( PdaS3Constants::REPLACED_POSTS_TYPES );
	$is_white_list_post_type =  is_array( $post_types_selected ) && in_array( get_post_type( get_the_ID() ), $post_types_selected );

	$selected_items = pda_s3_get_setting( PdaS3Constants::REPLACED_PAGES_POSTS );
	if ( is_array( $selected_items ) ) {
		$ids = array_map( 'intval', $selected_items );
	} else {
		$ids = array();
	}

	$is_white_list_post =  in_array( $id, $ids, true );

	if ( $is_white_list_post || $is_white_list_post_type ) {
		if ( ! class_exists( 'Pda_Gold_Functions' ) && ! class_exists( 'PDA_v3_Gold_Repository' ) ) {
			return false;
		}
		$gold_function = new Pda_Gold_Functions;
		if ( ! method_exists( $gold_function, 'check_file_access_permission_for_post') ) {
			return false;
		}

		$is_valid_role = $gold_function->check_file_access_permission_for_post( $attachment_id );

		return $is_valid_role;
	}
	return false;
}

/**
 * Check whether user has right permission to see the signed URL
 *
 * @param $attachment_id
 *
 * @return bool
 */
function pda_s3_is_valid_perm_to_show_signed_url( $attachment_id ) {
	if ( Yme_Plugin_Utils::is_plugin_activated( 'pda_v3' ) === - 1 || Yme_Plugin_Utils::is_plugin_activated( 'pda_gold' ) === - 1 ) {

		//Logic
		//1. If not synced return false
		//2. If not protected return false
		//3. If not Enable auto replace return false
		//4. If the current page and post in whitelist return false.
		if ( ! pda_s3_check_file_synced_s3( $attachment_id ) ) {
			return false;
		}

		$repo         = new PDA_v3_Gold_Repository();
		$is_protected = $repo->is_protected_file( $attachment_id );

		if ( ! $is_protected ) {
			return false;
		}

		return pda_s3_is_valid_pages_and_permission( $attachment_id );
	}
}

/**
 * Add file to garbage
 *
 * @param int $attachment_id Attachment's ID
 * @param string $filepath File path
 */
function pda_s3_add_to_garbage( $attachment_id, $filepath ) {
	$garbage = get_option( PdaS3Constants::PDA_S3_REMOVE_FILES, false );
	if ( false === $garbage ) {
		$garbage = array();
		$garbage[ $attachment_id ] = $filepath;
	}
	$garbage = (array) json_decode( $garbage );
	$garbage[ $attachment_id ] = $filepath;
	error_log( wp_json_encode( $garbage ) );
	update_option( PdaS3Constants::PDA_S3_REMOVE_FILES,  wp_json_encode( $garbage ) );
}

/**
 * Clear garbage
 *
 * @param string $filepath The file path
 *
 * @return bool
 */
function pda_s3_clean_garbage( $attachment_id ) {

	$garbage = get_option( PdaS3Constants::PDA_S3_REMOVE_FILES, false );
	if ( false === $garbage ) {
		return false;
	}
	$garbage = (array) json_decode( $garbage );
	if ( ! isset( $garbage[ $attachment_id ] ) ) {
		return false;
	}

	$filepath = $garbage[ $attachment_id ];

	if ( ! is_file( $filepath ) ) {
		return false;
	}

	return unlink( $garbage[ $attachment_id ] );

}

/**
 * Check whether the file is synced but not delete
 *
 * @param int $post_id Post ID
 *
 * @return bool
 */
function check_file_is_sync_but_not_delete( $post_id ) {
    $s3_util = new PDA_S3_UTILS();

    return pda_s3_check_file_synced_s3( $post_id ) && ! $s3_util->check_file_is_deleted( $post_id );
}

function remove_file_is_synced_in_server( $post_id, $utils, $file_paths ) {
    foreach ( $file_paths as $path ) {
        $utils->remove_local_file( $post_id, $path, false );
    }
}

/**
 * Get signed URL time out
 * @return bool|string
 */
function pda_get_signed_url_timeout() {
	$time_die = pda_s3_get_setting( PdaS3Constants::PDAS3_EXPIRED_TIME_OPTION );
	if ( false === $time_die ) {
		$time_die = PdaS3Constants::PDAS3_DEFAULT_EXPIRED_TIME;
	}
	return $time_die;
}

function handle_for_bulk_action_offload( $post_id, $utils ) {
    $is_sync = pda_s3_check_file_synced_s3( $post_id );
    if ( ! $is_sync ) {
        try {
            $utils->sync_file_to_s3( $post_id );
        } catch ( Exception $exception ) {
            PDA_S3_UTILS::Logger()->error( $exception->getMessage() );
        }
    }
    $is_synced_not_delete = check_file_is_sync_but_not_delete( $post_id );
    if ( $is_synced_not_delete ) {
        try {
            $file_paths = $utils->get_file_paths( $post_id, false );
            remove_file_is_synced_in_server( $post_id, $utils, $file_paths );
        } catch ( Exception $exception ) {
            PDA_S3_UTILS::Logger()->error( $exception->getMessage() );
        }
    }
}

/**
 * Should clear cache checker
 * If not configured for cdn then no need to clear cache
 * Else
 *   if not moving then clear
 *   else
 *     only clear the protecting file
 *
 * @param int $attachment_id Attachment's ID
 * @param bool $is_moving Is moving
 *
 * @return bool
 */
function should_clear_cache( $attachment_id, $is_moving ) {
	$cdn = pda_s3_get_setting( 'cdn' );
	if ( false === $cdn || empty( $cdn ) ) {
		return false;
	}

	if ( ! $is_moving ) {
		return true;
	}

	return pda_s3_is_protected_file( $attachment_id );
}

/**
 * Get attachment id by file path
 *
 * @param $file_path
 *
 * @return integer
 */
function get_attachment_id_by_file_path( $file_path ) {
    $file_url = str_replace( "\\", "", $file_path );
    $wp_upload_dir = wp_upload_dir();
    $basedir       = $wp_upload_dir['basedir'];
    $file_path = ltrim(str_replace( $basedir, '', $file_url), '/');
    $massaged_file = preg_replace( '/^(\S+)-[0-9]{1,4}x[0-9]{1,4}(\.[a-zA-Z0-9\.]{2,})?/', '$1$2', $file_path );

    $table = 'wp_postmeta';
    global $wpdb;
    $post = $wpdb->get_row( $wpdb->prepare( "SELECT post_id FROM $table WHERE meta_key = '_wp_attached_file' AND meta_value = %s", $massaged_file) );
    if ( is_null( $post ) ) {
    	return false;
    }
    return $post->post_id;
}

/**
 * Check role protection on s3
 *
 * @return boolean
 */
function pda_s3_check_role_protection() {
	if ( class_exists( 'Pda_Gold_Functions' ) ) {
		$gold_function = new Pda_Gold_Functions();
		if ( method_exists( $gold_function, 'pda_check_role_protection') ) {
			return $gold_function->pda_check_role_protection();
		}
	}
	return false;
}

/**
 * Convert all URLs from content to signed URLs if file synced.
 *
 * @param string $value Value.
 *
 * @return string
 */
function pda_s3_handle_the_content( $value ) {
	$pda_s3_helper = PDA_S3_Helper::get();
	if ( false === $pda_s3_helper ) {
		return $value;
	}

	return $pda_s3_helper->hand_urls_for_content( $value );
}
