<?php

/**
 * The plugin bootstrap file
 *
 * This file is read by WordPress to generate the plugin information in the plugin
 * admin area. This file also includes all of the dependencies used by the plugin,
 * registers the activation and deactivation functions, and defines a function
 * that starts the plugin.
 *
 * @link              https://preventdirectaccess.com/extensions/?utm_source=user-website&utm_medium=pluginsite_link&utm_campaign=pda-s3
 * @since            1.1.6
 * @package           pda_s3
 *
 * @wordpress-plugin
 * Plugin Name:       PDA S3 Integration
 * Plugin URI:        https://preventdirectaccess.com/extensions/?utm_source=user-website&utm_medium=pluginsite_link&utm_campaign=pda-s3
 * Description:       Sync your private protected files to Amazon S3 and use their expiring links on your content for faster load time and better security.
 * Version:          1.1.6
 * Author:            BWPS
 * Author URI:        https://preventdirectaccess.com/extensions/?utm_source=user-website&utm_medium=pluginsite_link&utm_campaign=pda-s3
 * License:           GPL-2.0+
 * License URI:       http://www.gnu.org/licenses/gpl-2.0.txt
 * Text Domain:       pda-s3
 * Domain Path:       /languages
 */


// If this file is called directly, abort.
if ( ! defined( 'WPINC' ) ) {
	die;
}

/**
 * Currently plugin version.
 * Start at version 1.0.0 and use SemVer - https://semver.org
 * Rename this for your plugin and update it as you release new versions.
 */
define( 'pda_s3_VERSION', '1.1.6' );
define( 'PDA_S3_DIRNAME', dirname( __FILE__ ) );
if ( ! defined( 'PDA_S3_DIR_PATH' ) ) {
	define( 'PDA_S3_DIR_PATH', plugin_dir_path( __FILE__ ) );
}
define( 'PDA_S3_PLUGIN_NAME', 'PDA S3 Integration' );

/**
 * The code that runs during plugin activation.
 * This action is documented in includes/class-pda-s3-activator.php
 */
function activate_pda_s3() {
	require_once plugin_dir_path( __FILE__ ) . 'includes/class-pda-s3-activator.php';
	pda_s3_Activator::activate(plugin_basename( __FILE__ ));
}

/**
 * The code that runs during plugin deactivation.
 * This action is documented in includes/class-ppda-s3-deactivator.php
 */
function deactivate_pda_s3() {
	require_once plugin_dir_path( __FILE__ ) . 'includes/class-pda-s3-deactivator.php';
	pda_s3_Deactivator::deactivate();
}

register_activation_hook( __FILE__, 'activate_pda_s3' );
register_deactivation_hook( __FILE__, 'deactivate_pda_s3' );

/**
 * The core plugin class that is used to define internationalization,
 * admin-specific hooks, and public-facing site hooks.
 */
require plugin_dir_path( __FILE__ ) . 'includes/class-pda-s3.php';

/**
 * Begins execution of the plugin.
 *
 * Since everything within the plugin is registered via hooks,
 * then kicking off the plugin from this point in the file does
 * not affect the page life cycle.
 *
 * @since    1.0.0
 */
function run_pda_s3() {

	$plugin = new pda_s3();
	$plugin->run();

    $configs = include PDA_S3_DIR_PATH . 'includes/class-pda-s3-configs.php';

	if ( method_exists( 'Puc_v4p8_Factory', 'buildUpdateChecker' ) ) {
		Puc_v4p8_Factory::buildUpdateChecker(
			$configs->update_url,
			__FILE__,
			'pda-s3'
		);
	} elseif ( method_exists( 'Puc_v4_Factory', 'buildUpdateChecker' ) ) {
		Puc_v4_Factory::buildUpdateChecker(
			$configs->update_url,
			__FILE__,
			'pda-s3'
		);
	}

	/**
	 * CronJobs.
	 */
	new \PDAS3\AsyncTasks\PDA_S3_Clear_CF_Cache();
	new \PDAS3\AsyncTasks\PDA_S3_Offload_Backup_Files();
	new \PDAS3\AsyncTasks\PDA_S3_Copy_Files_From_Bucket();
}

run_pda_s3();
