<?php

/**
 * The admin-specific functionality of the plugin.
 *
 * @link       preventdirectaccess.com
 * @since      1.0.0
 *
 * @package    Pda_Woocommerce
 * @subpackage Pda_Woocommerce/admin
 */

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    Pda_Woocommerce
 * @subpackage Pda_Woocommerce/admin
 * @author     BWPS <hello@preventdirectaccess.com>
 */
class Pda_Woocommerce_Admin {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string $plugin_name The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string $version The current version of this plugin.
	 */
	private $version;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 *
	 * @param      string $plugin_name The name of this plugin.
	 * @param      string $version The version of this plugin.
	 */
	public function __construct( $plugin_name, $version ) {

		$this->plugin_name = $plugin_name;
		$this->version     = $version;

	}

	/**
	 * Register the stylesheets for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_styles() {

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in Pda_Woocommerce_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The Pda_Woocommerce_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */
		if ( "toplevel_page_ppp_gold_options" === get_current_screen()->id ) {
			wp_enqueue_style( $this->plugin_name . '_ppp_woo_select2', plugin_dir_url( __FILE__ ) . 'css/select2.min.css', array(), $this->version, 'all' );
		}
		if ( "toplevel_page_pda-gold" == get_current_screen()->id ) {
			wp_enqueue_style( $this->plugin_name . '_ppp_woo', plugin_dir_url( __FILE__ ) . 'css/pda-woocommerce-settings.css', array(), $this->version, 'all' );
		}
	}

	/**
	 * Register the JavaScript for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_scripts() {

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in Pda_Woocommerce_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The Pda_Woocommerce_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */

		if ( "toplevel_page_ppp_gold_options" === get_current_screen()->id ) {
			wp_enqueue_script( $this->plugin_name . '_ppp_woo_select2', plugin_dir_url( __FILE__ ) . 'js/lib/select2.min.js', array( 'jquery' ), '4.0.3', true );

			wp_enqueue_script( 'ppp_woo_js' . $this->plugin_name, plugin_dir_url( __FILE__ ) . './js/pda-woocommerce-admin.js', array( 'jquery' ), $this->version, false );
			wp_localize_script( 'ppp_woo_js' . $this->plugin_name, 'ppp_woo_data', array( 'ajax_url' => admin_url( 'admin-ajax.php' ) ) );

			$id_protected = apply_filters( 'ppp_all_id_protected_page_post', array() );
			$id_protected = array_map( function ( $id ) {
				return (int) $id->post_id;
			}, $id_protected );
			wp_enqueue_script( 'search_page_js', plugin_dir_url( __FILE__ ) . './js/pda-woocommerce-search.js', array( 'jquery' ), $this->version, false );
			wp_localize_script( 'search_page_js', 'page_data', array(
				'ajax_url'     => admin_url( 'admin-ajax.php' ),
				'id_protected' => $id_protected,
			) );
		}
	}

	/**
	 * Replace private link in email content
	 *
	 * @param $email
	 */
	function add_private_link( $email ) {
		$link       = $this->get_link_page();
		$settings   = get_option( 'ppp_gold_woocommerce' );
		$text_input = '';
		if ( $settings ) {
			$arr_settings = unserialize( get_option( 'ppp_gold_woocommerce' ) );
			if ( array_key_exists( 'ppp_text_input', $arr_settings ) ) {
				$text_input = $arr_settings['ppp_text_input'];
			}
		}

		preg_match_all( '/\[+[\w\s]+\]/', $text_input, $matches );
		foreach ( $matches[0] as $value ) {
			$text_private_link = str_replace( '[', '', str_replace( ']', '', $value ) );
			$text_new          = "<a target='_blank' href='$link'>$text_private_link</a>";
			$post_id           = url_to_postid( $link );
			$private_link      = $this->get_default_private_link( $post_id );

			$settings_ppp = get_option( "ppp_setting_options" );
			$prefix_url   = 'privatepup/';
			if ( isset( $settings_ppp ) ) {
				$files_setting = unserialize( $settings_ppp );
				if ( is_array( $files_setting ) && array_key_exists( 'prefix_private_link', $files_setting ) ) {
					$prefix_url = $files_setting['prefix_private_link'] . "/";
				}
			}

			$home_url = home_url( '/' );
			if ( ! empty( $private_link ) ) {
				$text_new = "<a target='_blank' href='$home_url$prefix_url$private_link'>$text_private_link</a>";
			}
			$text_input = str_replace( $value, $text_new, $text_input );
		}
		echo "<div>" . $text_input . "</div><br>";
	}

	function add_tab_woo_page_post() {
		$wooActivated = is_plugin_active( 'woocommerce/woocommerce.php' );
		if ( $wooActivated ) {
			?>
			<div class="main_container">
				<?php
				$link  = $this->get_link_page();
				$title = $this->get_title_page();

				$setings        = get_option( 'ppp_gold_woocommerce' );
				$text_input     = '';
				$poisition_text = '';
				$templates      = [];
				if ( $setings ) {
					$arr_settings = unserialize( get_option( 'ppp_gold_woocommerce' ) );
					if ( array_key_exists( 'ppp_text_input', $arr_settings ) ) {
						$text_input = $arr_settings['ppp_text_input'];
					}
					if ( array_key_exists( 'ppp_poisition_text', $arr_settings ) ) {
						$poisition_text = $arr_settings['ppp_poisition_text'];
					}
					if ( array_key_exists( 'ppp_templates', $arr_settings ) ) {
						$templates = $arr_settings['ppp_templates'] !== "" ? $arr_settings['ppp_templates'] : [];
					}
				}

				$magic_settings        = get_option( 'pda_settings_magic_link' );
				$time_die_ptivate_link = "7 days";
				if ( $magic_settings ) {
					$magic_options = unserialize( $magic_settings );
					if ( is_array( $magic_options ) ) {
						if ( array_key_exists( "ppp_time_die_private_link", $magic_options ) ) {
							$time_die_ptivate_link = $magic_options["ppp_time_die_private_link"];
						}
					}
				}
				$time = explode( ' ', $time_die_ptivate_link )[0];
				$unit = explode( ' ', $time_die_ptivate_link )[1];

				$wc_roles = wc_get_order_statuses();
				?>
				<form id="ppp_gold_woo_form">
					<input type="hidden" nonce="<?php echo wp_create_nonce( 'ppp_ajax_woo_nonce' ); ?>"
					       id="ppp_woo_nonce"/>
					<table class="settings_table" cellpadding="4">
						<tr>
							Please refer to <a target="_blank"
							                   href="https://preventdirectaccess.com/docs/woocommerce-integration-extension/">this
								guide</a> on how to use WooCommerce Integration extension with Protect Pages & Posts
							plugin.
						</tr>
						<tr>
							<td class="feature-input"><span class="feature-input"></span></td>
							<td>
								<p>
									<label><?php echo esc_html__( 'Choose Email Templates', 'prevent-direct-access-gold' ) ?></label>
									<?php echo esc_html__( 'These email templates are created by WooCommerce plugin by default. Please configure them under Emails/Settings page.', 'prevent-direct-access-gold' ) ?>
								</p>
								<select multiple="multiple" id="pwc-templates" required>
									<?php foreach ( $wc_roles as $role_name => $role_info ): ?>
										<option <?php echo in_array( $role_name, $templates ) ? 'selected="selected"' : '' ?>
												value="<?php echo $role_name ?>"><?php echo $role_info ?></option>
									<?php endforeach; ?>
								</select>
							</td>
						</tr>
						<tr>
							<td class="feature-input"><span class="feature-input"></span></td>
							<td>
								<p>
									<label><?php echo esc_html__( 'Insert Custom Text', 'prevent-direct-access-gold' ) ?></label>
									<?php _e( 'Insert any text that you want to include in the chosen email templates. The text within the square brackets [] will become <a href="https://preventdirectaccess.com/docs/how-to-use-private-magic-links/" target="_blank">Private Magic Links</a>.', 'prevent-direct-access-gold' ) ?>
								</p>
								<textarea placeholder="Thank you for ordering from us. This is our [product-page]"
								          id="ppp_text_input" rows="4" cols="50"><?php echo $text_input; ?></textarea>
								<p id="pda_error_message">Please insert the correct format text</p>
							</td>
						</tr>
						<tr>
							<td class="feature-input"><span class="feature-input"></span></td>
							<td>
								<p>
									<label><?php echo esc_html__( 'Choose Private Content', 'prevent-direct-access-gold' ) ?></label>
									<?php echo esc_html__( 'Choose a protected page or post that you want to link the custom text above', 'prevent-direct-access-gold' ) ?>
								</p>
								<div>
									<?php wp_nonce_field( 'internal-linking', '_ajax_linking_nonce', false ); ?>
									<input required autocomplete="off" type="search" id="ppp_page"
									       placeholder="Search..."
									       value="<?php echo $title; ?>"/>
								</div>
								<div class="no-access-search-container">
									<ul id="pda_search_result">
									</ul>
								</div>
								<input id="ppp_title_page" type="hidden" value="<?php _e( $link . ';' . $title ); ?>"/>
							</td>
						</tr>
						<tr>
							<td class="feature-input"><span class="feature-input"></span></td>
							<td>
								<p>
									<label><?php echo esc_html__( 'Select Text Position', 'prevent-direct-access-gold' ) ?></label>
									<?php echo esc_html__( 'Select where to insert the custom text on the email templates', 'prevent-direct-access-gold' ) ?>
								</p>
								<select id="poisition_text">
									<?php if ( $poisition_text == "header" ) { ?>
										<option selected value="header">After Header</option>
										<option value="order">After Order</option>
										<option value="footer">In Footer</option>
									<?php } elseif ( $poisition_text == "order" ) { ?>
										<option value="header">After Header</option>
										<option selected value="order">After Order</option>
										<option value="footer">In Footer</option>
									<?php } elseif ( $poisition_text == "footer" ) { ?>
										<option value="header">After Header</option>
										<option value="order">After Order</option>
										<option selected value="footer">In Footer</option>
									<?php } else { ?>
										<option value="header">After Header</option>
										<option value="order">After Order</option>
										<option value="footer">In Footer</option>
									<?php } ?>
								</select>
							</td>
						</tr>
						<tr>
							<td class="feature-input"><span class="feature-input"></span></td>
							<td>
								<p>
									<label><?php echo esc_html__( 'Set Expiration Time', 'prevent-direct-access-gold' ) ?></label>
									<?php echo esc_html__( 'Set default expiration time for the private magic links embedded on WooCommerce emails. ', 'prevent-direct-access-gold' );
									if ( yme_plugin_utils::is_plugin_activated( 'magic_link' ) === - 1 ) {
									echo ' Please update this value under <a target=\'_blank\' href="' . home_url( '/' ) . 'wp-admin/admin.php?page=ppp_gold_options&tab=magic_link' . '">Magic Links</a> tab.';
									?>
								<div>
									<select disabled class="pda_expiration_time">
										<option value=""><?php echo $time; ?></option>
									</select>
									<select disabled class="pda_expiration_unit">
										<option value=""><?php echo $unit; ?></option>
									</select>
								</div>
								<?php } else { ?>
									Please purchase <a
											target="_blank"
											href="https://preventdirectaccess.com/extensions/private-magic-links/?utm_source=user-website&utm_medium=settings_link&utm_campaign=pda_woo">Private
										Magic Links</a> extension to change this value.
									<div>
										<select disabled class="pda_expiration_time">
											<option value="">7</option>
										</select>
										<select disabled class="pda_expiration_unit">
											<option>days</option>
										</select>
									</div>

								<?php } ?>
								</p>
							</td>
						</tr>
					</table>
					<?php
					submit_button();
					?>
				</form>
			</div>
			<?php
		} else {
			?>
			<div class="main_container">
				Please activate <a href="https://wordpress.org/plugins/woocommerce/" target="_blank" rel="noopener noreferrer">WooCommerce</a> plugin to use
				amazing features from <a
						href="https://preventdirectaccess.com/extensions/woocommerce-integration/?utm_source=user-website&utm_medium=plugin_link&utm_campaign=pda-woocommerce" target="_blank" rel="noopener noreferrer">PDA WooCommerce Integration</a>.
			</div>
			<?php
		}

		wp_enqueue_style( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'css/pda-woocommerce-settings.css', array(), $this->version, 'all' );

	}

	private function get_default_private_link( $post_id ) {
		$link = "";
		if ( class_exists( 'PPP_Repository' ) ) {
			$repo                   = new PPP_Repository;
			$page_post_is_protected = apply_filters( 'ppp_woo_check_page_post_is_protected', $post_id );
			if ( $page_post_is_protected !== true ) {
				return $link;
			} else {
				$private_link = $repo->get_default_private_link( $post_id );
				if ( ! is_null( $private_link ) ) {
					$link = $private_link->url;
				} else {
					$link = $repo->insert_defaul_private_link( $post_id );
				}
			}

			return $link;
		}

		return $link;
	}

	function get_title_page() {
		$setings = get_option( 'ppp_gold_woocommerce' );
		$title   = "";
		if ( $setings ) {
			$arr_settings = unserialize( get_option( 'ppp_gold_woocommerce' ) );
			if ( array_key_exists( 'ppp_page_link', $arr_settings ) ) {
				$option     = $arr_settings['ppp_page_link'];
				$title_link = explode( ";", $option );
				$title      = $title_link[1];
			}
		}

		return $title;
	}

	function get_link_page() {
		$setings = get_option( 'ppp_gold_woocommerce' );
		$link    = "";
		if ( $setings ) {
			$arr_settings = unserialize( get_option( 'ppp_gold_woocommerce' ) );
			if ( array_key_exists( 'ppp_page_link', $arr_settings ) ) {
				$option     = $arr_settings['ppp_page_link'];
				$title_link = explode( ";", $option );
				$link       = $title_link[0];
			}
		}

		return $link;
	}

	function woo_update_settings() {
		$nonce = $_REQUEST['security_check'];
		if ( ! wp_verify_nonce( $nonce, 'ppp_ajax_woo_nonce' ) ) {
			wp_die( 'invalid_nonce' );
		}
		$settings = $_REQUEST['settings'];
		$woo      = serialize( $settings );
		update_option( 'ppp_gold_woocommerce', $woo );
		wp_send_json( true );
		wp_die();
	}

	function handle_woocomerce_be4_send_email( $order ) {
		$status       = $order->get_status();
		$order_status = PDA_WC_Setting_Utils::get_settings( 'ppp_templates' );
		if ( $order_status === "" ) {
			return;
		}

		if ( in_array( "wc-$status", $order_status ) ) {
			$position = PDA_WC_Setting_Utils::get_settings( 'ppp_poisition_text' );
			if ( ! is_null( $position ) ) {
				switch ( $position ) {
					case "header":
						add_action( 'woocommerce_email_order_details', array( $this, 'add_private_link' ), 10, 1 );
						break;
					case "order":
						add_action( 'woocommerce_email_order_meta', array( $this, 'add_private_link' ), 10, 1 );
						break;
					case "footer":
						add_action( 'woocommerce_email_footer', array( $this, 'add_private_link' ), 10, 1 );
						break;

				}
			}
		}
	}

	function handle_order_completed() {
		$position = PDA_WC_Setting_Utils::get_settings( 'ppp_poisition_text' );
		if ( ! is_null( $position ) ) {
			switch ( $position ) {
				case "header":
					add_action( 'woocommerce_email_order_details', array( $this, 'add_private_link' ), 10, 1 );
					break;
				case "order":
					add_action( 'woocommerce_email_order_meta', array( $this, 'add_private_link' ), 10, 1 );
					break;
				case "footer":
					add_action( 'woocommerce_email_footer', array( $this, 'add_private_link' ), 10, 1 );
					break;

			}
		}
	}

	function check_plugin_ppp_gold_activate() {
		$message         = "";
		$papActivated    = Yme_Plugin_Utils::is_plugin_activated( 'pap_gold' );
		$papOldActivated = Yme_Plugin_Utils::is_plugin_activated( 'pap_gold_old' );
		$pdaActivated    = Yme_Plugin_Utils::is_plugin_activated( 'pda_gold' );
		$pdaV3Activated  = Yme_Plugin_Utils::is_plugin_activated( 'pda_v3' );

		if ( $papActivated !== - 1 && $papOldActivated !== - 1 && $pdaActivated !== - 1 && $pdaV3Activated !== - 1 ) {
			$message = "Please install and activate <a target=\"_blank\" rel=\"noopener noreferrer\" href=\"https://preventdirectaccess.com/pricing/\">Prevent Direct Access Gold</a> or <a target=\"_blank\" rel=\"noopener noreferrer\" href=\"https://preventdirectaccess.com/protect-wordpress-pages-posts/\">Protect Pages & Posts Gold</a> plugin";
		}

		if ( ! empty( $message ) ) {
			?>
			<div class="notice notice-error is-dismissible">
				<p><b>PDA WooCommerce Integration: </b><?php _e( $message, 'pda-woocommerce' ); ?></p>
			</div>
			<?php
		}

//	    $updates = plugin_has_updates();
//	    if ( ! empty( $updates ) ) {
//		    if ( count( $updates ) > 1 ) {
//			    $last = $updates[count( $updates ) -1];
//			    $tmp  = array_slice( $updates, 0, count( $updates ) - 1 );
//			    $text = implode( ', ', $tmp );
//			    $text .= " and $last";
//		    } else {
//			    $text = implode( ', ', $updates );
//		    }
//		    ?>
		<!--		    <div class="notice notice-warning is-dismissible">-->
		<!--			    <p>-->
		<!--				    <b>--><?php //echo "PDA Woocommerce Integration: "; ?><!--</b> Please upgrade --><?php //echo $text ?>
		<!--				    plugin to their latest versions to enjoy our new awesome features.-->
		<!--			    </p>-->
		<!--		    </div>-->
		<!--		    --><?php
//	    }

	}

	function reset_downloads_limit_woo_by_user_email( $order_id ) {
		global $wpdb;
		$woo_table = $wpdb->prefix . "woocommerce_downloadable_product_permissions";

		if ( ! empty( pda_woo_get_settings_value( PDA_Woo_Constants::PDA_PRODUCT_ID ) ) ) {
			$id = explode( ",", pda_woo_get_settings_value( PDA_Woo_Constants::PDA_PRODUCT_ID ) );
			if ( count( $id ) === 2 ) {
				$order = $this->get_data_order_by_order_id( $order_id, $woo_table, $wpdb );
				if ( ! empty( $order ) ) {
					$user_email = $order->user_email;
					$product_id = $order->product_id;

					if ( $id[1] == $product_id ) {
						$this->update_downloads_remaining_and_access_expires( $user_email, $id[0], $woo_table, $wpdb );
					}
				}
			}
		}
	}

	function update_downloads_remaining_and_access_expires( $user_email, $product_id, $woo_table, $wpdb ) {
		$products = $this->get_all_product_in_woo_by_email_user_and_product_id( $user_email, $product_id, $woo_table, $wpdb );

		foreach ( $products as $product ) {
			$order = new WC_Order( $product->order_id );
			$items = $order->get_items();

			foreach ( $items as $item ) {
				$quantity = $item['quantity'];
			}

			$wc_product = wc_get_product( $product->product_id );
			if ( $wc_product->get_download_limit() !== - 1 || $wc_product->get_download_expiry() !== - 1 ) {
				$data_update    = array();
				$download_limit = null;
				if ( $wc_product->get_download_limit() !== - 1 ) {
					$download_limit                     = ( wc_get_product( $product->product_id )->get_download_limit() ) * $quantity;
					$data_update['downloads_remaining'] = $download_limit;
				}

				$new_expires_date = null;
				if ( $wc_product->get_download_expiry() !== - 1 ) {
					$date        = $product->access_expires;
					$today       = date( 'Y-m-d' );
					$expires_day = wc_get_product( $product->product_id )->get_download_expiry();
					$date_format = 'Y-m-d H:i:s';
					if ( strtotime( $today ) > strtotime( $date ) ) {
						$new_expires_date = strtotime( '+' . $expires_day . ' day', strtotime( $today ) );
						$new_expires_date = date( $date_format, $new_expires_date );
					} else {
						$new_expires_date = strtotime( '+' . $expires_day . ' day', strtotime( $date ) );
						$new_expires_date = date( $date_format, $new_expires_date );
					}
					$data_update['access_expires'] = $new_expires_date;
				}

				$wpdb->update(
					$woo_table,
					$data_update,
					array(
						'permission_id' => $product->permission_id,
					)
				);
			}
		}
	}

	function get_data_order_by_order_id( $order_id, $woo_table, $wpdb ) {
		$query_string = $wpdb->prepare( "SELECT * FROM $woo_table WHERE order_id = %s", $order_id );

		return $wpdb->get_row( $query_string );
	}

	function get_all_product_in_woo_by_email_user_and_product_id( $user_email, $product_id, $woo_table, $wpdb ) {
		$query_string = $wpdb->prepare( "SELECT * FROM $woo_table WHERE user_email = %s and product_id = %s", $user_email, $product_id );

		return $wpdb->get_results( $query_string );
	}

	function handle_woocommerce_payment_complete( $order_id ) {
		$max_ip_access = pda_woo_get_settings_value( PDA_Woo_Constants::PDA_NUMBER_OF_DEVICES_ACCESS_GLOBAL_OPTION );
		if ( ! empty ( $max_ip_access ) ) {
			update_post_meta( $order_id, PDA_Woo_Constants::PDA_WOO_MAX_IP_ACCESS_META, $max_ip_access );
		}

		$this->reset_downloads_limit_woo_by_user_email( $order_id );
	}

	function pda_woocommerce_integration_render_settings() {
		$is_woo_activated = is_plugin_active( 'woocommerce/woocommerce.php' );
		?>
		<div class="main_container">
			<?php if ( $is_woo_activated ) { ?>
			<form id="pda_woocommerce_setting_form">
				<input type="hidden" nonce="<?php echo wp_create_nonce( 'pda_woocommerce_ajax_nonce' ); ?>"
				       id="nonce_pda_woocommerce"/>
				<table class="pda_woo_settings_table" cellpadding="4">
					<tr>
						<td colspan="2">
							<h3><?php echo esc_html__( 'RESET DOWNLOAD LIMIT', 'prevent-direct-access-gold' ) ?></h3>
						</td>
					</tr>
					<?php
					include PDA_WC_PLUGIN_PATH . "includes/views/view-pda-woocommerce-reset-download-limit.php";
					?>
					<tr>
						<td colspan="2">
							<hr>
						</td>
					</tr>
					<tr>
						<td colspan="2">
							<h3><?php echo esc_html__( 'SET IP RESTRICTION', 'prevent-direct-access-gold' ) ?></h3></td>
					</tr>
					<?php
					include PDA_WC_PLUGIN_PATH . "includes/views/view-pda-woocommerce-ip-limit-for-order.php";
					?>
					<tr>
						<td></td>
						<td>
							<?php submit_button(); ?>
						</td>
					</tr>
				</table>
			</form>
		<?php }else { ?>
			Please activate <a href="https://wordpress.org/plugins/woocommerce/" target="_blank" rel="noopener noreferrer">WooCommerce</a> plugin to use amazing features from
			<a href="https://preventdirectaccess.com/extensions/woocommerce-integration/?utm_source=user-website&utm_medium=plugin_link&utm_campaign=pda-woocommerce" target="_blank" rel="noopener noreferrer">PDA WooCommerce Integration</a>.
			<?php } ?>
		</div>
		<?php
	}

	function load_js_css_for_setting_woo() {
		$screen = get_current_screen();
		if ( PDA_Woo_Constants::PDA_WOO_SETTING_SCREEN === $screen->id ) {
			wp_enqueue_script( 'pad_woocommerce_integration_setting_js' . $this->plugin_name, plugin_dir_url( __FILE__ ) . './js/pda-woocommerce-settings-pdav3.js', array( 'jquery' ), $this->version );
			wp_localize_script( 'pad_woocommerce_integration_setting_js' . $this->plugin_name, 'pad_woocommerce_integration_data', array( 'ajax_url' => admin_url( 'admin-ajax.php' ) ) );
		}
	}

	public function woo_update_settings_pdav3() {
		$nonce = $_REQUEST['security_check'];
		if ( ! wp_verify_nonce( $nonce, 'pda_woocommerce_ajax_nonce' ) ) {
			wp_die( 'invalid_nonce' );
		}

		$settings = $_REQUEST['settings'];
		update_option( PDA_Woo_Constants::PDA_WOO_SETTINGS_FOR_PDAV3, wp_json_encode( $settings ) );
	}

	function display_all_product_woo() {
		if ( get_current_user_id() !== 0 ) {
			$posts = get_posts( apply_filters( 'woocommerce_my_account_my_orders_query', array(
				'numberposts' => - 1,
				'meta_key'    => '_customer_user',
				'meta_value'  => get_current_user_id(),
				'post_type'   => wc_get_order_types( 'view-orders' ),
				'post_status' => array_keys( wc_get_order_statuses() )
			) ) );
			?>
			<table>
				<tr>
					<th width="">Product</th>
					<th width="">Download</th>
					<th width="">Downloads remaining</th>
					<th width="">Expires</th>
					<?php
					foreach ( $posts

					as $post ) {
					$detail = wc_get_order( $post );
					$items  = $detail->get_downloadable_items()
					?>
					<?php
					foreach ( $items

					as $item ) {
					$downloads_remaining = '' === $item['downloads_remaining'] ? 'Unlimited' : $item['downloads_remaining'];
					$access_expires      = null === $item['access_expires'] ? 'Never' : $item['access_expires']->format( 'F d, Y' );
					?>
				<tr>
					<td><span><?php echo esc_html( $item['product_name'] ) ?></span></td>
					<td>
						<a href="<?php echo esc_attr( $item['download_url'] ) ?>"><?php echo esc_html( $item['download_name'] ) ?></a>
					</td>
					<td><?php echo esc_html( $downloads_remaining ) ?></td>
					<td><?php echo esc_html( $access_expires ) ?></td>
				</tr>
				<?php
				}
				?>
				<?php
				}
				?>
			</table>
			<?php
		}
	}

	/**
	 * @param $order_id
	 */
	public function block_access_to_woo_order_by_ip( $order_id ) {
		$ip_referer = $_SERVER['REMOTE_ADDR'];
		$is_max_ip_access_global = ! empty( pda_woo_get_settings_value( PDA_Woo_Constants::PDA_NUMBER_OF_DEVICES_ACCESS_GLOBAL_OPTION ) );
		if ( pda_woo_is_bot_detected() || ( $is_max_ip_access_global && ! pda_woo_handle_ip_to_allow_access( $order_id, $ip_referer ) ) ) {
			header( "HTTP/1.1 301 Moved Permanently" );
			header( "Location: " . get_bloginfo( 'url' ) . '/404.php' );
			exit();
		}
	}

	/**
	 * @param $order
	 */
	public function add_list_ip_access_after_shipping_item( $order ) {
		$is_max_ip_access_global = ! empty( pda_woo_get_settings_value( PDA_Woo_Constants::PDA_NUMBER_OF_DEVICES_ACCESS_GLOBAL_OPTION ) );
		if ( $is_max_ip_access_global ) {
			$order_id      = $order->get_id();
			$list_ip       = unserialize( get_post_meta( $order_id, PDA_Woo_Constants::PDA_WOO_CUSTOMER_IP_ACCESS_LIST_META, true ) );
			$max_ip_access = get_post_meta( $order_id, PDA_Woo_Constants::PDA_WOO_MAX_IP_ACCESS_META, true );
			?>
			<div class="form-field form-field-wide">
				<h3 style="margin-bottom: 0.5rem">IP Restrictions</h3>
				<?php if ( ! empty( $max_ip_access ) ) {
					$number_of_ip_accessed = ! empty ( $list_ip ) ? count( $list_ip ) : 0;
					?>
					The max number of IP addresses allowed: <b><?php echo $max_ip_access; ?></b>
					<div>The <?php echo $number_of_ip_accessed > 1 ? '<b>' . $number_of_ip_accessed . '</b> IP addresses': 'IP address' ?> below can access this order page and its product files:</div>
					<textarea wrap="off" rows="5" style="width: 100%" disabled><?php
						if ( ! empty( $list_ip ) ) {
							foreach ( $list_ip as $ip ) {
								echo $ip . '&#13;&#10;';
							}
						} ?></textarea>
				<?php } else { ?>
					<p>There is no IP restriction for this order</p>
				<?php }?>
			</div>
			<?php
		}
	}

	/**
	 * @param $product_url
	 * @param $product_name
	 */
	public function handle_woocommerce_download_file( $product_url, $product_name ) {
		if ( isset( $_GET['order'] ) ) {
			$order_id   = wc_get_order_id_by_order_key( $_GET['order'] );
			$is_max_ip_access_global = ! empty( pda_woo_get_settings_value( PDA_Woo_Constants::PDA_NUMBER_OF_DEVICES_ACCESS_GLOBAL_OPTION ) );
			$is_max_ip_access_order_meta = ! empty ( get_post_meta( $order_id, PDA_Woo_Constants::PDA_WOO_MAX_IP_ACCESS_META, true ) );
			$ip_referer = $_SERVER['REMOTE_ADDR'];
			if ( $is_max_ip_access_global && $is_max_ip_access_order_meta && ! pda_woo_is_allow_access_ip( $order_id, $ip_referer ) ) {
				header( "HTTP/1.1 301 Moved Permanently" );
				header( "Location: " . get_bloginfo( 'url' ) . '/404.php' );
				exit();
			}
		}
	}
}
