<?php

/**
 * @param $name_settings
 *
 * @return array|string
 */
function pda_woo_get_settings_value( $name_settings ) {
    $settings = get_option( PDA_Woo_Constants::PDA_WOO_SETTINGS_FOR_PDAV3 );
    if ( $settings ) {
        $options = json_decode( $settings );
        if( isset( $options->$name_settings ) ){
            if ( $options->$name_settings != '' ) {
                return $options->$name_settings;
            } else {
                return array();
            }
        }
    }

    return array();
}

/**
 * * Return plugin has updates
 *
 * @return array
 */
function plugin_has_updates() {
	$extensions  = [
		'protect_page_post_gold/prevent_ur_pages_gold.php'
	];
	$updates     = get_plugin_updates();
	$plugin_keys = array_keys( $updates );
	$tmp         = array_map(
		function ( $extension ) use ( $plugin_keys, $updates ) {
			if ( false === array_search( $extension, $plugin_keys ) ) {
				return null;
			}
			$plugin = $updates[ $extension ];
			$name = $plugin->Name;
			return "<b>$name" . ' (' . $plugin->update->new_version . ')</b>';
		},
		$extensions
	);

	return array_filter( $tmp, function ( $item ) {
		return ! is_null( $item );
	} );
}


/**
 * @param $order_id
 * @param $ip_referer
 *
 * @return array|mixed
 */
function pda_woo_handle_ip_to_allow_access( $order_id, $ip_referer) {
	$max_devices = get_post_meta( $order_id, PDA_Woo_Constants::PDA_WOO_MAX_IP_ACCESS_META, true );
	if ( empty( $max_devices ) ) {
		return true;
	}
	$max_devices = (int) $max_devices;
	$list_ip = unserialize( get_post_meta( $order_id, PDA_Woo_Constants::PDA_WOO_CUSTOMER_IP_ACCESS_LIST_META, true ) );
	if ( empty( $list_ip ) ) {
		$list_ip = array();
	}
	if ( ! in_array( $ip_referer, $list_ip ) ) {
		if ( count( $list_ip ) >= $max_devices ) {
			return false;
		} else {
			array_push( $list_ip, $ip_referer );
			update_post_meta( $order_id, PDA_Woo_Constants::PDA_WOO_CUSTOMER_IP_ACCESS_LIST_META, serialize( $list_ip ) );
		}
	}

	return true;
}

/**
 * @param $order_id
 * @param $ip_referer
 * @param $is_save
 *
 * @return array|mixed
 */
function pda_woo_is_allow_access_ip( $order_id, $ip_referer ) {
	$list_ip = unserialize( get_post_meta( $order_id, PDA_Woo_Constants::PDA_WOO_CUSTOMER_IP_ACCESS_LIST_META, true ) );
	if ( empty( $list_ip ) || ! in_array( $ip_referer, $list_ip ) ) {
		return false;
	}

	return true;
}

function pda_woo_is_bot_detected() {
	return (
		isset($_SERVER['HTTP_USER_AGENT'])
		&& preg_match('/bot|crawl|slurp|spider|mediapartners|facebookexternalhit|ia_archiver/i', strtolower( $_SERVER['HTTP_USER_AGENT'] ) )
	);
}
