<?php

/**
 * The plugin bootstrap file
 *
 * This file is read by WordPress to generate the plugin information in the plugin
 * admin area. This file also includes all of the dependencies used by the plugin,
 * registers the activation and deactivation functions, and defines a function
 * that starts the plugin.
 *
 * @link              preventdirectaccess.com
 * @since             1.0.5
 * @package           Pda_Woocommerce
 *
 * @wordpress-plugin
 * Plugin Name:       PDA Woocommerce Integration
 * Plugin URI:        https://preventdirectaccess.com/extensions/woocommerce-integration/?utm_source=user-website&utm_medium=plugin_link&utm_campaign=pda-woocommerce
 * Description:       Integrate Prevent Direct Access plugin and its extensions with WooCommerce. Allow paid users to access your private content and many more.
 * Version:           1.0.5
 * Author:            BWPS
 * Author URI:        https://preventdirectaccess.com/extensions/woocommerce-integration/?utm_source=user-website&utm_medium=plugin_link&utm_campaign=pda-woocommerce
 * License:           GPL-2.0+
 * License URI:       http://www.gnu.org/licenses/gpl-2.0.txt
 * Text Domain:       pda-woocommerce
 * Domain Path:       /languages
 */

// If this file is called directly, abort.
if ( ! defined( 'WPINC' ) ) {
	die;
}

/**
 * Currently plugin version.
 * Start at version 1.0.0 and use SemVer - https://semver.org
 * Rename this for your plugin and update it as you release new versions.
 */
define( 'PDA_WOO_VERSION', '1.0.5' );

define( 'PDA_WOO_BASE_NAME', plugin_basename( __FILE__ ) );

define( 'PDA_WC_PLUGIN_PATH',  plugin_dir_path( __FILE__ ) );

/**
 * The code that runs during plugin activation.
 * This action is documented in includes/class-pda-woocommerce-activator.php
 */
function activate_pda_woocommerce() {
	require_once plugin_dir_path( __FILE__ ) . 'includes/class-pda-woocommerce-activator.php';
	Pda_Woocommerce_Activator::activate();
}

/**
 * The code that runs during plugin deactivation.
 * This action is documented in includes/class-pda-woocommerce-deactivator.php
 */
function deactivate_pda_woocommerce() {
	require_once plugin_dir_path( __FILE__ ) . 'includes/class-pda-woocommerce-deactivator.php';
	Pda_Woocommerce_Deactivator::deactivate();
}

register_activation_hook( __FILE__, 'activate_pda_woocommerce' );
register_deactivation_hook( __FILE__, 'deactivate_pda_woocommerce' );

/**
 * The core plugin class that is used to define internationalization,
 * admin-specific hooks, and public-facing site hooks.
 */
require plugin_dir_path( __FILE__ ) . 'includes/class-pda-woocommerce.php';

/**
 * Begins execution of the plugin.
 *
 * Since everything within the plugin is registered via hooks,
 * then kicking off the plugin from this point in the file does
 * not affect the page life cycle.
 *
 * @since    1.0.0
 */
function run_pda_woocommerce() {

	$plugin = new Pda_Woocommerce();
	$plugin->run();
	$configs = require PDA_WC_PLUGIN_PATH . '/includes/class-pda-woocommerce-configs.php';
	$url     = '${UPDATE_URL}' === $configs->update_url
		? 'https://s3-ap-southeast-1.amazonaws.com/bwps.gold.plugins/pda-woocommerce/metadata.json'
		: $configs->update_url;
	Puc_v4_Factory::buildUpdateChecker(
		$url,
		__FILE__,
		'pda-woocommerce'
	);

}
run_pda_woocommerce();
