<?php

/**
 * The admin-specific functionality of the plugin.
 *
 * @link       buildwps.com
 * @since      1.0.0
 *
 * @package    Protect_Ur_Videos
 * @subpackage Protect_Ur_Videos/admin
 */

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    Protect_Ur_Videos
 * @subpackage Protect_Ur_Videos/admin
 * @author     Buildwps <gaupoit@outlook.com>
 */
class Protect_Ur_Videos_Admin {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string $plugin_name The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The slug of this plugin
	 *
	 * @access   private
	 * @var      string
	 */
	private $plugin_slug;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string $version The current version of this plugin.
	 */
	private $version;

	/**
	 * @var The private video utilities
	 */
	private $private_video;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @param string $plugin_name The name of this plugin.
	 * @param string $version The version of this plugin.
	 *
	 * @since    1.0.0
	 */
	public function __construct( $plugin_name, $version ) {

		$this->plugin_name   = $plugin_name;
		$this->plugin_slug   = 'pda-protect-wordpress-videos';
		$this->version       = $version;
		$this->private_video = new Private_Video();
	}

	/**
	 * Register the stylesheets for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_styles() {

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in Protect_Ur_Videos_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The Protect_Ur_Videos_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */

		wp_enqueue_style( $this->plugin_slug . '-admin-css', plugin_dir_url( __FILE__ ) . 'css/protect-ur-videos-admin.css', array(), $this->version, 'all' );
		if ( function_exists( 'get_current_screen' ) && ! is_null( get_current_screen() ) ) {
			if ( "prevent-direct-access-gold_page_pda-video" === get_current_screen()->id ) {
				wp_enqueue_style( $this->plugin_slug . '-configuration-css', plugin_dir_url( __FILE__ ) . 'css/protect-ur-videos-configuration.css', array(), $this->version, 'all' );
				wp_enqueue_style( $this->plugin_slug . 'toastr-css', '//cdnjs.cloudflare.com/ajax/libs/toastr.js/latest/css/toastr.min.css' );
			}
		}

	}

	/**
	 * Register the JavaScript for the admin area.
	 *
	 * @since    1.0.2 Remove a excess condition
	 * @since    1.0.1 Refactor code
	 * @since    1.0.0
	 */
	public function enqueue_scripts() {

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in Protect_Ur_Videos_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The Protect_Ur_Videos_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */
		$this->register_videojs_lib();

		if ( ! function_exists( 'get_current_screen' ) || is_null( get_current_screen() ) ) {
			return false;
		}
		$screen = get_current_screen();
		if ( 'prevent-direct-access-gold_page_pda-video' === $screen->id ) {
			$settings = Protect_Ur_Videos_Helpers::get_instance()->get_cloudfront_configuration();
			$none_obj = array(
				'ajaxurl'  => admin_url( 'admin-ajax.php' ),
				'security' => wp_create_nonce( 'pur_nonce' )
			);
			wp_enqueue_script( $this->plugin_slug . '-admin-js', plugin_dir_url( __FILE__ ) . 'js/protect-ur-videos-admin.js', array( 'jquery' ), $this->version, false );
			wp_localize_script( $this->plugin_slug . '-admin-js', 'ajax_obj', $none_obj );
			wp_localize_script( $this->plugin_slug . '-admin-js', 'pwv_data', array(
				'is_cf_configured' => ! empty( $settings[ PWV_Constants::CLOUDFRONT_URL_OPTION ] ),
			) );

			wp_register_script( $this->plugin_slug . '-toastr-js', '//cdnjs.cloudflare.com/ajax/libs/toastr.js/latest/js/toastr.min.js' );
			wp_enqueue_script( $this->plugin_slug . '-configuration-js', plugin_dir_url( __FILE__ ) . './js/protect-ur-videos-configuration.js', array(), $this->version );
			wp_localize_script( $this->plugin_slug . '-configuration-js', 'protect_video_configuration_data', array( 'ajax_url' => admin_url( 'admin-ajax.php' ) ) );
		}

		if ( Protect_Ur_Videos_Helpers::get_instance()->is_gutenberg_page( $screen ) ) {
			wp_enqueue_script( $this->plugin_slug . '-add-icon-to-block-gutenberg-js', plugin_dir_url( __FILE__ ) . './js/add-icon-to-gutenberg.js', array(
				'wp-blocks',
				'wp-components',
				'wp-element',
				'wp-i18n',
				'wp-editor'
			), $this->version );
			wp_enqueue_style( 'thickbox' );
			wp_enqueue_style( $this->plugin_slug . '-gutenberg-css', plugin_dir_url( __FILE__ ) . 'css/protect-ur-videos-gutenberg.css', array(), $this->version, 'all' );
		}

		return true;
	}

	/**
	 * Register videojs library
	 */
	private function register_videojs_lib() {
		/**
		 * Use CDN
		 */
		wp_register_script( 'videojs', '//vjs.zencdn.net/6.2.4/video.js' );
		wp_register_script( 'videojs', '//vjs.zencdn.net/6.2.4/video-js.css' );
		wp_register_style( 'hls', '//cdnjs.cloudflare.com/ajax/libs/videojs-contrib-hls/5.8.2/videojs-contrib-hls.js' );

		wp_enqueue_script( 'videojs' );
		wp_enqueue_script( 'hls' );
		wp_enqueue_script( 'wowza_player' );
		wp_enqueue_style( 'videojs' );
	}

	/**
	 * Register videojs button
	 */
	public function add_videojs_btn() {

		if ( ! pwv_can_add_videojs_btn() ) {
			return;
		}

		$service = new PWV_S3_Custom_Button_Service();
		$service->add_hooks();
		$service->set_cookie();

	}

	/**
	 * Register mce-button javascript file
	 *
	 * @param $plugin_array
	 *
	 * @return mixed
	 */
	public function videojs_mce_plugin( $plugin_array ) {
		$plugin_array['purvideojs'] = plugins_url( 'admin/js/', dirname( __FILE__ ) ) . 'pur-mce-button.js';

		return $plugin_array;
	}

	public function register_videojs_button( $buttons ) {
		array_push( $buttons, "|", "purvideojs" );
		echo( '<div style="display:none"><input type="hidden" id="videojs-autoplay-default"/><input type="hidden" id="videojs-autoplay-default"/></div>' );

		return $buttons;
	}

	/**
	 * Create video track
	 *
	 * @param $atts
	 * @param null $content
	 *
	 * @return string
	 */
	public function video_track( $atts, $content = null ) {
		extract( shortcode_atts( array(
			'kind'    => '',
			'src'     => '',
			'srclang' => '',
			'label'   => '',
			'default' => ''
		), $atts ) );

		if ( $kind ) {
			$kind = " kind='" . $kind . "'";
		}

		if ( $src ) {
			$src = " src='" . $src . "'";
		}

		if ( $srclang ) {
			$srclang = " srclang='" . $srclang . "'";
		}

		if ( $label ) {
			$label = " label='" . $label . "'";
		}

		if ( $default == "true" || $default == "default" ) {
			$default = " default";
		} else {
			$default = "";
		}

		$track = "<track" . $kind . $src . $srclang . $label . $default . " />";

		return $track;
	}

	/**
	 * Create video js short code
	 *
	 * @param $atts
	 * @param null $content
	 *
	 * @return string
	 */
	public function video_shortcode( $atts, $content = null ) {
		extract( shortcode_atts( array(
			'mp4'      => '',
			'webm'     => '',
			'ogg'      => '',
			'youtube'  => '',
			'poster'   => '',
			'width'    => 512,
			'height'   => 308,
			'preload'  => true,
			'autoplay' => false,
			'loop'     => '',
			'controls' => '',
			'id'       => '',
			'class'    => '',
			'muted'    => '',
			'video_id' => '',
			'post_id'  => '',
			'is_s3'    => '',
		), $atts ) );

		$video_id = empty( $post_id ) ? $video_id : $post_id;

		$dataSetup = array(
			'chromecast' => array(
				'appId'    => 'testMac',
				'metadata' => array(
					'title'
				)
			)
		);

		$style = " style='";
		if ( strpos( $width, '%' ) !== false ) {
			$style .= "width: " . $width . ";";
			$width = "";
		}

		if ( strpos( $height, '%' ) !== false ) {
			$style  .= "height: " . $height . ";";
			$height = "";
		}

		$style .= "'";

		if ( $style == " style='''" ) {
			$style = "";
		}

		if ( $id == '' ) {
			$id = 'protect-video-id_' . $video_id . '_' . time();
		}

		if ( $mp4 ) {
			$media_type = $this->get_media_type( $mp4 );
			$soure_type = "video" === $media_type ? "video/mp4" : "audio/mp3";
			$source     = '';

			if ( class_exists( 'PDA_Api_Gold' ) && ! empty( $video_id ) ) {
				$api    = new PDA_Api_Gold();
				$source = $api->generateExpired( array( 'id' => $video_id, 'clear_expired_link' => true ) );
			}

			$mp4_signed_video = $source;
			$mp4_source       = '<source src="' . $mp4_signed_video . '" type=\'' . $soure_type . '\' />';
		} else {
			$mp4_source = '';
		}
		// Preload the video?
		$preload_attribute = ' preload="false"';

		// Autoplay the video?
		if ( $autoplay == "true" || $autoplay == "on" ) {
			$autoplay_attribute = " autoplay";
		} else {
			$autoplay_attribute = "false";
		}

		// Tracks
		if ( ! is_null( $content ) ) {
			$track = do_shortcode( $content );
		} else {
			$track = "";
		}

		if ( $class ) {
			$class = ' ' . $class;
		}

		if ( $poster ) {
			$poster_attribute = ' poster="' . $poster . '"';
		} else {
			$poster_attribute = '';
		}

		if ( $controls == "false" ) {
			$controls_attribute = "";
		} else {
			$controls_attribute = " controls";
		}

		if ( $loop == "true" ) {
			$loop_attribute = " loop";
		} else {
			$loop_attribute = "";
		}

		if ( $muted == "true" ) {
			$muted_attribute = " muted";
		} else {
			$muted_attribute = "";
		}


		$jsonDataSetup = str_replace( '\\/', '/', json_encode( $dataSetup ) );

		if ( "video" === $media_type ) {
			//Output the <video> tag
			$videojs = <<<_end_

		<!-- Begin Video.js -->
		<video  oncontextmenu="return false" onselectstart="return false" ondragstart="return false" id="{$id}" class="video-js vjs-default-skin{$class}" width="{$width}" height="{$height}"{$poster_attribute}{$controls_attribute}{$preload_attribute}{$autoplay_attribute}{$loop_attribute}{$muted_attribute}{$style} data-setup='{$jsonDataSetup}'>
			{$mp4_source}
			{$track}
		</video>
<!-- Here will be the player setup script -->
		<!-- End Video.js -->
_end_;
		} else if ( "audio" === $media_type ) {
			//Output the <video> tag
			$videojs = <<<_end_

		<!-- Begin Video.js -->
		<audio poster="https://adrianogazza.files.wordpress.com/2016/08/oasis.jpg?w=768"  oncontextmenu="return false" onselectstart="return false" ondragstart="return false" id="{$id}" class="video-js vjs-default-skin{$class}" width="{$width}" height="{$height}"{$poster_attribute}{$controls_attribute}{$preload_attribute}{$autoplay_attribute}{$loop_attribute}{$muted_attribute}{$style} data-setup='{$jsonDataSetup}'>
			{$mp4_source}
			{$track}
		</audio>
		
<!-- Here will be the player setup script -->
		<!-- End Video.js -->
_end_;
		}

		$this->add_script_for_videos();

		return $videojs;
	}

	function add_script_for_videos() {
		$public = new Protect_Ur_Videos_Public( "", "" );
		$public->register_sdks();
		$public->register_videojs_lib();
		$none_obj = array(
			'ajaxurl'  => admin_url( 'admin-ajax.php' ),
			'security' => wp_create_nonce( 'pur_nonce_public' )
		);
		wp_enqueue_script( $this->plugin_name, WPPP_BASE_URL . 'public/js/protect-ur-videos-public.js', array( 'jquery' ), $this->version, false );
		wp_enqueue_script( $this->plugin_name . '_util', WPPP_BASE_URL . 'public/js/protect-ur-videos-util.js', array( 'jquery' ), $this->version, false );
		wp_localize_script( $this->plugin_name, 'ajax_obj', $none_obj );
		$public->register_videojs_plugin();
	}

	/**
	 * Upload file to protected storage
	 */
	public function upload_to_storage() {
		if ( pwv_is_s3_mode_on() ) {
			$file_url = urldecode( $_POST['file_url'] );;
			$s3_service = new PWV_S3_Service();
			$s3_service->upload_to_s3_bucket( $file_url );
		}
	}

	public function protect_wp_videos_admin_notice() {
		if ( Protect_Ur_Videos_Helpers::get_instance()->is_pda_gold_plugin_activated() ) {
			return;
		}
		$message = YME_MESSAGES::$PDA['PDA_NEVER_ACTIVATE'];
		if ( ! empty( $message ) ) {
			?>
			<div class="notice notice-error is-dismissible">
				<p><b><?php echo esc_html( $this->plugin_name ); ?>: </b><?php _e( $message, 'pda-protect-videos' ); ?>
				</p>
			</div>
			<?php
		}
	}

	public function pwv_rest_api_init_cb() {
		$api = new PUV_API();
		$api->register_rest_routes();
	}

	public function pda_protect_video_create_menu() {
		$setting_video     = new PDA_Video_Settings();
		$submenu_page_name = add_submenu_page( 'pda-gold', __( 'Videos', 'pda-settings-page' ), __( 'Videos', 'pda-settings-page' ), 'manage_options', PWV_Constants::STATUS_PAGE_VIDEO, array(
			$setting_video,
			'render_ui'
		) );
		add_action( 'admin_print_styles-' . $submenu_page_name, array(
			$this,
			'pda_protect_video_setting_assets'
		) );
	}

	public function pda_protect_video_setting_assets() {
		wp_enqueue_style( $this->plugin_slug . '-setting-css', plugin_dir_url( __FILE__ ) . '/dist/pwv-bundle.css', array(), $this->version, 'all' );

		wp_enqueue_script( $this->plugin_slug . 'setting-js', plugin_dir_url( __FILE__ ) . '/dist/pwv-bundle.js', array( 'jquery' ), $this->version );

		wp_localize_script( $this->plugin_slug . 'setting-js', 'pda_protect_video_data', array(
			'ajax_url'        => admin_url( 'admin-ajax.php' ),
			'ajax_nonce'      => wp_create_nonce( PWV_Constants::PWV_AJAX_NONCE ),
			'home_url'        => Protect_Ur_Videos_Helpers::get_instance()->get_home_url_with_ssl(),
			'rest_api_prefix' => 'wp-json',
			'rest_nonce'      => wp_create_nonce( 'wp_rest' ),
		) );
	}

	function pda_protect_video_update_settings() {
		$nonce = $_REQUEST['security_check'];
		if ( ! wp_verify_nonce( $nonce, PWV_Constants::SETTINGS_FORM_NONCE ) ) {
			wp_die( 'Invalid nonce' );
		}
		$data_settings = $_REQUEST['settings'];
		update_site_option( PWV_Constants::SETTING_OPTION, wp_json_encode( $data_settings ) );
	}

	function protect_video_update_configuration() {
		$nonce = $_REQUEST['security_check'];
		if ( ! wp_verify_nonce( $nonce, PWV_Constants::PWV_AJAX_NONCE ) ) {
			error_log( 'not verify nonce', 0 );
			wp_die( 'invalid_nonce' );
		}
		$settings      = $_REQUEST['settings'];
		$domain        = $_SERVER['HTTP_HOST'];
		$keys          = [];
		$options_s3_db = unserialize( get_option( PWV_Constants::CONFIGURATION_OPTIONS ) );
		if ( array_key_exists( "domain", $options_s3_db ) ) {
			unset( $options_s3_db['domain'] );
		}

		if ( $settings["is_aws_default"] == "true" ) {
			$keys = [
				'aws_key'        => 'AKIAIAOHVYLGYIR4CL7Q',
				'aws_secret'     => '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',
				'bucket'         => 'pda-s3-default',
				'prefix_key'     => 'pda',
				'region'         => 'us-west-1',
				'is_aws_default' => 'true',
				'domain'         => $domain,
			];
			$data = $options_s3_db ? array_merge( $options_s3_db, $keys ) : $keys;
		} else {
			foreach ( $settings as $key => $value ) {
				array_push( $keys, $key );
			}
			foreach ( $keys as $k ) {
				$options_s3_db[ $k ] = $settings[ $k ];
			}
			$data = $options_s3_db;
		}

		$pda_s3 = serialize( $data );

		update_option( PWV_Constants::CONFIGURATION_OPTIONS, $pda_s3 );
		wp_send_json( $settings );
		wp_die();
	}

	public function protect_video_test_configuration() {
		$nonce = $_REQUEST['security_check'];
		if ( ! wp_verify_nonce( $nonce, PWV_Constants::PWV_AJAX_NONCE ) ) {
			error_log( 'not verify nonce', 0 );
			wp_die( 'invalid_nonce' );
		}
		$settings   = $_REQUEST['settings'];
		$aws_helper = new Aws_S3_Helper();
		$isOk       = $aws_helper->testConfiguration( array(
			'aws_key'        => $settings["aws_key"],
			'aws_secret'     => $settings["aws_secret"],
			'bucket'         => $settings["bucket"],
			'prefix_key'     => $settings["prefix_key"],
			'region'         => $settings["region"],
			'is_aws_default' => $settings["is_aws_default"]
		) );
		wp_send_json( $isOk );
		wp_die();
	}

	public function handle_admin_notices() {
		$configuration = new Wp_Protect_Video_Configuration;
		$this->configs = $configuration->getAwsConfigs();

		if ( pwv_is_s3_mode_on() && ! $this->configs ) {
			$setting_url = network_admin_url( 'admin.php?page=pda-video&tab=pda_protect_video_configuration' );

			?>
			<div class="error is-dismissible notice">
				<p><b><?php echo "PDA Protect WordPress Videos: "; ?></b> You enabled the S3 mode. Please update your
					AWS configuration <a href="<?php echo $setting_url ?>">here</a></p>
			</div>
			<?php
		}
	}

	/**
	 * Detect IDM
	 *
	 * @param array $server Information of request
	 *
	 * @return bool
	 */
	public function is_block_idm( $server ) {
		$http_range = $server['HTTP_RANGE'];
		if ( $http_range === 'bytes=0-' ) {
			$private_link_patterns = explode( '/', $server['REQUEST_URI'] );
			$id_private_link       = $private_link_patterns[ count( $private_link_patterns ) - 1 ];
			if ( Protect_Ur_Videos_Repository::is_video_link_cached( $id_private_link ) ) {
				return true;
			}
			Protect_Ur_Videos_Repository::add_video_link_to_cache( $id_private_link );
		}

		return false;
	}

	/**
	 * Delete all cache link expired
	 *
	 * @param bool $temp Temp
	 *
	 * @return bool
	 */
	public function delete_all_cache_link_expired( $temp ) {
		Protect_Ur_Videos_Repository::delete_all_cache_link_expired();

		return $temp;
	}

	/**
	 * Handle database
	 */
	public function db_handle() {
		Protect_Ur_Videos_Repository::init_cache_database();
		// $cronjob_handler = new Protect_Ur_Videos_Cronjob_Handler();
		// $cronjob_handler->pwp_clear_expired_cache_link();\
		Protect_Ur_Videos_Cronjob_Handler::get_instance()->schedule_clear_cache_link_expired();
	}

	/**
	 * Cronjob to delete expired cache link
	 */
	public function pwp_delete_expired_cache_link_cron_exec() {
		Protect_Ur_Videos_Cronjob_Handler::get_instance()->pwp_clear_expired_cache_link();
	}

	/**
	 * @param $options
	 *
	 * @return array
	 */
	public function puc_add_token_to_headers( $options ) {
		$options['headers'] = array(
			'Accept'        => 'application/json',
			'Authorization' => 'helloymese',
		);

		return $options;
	}

}
