const { __ } = wp.i18n;
const { registerBlockType, BlockControls, AlignmentToolbar } = wp.blocks;
const el = wp.element.createElement;
const { MediaUpload, InspectorControls } = wp.editor;
const { Button, PanelBody } = wp.components;

registerBlockType('pda/pda-videos', {
    title: __('Protected Video'),
    description: __('Embed a protected video or audio from your media library or upload a new one.'),
    icon: 'video-alt2',
    category: 'common',
    attributes: {
        mediaUrl: {
            type: 'url',
            default: '',
        },
        mediaId: {
            type: 'number',
            default: '',
        },
        mediaTitle: {
            type: 'string',
            default: '',
        },
        mediaPoster: {
            type: 'string',
            default: '',
        },
        mediaWidth: {
            type: 'string',
            default: '',
        },
        mediaHeight: {
            type: 'string',
            default: '',
        },
        mediaIdHtml: {
            type: 'string',
            default: '',
        },
        mediaClass: {
            type: 'string',
            default: '',
        },
        mediaAutoplay: {
            type: 'string',
            default: '',
        },
        mediaControls: {
            type: 'string',
            default: 'true',
        }
    },

    edit( props ) {
        const alignment = props.attributes.alignment;

        const { mediaUrl, mediaWidth, mediaHeight, mediaIdHtml, mediaClass, mediaPoster, mediaAutoplay, mediaControls, mediaTitle } = props.attributes;

        const onSelectMedia = function ( media ) {
            return props.setAttributes({
                mediaUrl: media.url,
                mediaId: media.id,
                mediaTitle: media.title,
            })
        };

        const onSelectPoster = function ( media ) {
            return props.setAttributes({
                mediaPoster: media.url
            })
        };

        const onChangeHeight = function ( event ) {
            return props.setAttributes({
                mediaHeight: event.target.value
            })
        };

        const onChangeWidth = function ( event ) {
            return props.setAttributes({
                mediaWidth: event.target.value
            })
        };

        const onChangeId = function ( event ) {
            return props.setAttributes({
                mediaIdHtml: event.target.value
            })
        };

        const onChangeClass = function ( event ) {
            return props.setAttributes({
                mediaClass: event.target.value
            })
        };

        const onChangeAutoPlay = function ( event ) {
            return props.setAttributes({
                mediaAutoplay: event.target.checked ? 'true' : ''
            })
        };

        const onClickRemovePoster = function () {
            return props.setAttributes({
                mediaPoster: ''
            })
        };

        const removePoster = mediaPoster !== '' ? (
            el( 'div', { className: 'pda-wrap-attr-media' },
                el( 'span'),
                el( 'a', {
                        className: 'pda-remove-poster',
                        onClick: onClickRemovePoster,
                    },
                    'Remove Poster'
                ),
            )
        ) : '';

        function onChangeAlignment( newAlignment ) {
            props.setAttributes( { alignment: newAlignment } );
        }

        const checkAutoPlay = mediaAutoplay === 'true' ? true : false;

        const textPoster = mediaPoster === '' ? 'Select Poster' : 'Replace Poster';

        const textButtonUpload = mediaUrl === '' ? 'Media Library' : 'Change File';

        const selectMedia = (
            el( 'div', {},
                el( MediaUpload, {
                    onSelect: onSelectMedia,
                    allowedTypes: ['video', 'audio'],
                    render: function (obj) {
                        return el( Button, {
                                className: 'components-button editor-media-placeholder__button is-button is-default is-large',
                                onClick: obj.open,
                            }, textButtonUpload
                        );
                    }
                },
                    el( BlockControls, { key: 'controls' },
                        el( AlignmentToolbar,
                            {
                                value: alignment,
                                onChange: onChangeAlignment,
                            }
                        ),
                    )
                ),

                el( InspectorControls, { key: 'inspector' },
                    el( PanelBody, {
                            title: 'Protected Video Settings'
                        },

                        el( 'div', { className: 'pda-wrap-attr-media'},
                            el( 'span', { className: 'pda-auto-play' }, 'AutoPlay' ),
                            el( 'input', {
                                    type: 'checkbox',
                                    onChange: onChangeAutoPlay,
                                    checked: checkAutoPlay,
                                }
                            )
                        ),

                        el( 'div', { className: 'pda-wrap-attr-media'},
                            el( 'span', {}, 'Height' ),
                            el( 'input', {
                                    type: 'number',
                                    value: mediaHeight,
                                    onChange: onChangeHeight,
                                }
                            )
                        ),

                        el( 'div', { className: 'pda-wrap-attr-media'},
                            el( 'span', {}, 'Width' ),
                            el( 'input', {
                                    type: 'number',
                                    value: mediaWidth,
                                    onChange: onChangeWidth,
                                }
                            )
                        ),

                        el( 'div', { className: 'pda-wrap-attr-media'},
                            el( 'span', {}, 'ID' ),
                            el( 'input', {
                                    type: 'text',
                                    value: mediaIdHtml,
                                    onChange: onChangeId,
                                }
                            )
                        ),

                        el( 'div', { className: 'pda-wrap-attr-media'},
                            el( 'span', {}, 'Class' ),
                            el( 'input', {
                                    type: 'text',
                                    value: mediaClass,
                                    onChange: onChangeClass,
                                }
                            )
                        ),

                        el( 'div', { className: 'pda-wrap-attr-media'},
                            el( 'span', {}, 'Poster' ),
                            el( MediaUpload, {
                                    onSelect: onSelectPoster,
                                    allowedTypes: 'image',
                                    render: function( obj ) {
                                        return el( Button, {
                                                className: 'components-icon-button image-block-btn is-button is-default is-large',
                                                onClick: obj.open
                                            },
                                            textPoster
                                        );
                                    }
                                }
                            ),
                        ),

                        removePoster

                    ),
                )
            )
        );

        if ( mediaUrl !== '' ) {
            return (
                el( 'div', { className: 'components-placeholder editor-media-placeholder wp-block-video' },
                    el( 'div', { className: 'components-placeholder__instructions' },
                        el('span', { className: 'pda-selected-media' }, 'Selected Media: ' ),
                        mediaTitle
                    ),
                    el( 'figure', { className: 'wp-block-video' },
                        el( 'div', { className: 'components-disabled' },
                            el( 'video', { controls: mediaControls, src: mediaUrl, poster: mediaPoster } )
                        )
                    ),
                    selectMedia,
                )
            );
        } else {
            return(
                el( 'div', { className: 'components-placeholder editor-media-placeholder wp-block-video' },
                    el( 'div', { className: 'components-placeholder__label'},
                        el( 'svg', { className: 'pda-svg-icon', width: '20', height: '20' },
                            el( 'path', { d: 'M12 13V7c0-1.1-.9-2-2-2H3c-1.1 0-2 .9-2 2v6c0 1.1.9 2 2 2h7c1.1 0 2-.9 2-2zm1-2.5l6 4.5V5l-6 4.5v1z' } )
                        ),
                        el('span', {}, 'Protected Video' )
                    ),
                    el( 'div', { className: 'components-placeholder__instructions' }, 'Select a protected video or audio (MP3, MP4) file from your library.' ),
                    selectMedia,
                )
            );
        }
    },

    save( props ) {
        const { mediaUrl, mediaWidth, mediaHeight, mediaIdHtml, mediaClass, mediaPoster, mediaAutoplay, mediaId } = props.attributes;
        const width = mediaWidth !== '' ? 'width="' + mediaWidth + '"' : '';
        const height = mediaHeight !== '' ? 'height="' + mediaHeight + '"' : '';
        return (
            '[protected_video mp4="' + mediaUrl + '" ' + width + ' ' + height + ' id="' + mediaIdHtml + '"' + ' class="' + mediaClass + '"' + ' poster="' + mediaPoster + '"' + ' autoplay="' + mediaAutoplay + '"' + ' video_id=' + mediaId + ']'
        );
    },

});