<?php
/**
 * Created by PhpStorm.
 * User: gaupoit
 * Date: 7/31/18
 * Time: 14:50
 */

if ( ! class_exists( '' ) ) {
	class PUV_API {
		/**
		 * Register rest routes.
		 */
		public function register_rest_routes() {
			header( 'Access-Control-Allow-Origin: *' );
			header( 'Access-Control-Allow-Methods: POST' );
			register_rest_route( 'puv/v1', '/debug', array(
				'methods'  => 'GET',
				'callback' => array( $this, 'debug' ),
			) );

			register_rest_route( 'puv/v1', '/signed-url', array(
				'methods'  => 'POST',
				'callback' => array( $this, 'regenerate_private_url' ),
			) );

			register_rest_route( 'puv/v1', '/aws-configuration', array(
				'methods'             => 'GET',
				'callback'            => array( $this, 'get_aws_configuration' ),
				'permission_callback' => function () {
					return current_user_can( 'upload_files' );
				},
			) );

			register_rest_route( 'puv/v1', '/aws-configuration', array(
				'methods'             => 'POST',
				'callback'            => array( $this, 'update_aws_configuration' ),
				'permission_callback' => function () {
					return current_user_can( 'upload_files' );
				},
			) );

			register_rest_route( 'puv/v1', '/aws-configuration/test', array(
				'methods'             => 'GET',
				'callback'            => array( $this, 'test_aws_configuration' ),
				'permission_callback' => function () {
					return current_user_can( 'upload_files' );
				},
			) );

			register_rest_route( 'puv/v1', '/setting-options', array(
				'methods'             => 'POST',
				'callback'            => array( $this, 'update_setting_options' ),
				'permission_callback' => function () {
					return current_user_can( 'upload_files' );
				},
			) );

			register_rest_route( 'puv/v1', '/setting-options', array(
				'methods'             => 'GET',
				'callback'            => array( $this, 'get_setting_options' ),
				'permission_callback' => function () {
					return current_user_can( 'upload_files' );
				},
			) );

			register_rest_route( 'puv/v1', '/cloudfront-configuration', array(
				'methods'             => 'GET',
				'callback'            => array( $this, 'get_cloudfront_configuration' ),
				'permission_callback' => function () {
					return current_user_can( 'upload_files' );
				},
			) );

			register_rest_route( 'puv/v1', '/cloudfront-configuration', array(
				'methods'             => 'POST',
				'callback'            => array( $this, 'update_cloudfront_configuration' ),
				'permission_callback' => function () {
					return current_user_can( 'upload_files' );
				},
			) );

			register_rest_route( 'puv/v1', '/aws-custom-option', array(
				'methods'             => 'POST',
				'callback'            => array( $this, 'update_aws_custom_option' ),
				'permission_callback' => function () {
					return current_user_can( 'upload_files' );
				},
			) );
		}

		/**
		 * Debug
		 */
		public function debug() {
			return 'Hello world !';
		}

		/**
		 * Regenerate private url.
		 */
		public function regenerate_private_url() {
			$current_src = $_POST['src'];
			if ( 'pda-s3-videos' === $_POST['id'] ) {
				$service          = new PWV_S3_Service();
				$mp4_signed_video = $service->create_presigned_url( $current_src, '+3 seconds' );
			} else {
				$is_casting = false;
				if ( key_exists( 'isCasting', $_POST ) ) {
					$is_casting = urldecode( $_POST['isCasting'] );
				}
				if ( $is_casting ) {
					$timeout = 3600;
				} else {
					$timeout = 3;
				}
				$url              = strtok( $current_src, '?' );
				$private_video    = new Private_Video();
				$mp4_signed_video = $private_video->getSignedURL( $url, $timeout );
			}
			wp_send_json( $mp4_signed_video );
			wp_die();
		}

		/**
		 * Get aws configuration API.
		 */
		public function get_aws_configuration() {
			$aws_configuration = Protect_Ur_Videos_Helpers::get_instance()->get_aws_configuration();

			return array(
				'aws_configuration' => $aws_configuration
			);
		}

		/**
		 * Update setting options.
		 *
		 * @param object $data Data from request.
		 *
		 * @return array|WP_Error.
		 */
		public function update_setting_options( $data ) {
			$data_settings = $data['settings'];
			$data          = array(
				PWV_Constants::AWS_S3_SWITCH_TO_AWS_S3_OPTION => $data_settings[ PWV_Constants::AWS_S3_SWITCH_TO_AWS_S3_OPTION ],
			);
			update_option( PWV_Constants::SETTING_OPTION, wp_json_encode( $data ) );

			return array(
				'success' => true,
			);
		}

		/**
		 * Get setting options.
		 *
		 * @return array
		 */
		public function get_setting_options() {
			return json_decode( get_option( PWV_Constants::SETTING_OPTION, false ) );
		}

		/**
		 * Get CloudFront Configuration.
		 *
		 * @return array
		 */
		public function get_cloudfront_configuration() {
			$data = Protect_Ur_Videos_Helpers::get_instance()->get_cloudfront_configuration();

			return array(
				'url'     => $data[ PWV_Constants::CLOUDFRONT_URL_OPTION ],
				'rsa_key' => $data[ PWV_Constants::CLOUDFRONT_RSA_KEY_OPTION ],
				'cf_id'   => $data[ PWV_Constants::CLOUDFRONT_ID_OPTION ],
			);
		}

		/**
		 * Update CloudFront Configuration.
		 *
		 * @param object $data Data from request.
		 *
		 * @return array
		 */
		public function update_cloudfront_configuration( $data ) {
			$data_settings = $data['settings'];
			$data          = array(
				PWV_Constants::CLOUDFRONT_URL_OPTION     => $data_settings['url'],
				PWV_Constants::CLOUDFRONT_RSA_KEY_OPTION => $data_settings['rsa_key'],
				PWV_Constants::CLOUDFRONT_ID_OPTION      => $data_settings['cf_id'],
			);
			Protect_Ur_Videos_Helpers::get_instance()->update_cloudfront_configuration( $data );

			return array(
				'success' => true,
			);
		}

		/**
		 * Test aws configuration.
		 *
		 * @param object $data Data from request.
		 *
		 * @return object|WP_Error
		 */
		public function test_aws_configuration( $data ) {
			$settings     = $data['settings'];
			$aws_helper   = new Aws_S3_Helper();
			$is_connected = $aws_helper->testConfiguration( array(
				'aws_key'        => $settings['aws_key'],
				'aws_secret'     => $settings['aws_secret'],
				'bucket'         => $settings['bucket'],
				'prefix_key'     => $settings['prefix_key'],
				'region'         => $settings['region'],
				'is_aws_default' => $settings['is_aws_default']
			) );

			return $is_connected;
		}

		/**
		 * Update aws configuration.
		 *
		 * @param object $data Data from request.
		 *
		 * @return object|WP_Error
		 */
		public function update_aws_configuration( $data ) {
			$settings     = $data['settings'];
			$aws_helper   = new Aws_S3_Helper();
			$is_connected = $aws_helper->testConfiguration( array(
				'aws_key'        => $settings['aws_key'],
				'aws_secret'     => $settings['aws_secret'],
				'bucket'         => $settings['bucket'],
				'prefix_key'     => $settings['prefix_key'],
				'region'         => $settings['region'],
				'is_aws_default' => $settings['is_aws_default']
			) );
			error_log( 'DEBUG: ' . wp_json_encode( $is_connected == 'false' ) );
			if ( is_wp_error( $is_connected ) || ! $is_connected ) {
				return new WP_Error(
					'wrong_aws_configuration',
					__( 'Failed to check options', 'prevent-direct-access-gold' ),
					array(
						'status' => 400,
					)
				);
			}
			$domain   = $_SERVER['HTTP_HOST'];
			$keys     = [];
			$response = Protect_Ur_Videos_Helpers::get_instance()->update_aws_configurations( $settings, $domain, $keys );

			return $response;
		}

		/**
		 * Update aws option
		 *
		 * @param array $data Data.
		 *
		 * @return mixed
		 */
		public function update_aws_custom_option( $data ) {
			$settings                 = $data['settings'];
			$aws_helper               = new Aws_S3_Helper();
			$aws_bucket_configuration = Protect_Ur_Videos_Helpers::get_instance()->get_aws_configuration();
			$is_connected             = $aws_helper->testConfiguration( array(
				'aws_key'        => $aws_bucket_configuration['aws_key'],
				'aws_secret'     => $aws_bucket_configuration['aws_secret'],
				'bucket'         => $settings['bucket'],
				'prefix_key'     => $settings['prefix_key'],
				'region'         => $settings['region'],
				'is_aws_default' => $settings['is_aws_default']
			) );
			error_log( 'DEBUG: ' . wp_json_encode( $is_connected == 'false' ) );
			if ( is_wp_error( $is_connected ) || ! $is_connected ) {
				return new WP_Error(
					'wrong_aws_configuration',
					__( 'Failed to check options', 'prevent-direct-access-gold' ),
					array(
						'status' => 400,
					)
				);
			}
			$domain   = $_SERVER['HTTP_HOST'];
			$keys     = [];
			$response = Protect_Ur_Videos_Helpers::get_instance()->update_aws_configurations( $settings, $domain, $keys )
			            && Protect_Ur_Videos_Helpers::get_instance()->update_cloudfront_option( PWV_Constants::CLOUDFRONT_URL_OPTION, rawurlencode( $settings['cf_url'] ) );

			return $response;
		}
	}
}
