<?php
/**
 * Created by PhpStorm.
 * User: gaupoit
 * Date: 7/27/18
 * Time: 12:19
 */

/**
 * Check s3 is turn on
 *
 * @return bool
 */
function pwv_is_s3_mode_on() {
	return pwv_get_settings( PWV_Constants::AWS_S3_SWITCH_TO_AWS_S3_OPTION );
//	return get_site_option( PWV_Constants::$IS_S3_MODE ) === "1";
}

/**
 * Get site settings.
 *
 * @param string $name_settings Name settings.
 *
 * @return bool
 */
function pwv_get_settings( $name_settings ) {
	$settings = get_option( PWV_Constants::SETTING_OPTION );

	return pwv_check_option_settings( $settings, $name_settings );
}

/**
 * Check option settings.
 *
 * @param object $settings Settings.
 * @param string $name_settings Name settings.
 *
 * @return bool
 */
function pwv_check_option_settings( $settings, $name_settings ) {
	if ( $settings ) {
		$options = json_decode( $settings );
		if ( isset( $options->$name_settings ) && ( 'true' === $options->$name_settings || true === $options->$name_settings ) ) {
			return true;
		}
	}

	return false;
}

/**
 * Function to check whether the gutenberg editor is using
 *
 * @param string $wp_version WordPress version
 * @param bool $is_class_editor_activate Is class editor activated
 * @param bool $is_gutenberg_plugin_activated Is gutenberg plugin activated
 *
 * @return bool
 */
function pwv_is_using_gutenberg_editor( $wp_version, $is_class_editor_activate, $is_gutenberg_plugin_activated ) {

	if ( version_compare( $wp_version, '5.0.0' ) < 0 ) {
		return $is_gutenberg_plugin_activated;
	}

	return ! $is_class_editor_activate;
}

/**
 * Check whether it can add the plugin video button
 * @since 1.2.18
 * @return bool
 */
function pwv_can_add_videojs_btn() {
	if ( ! current_user_can( 'edit_posts' ) ||
	     ! current_user_can( 'edit_pages' ) ||
	     'true' !== get_user_option( 'rich_editing' ) ) {
		return false;
	}

	return true;
}

/**
 * Check that current post editor is classic
 *
 * @param array $get_params
 *
 * @return bool
 */
function is_using_classic_editor( $get_params ) {
	$allow_users = ( get_option( 'classic-editor-allow-users' ) === 'allow' );

	if ( $allow_users ) {
		return is_plugin_active( 'classic-editor/classic-editor.php' ) && isset( $get_params['classic-editor'] );
	} else {
		return is_plugin_active( 'classic-editor/classic-editor.php' );
	}
}

/**
 * Transform URL to peak ID format from:
 * http://hello.com/world.mp3 to http___hello.com_world.peaks
 *
 * @param $url
 *
 * @return string
 */
function massage_url_to_peak_track_id( $url ) {
	$components = wp_parse_url( $url );

	$extension = pathinfo( $components['path'], PATHINFO_EXTENSION );
	$replaced_ext = 'peaks';

	$path = empty($extension) ?
		$components['path'] . ".$replaced_ext" :
		str_replace("$extension" , $replaced_ext, $components['path'] );

	$path = ltrim( $path, '/' );

	return $components['scheme'] . '___' . $components['host'] . '_' . $path;
}
