<?php

/**
 * The protect video functionality of the plugin.
 *
 * @link       buildwps.com
 * @since      1.0.0
 *
 * @package    Protect_Ur_Videos
 * @subpackage Protect_Ur_Videos/includes
 */


class Protect_Ur_Videos_Helpers {
	/**
	 * Instance
	 *
	 * @var Protect_Ur_Videos_Helpers
	 */
	protected static $instance;

	/**
	 * Get instance of singleton
	 *
	 * @return Protect_Ur_Videos_Helpers
	 */
	public static function get_instance() {
		if ( null == self::$instance ) {
			self::$instance = new self();
		}

		return self::$instance;
	}

	/**
	 * Get aws configuration
	 */
	public function get_aws_configuration() {
		$settings   = get_option( PWV_Constants::CONFIGURATION_OPTIONS );
		$aws_key    = '';
		$aws_secret = '';
		$bucket     = '';
		$prefix_key = '';
		$regions    = [
			'us-east-1',
			'us-east-2',
			'us-west-1',
			'us-west-2',
			'ca-central-1',
			'eu-central-1',
			'eu-west-1',
			'eu-west-2',
			'eu-west-3',
			'eu-north-1',
			'ap-northeast-1',
			'ap-northeast-2',
			'ap-northeast-3',
			'ap-southeast-1',
			'ap-southeast-2',
			'ap-south-1',
			'sa-east-1',
		];
		$region     = $regions[0];
		if ( $settings ) {
			$options = ( ! $settings ) ? array() : unserialize( $settings );
			if ( is_array( $options ) ) {
				if ( array_key_exists( 'aws_key', $options ) ) {
					$aws_key    = $options['aws_key'];
					$aws_secret = $options['aws_secret'];
					$bucket     = $options['bucket'];
					$prefix_key = $options['prefix_key'];
				}
				$region = isset( $options['region'] ) ? $options['region'] : $regions[0];
			}
		}
		$cf_configuration  = $this->get_cloudfront_configuration();
		$aws_configuration = array(
			'aws_key'     => $aws_key,
			'aws_secret'  => $aws_secret,
			'bucket'      => $bucket,
			'prefix_key'  => $prefix_key,
			'region'      => $region,
			'list_region' => $regions,
			'cf_url'      => empty( $cf_configuration[ PWV_Constants::CLOUDFRONT_URL_OPTION ] ) ? '' : $cf_configuration[ PWV_Constants::CLOUDFRONT_URL_OPTION ],
		);

		return $aws_configuration;
	}

	/**
	 * Get home url with ssl.
	 *
	 * @return string|void
	 */
	public function get_home_url_with_ssl() {
		return is_ssl() ? home_url( '/', 'https' ) : home_url( '/' );
	}

	/**
	 * @param object $settings
	 * @param        $domain
	 * @param array  $keys
	 *
	 * @return array
	 */
	public function update_aws_configurations( $settings, $domain, array $keys ) {
		$options_s3_db = unserialize( get_option( PWV_Constants::CONFIGURATION_OPTIONS ) );

		if ( $settings["is_aws_default"] == "true" ) {
			$keys = [
				'aws_key'        => 'AKIAIAOHVYLGYIR4CL7Q',
				'aws_secret'     => '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',
				'bucket'         => 'pda-s3-default',
				'prefix_key'     => 'pda',
				'region'         => 'us-west-1',
				'is_aws_default' => 'true',
				'domain'         => $domain,
			];
			$data = $options_s3_db ? array_merge( $options_s3_db, $keys ) : $keys;
		} else {
			foreach ( $settings as $key => $value ) {
				array_push( $keys, $key );
			}
			foreach ( $keys as $k ) {
				$options_s3_db[ $k ] = $settings[ $k ];
			}
			$data = $options_s3_db;
		}

		$pda_s3 = serialize( $data );

		$is_updated = update_option( PWV_Constants::CONFIGURATION_OPTIONS, $pda_s3 );
		if ( $is_updated ) {
			return $settings;
		}

		return array(
			'is_existed' => true,
		);
	}

	/**
	 * Get options in CloudFront Configuration.
	 *
	 * @return mixed
	 */
	public function get_cloudfront_configuration() {
		$data = json_decode( get_option( PWV_Constants::CLOUDFRONT_CONFIGURATION, false ) );

		if (
			empty( $data ) ||
			! isset( $data->{PWV_Constants::CLOUDFRONT_ID_OPTION} ) ||
			! isset( $data->{PWV_Constants::CLOUDFRONT_RSA_KEY_OPTION} ) ||
			! isset( $data->{PWV_Constants::CLOUDFRONT_URL_OPTION} )
		) {
			return array(
				PWV_Constants::CLOUDFRONT_URL_OPTION     => '',
				PWV_Constants::CLOUDFRONT_RSA_KEY_OPTION => '',
				PWV_Constants::CLOUDFRONT_ID_OPTION      => '',
			);
		}

		$rsa_key_decoded = ! empty( $data->{PWV_Constants::CLOUDFRONT_RSA_KEY_OPTION} ) ? base64_decode( $data->{PWV_Constants::CLOUDFRONT_RSA_KEY_OPTION} ) : null;

		return array(
			PWV_Constants::CLOUDFRONT_URL_OPTION     => rawurldecode( $data->{PWV_Constants::CLOUDFRONT_URL_OPTION} ),
			PWV_Constants::CLOUDFRONT_RSA_KEY_OPTION => $rsa_key_decoded,
			PWV_Constants::CLOUDFRONT_ID_OPTION      => $data->{PWV_Constants::CLOUDFRONT_ID_OPTION},
		);
	}

	/**
	 * Update options in CloudFront Configuration.
	 *
	 * @param object $data Option of CloudFront.
	 *
	 * @return mixed
	 */
	public function update_cloudfront_configuration( $data ) {
		$options = array(
			PWV_Constants::CLOUDFRONT_URL_OPTION     => rawurlencode( $data[ PWV_Constants::CLOUDFRONT_URL_OPTION ] ),
			PWV_Constants::CLOUDFRONT_RSA_KEY_OPTION => base64_encode( trim( $data[ PWV_Constants::CLOUDFRONT_RSA_KEY_OPTION ] ) ),
			PWV_Constants::CLOUDFRONT_ID_OPTION      => $data[ PWV_Constants::CLOUDFRONT_ID_OPTION ],
		);

		return update_option( PWV_Constants::CLOUDFRONT_CONFIGURATION, wp_json_encode( $options ) );
	}

	/**
	 * @param $key
	 * @param $value
	 *
	 * @return bool
	 */
	public function update_cloudfront_option( $key, $value ) {
		$data         = json_decode( get_option( PWV_Constants::CLOUDFRONT_CONFIGURATION, false ) );
		$data->{$key} = $value;

		return update_option( PWV_Constants::CLOUDFRONT_CONFIGURATION, wp_json_encode( $data ) );
	}

	/**
	 * Generate cloudfront url
	 *
	 * @param string $mp4            Mp4 file.
	 * @param string $cloudfront_url .
	 *
	 * @return string
	 */
	public function generate_cloudfront_url( $mp4, $cloudfront_url ) {
		$url_path       = parse_url( $mp4, PHP_URL_PATH );
		$cloudfront_url = rtrim( $cloudfront_url, "/" );

		return $cloudfront_url . $url_path;
	}

	/**
	 * @since 1.2.18 First time this was introduced.
	 *
	 * Check if the current page is the Gutenberg block editor.
	 *
	 * @param object $screen
	 *
	 * @return bool
	 */
	public function is_gutenberg_page( $screen ) {
		if ( function_exists( 'is_gutenberg_page' ) &&
		     is_gutenberg_page()
		) {
			// The Gutenberg plugin is on.
			return true;
		}

		return method_exists( $screen, 'is_block_editor' ) && $screen->is_block_editor();
	}

	/**
	 * Only check PDA Gold plugin is activated and valid license in version smaller or equal 2.x & version 3.x
	 *
	 * @return bool
	 */
	public function is_pda_gold_plugin_activated_and_valid_license() {
		return ( defined( 'PDA_v3_Constants::LICENSE_OPTIONS' ) && get_option( PDA_v3_Constants::LICENSE_OPTIONS ) ) || ( class_exists( 'Pda_Gold_Admin' ) && get_option( 'pda_is_licensed' ) );
	}

	/**
	 * Only check PDA Gold plugin is activated in version smaller or equal 2.x & version 3.x
	 *
	 * @return bool
	 */
	public function is_pda_gold_plugin_activated() {
		return class_exists( 'Prevent_Direct_Access_Gold' ) || class_exists( 'Pda_Gold_Admin' );
	}

}
