<?php

/**
 * The file that defines the core plugin class
 *
 * A class definition that includes attributes and functions used across both the
 * public-facing side of the site and the admin area.
 *
 * @link       buildwps.com
 * @since      1.0.0
 *
 * @package    Protect_Ur_Videos
 * @subpackage Protect_Ur_Videos/includes
 */

/**
 * The core plugin class.
 *
 * This is used to define internationalization, admin-specific hooks, and
 * public-facing site hooks.
 *
 * Also maintains the unique identifier of this plugin as well as the current
 * version of the plugin.
 *
 * @since      1.0.0
 * @package    Protect_Ur_Videos
 * @subpackage Protect_Ur_Videos/includes
 * @author     Buildwps <gaupoit@outlook.com>
 */
class Protect_Ur_Videos {

	/**
	 * The loader that's responsible for maintaining and registering all hooks that power
	 * the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      Protect_Ur_Videos_Loader $loader Maintains and registers all hooks for the plugin.
	 */
	protected $loader;

	/**
	 * The unique identifier of this plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string $plugin_name The string used to uniquely identify this plugin.
	 */
	protected $plugin_name;

	/**
	 * The current version of the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string $version The current version of the plugin.
	 */
	protected $version;

	/**
	 * Define the core functionality of the plugin.
	 *
	 * Set the plugin name and the plugin version that can be used throughout the plugin.
	 * Load the dependencies, define the locale, and set the hooks for the admin area and
	 * the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function __construct() {

		$this->plugin_name = 'PDA Protect WordPress Videos';
		$this->version     = '1.1.2';

		$this->load_dependencies();
		$this->set_locale();
		$this->define_admin_hooks();
		$this->define_public_hooks();

	}

	/**
	 * Load the required dependencies for this plugin.
	 *
	 * Include the following files that make up the plugin:
	 *
	 * - Protect_Ur_Videos_Loader. Orchestrates the hooks of the plugin.
	 * - Protect_Ur_Videos_i18n. Defines internationalization functionality.
	 * - Protect_Ur_Videos_Admin. Defines all hooks for the admin area.
	 * - Protect_Ur_Videos_Public. Defines all hooks for the public side of the site.
	 *
	 * Create an instance of the loader which will be used to register the hooks
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function load_dependencies() {

		/**
		 * The class responsible for orchestrating the actions and filters of the
		 * core plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-protect-ur-videos-loader.php';

		/**
		 * The class responsible for defining internationalization functionality
		 * of the plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-protect-ur-videos-i18n.php';

		/**
		 * The class responsible for defining the constants
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-protect-ur-videos-constants.php';

		/**
		 * The class responsible for defining the helper functions
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-protect-ur-videos-functions.php';

		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-protect-ur-videos-s3-custom-button-service.php';

		/**
		 * The class responsible for S3 services
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-protect-ur-videos-s3-service.php';

		/**
		 * The class responsible for defining API routes.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-protect-ur-videos-api.php';

		/**
		 * The class responsible for defining all actions that occur in the admin area.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/class-protect-ur-videos-admin.php';

		/**
		 * The class responsible for defining all actions that occur in the public-facing
		 * side of the site.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'public/class-protect-ur-videos-public.php';

		/**
		 * The class responsible for protecting video
		 *
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-private-video.php';

		/**
		 * The class responsible for amazon S3 utilities
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-aws-s3-helper.php';

		/**
		 * This class responsible for require all sdk helper classes
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'yme-wp-plugins-sdk/require.php';

		/**
		 * The class render setting for video
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-protect-ur-videos-settings.php';

		/**
		 * The class render configuration tab
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-protect-ur-videos-configuration.php';

		/**
		 * The repository
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-protect-ur-videos-repository.php';

		/**
		 * Cronjob
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/cronjob-handlers/class-protect-ur-videos-cronjob-handler.php';
		/**
		 * Helpers
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-protect-ur-videos-helpers.php';

		$this->loader = new Protect_Ur_Videos_Loader();

	}

	/**
	 * Define the locale for this plugin for internationalization.
	 *
	 * Uses the Protect_Ur_Videos_i18n class in order to set the domain and to register the hook
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function set_locale() {

		$plugin_i18n = new Protect_Ur_Videos_i18n();

		$this->loader->add_action( 'plugins_loaded', $plugin_i18n, 'load_plugin_textdomain' );

	}

	/**
	 * Register all of the hooks related to the admin area functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_admin_hooks() {

		$plugin_admin = new Protect_Ur_Videos_Admin( $this->get_plugin_name(), $this->get_version() );
		if ( Protect_Ur_Videos_Helpers::get_instance()->is_pda_gold_plugin_activated_and_valid_license() ) {
			$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_styles' );
			$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_scripts' );

//            if (pwv_is_s3_mode_on()) {
//                $this->loader->add_action('wp_ajax_upload_to_storage', $plugin_admin, 'upload_to_storage');
//            }
			$this->loader->add_action( 'init', $plugin_admin, 'add_videojs_btn' );
			$this->loader->add_shortcode( 'track', $plugin_admin, 'video_track' );

			$this->loader->add_action( 'rest_api_init', $plugin_admin, 'pwv_rest_api_init_cb', 12, 2 );

			$this->loader->add_action( 'admin_menu', $plugin_admin, 'pda_protect_video_create_menu' );
			$this->loader->add_action( 'wp_ajax_pda_protect_video_update_settings', $plugin_admin, 'pda_protect_video_update_settings' );

			$this->loader->add_action( 'wp_ajax_protect_video_test_configuration', $plugin_admin, 'protect_video_test_configuration' );

			$this->loader->add_action( 'wp_ajax_protect_video_update_configuration', $plugin_admin, 'protect_video_update_configuration' );
			$this->loader->add_action( 'admin_notices', $plugin_admin, 'handle_admin_notices' );
			$this->loader->add_filter( 'pwp_block_download_managers', $plugin_admin, 'is_block_idm', 1, 1 );
			$this->loader->add_filter( 'pwp_video_delete_expired_cache_links', $plugin_admin, 'delete_all_cache_link_expired', 1, 1 );
			$this->loader->add_action( 'admin_init', $plugin_admin, 'db_handle', 10 );
			$this->loader->add_action( PWV_Constants::PWP_DELETE_EXPIRED_CACHE_LINK_CRON_JOB_NAME, $plugin_admin, 'pwp_delete_expired_cache_link_cron_exec' );
			$this->loader->add_filter( 'puc_request_info_options-pda-protect-videos', $plugin_admin, 'puc_add_token_to_headers', 10, 1 );

		}

		$this->loader->add_action( 'admin_notices', $plugin_admin, 'protect_wp_videos_admin_notice' );
	}

	/**
	 * Register all of the hooks related to the public-facing functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_public_hooks() {
		$plugin_public = new Protect_Ur_Videos_Public( $this->get_plugin_name(), $this->get_version() );
		$this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_styles' );
		$this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_scripts' );
		$this->loader->add_action( 'wp_ajax_regenerate_private_url', $plugin_public, 'regenerate_private_url' );
		$this->loader->add_filter( 'pre_handle_video_shortcode', $plugin_public, 'pre_handle_video_shortcode' );

		$this->loader->add_filter( 'fv_flowplayer_shortcode', $plugin_public, 'handle_wp_video_shortcode' );

		$this->loader->add_shortcode( 'protected_video', $plugin_public, 'video_shortcode' );
		$this->loader->add_shortcode( 'pwv_hls', $plugin_public, 'video_hls_shortcode' );
		$this->loader->add_shortcode( 'pda-tech-smith', $plugin_public, 'handle_pda_tech_smith_shortcode' );
		// Override waveplayer shortcode.
		$this->loader->add_shortcode( 'pwv_waveplayer', $plugin_public, 'waveplayer_shortcode' );
	}

	/**
	 * Run the loader to execute all of the hooks with WordPress.
	 *
	 * @since    1.0.0
	 */
	public function run() {
		$this->loader->run();
	}

	/**
	 * The name of the plugin used to uniquely identify it within the context of
	 * WordPress and to define internationalization functionality.
	 *
	 * @since     1.0.0
	 * @return    string    The name of the plugin.
	 */
	public function get_plugin_name() {
		return $this->plugin_name;
	}

	/**
	 * The reference to the class that orchestrates the hooks with the plugin.
	 *
	 * @since     1.0.0
	 * @return    Protect_Ur_Videos_Loader    Orchestrates the hooks of the plugin.
	 */
	public function get_loader() {
		return $this->loader;
	}

	/**
	 * Retrieve the version number of the plugin.
	 *
	 * @since     1.0.0
	 * @return    string    The version number of the plugin.
	 */
	public function get_version() {
		return $this->version;
	}

}
