<?php

/**
 *
 * @link              https://www.preventdirectaccess.com/
 * @since             1.0.0
 * @package           Protect_Wordpress_Videos
 *
 * @wordpress-plugin
 * Plugin Name:       PDA Protect WordPress Videos
 * Plugin URI:        https://preventdirectaccess.com/extensions/protect-wordpress-videos/?utm_source=user-website&utm_medium=pluginsite_link&utm_campaign=pda-protect-wp-videos
 * Description:       Allow users to view your WordPress videos online but not to download them.
 * Version:           1.3.1
 * Author:            BWPS
 * Author URI:        https://preventdirectaccess.com/extensions/protect-wordpress-videos/?utm_source=user-website&utm_medium=author_link&utm_campaign=pda-protect-wp-videos
 * License:           GPL-2.0+
 * License URI:       http://www.gnu.org/licenses/gpl-2.0.txt
 * Text Domain:       pda-protect-wp-videos
 * Domain Path:       /languages
 */
if ( ! defined( 'WPINC' ) ) {
	die;
}

define( 'WPPP_BASE_URL', plugin_dir_url( __FILE__ ) );
define( 'PDA_PROTECT_VIDEO_PLUGIN_BASE_FILE',  __FILE__ );
define( 'PDA_PROTECT_VIDEO_PLUGIN_BASE_DIR', plugin_dir_path( __FILE__ ) );
define( 'PDA_PROTECT_VIDEOS_BASE_DIR', plugin_dir_path( __FILE__ ) );
define( 'PDA_PROTECT_VIDEOS_PLUGIN_FOLDER', 'protect-wp-videos/protect-ur-videos.php' );

/**
 * Is pda gold deactivated
 *
 * @return bool
 */
function pwv_is_pda_gold_deactivated() {
	return Yme_Plugin_Utils::is_plugin_activated( 'pda_gold' ) !== - 1 && Yme_Plugin_Utils::is_plugin_activated( 'pda_v3' ) !== - 1;
}

/**
 * The code that runs during plugin activation.
 * This action is documented in includes/class-protect-ur-videos-activator.php
 */
function activate_protect_ur_videos() {
	require_once plugin_dir_path( __FILE__ ) . 'includes/class-protect-ur-videos-activator.php';
	Protect_Ur_Videos_Activator::activate(plugin_basename( __FILE__ ));
}

/**
 * The code that runs during plugin deactivation.
 * This action is documented in includes/class-protect-ur-videos-deactivator.php
 */
function deactivate_protect_ur_videos() {
	require_once plugin_dir_path( __FILE__ ) . 'includes/class-protect-ur-videos-deactivator.php';
	Protect_Ur_Videos_Deactivator::deactivate();
}

register_activation_hook( __FILE__, 'activate_protect_ur_videos' );
register_deactivation_hook( __FILE__, 'deactivate_protect_ur_videos' );

/**
 * The core plugin class that is used to define internationalization,
 * admin-specific hooks, and public-facing site hooks.
 */
require plugin_dir_path( __FILE__ ) . 'includes/class-protect-ur-videos.php';

/**
 * Filter plugin updates.
 *
 * @param object $value Data update cache.
 *
 * @return object $value Data update cache.
 */
function pwv_filter_plugin_updates( $value ) {
	if ( ! isset( $value->response[ PDA_PROTECT_VIDEOS_PLUGIN_FOLDER ] ) ) {
		return $value;
	}

	unset( $value->response[ PDA_PROTECT_VIDEOS_PLUGIN_FOLDER ] );

	return $value;
}


/**
 * Begins execution of the plugin.
 *
 * Since everything within the plugin is registered via hooks,
 * then kicking off the plugin from this point in the file does
 * not affect the page life cycle.
 *
 * @since    1.0.0
 */
function run_protect_ur_videos() {

	$plugin = new Protect_Ur_Videos();
	$plugin->run();

	pwv_update_checker();

}

function pwv_update_checker() {
	if ( pwv_is_pda_gold_deactivated() ) {
		add_filter( 'site_transient_update_plugins', 'pwv_filter_plugin_updates' );

		return;
	}

	$configs = require PDA_PROTECT_VIDEOS_BASE_DIR . '/includes/class-protect-ur-videos-configs.php';

	// Make sure the class Puc_v4p8_Factory exits.
	if ( method_exists( 'Puc_v4p8_Factory', 'buildUpdateChecker' ) ) {
		Puc_v4p8_Factory::buildUpdateChecker(
			$configs->update_url,
			__FILE__,
			'pda-protect-videos'
		);
	} elseif ( method_exists( 'Puc_v4_Factory', 'buildUpdateChecker' ) ) {
		Puc_v4_Factory::buildUpdateChecker(
			$configs->update_url,
			__FILE__,
			'pda-protect-videos'
		);
	} else {
		add_filter( 'site_transient_update_plugins', 'pwv_filter_plugin_updates' );
	}
}

run_protect_ur_videos();
