<?php

/**
 * The public-facing functionality of the plugin.
 *
 * @link       buildwps.com
 * @since      1.0.0
 *
 * @package    Protect_Ur_Videos
 * @subpackage Protect_Ur_Videos/public
 */

/**
 * The public-facing functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    Protect_Ur_Videos
 * @subpackage Protect_Ur_Videos/public
 * @author     Buildwps <gaupoit@outlook.com>
 */
class Protect_Ur_Videos_Public {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string $plugin_name The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The slug of this plugin
	 *
	 * @access   private
	 * @var      string
	 */
	private $plugin_slug;


	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string $version The current version of this plugin.
	 */
	private $version;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      object $private_video Private video utilities.
	 */
	private $private_video;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @param string $plugin_name The name of the plugin.
	 * @param string $version The version of this plugin.
	 *
	 * @since    1.0.0
	 */
	public function __construct( $plugin_name, $version ) {

		$this->plugin_name   = $plugin_name;
		$this->plugin_slug   = 'pda-protect-wordpress-videos';
		$this->version       = $version;
		$this->private_video = new Private_Video();

	}

	/**
	 * Register the stylesheets for the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_styles() {

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in Protect_Ur_Videos_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The Protect_Ur_Videos_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */

	}

	/**
	 * Register the JavaScript for the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_scripts() {

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in Protect_Ur_Videos_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The Protect_Ur_Videos_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */

	}

	/**
	 * Register videojs library
	 */
	public function register_videojs_lib() {
		wp_enqueue_style( $this->plugin_slug . '-errors-css', plugin_dir_url( __FILE__ ) . 'js/videojs-errors-3.0.2/videojs-errors.css' );
	}

	public function handle_wp_video_shortcode( $output ) {
		if ( class_exists( 'PDA_Services' ) ) {
			$attachment = PDA_Services::get_post_by_url( $output['src'] );
			if ( ! empty( $attachment ) && class_exists( 'PDA_Api_Gold' ) ) {
				$api           = new PDA_Api_Gold();
				$source        = $api->generateExpired( array( 'id' => $attachment[0]->ID, 'post_fix' => '.mp4' ) );
				$output['src'] = $source;
			}
		}

		return $output;
	}

	public function video_shortcode( $atts, $content = null ) {
		$message = apply_filters( PWV_Constants::HOOK_PRE_HANDLE_VIDEO_SHORTCODE, false );
		if ( false !== $message ) {
			return $message;
		}
		extract( shortcode_atts( array(
			'mp4'      => '',
			'webm'     => '',
			'ogg'      => '',
			'youtube'  => '',
			'poster'   => '',
			'width'    => 512,
			'height'   => 308,
			'preload'  => true,
			'autoplay' => false,
			'loop'     => '',
			'controls' => '',
			'id'       => '',
			'class'    => '',
			'muted'    => '',
			'video_id' => '',
			'post_id'  => '',
			's3_link'  => '',
			'is_s3'    => '',
		), $atts ) );
		if ( $s3_link ) {
			$video_id = empty( $video_id ) ? uniqid( 'pda-s3-videos_' ) : $video_id;
		}
		if ( 'true' === $is_s3 ) {
			$id = empty( $video_id ) ? uniqid( 'pda-s3-cf_' ) : $video_id;
		}
		$video_id = empty( $post_id ) ? $video_id : $post_id;

		$dataSetup = array(
			'chromecast'    => array(
				'appId'    => 'testMac',
				'metadata' => array(
					'title'
				)
			),
			"playbackRates" => array(
				0.25,
				0.5,
				0.75,
				1,
				1.5,
				2
			)
		);

		$style = " style='";
		if ( strpos( $width, '%' ) !== false ) {
			$style .= "width: " . $width . ";";
			$width = "";
		}

		if ( strpos( $height, '%' ) !== false ) {
			$style  .= "height: " . $height . ";";
			$height = "";
		}

		$style .= "'";

		if ( $style == " style='''" ) {
			$style = "";
		}

		if ( $id == '' ) {
			$id = 'protect-video-id_' . $video_id . '_' . uniqid();
		}

		$media_type = '';
		if ( $mp4 ) {
			$media_type       = $this->get_media_type( $mp4 );
			$soure_type       = 'video' === $media_type ? 'video/mp4' : 'audio/mp3';
			$source           = '';
			$mp4_signed_video = $mp4;
			if ( 'true' === $is_s3 ) {
				$cloudfront_configuration = Protect_Ur_Videos_Helpers::get_instance()->get_cloudfront_configuration();
				$cloudfront_url           = $cloudfront_configuration[ PWV_Constants::CLOUDFRONT_URL_OPTION ];
				if ( empty( $cloudfront_url ) ) {
					return;
				}
				$url_generated    = Protect_Ur_Videos_Helpers::get_instance()->generate_cloudfront_url( $mp4, $cloudfront_url );
				$mp4_signed_video = $this->private_video->getSignedURL( $url_generated, PWV_Constants::DEFAULT_EXPIRE_TIME );
			} else {
				if ( class_exists( 'PDA_Api_Gold' ) && ! empty( $video_id ) ) {
					$api              = new PDA_Api_Gold();
					$source           = $api->generateExpired(
						array(
							'id'                 => $video_id,
							'clear_expired_link' => true,
						) );
					$mp4_signed_video = $source;
				}
			}
			$mp4_source = '<source src="' . $mp4_signed_video . '" type=\'' . $soure_type . '\' />';
		} else {
			$mp4_source = '';
		}

		if ( $s3_link ) {
			$media_type       = $this->get_media_type( $s3_link );
			$soure_type       = "video" === $media_type ? "video/mp4" : "audio/mp3";
			$s3_service       = new PWV_S3_Service();
			$source           = $s3_service->create_presigned_url( $s3_link, '+3 seconds' );
			$mp4_signed_video = $source;
			$mp4_source       = '<source src="' . $mp4_signed_video . '" type=\'' . $soure_type . '\' />';
		}

		// Preload the video?
		$preload_attribute = ' preload="false"';

		// Autoplay the video?
		if ( $autoplay == "true" || $autoplay == "on" ) {
			$autoplay_attribute = " autoplay";
		} else {
			$autoplay_attribute = "false";
		}

		// Tracks
		if ( ! is_null( $content ) ) {
			$track = do_shortcode( $content );
		} else {
			$track = "";
		}

		if ( $class ) {
			$class = ' ' . $class;
		}

		if ( $poster ) {
			$poster_attribute = ' poster="' . $poster . '"';
		} else {
			$poster_attribute = '';
		}

		if ( $controls == "false" ) {
			$controls_attribute = "";
		} else {
			$controls_attribute = " controls";
		}

		if ( $loop == "true" ) {
			$loop_attribute = " loop";
		} else {
			$loop_attribute = "";
		}

		if ( $muted == "true" ) {
			$muted_attribute = " muted";
		} else {
			$muted_attribute = "";
		}


		$jsonDataSetup = str_replace( '\\/', '/', json_encode( $dataSetup ) );

		error_log( $media_type );

		if ( ! empty( $atts['playlist'] ) ) {
			$videojs =
				<<<_end
    <section class="main-preview-player">
      <video
        id="preview-player"
        class="video-js vjs-fluid"
        controls
        preload="auto"
      >
        <p class="vjs-no-js">
          To view this video please enable JavaScript, and consider upgrading to
          a web browser that
          <a href="http://videojs.com/html5-video-support/" target="_blank"
            >supports HTML5 video</a
          >
        </p>
      </video>

      <div class="playlist-container  preview-player-dimensions vjs-fluid">
        <ol id="item-playlist" class="vjs-playlist"></ol>
      </div>
    </section>
    <input id="list-video-id" type="hidden" value="{$atts['playlist']}">
_end;
		} else if ( "video" === $media_type ) {
			//Output the <video> tag
			$videojs = <<<_end_

		<!-- Begin Video.js -->
		<video  oncontextmenu="return false" onselectstart="return false" ondragstart="return false" id="{$id}" class="video-js vjs-default-skin{$class}" width="{$width}" height="{$height}"{$poster_attribute}{$controls_attribute}{$preload_attribute}{$autoplay_attribute}{$loop_attribute}{$muted_attribute}{$style} data-setup='{$jsonDataSetup}'>
			{$mp4_source}
			{$track}
		</video>
<!-- Here will be the player setup script -->
		<!-- End Video.js -->
_end_;
		} else if ( "audio" === $media_type ) {
			if ( $poster_attribute === '' ) {
				//Output the <video> tag
				$videojs = <<<_end_

		<!-- Begin Video.js -->
		<audio oncontextmenu="return false" onselectstart="return false" ondragstart="return false" id="{$id}" class="video-js vjs-default-skin{$class}" width="{$width}" height="{$height}"{$controls_attribute}{$preload_attribute}{$autoplay_attribute}{$loop_attribute}{$muted_attribute}{$style} data-setup='{$jsonDataSetup}'>
			{$mp4_source}
			{$track}
		</audio>
		
<!-- Here will be the player setup script -->
		<!-- End Video.js -->
_end_;
			} else {
				//Output the <video> tag
				$videojs = <<<_end_

		<!-- Begin Video.js -->
		<video oncontextmenu="return false" onselectstart="return false" ondragstart="return false" id="{$id}" class="video-js vjs-default-skin{$class}" width="{$width}" height="{$height}"{$poster_attribute}{$controls_attribute}{$preload_attribute}{$autoplay_attribute}{$loop_attribute}{$muted_attribute}{$style} data-setup='{$jsonDataSetup}'>
			{$mp4_source}
			{$track}
		</video>
		
<!-- Here will be the player setup script -->
		<!-- End Video.js -->
_end_;
			}
		} else {
			$videojs = <<<_end_
			<div></div>
_end_;
		}

		$this->add_script_for_videos();

		return $videojs;
	}

	/**
	 * Add scripts and styles for VideoJS.
	 */
	public function add_script_for_videos() {
		$none_obj = array(
			'ajaxurl'  => admin_url( 'admin-ajax.php' ),
			'rest_url' => get_rest_url(),
			'security' => wp_create_nonce( 'pur_nonce_public' ),
			'isS3Mode' => pwv_is_s3_mode_on(),
		);

		/**
		 * Need to load this event on Header because
		 *  - Load Video JS UI soon (Keep current UX)
		 *  - Register event avoid URL video is expired before load JS
		 */
		wp_enqueue_script( $this->plugin_slug . '-main-bundle', WPPP_BASE_URL . 'public/js/dist/main.bundle.js', array( 'jquery' ), $this->version, false );
		wp_localize_script( $this->plugin_slug . '-main-bundle', 'ajax_obj', $none_obj );

		// FIXME: Should combine CSS.
		wp_enqueue_style( $this->plugin_slug . '-main', plugin_dir_url( __FILE__ ) . 'js/dist/main.css', array(), $this->version, 'all' );

		$this->register_videojs_lib();
	}

	/**
	 * The function to change the URL's attribute before rendering the WaveForm
	 *
	 * @param $atts
	 *
	 * @return string
	 */
	public function waveplayer_shortcode( $atts ) {
		$message = apply_filters( PWV_Constants::HOOK_PRE_HANDLE_VIDEO_SHORTCODE, false );
		if ( false !== $message ) {
			return $message;
		}
		if ( ! function_exists( 'waveplayer_shortcode' ) ) {
			return 'Please install WavePlayer plugin to use this shortcode';
		}

		$args = shortcode_atts( array(
			'url'   => '',
			'is_s3' => '',
		), $atts );

		if ( ! isset( $args['url'] ) ) {
			return waveplayer_shortcode( $atts );
		}

		$url = $args['url'];
		if ( 'true' !== $args['is_s3'] ) {
			return waveplayer_shortcode( $atts );
		}

		$cloudfront_configuration = Protect_Ur_Videos_Helpers::get_instance()->get_cloudfront_configuration();
		$cloudfront_url           = $cloudfront_configuration[ PWV_Constants::CLOUDFRONT_URL_OPTION ];
		if ( empty( $cloudfront_url ) ) {
			return waveplayer_shortcode( $atts );
		}

		$url_generated = Protect_Ur_Videos_Helpers::get_instance()->generate_cloudfront_url( $url, $cloudfront_url );

		$track_id = massage_url_to_peak_track_id( $url_generated );
		$timeout  = $this->get_wave_file_timeout( $track_id );

		$atts['url'] = $this->private_video->getSignedURL( $url_generated, $timeout );

		return waveplayer_shortcode( $atts );
	}

	private function get_wave_file_timeout( $track_id ) {
		$peak_file = PEAK_FOLDER . $track_id;

		return file_exists( $peak_file ) ? PWV_Constants::DEFAULT_EXPIRE_TIME : DAY_IN_SECONDS;
	}

	private function get_media_type( $source ) {
		if ( preg_match( '/.*\.mp3$/', $source ) ) {
			return "audio";
		} else if ( preg_match( '/.*\.mp4$/', $source ) ) {
			return "video";
		} else {
			return "";
		}
	}

	/**
	 * @param array $atts
	 * @param null $content
	 * @return string
	 */
	public function handle_pda_tech_smith_shortcode( $atts, $content = null ) {
		$message = apply_filters( PWV_Constants::HOOK_PRE_HANDLE_VIDEO_SHORTCODE, false );
		if ( false !== $message ) {
			return $message;
		}
		extract( shortcode_atts( array(
			'url'    => '',
			'width'  => 512,
			'height' => 308,
		), $atts ) );
		$service = new PWV_S3_Service();
		$url     = $service->create_presigned_url( $url, '+3 seconds', array( 'ResponseCacheControl' => 'No-cache' ) );
		if ( ! function_exists( 'is_ssl' ) || ! is_ssl() ) {
			$url = preg_replace("/^https:/i", "http:", $url);
		}
		include plugin_dir_path( dirname( __FILE__ ) ) . 'includes/views/view-protect-wp-videos-tech-smith.php';

		return $video_content;
	}

	/**
	 * Export VideoJS shortcode.
	 *
	 * @param array $atts    Attributes of shortcode.
	 * @param null  $content Content of shortcode.
	 *
	 * @return string
	 */
	public function video_hls_shortcode( $atts, $content = null ) {
		$message = apply_filters( PWV_Constants::HOOK_PRE_HANDLE_VIDEO_SHORTCODE, false );
		if ( false !== $message ) {
			return $message;
		}
		extract(
			shortcode_atts(
				array(
					'url'    => '',
					'width'  => 512,
					'height' => 308,
					'expiry' => 5,
				),
				$atts
			)
		);

		// Create unique ID.
		$post_id  = get_the_ID();
		$video_id = "pwv-hls-{$post_id}";

		$url           = trim( $url );
		$expiry        = (int) $expiry;
		$aws_helper    = new Aws_S3_Helper();
		$presigned_url = $aws_helper->create_read_presigned_url( $url, "{$expiry} seconds" );
		if ( ! $presigned_url ) {
			// Keep original URL if can not convert to pre-signed URL.
			$presigned_url = $url;
		}

		$source = "
			<video-js id=\"{$video_id}\" width={$width} height={$height} class=\"video-js vjs-default-skin pwv-hls\" controls>
			  <source
			     src=\"{$presigned_url}\"
			     type=\"application/x-mpegURL\">
			</video-js>
		";

		$this->add_script_for_videos();

		return $source;
	}

	/**
	 * Handle before generate video shortcode.
	 *
	 * @param string $content Content.
	 *
	 * @return string
	 */
	public function pre_handle_video_shortcode( $content ) {
		if ( ! Protect_Ur_Videos_Helpers::get_instance()->is_pda_gold_plugin_activated_and_valid_license() ) {
			return '';
		}

		return $content;
	}
}
