import videojs from 'video.js';
import videojsErrors from './videojs-errors-3.0.2/videojs-errors.js';
import 'video.js/dist/video-js.min.css';
import 'videojs-playlist-ui/dist/videojs-playlist-ui.css'
import 'videojs-playlist/dist/videojs-playlist.min';
import 'videojs-playlist-ui/dist/videojs-playlist-ui.min';
import '@videojs/http-streaming/dist/videojs-http-streaming.min';
import '../css/protect-ur-videos-public.css';


(function( $ ) {
	'use strict';
    var curr = null;
    var countErr = 0;
    var maxCountErr = 20;
	$(document).ready( function() {
        var players = videojs.players;
        Object.keys(players).forEach(function (key) {
            var isSeeking = false;
            var player = players[key];

            player.on('ready', function() {
               console.log('Im ready');
            });

            player.on('seeking', function () {
                isSeeking = true;
                curr = player.currentTime();
                console.log("Seeking", curr);
            });

            player.on('error', function () {
                var error = this.error();
                console.log("Error", error.code);
                if(error.code === 2 || error.code === 4 || error.code === 0) {
                    if ( countErr >= maxCountErr ) {
                        return;
                    }
                    console.log('Loading again!');
                    countErr++;
                    _generate_url(this.src(), player);
                }
            });

            player.on('loadstart', function (evt) {
                console.log('Load start');
                if(curr) {
                    player.currentTime(curr);
                    player.play();
                    curr = null;
                } else {
                    // _generate_url(this.src(), player);
                }
            });

            player.on('playing', function () {
                console.log("Playing");
                countErr = 0;
            });

            player.on("stalled", function () {
                console.log("Stalled");
            });

            player.on("loadedmetadata", function () {
                console.log("loadedmetadata");
            });

            player.on("suspend", function () {
                // console.log("Stop");
                // this.load();
                // this.pause();
                // _generate_url(this.src(), player);
            });
            player.on('ended', function () {
                curr = null
            });

            function _generate_url(src, player) {
                var tmp = player.id_.split('_');
                var post_id = tmp[1];
                var rootDomain = ajax_obj.rest_url;
                if( 'pda-s3-cf' === tmp[0] ) {
                    if( 'pda-s3-videos' === tmp[0]) {
                        post_id = tmp[0];
                    }
                    var url = rootDomain + 'puv/v1/signed-url/';
                    $.ajax({
                        type: 'post',
                        dataType: 'json',
                        url: url,
                        data: {
                            action: 'regenerate_private_url',
                            security: ajax_obj.security,
                            src: src,
                            id: post_id,
                        },
                        success: function (response) {
                            curr = player.currentTime();
                            player.src(response);
                            isSeeking = false;
                            player.play();
                        }
                    });
                } else {
                    var url = rootDomain + 'pda/v1/generate-expired-link/' + post_id;
                    $.ajax({
                        type: 'post',
                        dataType: 'json',
                        url: url,
                        success: function (response) {
                            console.log("Response", response);
                            curr = player.currentTime();
                            player.src({type: 'video/mp4', src: response});
                            isSeeking = false;
                            player.play();
                        }
                    });
                }
            }
        });
    });

    function _handle_fv_players() {
        if("undefined" !== typeof flowplayer) {
            flowplayer(function (api, root) {

                api.on("load", function () {
                    // do something when a new video is about to be loaded
                }).on("ready", function () {
                    // do something when a video is loaded and ready to play
                }).on("error", function (evt, player, error) {
                    if(error.code === 2 || error.code === 4) {
                        api.error = api.loading = false;
                        $(".fp-message", root).css({opacity: 0});
                        $(".fp-waiting", root).css({opacity: 0});
                        $(root).removeClass("is-error");
                        _generate_signed_url(error.video.src, function (newLink) {
                            player.load({
                                sources: [
                                    { type: "video/mp4", src: newLink }
                                ]
                            }, function (e, api) {
                                // $(".fp-message:first", root).remove();
                                api.seekTo();
                            });
                        });
                    }
                });

            });
        }
    }

    function _generate_signed_url(src, cb) {
        var rootDomain = ajax_obj.rest_url;
        var url = rootDomain + 'pda/v1/generate-expired-from-private';
        $.ajax({
            type: 'post',
            dataType: 'json',
            data : {
             'url' : src
            },
            url: url,
            success: function (response) {
               cb(response);
            }
        })
    }

    $(function() {
        _handle_fv_players();
    });

    $(function() {
        $(document).ready(function() {
            let listVideoIds = $('#list-video-id');
            if (listVideoIds.val() && listVideoIds.val().length > 0){
                let listVideos = listVideoIds.val().split(';');
                const promises = [];
                listVideos.forEach(element => {
                    promises.push(generatePrivateLinkById(element));
                });
                Promise.all(promises).then(videos => {
                    if (videos.length > 0){
                        const playList = [];
                        const player = videojs("preview-player", {
                            fluid: true
                        });
                        videos.forEach((video,index) => {
                            const play = {
                                name: video.name,
                                duration: video.meta_value.length,
                                sources: [
                                    { src: `${video.link}?${index}`, type: video.meta_value.mime_type }
                                ],
                                thumbnail: false
                            };
                            playList.push(play);
                        });
                        player.playlist(playList);
                        player.playlist.autoadvance(0);
                        player.playlistUi();
                        $('.vjs-playlist-item').click(function () {
                            curr = null;
                        });
                    }
                }).catch(err => console.log(err));
            }
	        $('video-js.pwv-hls').each(function(){
		        videojs($(this)[0]).play();
	        });
        });

        function generatePrivateLinkById(id){
            const rootDomain = ajax_obj.rest_url;
            const settings = {
                "url": `${rootDomain}pda/v3/generate-expired-link-for-video/${id}`,
                "method": "POST",
                "headers": {
                    "content-type": "application/x-www-form-urlencoded",
                    "cache-control": "no-cache",
                }
            };
            return $.ajax(settings);
        }
    });

})( jQuery );
