<?php

namespace BWPS\Video\Aws3\Aws\Api\ErrorParser;

use BWPS\Video\Aws3\Aws\Api\Parser\JsonParser;
use BWPS\Video\Aws3\Aws\Api\Service;
use BWPS\Video\Aws3\Aws\Api\StructureShape;
use BWPS\Video\Aws3\Aws\CommandInterface;
use BWPS\Video\Aws3\Psr\Http\Message\ResponseInterface;
/**
 * Parses JSON-REST errors.
 */
class RestJsonErrorParser extends \BWPS\Video\Aws3\Aws\Api\ErrorParser\AbstractErrorParser
{
    use JsonParserTrait;
    private $parser;
    public function __construct(\BWPS\Video\Aws3\Aws\Api\Service $api = null, \BWPS\Video\Aws3\Aws\Api\Parser\JsonParser $parser = null)
    {
        parent::__construct($api);
        $this->parser = $parser ?: new \BWPS\Video\Aws3\Aws\Api\Parser\JsonParser();
    }
    public function __invoke(\BWPS\Video\Aws3\Psr\Http\Message\ResponseInterface $response, \BWPS\Video\Aws3\Aws\CommandInterface $command = null)
    {
        $data = $this->genericHandler($response);
        // Merge in error data from the JSON body
        if ($json = $data['parsed']) {
            $data = array_replace($data, $json);
        }
        // Correct error type from services like Amazon Glacier
        if (!empty($data['type'])) {
            $data['type'] = strtolower($data['type']);
        }
        // Retrieve the error code from services like Amazon Elastic Transcoder
        if ($code = $response->getHeaderLine('x-amzn-errortype')) {
            $colon = strpos($code, ':');
            $data['code'] = $colon ? substr($code, 0, $colon) : $code;
        }
        // Retrieve error message directly
        $data['message'] = isset($data['parsed']['message']) ? $data['parsed']['message'] : (isset($data['parsed']['Message']) ? $data['parsed']['Message'] : null);
        $this->populateShape($data, $response, $command);
        return $data;
    }
}
