<?php

/**
 * The admin-specific functionality of the plugin.
 *
 * @link       www.buildwps.com/prevent-direct-access-premium-plugin/?utm_source=user-website&utm_medium=pluginpage&utm_campaign=plugin-author-link
 * @since      1.0.0
 *
 * @package    Wp_Pda_Ip_Block
 * @subpackage Wp_Pda_Ip_Block/admin
 */

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    Wp_Pda_Ip_Block
 * @subpackage Wp_Pda_Ip_Block/admin
 * @author     BWPS <hello@ymese.com>
 */
class Wp_Pda_Ip_Block_Admin {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string $plugin_name The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string $version The current version of this plugin.
	 */
	private $version;

	/**
	 * IP BLock services
	 *
	 * @var Pda_Ip_Block_Services IP BLock services.
	 */
	private $services;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @param string $plugin_name The name of this plugin.
	 * @param string $version     The version of this plugin.
	 *
	 * @since    1.0.0
	 */
	public function __construct( $plugin_name, $version ) {
		$this->plugin_name = $plugin_name;
		$this->version     = $version;
		$this->services    = new Pda_Ip_Block_Services();
	}

	/**
	 * Register the stylesheets for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_styles() {
		if ( ! ip_block_have_main_plugin_version_const() ) {
			return;
		}
		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in Wp_Pda_Ip_Block_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The Wp_Pda_Ip_Block_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */

		wp_enqueue_style( $this->plugin_name . 'pda_vs_toastr_css', 'https://cdnjs.cloudflare.com/ajax/libs/toastr.js/latest/toastr.min.css', array(), $this->version, 'all' );
		if ( function_exists( 'get_current_screen' ) ) {
			if ( 'upload' === get_current_screen()->id ) {
				wp_enqueue_style( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'css/wp-pda-ip-block-admin.css', array(), $this->version, 'all' );
				wp_enqueue_style( $this->plugin_name . '-ip-block-component-css', plugin_dir_url( __FILE__ ) . 'js/dist/style/ipblock-form.css', array(), $this->version, false );
			}

			$tab = isset( $_GET['tab'] ) ? $_GET['tab'] : ''; // phpcs:ignore
			if ( Pda_Ip_Block_Constants::PPP_SETTING_TAB === $tab ) {
				wp_enqueue_style( 'ppp-gold-settings-css', PDA_IP_BLOCK_DIR_URL . 'includes/views/ppp-settings/assets/ppp-gold-settings.css', array(), $this->version, 'all' );
				wp_enqueue_script( 'ppp-gold-settings-js', PDA_IP_BLOCK_DIR_URL . 'includes/views/ppp-settings/assets/ppp-gold-settings.js', array( 'jquery' ), $this->version, false );
				wp_localize_script(
					'ppp-gold-settings-js',
					'ppp_gold_setting_data',
					array(
						'rest_url'    => get_rest_url(),
						'nonce'       => wp_create_nonce( 'wp_rest' ),
						'plugin_url'  => PDA_IP_BLOCK_DIR_URL,
						'plugin_name' => PDA_IP_BLOCK_NAME,
					)
				);
			}
		}
	}

	/**
	 * Register the JavaScript for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_scripts() {
		if ( ! ip_block_have_main_plugin_version_const() ) {
			return;
		}
		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in Wp_Pda_Ip_Block_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The Wp_Pda_Ip_Block_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */

		wp_enqueue_script( $this->plugin_name . '_tagsinput', plugin_dir_url( __FILE__ ) . 'js/jquery.tagsinput.min.js', array( 'jquery' ), $this->version, false );
		wp_enqueue_script(
			$this->plugin_name,
			plugin_dir_url( __FILE__ ) . 'js/wp-pda-ip-block-admin.js',
			array(
				'jquery',
				$this->plugin_name . '_tagsinput',
			),
			$this->version,
			false
		);

		wp_enqueue_script( $this->plugin_name . 'pda_s3_toastr_', 'https://cdnjs.cloudflare.com/ajax/libs/toastr.js/latest/toastr.min.js', $this->version );
		if ( function_exists( 'get_current_screen' ) ) {
			if ( "upload" === get_current_screen()->id ) {
				$pda_gold_script = 'prevent-direct-access-gold-v3-bundle-js';
				if ( wp_script_is( $pda_gold_script ) ) {
					wp_enqueue_script( $this->plugin_name . '-ip-block-component', plugin_dir_url( __FILE__ ) . 'js/dist/ipblock-form.js', array( $pda_gold_script ), $this->version, false );
				}

				wp_enqueue_script( 'ip_block_bulk_action_js' . $this->plugin_name, plugin_dir_url( __FILE__ ) . './js/wp-pda-ip-block-bulk-action.js', array( 'jquery' ), $this->version );
				wp_localize_script( 'ip_block_bulk_action_js' . $this->plugin_name, 'ip_block_bulk_action_data', array() );
			}
		}

	}

	/**
	 * Register API for PDA Access Restriction plugin
	 */
	public function rest_api_ip_block() {
		register_rest_route(
			'/pda-ip-block/',
			'(?P<post_id>[0-9-]+)',
			array(
				'methods'  => 'POST',
				'callback' => array( $this, 'insert_data_to_table' ),
			)
		);
		register_rest_route(
			'/pda-get-ip-block/',
			'(?P<post_id>[0-9-]+)',
			array(
				'methods'  => 'GET',
				'callback' => array( $this, 'get_ip_block' ),
			)
		);
		register_rest_route(
			'/pda-get-user-roles-ip-block/',
			'(?P<post_id>[0-9-]+)',
			array(
				'methods'  => 'GET',
				'callback' => array( $this, 'get_user_roles_ip_block' ),
			)
		);
		register_rest_route(
			'/pda-insert-user-roles/',
			'(?P<post_id>[0-9-]+)',
			array(
				'methods'  => 'POST',
				'callback' => array( $this, 'insert_user_roles_data_to_table' ),
			)
		);
		register_rest_route(
			'/pda-ar/',
			'search-user',
			array(
				'methods'             => 'POST',
				'callback'            => array( $this, 'search_user' ),
				'permission_callback' => function () {
					return current_user_can( 'upload_files' );
				},
			)
		);
		register_rest_route(
			'/pda-get-user-access/',
			'(?P<post_id>[0-9-]+)',
			array(
				'methods'             => 'GET',
				'callback'            => array( $this, 'get_user_access' ),
				'permission_callback' => function () {
					return current_user_can( 'upload_files' );
				},
			)
		);

		register_rest_route(
			'pup/v1',
			'ppp-update-ip-fap/(?P<post_id>[0-9-]+)',
			array(
				'methods'             => 'POST',
				'callback'            => array( $this, 'ppp_update_ip_fap' ),
				'permission_callback' => function () {
					return current_user_can( 'edit_posts' );
				},
			)
		);

		register_rest_route(
			'pup/v1',
			'ppp-get-ip-fap/(?P<post_id>[0-9-]+)',
			array(
				'methods'             => 'GET',
				'callback'            => array( $this, 'ppp_get_ip_fap' ),
				'permission_callback' => function () {
					return current_user_can( 'edit_posts' );
				},
			)
		);

		register_rest_route(
			'pup/v1',
			'ip-restriction',
			array(
				'methods'             => 'POST',
				'callback'            => array(
					$this,
					'ppp_setting_add_new_ip',
				),
				'permission_callback' => function () {
					return current_user_can( 'edit_posts' );
				},
			)
		);

		register_rest_route(
			'pup/v1',
			'ip-restriction',
			array(
				'methods'             => 'GET',
				'callback'            => array(
					$this,
					'ppp_setting_get_list_ip',
				),
				'permission_callback' => function () {
					return current_user_can( 'edit_posts' );
				},
			)
		);
	}

	/**
	 * API add new IP for Access Restriction tab in settings page of PPP Gold plugin
	 *
	 * @param array $request Request from client, includes: list_ip.
	 *
	 * @return WP_REST_Response
	 */
	public function ppp_setting_add_new_ip( $request ) {
		$list_ip = $request->get_param( 'list_ip' );
		if ( is_null( $list_ip ) ) {
			return $this->services->send_json( 400, Pda_Ip_Block_Constants::MESSAGE_DATA_IN_VALID, true );
		}
		$settings_data = array(
			Pda_Ip_Block_Constants::PPP_SETTING_LIST_IP => $list_ip,
		);
		update_option( Pda_Ip_Block_Constants::PPP_SETTING_IP_RESTRICTION_OPTION, $settings_data, 'no' );

		return $this->services->send_json( 200, Pda_Ip_Block_Constants::MESSAGE_UPDATE_IP_FAP_SUCCESS, false );
	}

	/**
	 * Get all IPs for Access Restriction tab in settings page of PPP Gold plugin
	 *
	 * @return array
	 */
	public function ppp_setting_get_list_ip() {
		return ip_block_list_ip_allow_access_protected_post();
	}

	/**
	 * Get IP FAP of for PPP Gold plugin
	 *
	 * @param array $data The data request from client.
	 *
	 * @return array|WP_REST_Response
	 */
	public function ppp_get_ip_fap( $data ) {
		$params = array( 'post_id' );
		if ( ! $this->services->is_valid_data( $data, $params ) ) {
			return $this->services->send_json( 400, Pda_Ip_Block_Constants::MESSAGE_DATA_IN_VALID, true );
		}

		return $this->services->ppp_get_ip_fap( $data['post_id'] );
	}

	/**
	 * Handle update IP FAP of PPP plugin for API
	 *
	 * @param array $data The data request from client.
	 *
	 * @return WP_REST_Response
	 */
	public function ppp_update_ip_fap( $data ) {
		$params = array( 'post_id', 'fap_type', 'ips' );
		if ( ! $this->services->is_valid_data( $data, $params ) ) {
			return $this->services->send_json( 400, Pda_Ip_Block_Constants::MESSAGE_DATA_IN_VALID, true );
		}

		$this->services->ppp_update_ip_fap( $data['post_id'], $data['fap_type'], $data['ips'] );

		return $this->services->send_json( 200, Pda_Ip_Block_Constants::MESSAGE_UPDATE_IP_FAP_SUCCESS, false );
	}

	/**
	 * API Search Users
	 *
	 * @param stdClass $data data on client. Have key: searchKey.
	 *
	 * @return array
	 */
	public function search_user( $data ) {
		if ( ! isset( $data['searchKey'] ) ) {
			return array();
		}

		return $this->services->search_users( $data['searchKey'] );
	}

	/**
	 * API get user access by post id
	 *
	 * @param array $data data on client. Have key: post_id.
	 *
	 * @return mixed
	 */
	public function get_user_access( $data ) {
		if ( ! isset( $data['post_id'] ) ) {
			return false;
		}

		return $this->services->get_user_access( $data['post_id'] );
	}

	/**
	 * Get IP Block data
	 *
	 * @param array $data data on client.
	 *
	 * @return mixed
	 */
	public function get_ip_block( $data ) {
		$ip_block_repo = new Pda_Ip_Block_Repository();

		return $ip_block_repo->get_post_id_db( $data['post_id'] );
	}

	/**
	 * Get File Access Permission for client
	 * Get Roles Access
	 * Get User Access
	 *
	 * @param array $data data on client.
	 *
	 * @return array
	 */
	public function get_user_roles_ip_block( $data ) {
		if ( ! isset( $data['post_id'] ) ) {
			return false;
		}
		$post_id       = $data['post_id'];
		$repo_ip_block = new Pda_Ip_Block_Repository();

		return $repo_ip_block->get_fap_info_by_post_id( $post_id );
	}

	/**
	 * Connect action in PDA Gold and Repository in PDA Access Restriction
	 *
	 * @param array $data data on client.
	 */
	public function pda_ip_block_set_fap( $data ) {
		$repo_ip_block = new Pda_Ip_Block_Repository();
		$repo_ip_block->set_fap_to_db( $data );
	}

	public function insert_user_roles_data_to_table( $data ) {
		global $wpdb;
		$table_name    = $wpdb->prefix . "pda_ip_block";
		$ip_block_repo = new Pda_Ip_Block_Repository();
		$data_ip_block = $ip_block_repo->get_post_id_db( $data["post_id"] );

		$type       = $data["file_access_permision"];
		$roles      = $data["user_roles"];
		$user_roles = [
			"type"  => $type,
			"roles" => $roles,
		];
		if ( empty( $data_ip_block ) ) {
			$wpdb->insert(
				$table_name,
				array(
					'post_id'    => $data["post_id"],
					'user_roles' => serialize( $user_roles ),
				)
			);
		} else {
			$wpdb->update(
				$table_name,
				array(
					'user_roles' => serialize( $user_roles ),
				),
				array(
					'post_id' => $data["post_id"],
				)
			);
		}
		if ( Yme_Plugin_Utils::is_plugin_activated( 'membership' ) === - 1 ) {
			if ( isset( $data["memberships_fap"] ) && ! empty( $data["memberships_fap"] ) ) {
				if ( $data["memberships_fap"] === 'memberships-default' || $data["memberships_fap"] === 'no-memberships' ) {
					update_post_meta( $data["post_id"], PDA_v3_Constants::$pda_meta_key_memberships_integration, $data["memberships_fap"] );
				} else {
					$data_member_keys = array(
						"user_access_manager",
						"memberships_2",
						"paid_memberships_pro",
						"woo_memberships",
						"woo_subscriptions",
						"ar_member",
						"restrict_content_pro",
					);
					$data_members     = array();
					foreach ( $data_member_keys as $key ) {
						if ( isset( $data[ $key ] ) ) {
							$data_members[ $key ] = $data[ $key ];
						}
					}
					if ( ! empty( $data_members ) ) {
						update_post_meta( $data["post_id"], PDA_v3_Constants::$pda_meta_key_memberships_integration, $data_members );
					}
				}
			}
		}
	}

	public function insert_data_to_table( $data ) {
		global $wpdb;
		$table_name    = $wpdb->prefix . "pda_ip_block";
		$ip_block_repo = new Pda_Ip_Block_Repository();
		$data_ip_block = $ip_block_repo->get_post_id_db( $data["post_id"] );

		if ( empty( $data_ip_block ) ) {
			$wpdb->insert(
				$table_name,
				array(
					'post_id'  => $data["post_id"],
					'ip_block' => $data["ip_block"],
				)
			);
		} else {
			$ip_block = $data["ip_block"];
			$wpdb->update(
				$table_name,
				array(
					'ip_block' => $ip_block,
				),
				array(
					'post_id' => $data["post_id"],
				)
			);
		}
	}

	public function check_plugin_pda_activate() {
		if ( ip_block_main_plugins_is_deactivated() ) {
			$message = "Please install and activate <a target=\"_blank\" rel=\"noopener noreferrer\" href=\"https://preventdirectaccess.com/pricing/\">Prevent Direct Access Gold</a> or <a target=\"_blank\" rel=\"noopener noreferrer\" href=\"https://preventdirectaccess.com/protect-wordpress-pages-posts/\">Protect Pages & Posts Gold</a> plugin";
			?>
			<div class="notice notice-error is-dismissible">
				<p><b><?php _e( $this->plugin_name ); ?>: </b><?php _e( $message, 'prevent-direct-access-gold' ); ?></p>
			</div>
			<?php
		}

		/**
		 * Notices for plugin update
		 */
		$updates = $this->is_yme_plugin_has_updates();
		if ( ! empty( $updates ) ) {
			global $pagenow;
			$position_notice = array( 'upload.php', 'plugins.php' );
			foreach ( $updates as $update ) {
				if ( 'Prevent Direct Access Gold' === $update->Name ) {
					if ( in_array( $pagenow, $position_notice ) ) {
						?>
						<div class="notice notice-warning is-dismissible">
							<p>
								<b><?php _e( $this->plugin_name ); ?>: </b> Please
								update<b> <?php _e( $update->Name ); ?></b>
								to its latest version (<?php esc_html_e( $update->update->new_version ); ?>) for our
								plugins to work properly.
							</p>
						</div>
						<?php
					}
				}
			}
		}
	}

	function render_ip_block() {
//        $ip_block = get_option('pda_gold_ip_block');
		$ip_white_list = get_option( 'pda_gold_ip_white_list' );
//        var_dump($ip_block);
//        var_dump($ip_white_list);
		?>
		<!--        Commented by thinhnp-->
		<!--        Reason: https://app.asana.com/0/383837169626397/588702644438026-->
		<!--        <div class="main_container">-->
		<!--            <form method="post" id="pda_gold_ip_block_form">-->
		<!--                <input type="hidden" nonce="--><?php //echo wp_create_nonce('pda_ajax_nonce_ip_block'); ?><!--" id="nonce_ip_block"/>-->
		<!--                <div class="ip-block-title">-->
		<!--                    <p>Prevent private link access or downloads from these unwanted IP addresses:</p>-->
		<!--                </div>-->
		<!--                <input id="pda_gold_ip_block" name="ip_block" value="--><?php //_e($ip_block); ?><!--" /><br>-->
		<!--                <p class="description">Use the asterisk (*) for wildcard matching. E.g: 7.7.7.* will match IP from 7.7.7.0 to 7.7.7.255</p>-->
		<!--                <p>-->
		<!--                    <input type="submit" value="--><?php //_e('Save IP Block'); ?><!--" class="button button-primary" name="btn_ip_lock">-->
		<!--                </p>-->
		<!--            </form>-->
		<!--        </div>-->
		<div class="main_container">
			<form method="post" id="pda_gold_ip_white_list_form">
				<input type="hidden" nonce="<?php echo wp_create_nonce( 'pda_ajax_nonce_ip_white_list' ); ?>"
				       id="nonce_ip_white_list"/>
				<div class="ip-block-title">
					<p>Whitelist these IP addresses: only allow the following IP addresses to access private download
						links</p>
				</div>
				<input id="pda_gold_ip_white_list" name="ip_white_list" value="<?php _e( $ip_white_list ); ?>"/><br>
				<p class="description">Use the asterisk (*) for wildcard matching, e.g. 7.7.7.* will match IP from 7.7.7.0 to 7.7.7.255</p>
				<p>
					<input type="submit" value="<?php _e( 'Save Changes' ); ?>" class="button button-primary"
					       name="btn_ip_white_list">
				</p>
			</form>
		</div>
		<?php
		$this->render_orignal_link_white_list_ip();
		$this->render_website_white_list_ip();
	}

	public function render_website_white_list_ip() {
		$ip_white_list = get_option( 'pda_gold_ip_website_white_list' );

		?>
		<div class="main_container">
			<h3>
				Restrict access to the entire website
			</h3>
			<form method="post" id="pda_gold_ip_website_whitelist">
				<!--                <input type="hidden" nonce="-->
				<?php //echo wp_create_nonce('pda_ajax_nonce_website_ip_white_list'); ?><!--" id="nonce_ip_website_white_list"/>-->
				<div class="ip-block-title">
					<p>Whitelist these IP addresses: only allow the following IP addresses to access this website</p>
				</div>
				<input id="pda_gold_ip_website_white_list" name="ip_website_white_list"
				       value="<?php _e( $ip_white_list ); ?>"/><br>
				<p class="description">Use the asterisk (*) for wildcard matching, e.g. 7.7.7.* will match IP from 7.7.7.0 to 7.7.7.255</p>
				<p>
					<input type="submit" value="<?php _e( 'Save Changes' ); ?>" class="button button-primary"
					       name="btn_ip_white_list">
				</p>
			</form>
		</div>
		<?php
	}

	/**
	 * Render UI for Origin Link whitelisted IP.
	 */
	public function render_orignal_link_white_list_ip() {
		$ip_white_list = get_option( 'pda_gold_ip_original_link_white_list' );

		?>
		<div class="main_container">
			<h3>
				Restrict access to protected links
			</h3>
			<form method="post" id="pda_gold_ip_original_link_whitelist">
				<!--                <input type="hidden" nonce="-->
				<div class="ip-block-title">
					<p>Allow the following IP addresses to <a target="_blank" rel="noopener" href="https://preventdirectaccess.com/docs/restrict-wordpress-access-ip-address/#protected-links">access all protected links</a></p>
				</div>
				<input id="pda_gold_ip_original_link_white_list" name="ip_original_link_white_list"
				       value="<?php _e( $ip_white_list ); ?>"/><br>
				<p class="description">Use the asterisk (*) for wildcard matching, e.g. 7.7.7.* will match IP from 7.7.7.0 to 7.7.7.255</p>
				<p>
					<input type="submit" value="<?php _e( 'Save Changes' ); ?>" class="button button-primary"
					       id="btn_ip_original_btn" name="btn_ip_original_list">
				</p>
			</form>
		</div>
		<?php
	}

	public function get_ip_white_list() {
		$str_ip_whhite_list = get_option( 'pda_gold_ip_white_list' );
		$arr_ip_white_list  = explode( ";", $str_ip_whhite_list );
		$newRule            = '';
		if ( $arr_ip_white_list[0] != null ) {
			// Return null when isset *.*.*.*
			for ( $i = 0; $i < count( $arr_ip_white_list ); $i ++ ) {
				if ( $arr_ip_white_list[ $i ] == "*.*.*.*" ) {
					return '';
				}
			}
			$ip = [ '*.*.*.*', '*.*.*', '*.*', '*' ];
			for ( $i = 0; $i < count( $arr_ip_white_list ) - 1; $i ++ ) {
				$newRule .= "RewriteCond %{REMOTE_ADDR} " . str_replace( $ip, '', $arr_ip_white_list[ $i ] ) . ' [OR]' . PHP_EOL;
			}
			$newRule .= "RewriteCond %{REMOTE_ADDR} " . str_replace( $ip, '', $arr_ip_white_list[ count( $arr_ip_white_list ) - 1 ] ) . PHP_EOL;
		}

		return $newRule;
	}

	public function get_ip_white_list_for_pdav3() {
		$str_ip_whhite_list = get_option( 'pda_gold_ip_white_list' );
		$arr_ip_white_list  = explode( ";", $str_ip_whhite_list );
		$rules              = [];
		if ( $arr_ip_white_list[0] != null ) {
			// Return null when isset *.*.*.*
			for ( $i = 0; $i < count( $arr_ip_white_list ); $i ++ ) {
				if ( $arr_ip_white_list[ $i ] == "*.*.*.*" ) {
					return [];
				}
			}
			array_push( $rules, '# Prevent Direct Access IP Whitelist Rules' );
			$ip = [ '*.*.*.*', '*.*.*', '*.*', '*' ];
			for ( $i = 0; $i < count( $arr_ip_white_list ) - 1; $i ++ ) {
				array_push( $rules, "RewriteCond %{REMOTE_ADDR} " . str_replace( $ip, '', $arr_ip_white_list[ $i ] ) . ' [OR]' );
			}
			array_push( $rules, "RewriteCond %{REMOTE_ADDR} " . str_replace( $ip, '', $arr_ip_white_list[ count( $arr_ip_white_list ) - 1 ] ) );
			array_push( $rules, '# Prevent Direct Access IP Whitelist Rules End' . PHP_EOL );
		}

		return $rules;
	}

	public function so_wp_ajax_update_ip_white_list() {
		$nonce = $_REQUEST['security_check'];
		if ( ! wp_verify_nonce( $nonce, 'pda_ajax_nonce_ip_white_list' ) ) {
			error_log( 'not verify nonce', 0 );
			wp_die( 'invalid_nonce' );
		}
		$settings = $_REQUEST['settings'];
		if ( array_key_exists( 'pda_gold_ip_white_list', $settings ) ) {
			update_option( 'pda_gold_ip_white_list', $settings['pda_gold_ip_white_list'] );
		}
		if ( array_key_exists( 'pda_gold_ip_website_white_list', $settings ) ) {
			update_option( 'pda_gold_ip_website_white_list', $settings['pda_gold_ip_website_white_list'] );
		}
		if ( array_key_exists( 'pda_gold_ip_original_link_white_list', $settings ) ) {
			update_option( 'pda_gold_ip_original_link_white_list', $settings['pda_gold_ip_original_link_white_list'] );
		}

		if ( class_exists( "Prevent_Direct_Access_Gold_Htaccess" ) ) {
			Prevent_Direct_Access_Gold_Htaccess::register_rewrite_rules();
		}
		wp_send_json( true );
		wp_die();
	}

	public function ip_block_htaccess_contents( $rewriteRules ) {
		$ip_white_list = get_option( 'pda_gold_ip_website_white_list' );
		if ( $ip_white_list === false || empty( $ip_white_list ) ) {
			return $rewriteRules;
		}
		$ips = explode( ";", $ip_white_list );
		if ( ! empty( $ips ) ) {
			$rewriteRules .= "order deny,allow" . PHP_EOL;
			$rewriteRules .= "deny from all" . PHP_EOL;
			$rewriteRules .= "allow from ";
			foreach ( $ips as $ip ) {
				$rewriteRules .= $ip . " ";
			}
			$rewriteRules .= PHP_EOL;
		}

		return $rewriteRules;
	}

	public function get_ip_white_list_website_for_pdav3() {
		$ip_white_list = get_option( 'pda_gold_ip_website_white_list' );
		$rules         = [];
		if ( ! empty( $ip_white_list ) ) {
			$ips = explode( ";", $ip_white_list );
			$ip  = implode( " ", $ips );
			array_push( $rules, "# Prevent Direct Access IP Blacklist Website Rules", "order deny,allow", "deny from all", "allow from " . $ip, "# Prevent Direct Access IP Blacklist Website Rules End" . PHP_EOL );
		}

		return $rules;
	}

	static function get_ip_block_by_post_id( $post_id ) {
		global $wpdb;
		$result = $wpdb->get_row( " SELECT * FROM {$wpdb->prefix}pda_ip_block WHERE post_id = $post_id " );

		return $result;
	}

	function delete_table_site_website( $blog_id, $drop ) {
		global $wpdb;
		$table_name = $wpdb->prefix . "pda_ip_block";
		$sql        = "DROP TABLE IF EXISTS $table_name";
		$wpdb->query( $sql );
	}

	function pda_ip_block_custom_bulk_actions( $actions ) {
		if ( - 1 !== Yme_Plugin_Utils::is_plugin_activated( 'pda_gold' ) && - 1 !== Yme_Plugin_Utils::is_plugin_activated( 'pda_v3' ) ) {
			return $actions;
		}
		if ( pda_ip_block_check_role_protection() ) {
			$actions['pda_assign_access_permission'] = __( 'Assign Permission', 'prevent-direct-access-gold' );
		}

		return $actions;
	}

	function pda_ip_block_bulk_action_handler( $redirect_to, $doaction, $post_ids ) {
		if ( $doaction !== 'pda_assign_access_permission' ) {
			return $redirect_to;
		}
		if ( ! empty( $post_ids ) ) {
			if ( ! empty( $_REQUEST["pda-ip-block-bulk-action"] ) ) {
				$repo_v3 = new PDA_v3_Gold_Repository;
				foreach ( $post_ids as $post_id ) {
					if ( $repo_v3->is_protected_file( $post_id ) ) {
						$data["post_id"]               = $post_id;
						$data["file_access_permision"] = $_REQUEST["pda-ip-block-bulk-action"];
						$data["user_roles"]            = $_REQUEST["pda_ip_block_value_custom_roles"] !== "" ? str_replace( ",", ";", $_REQUEST["pda_ip_block_value_custom_roles"] ) : "";

						$repo_ip_block = new Pda_Ip_Block_Repository();
						$repo_ip_block->insert_fap_to_db( $data );
						/*
						$this->insert_user_roles_data_to_table( $data );
						*/
					}
				}
			}
		}

		return $redirect_to;
	}

	function pda_ip_block_handle_footer_scripts() {
		if ( "upload" === get_current_screen()->id ) {
			ob_start();
			require_once PDA_IP_BLOCK_PLUGIN_DIR_PATH . 'includes/class-wp-pda-ip-block-bulk-action.php';
			$bulkaction = new Pda_ip_block_Bulk_Action();
			$bulkaction->template();
			$output = ob_get_clean();

			if ( ! $output ) {
				return;
			}

			echo '<script type="text/html" class="pda-ip-block-template" id="pda-ip-block-template-upload">';
			echo $output;
			echo '</script>';
		}
	}

	/**
	 * Render setting for protect files in folder tab
	 */
	public function render_ui_for_protect_folder_tab() {
		global $is_nginx;
		$folder_selected  = get_data_protect_folder_type_is_array( Pda_Ip_Block_Constants::PDA_PROTECT_ALL_FILE_IN_FOLDER );
		$wp_upload_folder = ip_block_get_wp_upload_folder();
		?>
		<div class="main_container">
			<form id="pda_protect_folder_form" autocomplete="off">
				<input type="hidden" id="pda_protect_folder_nonce"
				       value="<?php echo wp_create_nonce( Pda_Ip_Block_Constants::PDA_PROTECT_FOLDER_FORM_NONCE ); ?>"/>
				<table class="pda_v3_settings_table pda_v3_protect_folder_table" cellpadding="4">
					<tr>
						<td colspan="2">
							<h3 class="pda-title-folder-protection"><?php echo esc_html__( 'FOLDER PROTECTION', 'prevent-direct-access-gold' ) ?></h3>
							<p>Protect all files under selected folders below</p>
						</td>
					</tr>
					<?php
					include PDA_IP_BLOCK_PLUGIN_DIR_PATH . 'includes/views/view-wp-pda-ip-block-protect-folder.php';
					include PDA_IP_BLOCK_PLUGIN_DIR_PATH . 'includes/views/view-wp-pda-ip-block-fap.php';
					include PDA_IP_BLOCK_PLUGIN_DIR_PATH . 'includes/views/view-wp-pda-ref-block.php';
					if ( $is_nginx ) {
						include PDA_IP_BLOCK_PLUGIN_DIR_PATH . 'includes/views/view-wp-pda-ip-block-nginx-server.php';
					}
					?>
				</table>
				<p class="pda-submit-btn">
					<input type="submit" name="submit" id="submit" class="button button-primary" value="Save Changes">
				</p>
			</form>
		</div>
		<?php
	}

	/**
	 * Load js and css for protect files in folder tab
	 */
	public function load_js_css_for_protect_folder_tab() {
		global $is_nginx;
		wp_enqueue_script( 'load_vue_lib' . $this->plugin_name, plugin_dir_url( __FILE__ ) . 'js/vue.js', $this->version, false );
		wp_enqueue_script( 'load_js_for_protect_folder' . $this->plugin_name, plugin_dir_url( __FILE__ ) . 'js/wp-pda-ip-block-protect-folder.js', array( 'jquery' ), $this->version, false );
		wp_localize_script( 'load_js_for_protect_folder' . $this->plugin_name, 'pda_data_protect_folder', array(
				'ajax_url'      => admin_url( 'admin-ajax.php' ),
				'param_in_url'  => PDA_v3_Constants::$secret_param,
				'is_nginx'      => $is_nginx,
				'upload_path'   => ip_block_get_wp_upload_folder(),
				'ref_urls_conf' => ip_block_get_ref_folder_protection_conf(),
			)
		);

		wp_register_style( 'pda_protect_folder_css_' . $this->plugin_name, plugin_dir_url( __FILE__ ) . './css/wp-pda-ip-block-protect-folder.css', array(), $this->version );
		wp_enqueue_style( 'pda_protect_folder_css_' . $this->plugin_name );
	}

	/**
	 * Update data for protect files in folder tab
	 */
	public function pda_gold_update_data_for_protect_folder() {
		$nonce = $_REQUEST['security_check'];
		if ( ! wp_verify_nonce( $nonce, Pda_Ip_Block_Constants::PDA_PROTECT_FOLDER_FORM_NONCE ) ) {
			wp_die( 'Invalid nonce' );
		}
		$settings = $_REQUEST['settings'];

		$settings['pda_protect_all_file_in_folder'] = isset( $settings['pda_protect_all_file_in_folder'] ) && is_array( $settings['pda_protect_all_file_in_folder'] ) ? $settings['pda_protect_all_file_in_folder'] : array();
		$settings['pda_ref_url_folder_protect'] = isset( $settings['pda_ref_url_folder_protect'] ) && is_array( $settings['pda_ref_url_folder_protect'] ) ? $settings['pda_ref_url_folder_protect'] : array();

		// Remove the deleted folder from Referral URLs configuration.
		$selected_folders = $settings['pda_protect_all_file_in_folder'];
		$rel_url_confs    = $settings['pda_ref_url_folder_protect'];

		$massaged_ref_urls = array_filter( $rel_url_confs, function ( $conf ) use ( $selected_folders ) {
			return in_array( $conf['folder'], $selected_folders, true );
		} );

		// Use array_values to re-arrange index.
		$settings['pda_ref_url_folder_protect'] = array_values( $massaged_ref_urls );

		update_option( Pda_Ip_Block_Constants::PDA_OPTION_PROTECT_FOLDER, serialize( $settings ), 'no' );
		Prevent_Direct_Access_Gold_Htaccess::register_rewrite_rules();

		wp_send_json( ip_block_get_ref_folder_protection_conf() );
		wp_die();
	}

	/**
	 * Add rules folder protect to htaccess file
	 *
	 * @param array $rules rules in pda gold.
	 *
	 * @return array rules for htaccess file.
	 */
	public function pda_add_rules( $rules ) {
		$selected_folders = get_data_protect_folder_type_is_array( Pda_Ip_Block_Constants::PDA_PROTECT_ALL_FILE_IN_FOLDER );
		$types            = $this->services->get_folder_types( $selected_folders );
		$root_folder      = $types['root'];
		$upload_folder    = $types['wp_upload'];
		$defined_folders  = ip_block_maybe_get_all_folder_protections();

		if ( ! empty( $defined_folders['wp_upload'] ) ) {
			$upload_folder = array_merge( $upload_folder, $defined_folders['wp_upload'] );
		}
		if ( ! empty( $defined_folders['root'] ) ) {
			$root_folder = array_merge( $root_folder, $defined_folders['root'] );
		}

		if ( empty( $root_folder ) && empty( $upload_folder ) ) {
			return $rules;
		}

		// Add comment start rule for Folder Protection.
		array_push( $rules, PHP_EOL . '# IP Block Rewrite Rules for Folder Protection' );

		// Handle the root folder here.
		if ( ! empty( $root_folder ) ) {
			$root_rule = $this->convert_folder_to_rule( $root_folder, '(' );
			array_push( $rules, $root_rule );
		}

		// Handle the uploads folder here.
		if ( ! empty( $upload_folder ) ) {
			$upload_path = ip_block_get_wp_upload_folder();
			$upload_rule = $this->convert_folder_to_rule( $upload_folder, $upload_path . '(/' );
			array_push( $rules, $upload_rule );
		}

		// Add comment end rule for Folder Protection.
		array_push( $rules, '# IP Block Rewrite Rules for Folder Protection End' );

		return $rules;
	}

	/**
	 * Convert folder to rule for htaccess file
	 *
	 * @param array  $folder The folder name.
	 * @param string $path   Path.
	 *
	 * @return string htaccess rule.
	 */
	public function convert_folder_to_rule( $folder, $path = '' ) {
		$folder = implode( '|', $folder );
		$folder = preg_replace( '!\s+!', '\s+', $folder );

		return sprintf( 'RewriteRule ^%1$s(%2$s)/.*\.\w+)$ index.php?%3$s=$1 [QSA,L]', $path, $folder, PDA_v3_Constants::$secret_param );
	}

	/**
	 * Handle before send file in folder protect
	 *
	 * @param string $file_path file path in pda v3.
	 */
	public function handle_before_send_file( $file_path ) {
		$dirname         = ltrim( dirname( $file_path ), '/' );
		$root_dir        = explode( '/', $dirname )[0];
		$folder_selected = get_data_protect_folder_type_is_array( Pda_Ip_Block_Constants::PDA_PROTECT_ALL_FILE_IN_FOLDER );

		$is_protect_upload = in_array( $root_dir, $folder_selected, true );
		$is_protect_root   = in_array( '^' . $root_dir, $folder_selected, true );
		$folder_protection = ip_block_maybe_get_folder_protection( $dirname, $file_path );

		if ( $is_protect_upload || $is_protect_root || $folder_protection ) {
			if ( $is_protect_root ) {
				$file        = ABSPATH . $file_path;
				$file_folder = '^' . $root_dir;
			} else {
				$file        = strtok( rtrim( wp_upload_dir()['basedir'], '/' ) . str_replace( '..', '', $file_path ), '?' );
				$file_folder = $root_dir;
			}

			if ( ! is_file( $file ) ) {
				file_not_found();
			}

			/**
			 * Allow to set different permission for each folder.
			 */
			if ( isset( $folder_protection['has_permission'] ) ) {
				if ( true === $folder_protection['has_permission'] ) {
					$this->send_file_to_client( $file );
				} else {
					file_not_found();
				}
			}

			$result = apply_filters( Pda_Ip_Block_Constants::HOOK_BEFORE_CHECK_FAP_FOLDER_PROTECTION, $file, $file_folder );

			if ( ! $result && ! check_fap_for_folder_protect() ) {
				file_not_found();
			}

			$this->send_file_to_client( $file );
		}
	}

	/**
	 * Check plugin create by Ymese team having the new updates.
	 *
	 * @return array
	 */
	private function is_yme_plugin_has_updates() {
		$updates = get_plugin_updates();
		$results = array();
		foreach ( $updates as $plugin ) {
			if ( 'Prevent Direct Access Gold' === $plugin->Name ) {
				if ( version_compare( $plugin->Version, '3.1.2' ) < 0 ) {
					array_push( $results, $plugin );
				}
			}
		}

		return $results;
	}

	/**
	 * Set permission for file is protected.
	 *
	 * @param string $attachment_id The attachment's id.
	 * @param array  $fap           Permission.
	 *
	 * @return bool
	 */
	public function pda_handle_set_fap( $attachment_id, $fap ) {
		if ( ! array_key_exists( 'roles', $fap ) || ! is_array( $fap['roles'] ) ) {
			return false;
		}
		$ip_block_repo = new Pda_Ip_Block_Repository();

		return $ip_block_repo->set_fap_for_file_is_protected( $attachment_id, $fap );
	}

	/**
	 * @param $options
	 *
	 * @return mixed
	 */
	public function puc_add_token_to_headers( $options ) {
		$options['headers'] = array(
			'Accept'        => 'application/json',
			'Authorization' => 'helloymese',
		);

		return $options;
	}

	/**
	 * Check File Access Permission of User
	 *
	 * @param bool   $fap           file access permission.
	 * @param string $attachment_id attachment id.
	 *
	 * @return bool
	 */
	public function check_fap_for_ip_block( $fap, $attachment_id ) {
		return $this->services->check_fap_for_user( $fap, $attachment_id );
	}

	/**
	 * Add new to tab settings PPP Gold plugin.
	 *
	 * @param array $default_tab Default tab in PPP Gold plugin.
	 *
	 * @return mixed
	 */
	public function add_new_tab_for_ppp( $default_tab ) {
		array_push(
			$default_tab,
			array(
				'tab'      => Pda_Ip_Block_Constants::PPP_SETTING_TAB,
				'tab_name' => 'Restriction',
			)
		);

		return $default_tab;
	}

	/**
	 * Custom tab to PPP Gold render content.
	 *
	 * @param array $default_tab Default tab in PPP Gold plugin.
	 *
	 * @return mixed
	 */
	public function custom_tab_for_ppp( $default_tab ) {
		array_push( $default_tab, Pda_Ip_Block_Constants::PPP_SETTING_TAB );

		return $default_tab;
	}

	/**
	 * Render UI for Access Restriction tab in PPP Gold plugin
	 */
	public function render_content_access_restriction_tab() {
		include PDA_IP_BLOCK_PLUGIN_DIR_PATH . 'includes/views/ppp-settings/ppp-view-main-settings-page.php';
	}

	public function handle_admin_init() {
		Wp_Pda_Ip_Block_Db::create_table();
	}

	/**
	 * Allow the whitelisted IP to access the protected links.
	 *
	 * @since 1.2.1
	 *
	 * @param bool $valid         Is valid protected link. If it's true then can see the link right away.
	 * @param int  $attachment_id The attachment ID.
	 *
	 * @return bool
	 * True: current IP is whitelisted. Otherwise do nothing.
	 */
	public function allow_ip( $valid, $attachment_id ) {
		$ips_str = get_option( 'pda_gold_ip_original_link_white_list' );
		$ips     = explode( ';', $ips_str );
		if ( $this->services->is_whitelisted_ip( $ips ) ) {
			$valid = true;
		}

		// TODO: Later can handle individual attachment IP block here.
		return $valid;
	}

	public function check_ref_url_folder_protection( $file, $file_folder ) {
		return $this->services->is_valid_ref_url_folder_protection( $file_folder );
	}

	/**
	 * Response file to client.
	 *
	 * @param string $file File path.
	 */
	private function send_file_to_client( $file ) {
		$pda_helper = new Pda_v3_Gold_Helper();
		$mime_type  = $pda_helper->pda_mime_content_type( $file );
		if ( ! $mime_type ) {
			file_not_found();
		}
		send_file_to_client(
			$mime_type,
			$file,
			array(
				'user_id'   => get_current_user_id(),
				'link_id'   => - 1,
				'link_type' => PDA_v3_Constants::PDA_ORIGINAL_LINK,
				'mime_type' => $mime_type,
				'file'      => $file,
			)
		);
	}
}
