(function ($) {
  'use strict';

  /**
   * All of the code for your admin-facing JavaScript source
   * should reside in this file.
   *
   * Note: It has been assumed you will write jQuery code here, so the
   * $ function reference has been prepared for usage within the scope
   * of this function.
   *
   * This enables you to define handlers, for when the DOM is ready:
   *
   * $(function() {
	 *
	 * });
   *
   * When the window is loaded:
   *
   * $( window ).load(function() {
	 *
	 * });
   *
   * ...and/or other possibilities.
   *
   * Ideally, it is not considered best practise to attach more than a
   * single DOM-ready or window-load handler for a particular page.
   * Although scripts in the WordPress core, Plugins and Themes may be
   * practising this, we should strive to set a better example in our own work.
   */
  var regex = /^((?!0)(?!.*\.$)((1?\d?\d|25[0-5]|2[0-4]\d|\*)(\.|$)){4})|(([0-9a-f]|:){1,4}(:([0-9a-f]{0,4})*){1,7})$/;
  (function (window, $) {
    $(document).ready(function ($) {
      $('#pda_gold_ip_white_list').tagsInput({
        defaultText: '',
        delimiter: ';',
        width: '765px',
        pattern: regex,
      });

      var form_white_list = $("#pda_gold_ip_white_list_form");
      form_white_list.submit(function (e) {
        e.preventDefault();
        var ips_white_list = $("#pda_gold_ip_white_list").val();
        updateSettings_ip_white_list({
          pda_gold_ip_white_list: ips_white_list
        }, function (error) {
          if (error) {
            console.error(error);
          }
          // location.reload(true);
        });
      });

      function updateSettings_ip_white_list(settings, cb) {
        var _data = {
          action: 'update_ip_white_list',
          settings: settings,
          security_check: $("#nonce_ip_white_list").attr('nonce')
        }
        $("#submit").val("Submitting");
        $("#submit").prop("disabled", true);
        $.ajax({
          url: ip_block_server_data.ajaxurl, // this is the object instantiated in wp_localize_script function
          type: 'POST',
          data: _data,
          success: function (data) {
            $("#submit").prop("disabled", false);
            //Do something with the result from server
            if (data == 'invalid_nonce') {
              alert('No! No! No! Verify Nonce Fails!');
            } else if (data) {
              //success here
              console.log("Success", data);
              toastr.success('Your settings have been updated successfully!');
            } else {
              console.log("Failed", data);
            }
            cb();
          },
          error: function (error) {
            $("#submit").prop("disabled", false);
            console.log("Errors", error);
            cb(error);
          },
          timeout: 5000
        });
      }

      _handleIPWebsiteWhiteList();
      _handleIPOriginaLinkWhiteList();
    });
  })(window, jQuery.noConflict());

  function _handleIPWebsiteWhiteList() {
    $('#pda_gold_ip_website_white_list').tagsInput({
      defaultText: '',
      delimiter: ';',
      width: '765px',
      pattern: regex,
    });
    var form_white_list = $("#pda_gold_ip_website_whitelist");
    form_white_list.submit(function (e) {
      e.preventDefault();
      var ips_white_list = $("#pda_gold_ip_website_white_list").val();
      updateSettings_ip_white_list({
        pda_gold_ip_website_white_list: ips_white_list
      }, function (error) {
        if (error) {
          console.error(error);
        }
        // location.reload(true);
      });
    });

    function updateSettings_ip_white_list(settings, cb) {
      var _data = {
        action: 'update_ip_white_list',
        settings: settings,
        security_check: $("#nonce_ip_white_list").attr('nonce')
      }
      $("#submit").val("Submitting");
      $("#submit").prop("disabled", true);
      $.ajax({
        url: ip_block_server_data.ajaxurl, // this is the object instantiated in wp_localize_script function
        type: 'POST',
        data: _data,
        success: function (data) {
          $("#submit").prop("disabled", false);
          //Do something with the result from server
          if (data == 'invalid_nonce') {
            alert('No! No! No! Verify Nonce Fails!');
          } else if (data) {
            //success here
            console.log("Success", data);
            toastr.success('Your settings have been updated successfully!');
          } else {
            console.log("Failed", data);
          }
          cb();
        },
        error: function (error) {
          $("#submit").prop("disabled", false);
          console.log("Errors", error);
          cb(error);
        },
        timeout: 5000
      });
    }
  }

  function _handleIPOriginaLinkWhiteList() {
    $('#pda_gold_ip_original_link_white_list').tagsInput({
      defaultText: '',
      delimiter: ';',
      width: '765px',
      pattern: regex,
    });

    $("#pda_gold_ip_original_link_whitelist").submit(function (e) {
      e.preventDefault();
      var ips_white_list = $("#pda_gold_ip_original_link_white_list").val();
      updateSettings_original_link_white_list({
        pda_gold_ip_original_link_white_list: ips_white_list
      }, function (error) {
        if (error) {
          console.error(error);
        }
      });
    });

    function updateSettings_original_link_white_list(settings, cb) {
      var _data = {
        action: 'update_ip_white_list',
        settings: settings,
        security_check: $("#nonce_ip_white_list").attr('nonce')
      }
      $("#btn_ip_original_btn").val("Submitting");
      $("#btn_ip_original_btn").prop("disabled", true);
      $.ajax({
        url: ip_block_server_data.ajaxurl, // this is the object instantiated in wp_localize_script function
        type: 'POST',
        data: _data,
        success: function (data) {
          $("#btn_ip_original_btn").prop("disabled", false);
          //Do something with the result from server
          if (data == 'invalid_nonce') {
            alert('No! No! No! Verify Nonce Fails!');
          } else if (data) {
            //success here
            console.log("Success", data);
            toastr.success('Your settings have been updated successfully!');
          } else {
            console.log("Failed", data);
          }
          cb();
        },
        error: function (error) {
          console.log("Errors", error);
          cb(error);
        },
        complete: function () {
          $("#btn_ip_original_btn").prop("disabled", false);
          $("#btn_ip_original_btn").val("Save changes");
        },
        timeout: 5000
      });
    }
  }
})(jQuery);
