(function ($) {
  'use strict';

  let _shared = {
    vueApp: {},
  }

  $(function () {
    $(".pda_folder_select2").select2({
      width: '100%',
      templateSelection: _formatOpt,
    });

    $('#pda_fap_for_folder').change(function () {
      if ($(this).val() === 'custom_roles') {
        $('#pda_access_custom_roles').show('slow');
        $('#pda_access_custom_roles_select2').prop('required', true);
      } else {
        $('#pda_access_custom_roles').hide('slow');
        $('#pda_access_custom_roles_select2').prop('required', false);
      }
    });
    $('#pda_fap_for_folder').trigger('change');

    $('#pda_protect_folder_form').submit(function (evt) {
      evt.preventDefault();
      const refUrls = _set_ref_data_for_submit();
      if (!refUrls) {
        toastr.error('There are some empty or invalid referral links.', 'Prevent Direct Access Gold')
        return;
      } else {
        $('#pda-ref-app .form-errors').text()
        $('#pda-ref-app .form-errors').hide();
      }
      _updateDataForProtectFolder({
          pda_protect_all_file_in_folder: $("#pda_protect_all_file_in_folder").val(),
          pda_fap_for_folder_protect: $("#pda_fap_for_folder").val(),
          pda_white_list_role_folder_protect: $("#pda_access_custom_roles_select2").val(),
          pda_ref_url_folder_protect: refUrls,
        }, function (error, data) {
          if (error) {
            console.error(error);
            toastr.error('There is an error: ' + error, 'Prevent Direct Access Gold');
          }
        }
      );
    });

    _handle_ref_url_ui();
  });

  function _updateDataForProtectFolder(settings, cb) {
    var _data = {
      action: 'pda_gold_update_data_for_protect_folder',
      settings: settings,
      security_check: $('#pda_protect_folder_nonce').val(),
    };
    $('#summit').val('Submitting');
    $("#submit").prop("disabled", true);
    $.ajax({
      url: pda_data_protect_folder.ajax_url,
      type: 'POST',
      data: _data,
      success: function (data) {
        $("#submit").prop("disabled", false);
        if (data == 'Invalid nonce') {
          alert('No! No! No! Verify Nonce Fails!');
        } else if (data) {
          if (pda_data_protect_folder.is_nginx) {
            refreshRules();
          }
          _force_render_vue_app(data);
          toastr.success('Your settings have been updated successfully!', 'Prevent Direct Access Gold')
        } else {
          console.log("Failed", data);
        }
      },
      error: function (error) {
        $("#submit").prop("disabled", false);
        console.log("Errors", error);
        cb(error);
      },
      timeout: 5000
    });
  }

  function refreshRules() {
    const foldersSelected = $("#pda_protect_all_file_in_folder").val();
    if (foldersSelected === null) {
      $('#pda_check_server_is_nginx').hide();
      return;
    }

    let rules = [];
    const rootRule = _generateRootRule(foldersSelected, pda_data_protect_folder.param_in_url);
    if (rootRule) {
      rules.push(rootRule);
    }

    const uploadFolderRule = _generateWpUploadRule(
      foldersSelected,
      pda_data_protect_folder.param_in_url,
      pda_data_protect_folder.upload_path
    );

    if (uploadFolderRule) {
      rules.push(uploadFolderRule);
    }

    if (!rules.length) {
      $('#pda_check_server_is_nginx').hide();
      return;
    }
    $('#pda_rules_nginx').attr('rows', 2 * rules.length);
    $('#pda_rules_nginx').val(rules.join('\n'));
    $('#pda_check_server_is_nginx').show();
  }

  function _generateRootRule(foldersSelected, paramInUrl) {
    const rootFolders = foldersSelected
      .filter(f => f.startsWith('^'))
      .map(f => f.substring(1))
      .join('|')
      .replace(/\s+/g, '\\s+');

    if (!rootFolders) {
      return '';
    }
    return 'rewrite ((' + rootFolders + ')/.*\\.\\w+)$ "/index.php?' + paramInUrl + '=$1" last;'
  }

  function _generateWpUploadRule(foldersSelected, paramInUrl, upload_path) {
    let wpUploadFolders = foldersSelected
      .filter(f => !f.startsWith('^'))
      .join('|')
      .replace(/\s+/g, '\\s+');

    if (!wpUploadFolders) {
      return '';
    }

    return 'rewrite ' + upload_path + '(/(' + wpUploadFolders + ')/.*\\.\\w+)$ "/index.php?' + paramInUrl + '=$1" last;';
  }

  function _formatOpt(item) {
    const og = $(item.element).parent('optgroup').attr('id');
    return og + '/' + item.text;
  }

  function _handle_ref_url_ui() {
    _shared.vueApp = new Vue({
      el: '#pda-ref-app',
      data: {
        selectedFolder: '',
        selectedConfigOpt: '',
        selectedConfig: {
          folder: '',
          ref: {}
        },
        configs: pda_data_protect_folder.ref_urls_conf.map(function (conf) {
          conf.ref.values = !conf.ref.values ? '' : conf.ref.values.join('\n');
          return conf;
        }),
        errorMessage: '',
      },
      methods: {
        massageFolder(folder) {
          return folder.includes('^')
            ? folder.replace('^', 'root/')
            : pda_data_protect_folder.upload_path + '/' + folder;
        },
        onBlurRefUrl(evt) {
          evt.preventDefault();
          this.validateValues();
        },
        onInput(evt) {
          evt.preventDefault();
          console.log('Input');
          this.validateValues();
        },
        changeRefOpt(evt) {
          evt.preventDefault();
          if ('custom' === this.selectedConfig.ref.opt) {
            this.validateValues();
          } else {
            this.resetErrorState();
          }
        },
        changeFolder(evt) {
          evt.preventDefault();
          this.resetErrorState();
        },
        validateValues() {
          const values = Array.from(new Set(this.selectedConfig.ref.values.split(/\n/).filter(u => u !== '')));
          let inValid = true;
          if (values.length > 0) {
            inValid = values.some(function (val) {
              return !_isValidURL(val);
            });
          }

          if (inValid) {
            // use HTML to prevent the HTML flashed when first loading the page.
            $("#submit").prop("disabled", true);
            this.errorMessage = '<span class="errors">Please enter the valid links.</span>';
          } else {
            this.resetErrorState();
          }
        },
        resetErrorState() {
          $("#submit").prop("disabled", false);
          this.errorMessage = '';
        },
        resetSubmitError() {
          $('#pda-ref-app .form-errors').text()
          $('#pda-ref-app .form-errors').hide();
        }
      },
      watch: {
        selectedFolder: function () {
          const self = this;
          const config = this.configs.find(function (conf) {
            return self.selectedFolder === conf.folder;
          });
          if (!config) {
            return;
          }
          this.selectedConfig = config;
          this.selectedConfigOpt = config.ref.opt;
        },
      }
    })
  }

  function _set_ref_data_for_submit() {
    const configs = [];
    let inValidData = false;
    pda_data_protect_folder.ref_urls_conf.forEach(function (conf) {
      let clone = {
        folder: conf.folder,
        ref: {
          opt: conf.ref.opt,
          values: conf.ref.values,
        }
      };

      if ('custom' !== clone.ref.opt) {
        clone.ref.values = [];
        configs.push(clone);
        return;
      }
      // Clear URLs if it's not custom option.
      clone.ref.values = !clone.ref.values
        ? []
        : Array.from(new Set(clone.ref.values.split(/\n/).filter(function (s) {
          return !!s;
        })));
      if (0 === clone.ref.values.length) {
        inValidData = true;
        return;
      }
      if (clone.ref.values.some(function (val) {
          return !_isValidURL(val);
        })) {
        inValidData = true;
        return;
      }
      configs.push(clone);
    });

    return inValidData ? false : configs;
  }

  function _force_render_vue_app(data) {
    pda_data_protect_folder.ref_urls_conf = data.map(function (conf) {
      conf.ref.values = !conf.ref.values ? '' : conf.ref.values.join('\n');
      return conf;
    });
    _shared.vueApp.configs = pda_data_protect_folder.ref_urls_conf;
    const found = _shared.vueApp.configs.find(function (ref) {
      return ref.folder === _shared.vueApp.selectedConfig.folder;
    });
    if (found) {
      _shared.vueApp.selectedConfig = found;
    } else {
      _shared.vueApp.selectedFolder = '';
      _shared.vueApp.selectedConfig.folder = '';
      _shared.vueApp.selectedConfig.ref.opt = '';
      _shared.vueApp.selectedConfig.ref = {};
    }
  }

  /**
   * Clone from PPP
   * TODO: need to create the util for it.
   *
   * @param string url The URL to validate
   *
   * @returns {boolean}
   * @private
   */
  function _isValidURL(url) {
    const reUrl = new RegExp(
      "^" +
      // protocol identifier (optional)
      // short syntax // still required
      "(?:(?:(?:https?|ftp):)?\\/\\/)" +
      // user:pass BasicAuth (optional)
      "(?:\\S+(?::\\S*)?@)?" +
      "(?:" +
      // IP address exclusion
      // private & local networks
      "(?!(?:10|127)(?:\\.\\d{1,3}){3})" +
      "(?!(?:169\\.254|192\\.168)(?:\\.\\d{1,3}){2})" +
      "(?!172\\.(?:1[6-9]|2\\d|3[0-1])(?:\\.\\d{1,3}){2})" +
      // IP address dotted notation octets
      // excludes loopback network 0.0.0.0
      // excludes reserved space >= 224.0.0.0
      // excludes network & broacast addresses
      // (first & last IP address of each class)
      "(?:[1-9]\\d?|1\\d\\d|2[01]\\d|22[0-3])" +
      "(?:\\.(?:1?\\d{1,2}|2[0-4]\\d|25[0-5])){2}" +
      "(?:\\.(?:[1-9]\\d?|1\\d\\d|2[0-4]\\d|25[0-4]))" +
      "|" +
      // host & domain names, may end with dot
      // can be replaced by a shortest alternative
      // (?![-_])(?:[-\\w\\u00a1-\\uffff]{0,63}[^-_]\\.)+
      "(?:" +
      "(?:" +
      "[a-z0-9\\u00a1-\\uffff]" +
      "[a-z0-9\\u00a1-\\uffff_-]{0,62}" +
      ")?" +
      "[a-z0-9\\u00a1-\\uffff]\\." +
      ")+" +
      // TLD identifier name, may end with dot
      "(?:[a-z\\u00a1-\\uffff]{2,}\\.?)" +
      ")" +
      // port number (optional)
      "(?::\\d{2,5})?" +
      // resource path (optional)
      "(?:[/?#]\\S*)?" +
      "$", "i"
    );
    return reUrl.test(url);
  }

})(jQuery);
