<?php

class Wp_Pda_Ip_Block_Db {

	/**
	 * Create table
	 */
	public static function create_table() {
		global $wpdb;
		$jal_db_version = '1.0';
		$table_name     = $wpdb->prefix . Pda_Ip_Block_Constants::PDA_IP_BLOCK_TABLE_NAME;
		if ( $wpdb->get_var( "SHOW TABLES LIKE '$table_name'" ) != $table_name ) {
			$charset_collate = $wpdb->get_charset_collate();
			$sql             = "CREATE TABLE $table_name (
	    	ID mediumint(9) NOT NULL AUTO_INCREMENT,
	    	post_id mediumint(9) NOT NULL,	      
	    	ip_block varchar(100) DEFAULT '' NOT NULL,
	    	UNIQUE KEY id (id)
	    ) $charset_collate;";
			require_once ABSPATH . 'wp-admin/includes/upgrade.php';
			dbDelta( $sql );
			add_option( 'jal_db_version_ip_block', $jal_db_version );
		}
		self::update_tbl_ip_block_v1_1( $table_name, $wpdb );
		self::update_tbl_ip_block_v1_2( $table_name, $wpdb );
		self::update_tbl_ip_block_v1_3( $table_name, $wpdb );
	}

	/**
	 * Update table version 1.1
	 * Add user_roles column
	 *
	 * @param string   $table_name pda_ip_block.
	 * @param stdClass $wpdb       Wordpress Databases.
	 */
	private static function update_tbl_ip_block_v1_1( $table_name, $wpdb ) {
		$current_db_version = get_option( 'jal_db_version_ip_block' );
		if ( '1.0' === $current_db_version ) {
			$charset_collate = $wpdb->get_charset_collate();
			$sql             = "CREATE TABLE $table_name (
		    	  user_roles varchar(100) NULL DEFAULT '',
		        ) $charset_collate;";
			require_once ABSPATH . 'wp-admin/includes/upgrade.php';
			dbDelta( $sql );
			$db_version = '1.1';
			update_option( 'jal_db_version_ip_block', $db_version );
		}
	}

	/**
	 * Update table version 1.2
	 * Change data type user_roles column
	 *
	 * @param string   $table_name pda_ip_block.
	 * @param stdClass $wpdb       Wordpress Databases.
	 */
	private static function update_tbl_ip_block_v1_2( $table_name, $wpdb ) {
		$current_db_version = get_option( 'jal_db_version_ip_block' );
		if ( '1.1' === $current_db_version ) {
			$sql = "ALTER TABLE $table_name CHANGE user_roles user_roles text NULL DEFAULT ''";
			require_once ABSPATH . 'wp-admin/includes/upgrade.php';
			$wpdb->query( $sql );
			$db_version = '1.2';
			error_log( 'Ip Block updating db 1.2' );
			update_option( 'jal_db_version_ip_block', $db_version );
		}
	}

	/**
	 * Update table version 1.3
	 * Add user_access column
	 *
	 * @param string   $table_name pda_ip_block.
	 * @param stdClass $wpdb       Wordpress Databases.
	 */
	private static function update_tbl_ip_block_v1_3( $table_name, $wpdb ) {
		$current_db_version = get_option( 'jal_db_version_ip_block' );
		if ( '1.2' === $current_db_version ) {
			$charset_collate = $wpdb->get_charset_collate();
			$sql             = "CREATE TABLE $table_name (
		    	  user_access text NULL DEFAULT '',
		        ) $charset_collate;";
			require_once ABSPATH . 'wp-admin/includes/upgrade.php';
			dbDelta( $sql );
			$db_version = '1.3';
			update_option( 'jal_db_version_ip_block', $db_version );
		}
	}

}
