<?php

/**
 * Copy permission to post meta for pda gold when deactivate plugin IP Block
 */
function sync_file_access_permission_when_deactivated_plugin() {
	$repo = new Pda_Ip_Block_Repository();
	$data = $repo->get_all_data_in_table();
	foreach ( $data as $value ) {
		$user_role = unserialize( $value->user_roles );
		if ( $user_role['type'] !== 'custom-roles' ) {
			update_post_meta( $value->post_id, PDA_v3_Constants::$pda_meta_key_user_roles, $user_role['type'] );
		} else {
			update_post_meta( $value->post_id, PDA_v3_Constants::$pda_meta_key_user_roles, 'default' );
		}
	}
}

/**
 * Get data in setting type is array
 *
 * @param $name key in options
 *
 * @return array
 */
function get_data_protect_folder_type_is_array( $name ) {
	$settings = get_option( Pda_Ip_Block_Constants::PDA_OPTION_PROTECT_FOLDER );
	if ( $settings ) {
		$options = unserialize( $settings );
		$results = array_key_exists( $name, $options ) ? $options[ $name ] : '';
		if ( '' !== $results && is_array( $results ) ) {
			return $results;
		} else {
			return array();
		}
	}

	return array();
}

/**
 * Get data in setting type is string
 *
 * @param $name key in options
 *
 * @return string
 */
function get_data_protect_folder_type_is_string( $name ) {
	$settings = get_option( Pda_Ip_Block_Constants::PDA_OPTION_PROTECT_FOLDER );
	if ( $settings ) {
		$options = unserialize( $settings );
		if ( array_key_exists( $name, $options ) ) {
			return $options[ $name ];
		}
	}

	return '';
}

/**
 * Check file access for folder protect
 *
 * @return bool
 */
function check_fap_for_folder_protect() {
	$fap = get_data_protect_folder_type_is_string( Pda_Ip_Block_Constants::PDA_FAP_FOR_FOLDER_PROTECT );
	switch ( $fap ) {
		case 'custom_roles':
			return check_fap_case_custom_roles();
		case 'admin_users':
			if ( current_user_can( 'manage_options' ) ) {
				return true;
			}
			break;
		case 'logged_users':
			if ( is_user_logged_in() ) {
				return true;
			}
			break;
		default:
			return false;
	}
}

/**
 * Check file access permission case custom roles
 *
 * @return bool
 */
function check_fap_case_custom_roles() {
	$user_login     = wp_get_current_user()->roles;
	$pda_white_list = get_data_protect_folder_type_is_array( Pda_Ip_Block_Constants::PDA_WHITE_LIST_ROLE_FOLDER_PROTECT );
	if ( ! empty( array_intersect( $user_login, $pda_white_list ) ) ) {
		return true;
	} elseif ( is_super_admin( wp_get_current_user()->ID ) ) {
		if ( in_array( "administrator", $pda_white_list ) ) {
			return true;
		}
	}

	return false;
}

/**
 * Check special parameter in folder name
 *
 * @param $folder_name name of folder
 *
 * @return bool
 */
function check_folder_name_before_render( $folder_name ) {
	preg_match( '/^[A-Za-z0-9_\-\s]+$/', $folder_name, $matches, PREG_OFFSET_CAPTURE );
	if ( ! empty( $matches ) ) {
		return true;
	}

	return false;
}

/**
 * Change role of all page|post to default setting option
 */
function pda_ar_change_to_default_setting_for_protected_page_post() {
	$repo  = new Pda_Ip_Block_Repository();
	$posts = $repo->get_all_post_and_page();
	$roles = array( 'logged-users', 'custom-roles', 'blank', 'anyone' );
	foreach ( $posts as $post ) {
		$access_permission = get_post_meta( $post->ID, 'ppp_access_permission_private', true );
		if ( ! empty( $access_permission['type'] ) && in_array( $access_permission['type'], $roles ) ) {
			$access_permission = array(
				'type'             => 'default',
				'white_list_roles' => [],
			);
			update_post_meta( $post->ID, 'ppp_access_permission_private', $access_permission );
		}
	}
}

function pda_ip_block_check_role_protection() {
	if ( class_exists( 'Pda_Gold_Functions' ) ) {
		$gold_function = new Pda_Gold_Functions();
		if ( method_exists( $gold_function, 'pda_check_role_protection' ) ) {
			return $gold_function->pda_check_role_protection();
		}
	}

	return false;
}

function wp_pda_ip_block_component_folder() {
	return PDA_IP_BLOCK_PLUGIN_DIR_PATH . '__client__';
}

/**
 * Check condition to hide "Check for updates" button.
 *
 * @return bool
 * Return false: If all main plugins is deactivate
 * Else: return true
 */
function ip_block_main_plugins_is_deactivated() {
	$main_plugins = array( 'pda_gold', 'pap_gold', 'pda_v3', 'pap_gold_old' );
	foreach ( $main_plugins as $main_plugin ) {
		if ( - 1 === Yme_Plugin_Utils::is_plugin_activated( $main_plugin ) ) {
			return false;
		}
	}

	return true;
}

/**
 * This is the quick way to check whether the main plugin loaded and activated.
 * Notes: Used it after plugin_loaded hook fired only.
 *
 * @return bool
 *      True: PDA_GOLD_V3_VERSION or PPP_GOLD_VERSION is defined.
 */
function ip_block_have_main_plugin_version_const() {
	return defined( 'PDA_GOLD_V3_VERSION' ) || defined( 'PPP_GOLD_VERSION' );
}

/**
 * Get directory
 *
 * @param string $path dir path.
 *
 * @return false|resource
 */
function ip_block_get_directory( $path ) {
	if ( ! is_dir( $path ) ) {
		return false;
	}

	return opendir( $path );
}

/**
 * Get all folder in uploads dir ignore _pda folder
 *
 * @return array
 */
function ip_block_get_dir_under_uploads_path() {
	$base_dir  = wp_upload_dir()['basedir'];
	$directory = ip_block_get_directory( $base_dir );
	if ( false === $directory ) {
		return array();
	}
	$directories = array();
	while ( ( $file = readdir( $directory ) ) !== false ) {
		if ( ! check_folder_name_before_render( $file ) || '_pda' === $file || 'dir' !== filetype( $base_dir . '/' . $file ) ) {
			continue;
		}

		array_push( $directories, $file );
	}
	closedir( $directory );

	return $directories;
}

/**
 * Get all folder in uploads dir ignore default folder of WP
 *
 * @return array
 */
function ip_block_get_dir_under_home_path() {
	$home_path = get_home_path();
	$directory = ip_block_get_directory( $home_path );
	if ( false === $directory ) {
		return array();
	}
	$directories = array();
	while ( ( $file = readdir( $directory ) ) !== false ) {
		$is_wp_folder = strpos( $file, 'wp' ) === 0;
		if ( ! check_folder_name_before_render( $file ) || $is_wp_folder || 'dir' !== filetype( $home_path . '/' . $file ) ) {
			continue;
		}

		array_push( $directories, $file );
	}
	closedir( $directory );

	return $directories;
}

/**
 * Get WP upload folder without base URL: example wp-content/uploads
 *
 * @return string
 */
function ip_block_get_wp_upload_folder() {
	if ( ! class_exists( 'Pda_v3_Gold_Helper' ) ) {
		return '';
	}

	return str_replace( Pda_v3_Gold_Helper::get_home_url_with_ssl(), '', wp_upload_dir()['baseurl'] );
}

/**
 * List IPs allow access protected page/post
 *
 * @return array
 */
function ip_block_list_ip_allow_access_protected_post() {
	$ppp_access_restriction_option = get_option( Pda_Ip_Block_Constants::PPP_SETTING_IP_RESTRICTION_OPTION );
	if ( false === $ppp_access_restriction_option ) {
		return array();
	}

	if ( ! isset( $ppp_access_restriction_option[ Pda_Ip_Block_Constants::PPP_SETTING_LIST_IP ] ) || ! is_array( $ppp_access_restriction_option[ Pda_Ip_Block_Constants::PPP_SETTING_LIST_IP ] ) ) {
		return array();
	}

	return $ppp_access_restriction_option[ Pda_Ip_Block_Constants::PPP_SETTING_LIST_IP ];
}

/**
 * Get rel URL folder protection configurations.
 *
 * @return array
 */
function ip_block_get_ref_folder_protection_conf() {
	$ref_conf        = get_data_protect_folder_type_is_array( Pda_Ip_Block_Constants::PDA_WHITE_LIST_REF_URL_FOLDER_PROTECT );
	$ref_folders     = is_array( $ref_conf ) ? array_column( $ref_conf, 'folder' ) : array();
	$folder_selected = get_data_protect_folder_type_is_array( Pda_Ip_Block_Constants::PDA_PROTECT_ALL_FILE_IN_FOLDER );

	// Default configurations.
	return array_map(
		function ( $folder ) use ( $ref_folders, $ref_conf ) {
			$found = array_search( $folder, $ref_folders, true );
			$rel   = $found > - 1
				? $ref_conf[ $found ]['ref']
				: array(
					'opt'    => 'disable',
					'values' => [],
				);

			return array(
				'folder' => $folder,
				'ref'    => $rel,
			);
		},
		$folder_selected
	);
}

/**
 * Massage directory.
 *
 * @param string $dir Directory Path.
 *
 * @return string
 */
function ip_block_massage_directory( $dir ) {
	$dir = rtrim( $dir, '/' );
	$dir = ltrim( $dir, '/' );

	return $dir;
}

/**
 * Get all folder protections from PDA_FOLDER_PROTECTION const if it is exist.
 *
 * @return array
 */
function ip_block_maybe_get_all_folder_protections() {
	$results = array(
		'wp_upload' => [],
		'root'      => [],
	);
	if ( ! defined( 'PDA_FOLDER_PROTECTION' ) || ! is_array( PDA_FOLDER_PROTECTION ) ) {
		return $results;
	}

	$wp_upload_dirs = [];
	$root_dirs      = [];

	foreach ( PDA_FOLDER_PROTECTION as $folders ) {
		if ( ! isset( $folders['folder'] ) || ! is_string( $folders['folder'] ) || '' === $folders['folder'] ) {
			continue;
		}
		$folder_massaged = ip_block_massage_directory( $folders['folder'] );
		if ( ! empty( $folders['type'] ) && 'root' === $folders['type'] ) {
			$root_dirs[] = $folder_massaged;
		} else {
			$wp_upload_dirs[] = $folder_massaged;
		}
	}
	$results['wp_upload'] = $wp_upload_dirs;
	$results['root']      = $root_dirs;

	return $results;
}

/**
 * Check folder protection is defined and check user can view it.
 *
 * @param string $dir       Directory path.
 * @param string $file_path File path of URL.
 *
 * @return false|array False when variables is not defined
 */
function ip_block_maybe_get_folder_protection( $dir, $file_path ) {
	if ( ! defined( 'PDA_FOLDER_PROTECTION' ) || ! is_array( PDA_FOLDER_PROTECTION ) ) {
		return false;
	}

	$dir = ip_block_massage_directory( $dir );

	$folder_protection     = false;
	$sub_folder_protection = false;

	foreach ( PDA_FOLDER_PROTECTION as $rule ) {
		if ( empty( $rule['folder'] ) ) {
			continue;
		}

		// Check directory of file is contains in PDA_FOLDER_PROTECTION.
		$dir_rule = ip_block_massage_directory( $rule['folder'] );
		if ( $dir === $dir_rule ) {
			$folder_protection = $rule;

			break;
		}

		if ( ! $sub_folder_protection && false !== strpos( $dir, $dir_rule . '/' ) ) {
			$sub_folder_protection = $rule;
		}
	}

	/**
	 * $folder_protection is equal to defined folder.
	 * If $folder_protection is not exist then check folder protection is inside defined folder.
	 */
	$folder_protection = empty( $folder_protection ) ? $sub_folder_protection : $folder_protection;
	if ( empty( $folder_protection ) ) {
		return false;
	}

	$folder_protection['has_permission'] = true;
	if ( ! ip_block_has_access_to_folder( $folder_protection, $file_path ) ) {
		$folder_protection['has_permission'] = false;

		return $folder_protection;
	}

	return $folder_protection;
}


/**
 * Check permission
 *
 * @param array|boolean $folder_protection A folder protection.
 * @param string        $file_path       File path.
 *
 * @return boolean
 */
function ip_block_has_access_to_folder( $folder_protection, $file_path ) {
	if (  ! is_array( $folder_protection ) || empty( $folder_protection['permission'] ) ) {
		return false;
	}

	$has_permission = apply_filters('pda_ar_folder_protection_before_checking_access_permission', null, $folder_protection, $file_path);
	if ( is_bool( $has_permission ) ) {
		return $has_permission;
	}

	return ip_block_check_access_permission( $folder_protection['permission'] );
}

/**
 * Logic to check user can view file.
 *
 * @param array|string $permission Folder permission.
 *
 * @return bool
 */
function ip_block_check_access_permission( $permission ) {
	if ( 'logged_in_users' === $permission) {
		return is_user_logged_in();
	}

	if ( ! is_array( $permission ) ) {
		return false;
	}
	// User logged in then roles are available.
	if ( ! is_user_logged_in() ) {
		return false;
	}

	$current_user_roles = ip_block_get_current_user_roles();

	// Check user role is exist in config.
	$roles_intersection = array_intersect( $current_user_roles, $permission );

	return ! empty( $roles_intersection );
}

/**
 * Get current roles if user is login.
 *
 * @return array Current roles.
 */
function ip_block_get_current_user_roles() {
	if ( ! is_user_logged_in() ) {
		return array();
	}

	$user  = wp_get_current_user();
	$roles = ( array ) $user->roles;

	if ( is_multisite() && is_super_admin( $user->ID ) ) {
		$roles[] = 'administrator';
	}

	return $roles;
}
