<?php

if ( ! class_exists( 'Pda_Ip_Block_Repository' ) ) {
	class Pda_Ip_Block_Repository {

		/**
		 * Table name: pda_ip_block
		 *
		 * @var string
		 */
		private $table_name;

		/**
		 * Variable global wordpress databases
		 *
		 * @var object
		 */
		private $wpdb;

		/**
		 * Pda_Ip_Block_Repository constructor.
		 */
		public function __construct() {
			global $wpdb;
			$this->wpdb       = $wpdb;
			$this->table_name = $wpdb->prefix . Pda_Ip_Block_Constants::PDA_IP_BLOCK_TABLE_NAME;
		}

		/**
		 * Get post meta by meta key
		 *
		 * @param string $meta_key meta key.
		 *
		 * @return mixed
		 */
		public function get_post_meta_by_meta_key( $meta_key ) {
			$post_meta_table = $this->wpdb->postmeta;

			return $this->wpdb->get_results( $this->wpdb->prepare( "SELECT * FROM $post_meta_table WHERE meta_key = %s", $meta_key ) );
		}

		function get_data_in_pda() {
			if ( - 1 === Yme_Plugin_Utils::is_plugin_activated( 'pda_gold' ) || - 1 === Yme_Plugin_Utils::is_plugin_activated( 'pda_v3' ) ) {
				$data = $this->get_post_meta_by_meta_key( PDA_v3_Constants::$pda_meta_key_user_roles );

				foreach ( $data as $value ) {
					$old_data = $this->get_data_by_post_id( $value->post_id );
					if ( empty( $old_data ) ) {
						$this->wpdb->insert(
							$this->table_name,
							array(
								'post_id'    => $value->post_id,
								'user_roles' => serialize(
									array(
										'type'  => $value->meta_value,
										'roles' => '',
									)
								),
							)
						);
					} else {
						$this->wpdb->update(
							$this->table_name,
							array(
								'user_roles' => serialize(
									array(
										'type'  => $value->meta_value,
										'roles' => '',
									)
								),
							),
							array(
								'post_id' => $old_data->post_id,
							)
						);
					}
					delete_post_meta( $value->post_id, $value->meta_key );
				}
			}
		}

		/**
		 * Get data from IP Block table by Post ID.
		 *
		 * @param int|string $post_id The post ID.
		 *
		 * @return mixed
		 */
		function get_data_by_post_id( $post_id ) {
			$sql = $this->wpdb->prepare( "SELECT * FROM $this->table_name WHERE post_id = %d", absint( $post_id ) );

			return $this->wpdb->get_row( $sql );
		}

		function get_all_data_in_table() {
			return $this->wpdb->get_results( "SELECT * FROM $this->table_name" );
		}

		/**
		 * Get all post and page.
		 *
		 * @return mixed
		 */
		function get_all_post_and_page() {
			$postmeta_table = $this->wpdb->prefix . 'posts';
			$query_string   = "SELECT ID FROM $postmeta_table WHERE ( post_status = 'publish' OR post_status = 'draft' ) AND ( post_type ='post' OR post_type = 'page' )";
			$results        = $this->wpdb->get_results( $query_string );

			return $results;
		}

		/**
		 * Check condition and insert or update FAP
		 *
		 * @param array $data data on client.
		 */
		public function insert_fap_to_db( $data ) {
			$post_id    = $data['post_id'];
			$old_data   = $this->get_post_id_db( $post_id );
			$type       = $data['file_access_permision'];
			$roles      = $data['user_roles'];
			$user_roles = array(
				'type'  => $type,
				'roles' => $roles,
			);

			if ( empty( $old_data ) ) {
				$this->wpdb->insert(
					$this->table_name,
					array(
						'post_id'    => $post_id,
						'user_roles' => serialize( $user_roles ),
					)
				);
			} else {
				$this->wpdb->update(
					$this->table_name,
					array(
						'user_roles' => serialize( $user_roles ),
					),
					array(
						'post_id' => $post_id,
					)
				);
			}
		}

		/**
		 * Save File Access Permission to DB
		 * Check if isset old data then update else insert
		 *
		 * @param array $data data on client. Have key: post_id, file_access_permision, user_roles, type, users.
		 */
		public function set_fap_to_db( $data ) {
			if ( ! isset( $data['post_id'] ) || ! isset( $data['file_access_permision'] ) || ! isset( $data['user_roles'] ) || ! isset( $data['type'] ) || ! isset( $data['users'] ) ) {
				return;
			}

			$post_id     = $data['post_id'];
			$old_data    = $this->get_post_id_db( $post_id );
			$users       = 'custom-users' === $data['type']['value'] ? $this->massage_users_value( $data['users'] ) : '';
			$user_roles  = array(
				'type'  => $data['file_access_permision'],
				'roles' => $data['user_roles'],
			);
			$user_access = array(
				'type'  => $data['type'],
				'users' => $users,
			);

			if ( empty( $old_data ) ) {
				$this->wpdb->insert(
					$this->table_name,
					array(
						'post_id'     => $post_id,
						'user_roles'  => serialize( $user_roles ),
						'user_access' => serialize( $user_access ),
					)
				);
			} else {
				$this->wpdb->update(
					$this->table_name,
					array(
						'user_roles'  => serialize( $user_roles ),
						'user_access' => serialize( $user_access ),
					),
					array(
						'post_id' => $post_id,
					)
				);
			}
		}

		/**
		 * Massage user value before save to databases
		 * Remove deleted and duplicated users
		 * Input:   array(
		 *              array(
		 *                  label: abc,
		 *                  value: 1
		 *              ),
		 *              array(
		 *                  label: def,
		 *                  value: 2
		 *              ),
		 *              array(
		 *                  label: abc,
		 *                  value: 3
		 *              )
		 *          );
		 * Note: User label: abc, value: 1 is deleted
		 * Output: array(
		 *              array(
		 *                  label: def,
		 *                  value 2
		 *              ),
		 *              array(
		 *                  label: abc,
		 *                  value: 3
		 *              )
		 *          );
		 *
		 * @param array $users user on client request.
		 *
		 * @return array
		 */
		private function massage_users_value( $users ) {
			$users_name      = array_column( $users, 'label' );
			$users_duplicate = array_diff_assoc( $users_name, array_unique( $users_name ) );

			return array_values(
				array_filter(
					$users,
					function ( $user ) use ( $users_duplicate ) {
						if ( in_array( $user['label'], $users_duplicate, true ) ) {
							$user_id   = $user['value'];
							$user_info = get_userdata( $user_id );

							return false !== $user_info;
						} else {
							return $user;
						}
					}
				)
			);
		}

		/**
		 * Set permission for file is protected
		 *
		 * @param $attachment_id
		 * @param $fap
		 *
		 * @return bool
		 */
		public function set_fap_for_file_is_protected( $attachment_id, $fap ) {
			$old_data   = $this->get_post_id_db( $attachment_id );
			$user_roles = array(
				'type'  => $fap['type'],
				'roles' => implode( ';', $fap['roles'] ),
			);

			if ( empty( $old_data ) ) {
				$this->wpdb->insert(
					$this->table_name,
					array(
						'post_id'    => $attachment_id,
						'user_roles' => serialize( $user_roles ),
					)
				);
			} else {
				$this->wpdb->update(
					$this->table_name,
					array(
						'user_roles' => serialize( $user_roles ),
					),
					array(
						'post_id' => $attachment_id,
					)
				);
			}

			return true;
		}

		/**
		 * Get data in ip block table by post id
		 *
		 * @param $post_id
		 *
		 * @return mixed
		 */
		public function get_post_id_db( $post_id ) {
			$result = $this->wpdb->get_row( " SELECT * FROM $this->table_name WHERE post_id = $post_id " );

			return $result;
		}

		/**
		 * Get user info by user login or user email
		 *
		 * @param string $search_key key on client.
		 *
		 * @return mixed
		 */
		public function get_user_info_by_user_login( $search_key ) {
			return get_users(
				array(
					'search'         => '*' . $search_key . '*',
					'search_columns' => array(
						'user_login',
					),
					'blog_id'        => get_current_blog_id(),
				)
			);
		}

		/**
		 * Get user access by post id
		 *
		 * @param string $post_id post id.
		 *
		 * @return mixed
		 */
		public function get_user_access_by_post_id( $post_id ) {
			$query_string = $this->wpdb->prepare( "SELECT user_access FROM $this->table_name WHERE post_id LIKE %s", $post_id );
			$results      = $this->wpdb->get_row( $query_string );
			if ( is_null( $results ) ) {
				return Pda_Ip_Block_Constants::PDA_USERS_FAP_SAMPLE;
			}
			$user_access = @unserialize( $results->user_access );

			return $user_access ? $user_access : Pda_Ip_Block_Constants::PDA_USERS_FAP_SAMPLE;
		}

		/**
		 * Get roles and users info by post
		 *
		 * @param string $post_id post id.
		 *
		 * @return array
		 */
		public function get_fap_info_by_post_id( $post_id ) {
			$query_result = $this->wpdb->get_row( $this->wpdb->prepare( "SELECT * FROM $this->table_name WHERE post_id = %s", $post_id ) );
			if ( empty( $query_result ) ) {
				return array(
					'type'        => 'default',
					'roles'       => '',
					'user_access' => Pda_Ip_Block_Constants::PDA_USERS_FAP_SAMPLE,
				);
			}
			$user_access           = @unserialize( $query_result->user_access );
			$result['user_access'] = $user_access ? $user_access : Pda_Ip_Block_Constants::PDA_USERS_FAP_SAMPLE;
			$user_roles            = @unserialize( $query_result->user_roles );
			if ( $user_roles ) {
				$result = array_merge( $user_roles, $result );
			} else {
				$result['type']  = 'default';
				$result['roles'] = '';
			}

			return $result;
		}

		/**
		 * Get roles and users info by post
		 *
		 * @param string $post_ids Many post ID.
		 *
		 * @return array
		 */
		public function get_fap_info_by_post_ids( $post_ids ) {
			$results = $this->wpdb->get_results( "SELECT * FROM {$this->table_name} WHERE post_id IN ({$post_ids})" );
			if ( empty( $results ) ) {
				return array();
			}

			return $results;
		}

		/**
		 * Update IP FAP for PPP Gold plugin
		 *
		 * @param int|string $post_id The post ID.
		 * @param array      $ips     List IP allow access article.
		 */
		public function update_ip_fap( $post_id, $ips ) {
			$old_data = $this->get_post_id_db( $post_id );
			if ( empty( $old_data ) ) {
				$this->wpdb->insert(
					$this->table_name,
					array(
						'post_id'  => $post_id,
						'ip_block' => implode( ';', $ips ),
					)
				);
			} else {
				$this->wpdb->update(
					$this->table_name,
					array(
						'ip_block' => implode( ';', $ips ),
					),
					array(
						'post_id' => $post_id,
					)
				);
			}
		}

	}
}
