<?php

if ( class_exists( 'Pda_Ip_Block_Shortcode_Integration_Service' ) ) {
	return;
}

/**
 *
 * Class Pda_Ip_Block_Shortcode_Integration_Service
 */
class Pda_Ip_Block_Shortcode_Integration_Service {
	/**
	 * @var Pda_Ip_Block_Repository
	 */
	private $repository;

	/**
	 * @var Pda_Gold_Functions
	 */
	private $pda_gold_functions;

	/**
	 * Class instance
	 *
	 * @var Pda_Ip_Block_Shortcode_Integration_Service
	 */
	public static $instance;

	/**
	 * Pda_Ip_Block_Shortcode_Service constructor.
	 */
	public function __construct() {
		if ( ! class_exists( 'Pda_Gold_Functions' ) ) {
			return;
		}

		$this->repository         = new Pda_Ip_Block_Repository();
		$this->pda_gold_functions = new Pda_Gold_Functions();

		add_filter( 'pda_shortcode_get_accessible_files', array( $this, 'get_accessible_files' ), 10, 2 );
	}

	/**
	 * Get service instance.
	 *
	 * @return Pda_Ip_Block_Shortcode_Integration_Service
	 */
	public static function get_instance() {
		if ( is_null( self::$instance ) ) {
			// Use static instead of self due to the inheritance later.
			// For example: ChildSC extends this class, when we call get_instance
			// it will return the object of child class. On the other hand, self function
			// will return the object of base class.
			self::$instance = new self();
		}

		return self::$instance;
	}

	/**
	 * Get files which user can see.
	 *
	 * @param array $accessible_files List accessible files.
	 * @param array $params           Parameters from PDA Gold.
	 *
	 * @return array
	 */
	public function get_accessible_files( $accessible_files, $params ) {
		$protected_files = isset( $params['protected_files'] ) && is_array( $params['protected_files'] ) ? $params['protected_files'] : array();
		$post_ids        = array_map(
			function ( $file_protection ) {
				return absint( $file_protection['id'] );
			},
			$protected_files
		);

		// Get all protected post IDs which have role in PDA AR table.
		$post_ids_string     = implode( ',', $post_ids );
		$file_permissions    = array();
		if ( ! empty( $post_ids ) ) {
			$file_permissions = $this->repository->get_fap_info_by_post_ids( $post_ids_string );
		}

		// Get post IDs for compare purpose.
		$file_id_permissions = array();
		if ( ! empty( $file_permissions ) ) {
			$file_id_permissions = array_map(
				function ( $file_permission ) {
					return absint( $file_permission->post_id );
				},
				$file_permissions
			);
		}

		$accessible_files = [];
		foreach ( $protected_files as $protected_file ) {
			if ( $this->is_allowed_permission( $protected_file, $file_permissions, $file_id_permissions ) ) {
				$accessible_files[] = $protected_file;
			}
		}

		return $accessible_files;
	}

	/**
	 * Check user can access file.
	 *
	 * @param array $protected_file      Protected file.
	 * @param array $file_permissions    File permissions.
	 * @param array $file_id_permissions Filed id permissions.
	 *
	 * @return bool
	 */
	private function is_allowed_permission( $protected_file, $file_permissions, $file_id_permissions ) {
		if ( true === $this->pda_gold_functions->is_post_author( $protected_file['id'] ) ) {
			return true;
		}

		$found_idx       = array_search( $protected_file['id'], $file_id_permissions, true );
		$file_permission = false !== $found_idx ? $file_permissions[ $found_idx ] : array();
		$fap_type        = '';

		/**
		 * If can not find index:
		 *    - Check with global setting. (Default)
		 * Else:
		 *    - Check with FAP of IP Block. (Custom)
		 */
		$is_allowed_access = $this->pda_gold_functions->check_user_access_for_ip_block_addon(
			$protected_file['id'],
			$fap_type,
			$file_permission
		);

		return apply_filters(
			'pda_file_access_permission',
			$is_allowed_access,
			$protected_file['id']
		);
	}

}
