<?php

/**
 * The public-facing functionality of the plugin.
 *
 * @link       www.buildwps.com/prevent-direct-access-premium-plugin/?utm_source=user-website&utm_medium=pluginpage&utm_campaign=plugin-author-link
 * @since      1.0.0
 *
 * @package    Wp_Pda_Ip_Block
 * @subpackage Wp_Pda_Ip_Block/public
 */

/**
 * The public-facing functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the public-facing stylesheet and JavaScript.
 *
 * @package    Wp_Pda_Ip_Block
 * @subpackage Wp_Pda_Ip_Block/public
 * @author     BWPS <hello@ymese.com>
 */
class Wp_Pda_Ip_Block_Public {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string $plugin_name The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string $version The current version of this plugin.
	 */
	private $version;

	/**
	 * Service class.
	 *
	 * @var stdClass Pda_Ip_Block_Services Service class.
	 */
	private $services;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @param string $plugin_name The name of the plugin.
	 * @param string $version     The version of this plugin.
	 *
	 * @since    1.0.0
	 */
	public function __construct( $plugin_name, $version ) {
		$this->plugin_name = $plugin_name;
		$this->version     = $version;
		$this->services    = new Pda_Ip_Block_Services();
	}

	/**
	 * Register the stylesheets for the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_styles() {
		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in Wp_Pda_Ip_Block_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The Wp_Pda_Ip_Block_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */
	}

	/**
	 * Register the JavaScript for the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_scripts() {
		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in Wp_Pda_Ip_Block_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The Wp_Pda_Ip_Block_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */
	}

	/**
	 * Handle File Access Permission for PPP Gold plugin
	 *
	 * @param bool       $default_fap File Access Permission default in PPP Gold or other plugin handled before.
	 * @param int|string $post_id     The post id.
	 *
	 * @return bool
	 */
	public function ppp_handle_fap_follow_ip( $default_fap, $post_id ) {
		return $this->services->ppp_handle_fap_follow_ip( $default_fap, $post_id );
	}

	/**
	 * Plugins loaded.
	 */
	public function handle_plugins_loaded() {
		Pda_Ip_Block_Shortcode_Integration_Service::get_instance();
	}

}
