<?php

/**
 * The plugin bootstrap file
 *
 * This file is read by WordPress to generate the plugin information in the plugin
 * admin area. This file also includes all of the dependencies used by the plugin,
 * registers the activation and deactivation functions, and defines a function
 * that starts the plugin.
 *
 * @link              https://preventdirectaccess.com/extensions/wordpress-restrict-page-file-access/
 * @since             1.3.1
 * @package           Wp_Pda_Ip_Block
 *
 * @wordpress-plugin
 * Plugin Name:       PDA Access Restriction
 * Plugin URI:        https://preventdirectaccess.com/extensions/wordpress-restrict-page-file-access/?utm_source=user-website&utm_medium=notification&utm_campaign=plugins-visitpluginsite-link
 * Description:       Restrict access to your protected files by IP addresses or specific user roles on each and every file
 * Version:           1.3.1
 * Author:            BWPS
 * Author URI:        https://preventdirectaccess.com/extensions/wordpress-restrict-page-file-access/?utm_source=user-website&utm_medium=pluginpage&utm_campaign=plugin-author-link
 * License:           GPL-2.0+
 * License URI:       http://www.gnu.org/licenses/gpl-2.0.txt
 * Text Domain:       wp-pda-ip-block
 * Domain Path:       /languages
 */

// If this file is called directly, abort.
if ( ! defined( 'WPINC' ) ) {
	die;
}

/**
 * Currently plugin version.
 * Start at version 1.0.0 and use SemVer - https://semver.org
 * Rename this for your plugin and update it as you release new versions.
 */
define( 'PDA_IP_BLOCK_VERSION', '1.3.1' );
define( 'PDA_IP_BLOCK_PLUGIN_DIR_PATH', plugin_dir_path( __FILE__ ) );
if ( ! defined( 'PDA_IP_BLOCK_DIR_URL' ) ) {
	define( 'PDA_IP_BLOCK_DIR_URL', plugin_dir_url( __FILE__ ) );
}
if ( ! defined( 'PDA_IP_BLOCK_NAME' ) ) {
	define( 'PDA_IP_BLOCK_NAME', 'PDA Access Restriction' );
}
if ( ! defined( 'PDA_IP_BLOCK_PLUGIN_FOLDER' ) ) {
	define( 'PDA_IP_BLOCK_PLUGIN_FOLDER', 'wp-pda-ip-block/wp-pda-ip-block.php' );
}
/**
 * The code that runs during plugin activation.
 * This action is documented in includes/class-wp-pda-ip-block-activator.php
 */
function activate_wp_pda_ip_block() {
	require_once plugin_dir_path( __FILE__ ) . 'includes/class-wp-pda-ip-block-activator.php';
	Wp_Pda_Ip_Block_Activator::activate( plugin_basename( __FILE__ ) );
}

/**
 * The code that runs during plugin deactivation.
 * This action is documented in includes/class-wp-pda-ip-block-deactivator.php
 */
function deactivate_wp_pda_ip_block() {
	require_once plugin_dir_path( __FILE__ ) . 'includes/class-wp-pda-ip-block-deactivator.php';
	Wp_Pda_Ip_Block_Deactivator::deactivate();
}

register_activation_hook( __FILE__, 'activate_wp_pda_ip_block' );
register_deactivation_hook( __FILE__, 'deactivate_wp_pda_ip_block' );

/**
 * The core plugin class that is used to define internationalization,
 * admin-specific hooks, and public-facing site hooks.
 */
require plugin_dir_path( __FILE__ ) . 'includes/class-wp-pda-ip-block.php';

/**
 * Remove PDA Access Restriction plugin in list plugins update of WP.
 *
 * @param object $value Data update cache.
 *
 * @return object $value Data update cache.
 */
function ip_block_filter_plugin_updates( $value ) {
	if ( ! isset( $value->response[ PDA_IP_BLOCK_PLUGIN_FOLDER ] ) ) {
		return $value;
	}
	unset( $value->response[ PDA_IP_BLOCK_PLUGIN_FOLDER ] );

	return $value;
}

/**
 * Begins execution of the plugin.
 *
 * Since everything within the plugin is registered via hooks,
 * then kicking off the plugin from this point in the file does
 * not affect the page life cycle.
 *
 * @since    1.0.0
 */
function run_wp_pda_ip_block() {
	$plugin = new Wp_Pda_Ip_Block();
	$plugin->run();
	if ( ! ip_block_main_plugins_is_deactivated() ) {
		$configs = require PDA_IP_BLOCK_PLUGIN_DIR_PATH . '/includes/class-wp-pda-ip-configs.php';
		if ( method_exists( 'Puc_v4p8_Factory', 'buildUpdateChecker' ) ) {
			Puc_v4p8_Factory::buildUpdateChecker(
				$configs->update_url,
				__FILE__,
				'pda-ip-block'
			);
		} elseif ( method_exists( 'Puc_v4_Factory', 'buildUpdateChecker' ) ) {
			Puc_v4_Factory::buildUpdateChecker(
				$configs->update_url,
				__FILE__,
				'pda-ip-block'
			);
		} else {
			add_filter( 'site_transient_update_plugins', 'ip_block_filter_plugin_updates' );
		}
	} else {
		add_filter( 'site_transient_update_plugins', 'ip_block_filter_plugin_updates' );
	}
}

run_wp_pda_ip_block();
