<?php

/**
 * The admin-specific functionality of the plugin.
 *
 * @link       https://preventdirectaccess.com/
 * @since      1.0.0
 *
 * @package    Prevent_Direct_Access_By_S3
 * @subpackage Prevent_Direct_Access_By_S3/admin
 */

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    Prevent_Direct_Access_By_S3
 * @subpackage Prevent_Direct_Access_By_S3/admin
 * @author     BWPS <hello@preventdirectaccess.com>
 */
class Prevent_Direct_Access_By_S3_Admin {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;


    /**
     * @var DB Repository
     */
	private $repository;

	private $aws_helper;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string    $plugin_name       The name of this plugin.
	 * @param      string    $version    The version of this plugin.
	 */
	public function __construct( $plugin_name, $version ) {

		$this->plugin_name = $plugin_name;
		$this->version = $version;
		$this->repository = new PDA_S3_Repository();
        $this->aws_helper = new Aws_Helper();

	}

	/**
	 * Register the stylesheets for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_styles() {

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in Prevent_Direct_Access_By_S3_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The Prevent_Direct_Access_By_S3_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */

		wp_enqueue_style( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'css/prevent-direct-access-by-s3-admin.css', array(), $this->version, 'all' );
        wp_enqueue_style( $this->plugin_name . 'css-bundle', plugin_dir_url( __FILE__ ) . '../assets/dist/style/wp_pda_s3_bundle.css', array(), $this->version, 'all' );
        wp_enqueue_style('select2_css', 'https://cdnjs.cloudflare.com/ajax/libs/select2/4.0.3/css/select2.min.css', null, "4.0.3");
        wp_register_style('jquery_tagsinput_min_css' . $this->plugin_name, plugin_dir_url( __FILE__ ) . './css/tagsinput.css', array(), $this->version, 'all');
        wp_enqueue_style('jquery_tagsinput_min_css' . $this->plugin_name);

        wp_register_style('css_jquery_ui'. $this->plugin_name, plugin_dir_url(__FILE__) . ('./css/jquery-ui.min.css'), array(), $this->version, 'all');
        wp_enqueue_style('css_jquery_ui'. $this->plugin_name);
	}

	/**
	 * Register the JavaScript for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_scripts() {

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in Prevent_Direct_Access_By_S3_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The Prevent_Direct_Access_By_S3_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */
		wp_enqueue_script( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'js/prevent-direct-access-by-s3-admin.js', array( 'jquery' ), $this->version, false );
		wp_enqueue_script( $this->plugin_name . 'js_bundle', plugin_dir_url( __FILE__ ) . '../assets/dist/wp_pda_s3_bundle.js', array(), $this->version, 'all' );
        wp_localize_script( $this->plugin_name . 'js_bundle', 'wps3AjaxObj', array( 'ajaxurl' => admin_url('admin-ajax.php') ) );
        wp_enqueue_script('select2', 'https://cdnjs.cloudflare.com/ajax/libs/select2/4.0.3/js/select2.min.js',array('jquery'), '4.0.3', true);
        if( !wp_script_is( 'jquery_tagsinput_min' ) ) {
            wp_enqueue_script('jquery_tagsinput_min' .$this->plugin_name, plugin_dir_url(__FILE__). './js/jquery.tagsinput.min.js', array(), $this->version);
        }

        if( !wp_script_is( 'jquery_ui_min' ) ) {
            wp_enqueue_script('jquery_ui_min'.$this->plugin_name, plugin_dir_url(__FILE__). './js/jquery-ui.min.js', array(), $this->version);
        }
    }

    public function wpps3_add_upload_columns( $columns ) {
	    $columns['wps3_custom_column'] = '<label>' . __("Prevent Direct Access S3", "prevent-direct-access-by-s3") . '</label>';
	    return $columns;
    }

    /**
     * @param $column_name
     * @param $post_id
     */
	public function wpps3_media_custom_column( $column_name, $post_id ) {
	    if($column_name == 'wps3_custom_column') {
            WPS3_ViewLoader::render_custom_column( $post_id );
        }
    }

    public function wps3_rest_api_init_cb() {
	    $api = new WPS3_API;
	    $api->register_rest_routes();
    }

    public function wps3_add_menu_setting() {
        require_once plugin_dir_path( dirname( __FILE__ )  ) . '/includes/class-prevent-direct-access-by-s3-setting.php';
        $setting_page = new PDA_WPS3_Settings();
        $menu_page_name = add_menu_page('PDA S3 Setting', 'Prevent Direct Access S3', 'administrator', 'wp_pdas3_options', array($setting_page, 'render_ui'), 'dashicons-unlock');
        add_action( 'admin_print_styles-' . $menu_page_name, array( $this, 'pdas3_setting_assets' ) );
    }

    public function pdas3_setting_assets() {
        
        wp_enqueue_style('pda_s3_toastr_css' . $this->plugin_name, 'https://cdnjs.cloudflare.com/ajax/libs/toastr.js/latest/toastr.min.css');
        wp_register_style('pda_s3_setting_' . $this->plugin_name, plugin_dir_url( __FILE__ ) . './css/pda_s3_setting.css', array(), $this->version, 'all' );
        wp_enqueue_style('pda_s3_setting_' . $this->plugin_name);

        wp_enqueue_script('pda_s3_toastr_' . $this->plugin_name, 'https://cdnjs.cloudflare.com/ajax/libs/toastr.js/latest/toastr.min.js', $this->version);
        wp_enqueue_script('pda_s3_setting_general_js' . $this->plugin_name, plugin_dir_url(__FILE__) . './js/prevent-direct-access-by-s3-setting-general.js', array(), $this->version);
        wp_localize_script('pda_s3_setting_general_js' . $this->plugin_name, 'pda_s3_setting_data', array('ajax_url' => admin_url('admin-ajax.php')));

        wp_enqueue_script('prevent-direct-access-by-s3-license' . $this->plugin_name, plugin_dir_url(__FILE__) . './js/prevent-direct-access-by-s3-license.js', array(), $this->version);
        wp_localize_script('pda_s3_setting_general_js' . $this->plugin_name, 'pda_s3_settings_general_data', array('ajax_url' => admin_url('admin-ajax.php')));
        wp_localize_script('prevent-direct-access-by-s3-license' . $this->plugin_name, 'prevent_direct_access_by_s3_license_data', array('ajax_url' => admin_url('admin-ajax.php')));

        wp_enqueue_script('search_gold_js' . $this->plugin_name, plugin_dir_url(__FILE__). './js/pdas3_setting_search.js',  array(), $this->version);
        wp_localize_script('search_gold_js'. $this->plugin_name, 'server_data', array ('ajax_url' => admin_url ('admin-ajax.php')));

        wp_enqueue_script('pda_gold_ip_block' . $this->plugin_name, plugin_dir_url(__FILE__). './js/pdas3_iplock.js', array(), $this->version);
        wp_localize_script( 'pda_gold_ip_block'. $this->plugin_name, 'ip_block_server_data', array ( 'ajaxurl' => admin_url ('admin-ajax.php' )));
        
        wp_enqueue_script('pdas3_migrate_js' . $this->plugin_name, plugin_dir_url(__FILE__). './js/pdas3_migrate.js', array(), $this->version);
        wp_localize_script( 'pdas3_migrate_js'. $this->plugin_name, 'migrate_server_data', array ( 'home_url' => home_url()));
    }

    public function pda_s3_update_general_settings() {
        $nonce = $_REQUEST['security_check'];
        if ( ! wp_verify_nonce( $nonce, 'pda_ajax_nonce_s3' ) ) {
            error_log('not verify nonce', 0);
            wp_die( 'invalid_nonce' );
        }
        $settings = $_REQUEST['settings'];

        $keys = [];
        foreach ($settings as $key => $value) {
            array_push($keys, $key);
        }

        $options_s3_db = unserialize(get_option('pdas3_options'));

        foreach ($keys as $k) {
            $options_s3_db[$k] = $settings[$k];
        }

        $pda_s3 = serialize($options_s3_db);

        update_option( 'pdas3_options', $pda_s3 );
        wp_send_json( $settings );
        wp_die();
    }

    public function wps3_customize_htaccess_content( $rules ) {
        $links = $this->repository->fetch_s3_info();
        $htaccess_utils = new WPS3_Htaccess;
        $newRules = $htaccess_utils->generate_rules($links);
        return $newRules . PHP_EOL . $this->wps3_customize_htaccess_pretty_url() . PHP_EOL . $rules;
    }

    public function wps3_customize_htaccess_pretty_url() {
        $files =  $this->repository->get_pretty_url();
        $pretty_url = array_map(function($p_url){
            $pretty =  $p_url->pretty_url;
            $signed_url = html_entity_decode(urldecode($p_url->signed_url));
            return "RewriteRule pda-s3/$pretty$ $signed_url [R=301,L]";
        },$files);

        $newRules = array_reduce($pretty_url, function($link, $item) {
            return  $link . PHP_EOL . $item . PHP_EOL;
        });

        return $newRules;
    }

    public function wps3_htaccess_updated() {
	    $writable = Yme_Htaccess::htaccess_writable();
	    if($writable === true) {
	        flush_rewrite_rules();
        }
        $wizard = new PDA_S3_WIZARD();
	    $wizard->redirect_to_wizard();
    }

    public function wps3_admin_notices()
    {
        global $pagenow;

        if ($pagenow == 'plugins.php' || $pagenow == 'upload.php') {
            $writable = Yme_Htaccess::htaccess_writable();
            if($writable !== true) {
                WPS3_ViewLoader::renderHtaccessWarning();
            }
            $licenseKey = Yme_Plugin::getLicenseKey('pda-s3');
            if ( empty($licenseKey) ) {
                ?>
                <div class="error is-dismissible notice">
                    <p><b><?php $this->plugin_name ?></b> Please enter the license key that you received after purchasing our plugin under <b>Prevent Direct Access S3</b> tab. Otherwise, the plugin's premium features are not activated. </p>
                </div>
                <?php
            }
        }
    }

    public function delete_private_link_s3($post_id) {
	    $repository = new PDA_S3_Repository;
	    $result = $repository->delete_private_link_by_post_id($post_id);

        $aws_helper = new Aws_Helper;
        $file_path = get_attached_file($post_id);
        $aws_helper->deleteS3File($file_path);

        return $result;
    }

    public function set_custom_cookie() {
	    if( is_user_logged_in() && current_user_can('edit_posts') ) {
	        setcookie('wppdas3_can_edit_post', true, strtotime('+1 day'), COOKIEPATH, COOKIE_DOMAIN);
        }
    }

    public function unset_custom_cookie() {
	    unset($_COOKIE['wppdas3_can_edit_post']);
	    setcookie('wppdas3_can_edit_post', true,time() - ( 15 * 60 ) );
    }

    public function pda_s3_license_checking() {
        $nonce = $_REQUEST['security_check'];
        if ( ! wp_verify_nonce( $nonce, 'pda_ajax_nonce_s3' ) ) {
            error_log('not verify nonce', 0);
            wp_die( 'invalid_nonce' );
        }
        $license = $_REQUEST['license'];
        $result = YME_LICENSE::checkLicense($license, 'pda-s3');
        $data = $result['data'];
        if( $data->isError || property_exists($data, 'errorMessage') ) {
            update_option(PdaS3Constants::LICENSE_OPTIONS, false);
            update_option(PdaS3Constants::LICENSE_ERROR, $data->errorMessage);
            update_option(PdaS3Constants::LICENSE_KEY, '');
        } else {
            update_option(PdaS3Constants::LICENSE_KEY, $license);
            update_option(PdaS3Constants::LICENSE_OPTIONS, true);
            update_option(PdaS3Constants::LICENSE_ERROR, '');
        }
        wp_send_json( $result );
        wp_die();
    }

    public function pda_s3_update_ip_block() {
        $nonce = $_REQUEST['security_check'];
        if ( ! wp_verify_nonce( $nonce, 'pda_ajax_nonce_ip_block' ) ) {
            error_log('not verify nonce', 0);
            wp_die( 'invalid_nonce' );
        }
        $settings = $_REQUEST['settings_ip'];
        $keys = [];
        foreach ($settings as $key => $value) {
            array_push($keys, $key);
        }

        $options_s3_db = unserialize(get_option('pdas3_ip_block'));

        foreach ($keys as $k) {
            $options_s3_db[$k] = $settings[$k];
        }

        $pda_s3 = serialize($options_s3_db);
        update_option( 'pdas3_ip_block', $pda_s3);
        wp_send_json( true );
        wp_die();
    }

    public function auto_synced_new_file_to_s3($post_id) {
        $utils = new WPS3_UTILS;
        $settings = get_option('pdas3_options');
        $auto_synced = false;
        if($settings) {
            $options = (!$settings) ? array() : unserialize($settings);
            if(array_key_exists('auto_synced', $options)) {
                $auto_synced = $options['auto_synced'];
            }
        }
        if($auto_synced == "true") {
            return $utils->sync_file_to_S3($post_id);
        }
    }

    public function pda_s3_test_configuration() {
        $nonce = $_REQUEST['security_check'];
        if ( ! wp_verify_nonce( $nonce, 'pda_ajax_nonce_s3' ) ) {
            error_log('not verify nonce', 0);
            wp_die( 'invalid_nonce' );
        }
	    $settings = $_REQUEST['settings'];
        $isOk = $this->aws_helper->testConfiguration(array(
            'aws_key' => $settings["aws_key"],
            'aws_secret' => $settings["aws_secret"],
            'bucket' => $settings["bucket"],
            'prefix_key' => $settings["prefix_key"],
            'region' => $settings["region"],
            'is_aws_default' => $settings["is_aws_default"]
        ));
	    wp_send_json($isOk);
	    wp_die();
    }

    public function admin_menu_handler() {
        add_dashboard_page('', '', 'manage_options', 'pdas3-wizard-setup', '');
    }

    public function setup_wizard() {

        if ( empty( $_GET['page'] ) || PdaS3Constants::WIZARD_PAGES['main_page'] !== $_GET['page'] ) { // WPCS: CSRF ok, input var ok.
            return;
        }
        $wizard = new PDA_S3_WIZARD;
        ob_start();
        $wizard->setup();
        $wizard->setup_header();
        $wizard->setup_steps();
        $wizard->setup_content();
        $wizard->setup_wizard_footer();
        exit();

    }

    function pda_s3_cron_interval($schedules) {
        $interval = 60*60*24;
        $schedules['pda_s3_check_license_expired'] = array(
            'interval' => $interval,
            'display' => esc_html__('PDA s3: Check license expired once a day'),
        );
        return $schedules;
    }

    function pda_s3_cron_le_exec() {
        $license = get_option(PdaS3Constants::LICENSE_KEY);
        YME_LICENSE::checkExpiredLicense($license);
    }

}
