<?php
/**
 * Created by PhpStorm.
 * User: gaupoit
 * Date: 4/12/18
 * Time: 16:17
 */

if (!defined('ABSPATH')) die('You do not have sufficient permissions to access this file.');

if ( !class_exists('WPS3_API') ) {

    class WPS3_API {

        private $aws_helper;

        private $repository;

        private $check_size;


        function __construct()
        {
            $this->aws_helper = new Aws_Helper();
            $this->repository = new PDA_S3_Repository();
            $this->check_size = new WPS3_UTILS();
        }

        function register_rest_routes() {

            register_rest_route('wps3/v1', '/s3/(?P<id>\d+)', array(
                'methods' => 'GET',
                'callback' => array($this, 'get_syncS3')
            ));

            register_rest_route('wps3/v1', '/s3/(?P<id>\d+)', array(
                'methods' => 'POST',
                'callback' => array($this, 'syncS3')
            ));

            register_rest_route('wps3/v1', '/s3/(?P<id>\d+)', array(
                'methods' => 'PUT',
                'callback' => array($this, 'changeACL')
            ));

            register_rest_route('wps3/v1', '/private-links/(?P<id>\d+)', array(
                'methods' => 'POST',
                'callback' => array($this, 'insert_new_private_links')
            ));

            register_rest_route('wps3/v1', '/private-links/(?P<id>\d+)', array(
                'methods' => 'GET',
                'callback' => array($this, 'get_private_link')
            ));

            register_rest_route('wps3/v1', '/private-links/(?P<id>\d+)', array(
                'methods' => 'DELETE',
                'callback' => array($this, 'delete_private_link')
            ));

            register_rest_route('wps3/v1', '/s3-info/(?P<id>\d+)', array(
               'methods' => 'GET',
               'callback' => array($this, 'get_s3_info')
            ));

            register_rest_route('wps3/v1', '/check-is-public/(?P<id>\d+)', array(
                'methods' => 'GET',
                'callback' => array($this, 'check_is_public')
            ));

            register_rest_route('wps3/v1', '/delete-links-s3/(?P<id>\d+)', array(
                'methods' => 'DELETE',
                'callback' => array($this, 'delete_link_s3')
            ));

            register_rest_route('wps3/v1', '/custom-link/', array(
               'methods' => 'POST',
                'callback' => array($this, 'check_custom_link')
            ));

            register_rest_route('wps3/v1', '/migrate_data/', array(
                'methods' => 'GET',
                 'callback' => array($this, 'handle_migration')
             ));

             register_rest_route( 'wps3/v1', '/update/set-default-private-url/(?P<id>\d+)', array(
                'methods' => 'POST',
                'callback' => array($this, 'setDefaultPrivateUrl'),
             ) );

        }

        function delete_private_link($data) {
            $this->repository->delete_private_link_by_id($data['id']);
        }

        function get_private_link($data) {
            $writable = Yme_Htaccess::htaccess_writable();
            if($writable === true) {
                flush_rewrite_rules( true );
            }
             $links = $this->repository->get_private_link_by_post_id($data['id']);

             return array_map(function($link) {
                 if( isset( $link->pretty_url) ) {
//                     $link->available = $this->check_private_link_in_rules($link->pretty_url);
                     $link->pretty_url = get_site_url() . '/' . PdaS3Constants::PDAS3_PREFIX_PRETTY_URL . "/$link->pretty_url";
//                     $link->available = $this->check_url($link->pretty_url);
                 }
                $link->signed_url = html_entity_decode($link->signed_url);
                 return $link;
             }, $links);
        }

        function insert_new_private_links($data) {
            if( is_null($data['expiredTimeValue']) ) {
                $settings = get_option( PdaS3Constants::PDAS3_SETTINGS_NAME );
                $time_die = PdaS3Constants::PDAS3_DEFAULT_EXPIRED_TIME;
                if ($settings) {
                    $options = unserialize($settings);
                    if(array_key_exists('time_die_private_link', $options)) {
                        $time_die = $options['time_die_private_link'];
                    }
                }
            } else {
                $time_die = $data['expiredTimeValue'] . ' ' . $data['expiredTimeUnit'];
            }

            $pretty_url = $data['pretty_url'];
            $signed_url = $this->presignedFile($data['id'], $time_die);
            $expired_time_stamp = new DateTime();
            $expired_time_stamp->modify("+" . $time_die);
            $timestamp = $expired_time_stamp->getTimestamp();
            $is_default = $this->repository->is_default_private_link($data['id']);
            $data = [
                'post_id' => $data['id'],
                'signed_url' => $signed_url,
                'expired_time_stamp' => $timestamp,
                'is_default' => $is_default,
                'pretty_url' => $pretty_url,
            ];
             $result = $this->repository->createCustomPrivateUrl($data);
             if($result === false) {
                 return $result;
             }
             return $data;
        }

        public function syncS3( $data ) {
            $utils = new WPS3_UTILS;
            return $utils->sync_file_to_S3($data['id']);
        }

        function delete_link_s3($data) {
            $filePath = get_attached_file($data['id']);
            $repository = new PDA_S3_Repository;
            $result = $repository->delete_private_link_by_post_id($data['id']);
            $this->aws_helper->deleteS3File($filePath);
            return $result;
        }

        function get_syncS3($data) {
            $filePath = get_attached_file($data['id']);
            if( is_file( $filePath ) ) {
                if( $this->check_size->check_file_size( $filePath ) ) {
                    return $this->aws_helper->isFileExisted($filePath);
                }
            }
            return false;
        }

        function changeACL($data) {
            $filePath = get_attached_file($data['id']);
            $this->aws_helper->updateObjACL($filePath, $data['isPublic']);
            return $this->repository->update_public_s3_info($data['id'], array(
                'is_public' => $data['isPublic']
            ));
        }

        private function presignedFile( $id, $time_die )
        {
            $filePath = get_attached_file( $id );
            if (is_file($filePath)) {
                if ($this->check_size->check_file_size($filePath)) {
                    $aws_helper = new Aws_Helper();
                    return $aws_helper->createPresignedUrl($filePath, $time_die);
                }
            }
        }

        public function get_s3_info( $data ) {
            return $this->repository->get_s3_info( $data['id'] );
        }

        public function check_is_public ($data) {
            return $this->repository->get_advance_file_s3_info($data['id']);
        }

        public function check_custom_link($data) {
            $uri = $data['uri'];
            $url = get_site_url() . '/' . PdaS3Constants::PDAS3_PREFIX_PRETTY_URL . "/$uri";
            return $this->check_url($url);
        }

        private function check_url($url) {
            $headers = get_headers($url);
            $headers = (is_array($headers)) ? implode( "\n ", $headers) : $headers;

            return (bool)preg_match('#^HTTP/.*\s+[(200|301|302)]+\s#i', $headers);
        }

        private function check_private_link_in_rules($uri) {
            global $wp_rewrite;
            $rules = $wp_rewrite->mod_rewrite_rules();
            return (bool)preg_match("/($uri)/", $rules);
        }

        public function handle_migration() {
            $data = $this->repository->get_postid_pda();
            foreach ($data as $value) {
                $postid = $value->post_id; 
                $this->check_size->sync_file_to_S3($postid);
            }
            deactivate_plugins("prevent-direct-access-gold/prevent-direct-access.php");
        }

        function setDefaultPrivateUrl ( $data ) {
            $private_file = $this->repository->get_advance_file_by_id( $data['id'] );
            if ( isset( $private_file ) ) {
                $default_file = $this->repository->get_default_advance_file_by_host_id( $private_file->post_id );
                if( isset($default_file) ) {
                    $this->repository->unset_default_advance_file( $default_file->ID );
                }
                $this->repository->update_advance_file_by_id( $data['id'], array('is_default' => true));
            } else {
                return false;
            }
        }
    }
}