<?php
/**
 * Created by PhpStorm.
 * User: gaupoit
 * Date: 4/12/18
 * Time: 09:19
 */

use Aws\S3\S3Client;

class Aws_Helper {

    protected $s3;

    protected $options;

    public function __construct()
    {
        $settings = new PDA_S3_Settings_Util;
        $this->options = $settings->getAwsConfigs();

        if (!empty($this->options)) {
            $this->s3 = new S3Client([
                'version' => 'latest',
                'region' => $this->options['region'],
                'credentials' => [
                    'key' => $this->options['aws_key'],
                    'secret' => $this->options['aws_secret']
                ]
            ]);
        }
    }

    public function getFiles() {
        return $this->s3->listObjects(array(
            'Bucket' => 'pur-free-asia'
        ));
    }

    /**
     * @param $filePath File path
     * @return mixed|null|string
     * empty string if user have never did the aws configuration
     */
    public function putFile( $filePath, $isPublic ) {

        if( is_null( $this->s3) ) {
            return "";
        }

        $key = $this->getFileKey( $filePath );
        $acl = $this->getAcl($isPublic);
        $result = $this->s3->putObject(array(
            'Bucket' => $this->options['bucket'],
            'Key' => $key,
            'SourceFile' => $filePath,
            'region' => 'ap-southeast-1',
            'ACL' => $acl ,
        ));
        return $result->get('ObjectURL');

    }

    /**
     * @param $filePath
     * @return string
     * empty string if user have never did the aws configuration
     */
    public function createPresignedUrl( $filePath, $time_die ) {

        if( is_null( $this->s3) ) {
            return "";
        }

        $key = $this->getFileKey( $filePath );
        $cmd = $this->s3->getCommand('GetObject', [
            'Bucket' => $this->options['bucket'],
            'Key' => $key
        ]);
        $request = $this->s3->createPresignedRequest($cmd, $time_die);
        $presignedUrl = (string) $request->getUri();

        return $presignedUrl;

    }

    public function updateObjACL( $filePath, $isPublic ) {

        if( is_null( $this->s3) ) {
            return "";
        }

        $key = $this->getFileKey( $filePath );
        $acl = $isPublic == "1" ? 'public-read' : 'private';
         $result = $this->s3->putObjectAcl(array(
            'Bucket' => $this->options['bucket'],
            'Key' => $key,
            'ACL' => $acl,
        ));
         return $result;
    }

    private function getFileKey( $filePath ) {
        $file_name = basename( $filePath );
        $prefix_key = $this->options['prefix_key'] . '/' . $this->options['domain'];
        $key = "$prefix_key/$file_name";

        return $key;
    }

    public function isFileExisted($filePath) {
        $key = $this->getFileKey( $filePath );
        $result = $this->s3->doesObjectExist(
            $this->options['bucket'],
            $key
        );
        return $result;
    }

    private function getAcl($is_public) {
     return $is_public ? 'public-read' : 'private';
    }

    public function deleteS3File($filePath) {
        $key = $this->getFileKey( $filePath );
        $result = $this->s3->deleteObject(array(
            'Bucket' => $this->options['bucket'],
            'Key' => $key
        ));
        return $result;
    }

    public function getExpiredTime() {
//        $settings = get_option( PdaS3Constants::PDAS3_SETTINGS_NAME );
//        $time_die = PdaS3Constants::PDAS3_DEFAULT_EXPIRED_TIME;
//        if ($settings) {
//            $options = unserialize($settings);
//            if(array_key_exists('time_die_private_link', $options)) {
//                $time_die = $options['time_die_private_link'];
//            }
//        }
//        return $time_die;

        $settings = get_option( "pda_settings_magic_link" );
        $time_die = PdaS3Constants::PDAS3_DEFAULT_EXPIRED_TIME;
        if ($settings) {
            $options = unserialize($settings);
            if(is_array($options)){
                if(array_key_exists('mg_time_die_private_link', $options)) {
                    $time_die = $options['mg_time_die_private_link'];
                }
            }
        }
        return $time_die;

    }

    public function updateBucketPolicy( $whiteListIps ) {
        $bucket = $this->options['bucket'];
        $isBucketExisted = $this->s3->doesBucketExist($bucket);
        if($isBucketExisted) {
            return $this->s3->putBucketPolicy(array(
                'Bucket' => $bucket,
                'Policy' => json_encode(array(
                    'Statement' => array(
                        array(
                            'Sid' => 'pdas3-policy-ipallow',
                            'Action' => array(
                                's3:*'
                            ),
                            'Effect' => 'Allow',
                            "Principal" => "*",
                            'Resource' => array(
                                "arn:aws:s3:::{$bucket}/pda/*"
                            ),
                            'Condition' => array(
                                "IpAddress" => array(
                                    "aws:SourceIp" => $whiteListIps
                                )
                            )

                        )
                    )
                ))
            ));
        }
    }
    
    public function testConfiguration( $options ) {
        try {
            $s3 = new S3Client([
                'version' => 'latest',
                'region' => $options['region'],
                'credentials' => [
                    'key' => $options['aws_key'],
                    'secret' => $options['aws_secret']
                ]
            ]);
            return $s3->doesBucketExist($options['bucket']);
        } catch(Exception $ex) {
            error_log('Test Configuration: ' . serialize($ex));
            return new WP_Error('wrong_aws_configration_', __('Please double check your aws configuration at the plugin setting', 'wp-pda-s3'), array('status' => 400));
        }
    }
}
