<?php

if ( !class_exists('PDA_S3_Repository') ) {
     
    class PDA_S3_Repository {

        public function createCustomPrivateUrl($data) {

            if ($this->check_existed_url($data['signed_url'])) {
                return false;
            }

            $values = array(
                'expired_time_stamp' => $data['expired_time_stamp'],
                'is_default' => $data['is_default']
            );

            if(isset($data['pretty_url'])) {
                $isExisted = $this->check_existed_custom_url($data['pretty_url']);
                if ($isExisted) {
                    return false;
                }
                $values['pretty_url'] = $data['pretty_url'];
            }

            return $this->insert($data['post_id'], $data['signed_url'], $values);
        }

        function get_private_link_by_post_id($id) {
            global $wpdb;
            $table_name = $wpdb->prefix . 'prevent_direct_access_by_s3';
            $query = $wpdb->get_results("SELECT * FROM $table_name where post_id = {$id} order by created_time desc");

            return $query;
        }

        function delete_private_link_by_id($id) {
            global $wpdb;
            $table_name = $wpdb->prefix . 'prevent_direct_access_by_s3';
            return $wpdb->delete($table_name, array('ID' => $id), array('%d'));
        }

        public function insert($postId, $signedUrl, $values) {
            global $wpdb;
            $table_name = $wpdb->prefix . 'prevent_direct_access_by_s3';

            $present_date = new DateTime();

            $expired_date = is_array($values) && array_key_exists('expired_time_stamp', $values)
                ? $values["expired_time_stamp"] : null;
            $pretty_url = is_array($values) && array_key_exists('pretty_url', $values)
                ? $values["pretty_url"] : null;
            $wpdb->insert($table_name, array(
                'post_id' => $postId,
                'created_time' => $present_date->getTimeStamp(),
                'signed_url' => htmlentities($signedUrl),
                'expired_time_stamp' => $expired_date,
                'is_default' => $values['is_default'],
                'pretty_url' => $pretty_url,
            ));
            return $signedUrl;
        }

        function check_existed_custom_url($url) {
            global $wpdb;
            $table_name = $wpdb->prefix . 'prevent_direct_access_by_s3';
            $query_string = "SELECT * FROM $table_name WHERE pretty_url = %s";
            $preparation = $wpdb->prepare( $query_string, $url );

            $result = $wpdb->get_row( $preparation );

            return isset($result);
        }

        function check_existed_url($signed_url) {
            global $wpdb;
            $table_name = $wpdb->prefix . 'prevent_direct_access_by_s3';
            $query_string = "SELECT * FROM $table_name WHERE signed_url = %s";
            $preparation = $wpdb->prepare( $query_string, $signed_url );

            $result = $wpdb->get_row( $preparation );

            return isset($result);
        }

        function get_private_url($signed_url) {
            global $wpdb;
            $table_name = $wpdb->prefix . 'prevent_direct_access_by_s3';
            $query_string = "SELECT * FROM $table_name WHERE signed_url = '" . $signed_url ."'";
//            $preparation = $wpdb->prepare( $query_string, $url );
            $result = $wpdb->get_row( $query_string );
            $wpdb->show_errors = true;
            return $result;
        }

        function get_default_private_link($post_id) {
            global $wpdb;
            $table_name = $wpdb->prefix . 'prevent_direct_access_by_s3';
            $query_string = "SELECT * FROM $table_name where post_id = %d and is_default = 1 and UNIX_TIMESTAMP() < expired_time_stamp";
            $preparation = $wpdb->prepare( $query_string, $post_id );

            $result = $wpdb->get_row( $preparation );
            return $result;
        }

        function insert_default_private_link( $post_id ) {
            $aws_helper = new Aws_Helper;

            $file_path = get_attached_file( $post_id );
            $time_die = $aws_helper->getExpiredTime();
            $expired_time_stamp = new DateTime();
            $expired_time_stamp->modify($time_die);
            $timestamp = $expired_time_stamp->getTimestamp();

            $s3_link = $aws_helper->createPresignedUrl($file_path, $time_die);

             $this->insert($post_id, $s3_link, array(
                'is_default' => true,
                'expired_time_stamp' => $timestamp
            ));

             return $s3_link;
        }

        function is_default_private_link($post_id) {
            $links = $this->get_private_link_by_post_id($post_id);
            return empty($links);
        }

        public function insert_s3_info($post_id, $data) {
            global $wpdb;
            $table_name = $wpdb->prefix . 'prevent_direct_access_by_s3_info';
            $present_date = new DateTime();

            return $wpdb->insert($table_name, array(
                'post_id' => $post_id,
                'created_time' => $present_date->getTimeStamp(),
                's3_link' => $data['s3_link'],
                'is_public' => $data['is_public']
            ));
        }

        public function fetch_s3_info() {
            global $wpdb;
            $table_name = $wpdb->prefix . 'prevent_direct_access_by_s3_info';

            $results = $wpdb->get_results("SELECT * FROM $table_name");
            return $results;
        }

        public function get_s3_info( $post_id ) {
            global $wpdb;
            $table_name = $wpdb->prefix . 'prevent_direct_access_by_s3_info';
            $post_table = $wpdb->prefix . 'posts';
            $query_string = "SELECT s3.*, p.post_title, p.guid FROM $table_name as s3 INNER JOIN $post_table as p WHERE s3.post_id = p.ID AND s3.post_id = %d";
            $preparation = $wpdb->prepare( $query_string, $post_id );
            $result = $wpdb->get_row( $preparation );

            return $result;
        }

        public function update_public_s3_info($id, $values) {
            global $wpdb;
            $table_name = $wpdb->prefix . 'prevent_direct_access_by_s3_info';
            return $wpdb->update($table_name, $values, array(
                'post_id' => $id
            ));
        }

        public function get_advance_file_s3_info($post_id) {
            global $wpdb;
            $table_name = $wpdb->prefix . 'prevent_direct_access_by_s3_info';

            $results = $wpdb -> get_results("SELECT * FROM {$table_name} WHERE post_id = {$post_id}");
            return $results;
        }

        public function delete_private_link_by_post_id($post_id){
            global $wpdb;
            $table_name1 = $wpdb->prefix . 'prevent_direct_access_by_s3';
            $table_name2 = $wpdb->prefix . 'prevent_direct_access_by_s3_info';
            $result1 = $wpdb->delete($table_name1, array('post_id' => $post_id), array('%d'));
            $result2 = $wpdb->delete($table_name2, array('post_id' => $post_id), array('%d'));
            return $result1 . $result2;
        }

        public function get_pretty_url() {
            global $wpdb;
            $table_name = $wpdb->prefix . 'prevent_direct_access_by_s3';
            $query = "SELECT * FROM $table_name WHERE pretty_url != ''";
            $results = $wpdb->get_results($query);
            return $results;
        }

        public function get_postid_pda() {
            if(class_exists('PDA_Gold_Repository')) {
                $post_id =new PDA_Gold_Repository;
                $data = $post_id->get_post_id_pda_gold();
                return $data;
            }
        }

        //function api private links
        function get_advance_file_by_id( $id ) {
            global $wpdb;
            $table_name = $wpdb->prefix . 'prevent_direct_access_by_s3';
            $advance_file = $wpdb->get_row( $wpdb->prepare( "SELECT * FROM $table_name WHERE ID = %s", $id ) );
            return $advance_file;
        }

        function get_default_advance_file_by_host_id ( $id ) {
            global $wpdb;
            $table_name = $wpdb->prefix . 'prevent_direct_access_by_s3';
            $advance_file = $wpdb->get_row( $wpdb->prepare( "SELECT * FROM $table_name WHERE post_id = %s AND is_default = %d", $id, 1 ) );
            return $advance_file;
        }

        function unset_default_advance_file( $id ) {
            global $wpdb;
            $table_name = $wpdb->prefix . 'prevent_direct_access_by_s3';
            $where = array('ID' => $id);
            $data = array( 'is_default' => 0 );
            $result = $wpdb->update( $table_name, $data, $where );
            var_dump($result);
            return $result;
        }

        function update_advance_file_by_id( $id, $data ) {
            global $wpdb;
            $table_name = $wpdb->prefix . 'prevent_direct_access_by_s3';
            $where = array('ID' => $id);
            $result = $wpdb->update( $table_name, $data, $where );
            return $result;
        }

        public function get_all_file() {
            global $wpdb;
            $table_name = $wpdb->prefix . 'prevent_direct_access_by_s3';
            $query_string = "SELECT * FROM $table_name where is_default = 1";
            $result = $wpdb->get_results( $query_string );
            return $result;
        }
    }
}

