<?php

if(!class_exists('PDA_S3_Setting_Widgets')) {

    class PDA_S3_Setting_Widgets{

        protected $settings_utils;

        public function __construct()
        {
            $this->settings_utils = new PDA_S3_Settings_Util();
        }

        function render_general_tab() {
            return require_once('views/view-aws-configuration-setting-tab.php');
        }

        function tooltip_content($key) {
            $contents = array(
               'enable_remote_logs' => 'Allow us to track plugin usage and help you debug when something goes wrong. No sensitive data is tracked.',
               'grant_access_to_all_logged_users' => 'All logged-in users including admins and subscribers will be able to access your protected files.',
               'grant_access_to_these_roles_only' => 'Only allow the following roles to access your protected files.',
               'prevent_hotlinking' => 'Stop unwanted users from stealing and using your images and files without permission.',
               'disable_directory_listing' => 'Disable directory browsing of all folders and subdirectories.',
               'no_access_page' => 'What page to display when users have no access to protected files',
                'pda_enable_protection' => "While this option blocks Google from indexing your protected files through robots.txt, it doesn't block their original links, which are still accessible by anyone."
           );
   
            return $contents[$key];
        }

        function render_tooltip($key)
        {
            $content = PDA_S3_Setting_Widgets::tooltip_content($key);
            ?>
            <span title="<?php echo $content ?>" class="dashicons dashicons-warning pda-gold-tooltip"></span>
            <?php
        }

        function is_checkbox()
        {
            $settings = get_option('pdas3_options');
            if($settings) {
                $options = (!$settings) ? array() : unserialize($settings);
                $logged_in_user = array_key_exists('logged_in_user', $options) ? $options['logged_in_user'] : false;
                if ($logged_in_user === "true") {
                    return "none";
                } else {
                    return "";
                }
            }
        }

        function get_title_page_404()
        {
            $search_result_page_404 = get_option('pdas3_options');          
            if($search_result_page_404) {
                $options = ( !$search_result_page_404 ) ? array() : unserialize($search_result_page_404);
                $pdas3_Settings = array_key_exists('pda_gold_search', $options) ? $options['pda_gold_search'] : '';
                if($pdas3_Settings != NULL) {
                    $title = explode(";", $pdas3_Settings );
                    return array("link" => $title[0], "title" => $title[1]);
                }    
            }
            return NULL;
        }
        /**
         * prams : $name
         * return : Array
         */
        function selected_roles($name) {
            $settings = get_option('pdas3_options');
            if($settings) {
                $options = ( !$settings ) ? array() : unserialize($settings);
                $selected_roles = array_key_exists($name, $options) ? $options[$name] : [];
                if($selected_roles != "") {
                    $arr = array($selected_roles);
                    return $arr[0];
                }
                else {
                    return array();
                }
            }  
            return array();
        }

        function render_default_public_tab() {
            $roles = get_editable_roles();
            $check = $this->settings_utils->getSettings('default_public') ;
            $auto_synced = $this->settings_utils->getSettings('auto_synced');

            $time = "";
            $units = "";
            $time_die = $this->selected_roles('time_die_private_link');
            if($time_die != NULL) {
                $arr_time_die = explode(' ', $time_die);
                $time = $arr_time_die[0];
                $units = $arr_time_die[1];
            }
            
            ?>
            <div class="main_container">
                <form id="pda_s3_default_form">
                    <input type="hidden" nonce="<?php echo wp_create_nonce('pda_ajax_nonce_s3'); ?>" id="nonce_pda_s3"/>
                    <table class="settings_table" cellpadding="8">
                        <tr>
                            <td>Enable Remote Logs:<?php $this->render_tooltip('enable_remote_logs') ?></td>
                            <td>
                                <label class="pda_switch" for="enable_remote_log">
                                    <?php
                                    if($this->settings_utils->getSettings('enable_remote_log')) {
                                        ?>
                                        <input type="checkbox" id="enable_remote_log" checked/>
                                        <?php
                                    } else {
                                        ?>
                                        <input type="checkbox" id="enable_remote_log"/>
                                        <?php
                                    }
                                    ?>
                                    <span class="slider round"></span>
                                </label>
                            </td>
                        </tr>
                        <tr>
                            <td>Grant Access to all logged-in Users?<?php $this->render_tooltip('grant_access_to_all_logged_users') ?></td>
                            <td>
                                <label class="pda_switch" for="logged_in_user">
                                    <?php
                                        if($this->settings_utils->getSettings('logged_in_user')) {
                                            ?>
                                            <input type="checkbox" id="logged_in_user" checked/>
                                            <?php
                                        } else {
                                            ?>
                                            <input type="checkbox" id="logged_in_user"/>
                                            <?php
                                        }
                                    ?>
                                    <span class="slider round"></span>
                                </label>
                            </td>
                        </tr>
                        <tr id="grant-access" style="display: <?php echo $this->is_checkbox(); ?>">
                            <td scope="row">Grant Access to these Roles Only:<?php $this->render_tooltip('grant_access_to_these_roles_only') ?></td>
                            <td>
                                <select multiple="multiple" id="pdas3_role_select2" style="width:100%">
                                <?php 
                                    foreach ($roles as $role_name => $role_info): 
                                        $arrRole = array($role_name);
                                ?>
                                        <option <?php echo array_intersect($arrRole, $this->selected_roles("list_roles") ) ? 'selected="selected"' : '' ?> value="<?php echo $role_name ?>"><?php echo $role_name ?></option>
                                <?php 
                                    endforeach; 
                                ?>
                                </select>
                            </td>
                        </tr>
                        <tr>
                            <td>Disable Directory Listing<?php $this->render_tooltip('disable_directory_listing') ?></td>
                            <td>
                                <label class="pda_switch" for="directory_listing">
                                    <?php
                                        if($this->settings_utils->getSettings('directory_listing')) {
                                            ?>
                                            <input type="checkbox" id="directory_listing" checked/>
                                            <?php
                                        } else {
                                            ?>
                                            <input type="checkbox" id="directory_listing"/>
                                            <?php
                                        }
                                    ?>
                                    <span class="slider round"></span>
                                </label>
                            </td>
                        </tr>
                        <tr>
                            <td>Block Search Indexing Only:<?php $this->render_tooltip('pda_enable_protection') ?></td>
                            <td>
                                <label class="pda_switch" for="block_search_index">
                                    <?php
                                        if($this->settings_utils->getSettings('block_search_index')) {
                                            ?>
                                            <input type="checkbox" id="block_search_index" checked/>
                                            <?php
                                        } else {
                                            ?>
                                            <input type="checkbox" id="block_search_index"/>
                                            <?php
                                        }
                                    ?>
                                    <span class="slider round"></span>
                                </label>
                            </td>
                        </tr>
                        <tr>
                            <td>Prevent Hotlinking<?php $this->render_tooltip('prevent_hotlinking') ?></td>
                            <td>
                                <label class="pda_switch" for="prevent_hotlinking">
                                    <?php
                                        if($this->settings_utils->getSettings('prevent_hotlinking')) {
                                            ?>
                                            <input type="checkbox" id="prevent_hotlinking" checked/>
                                            <?php
                                        } else {
                                            ?>
                                            <input type="checkbox" id="prevent_hotlinking"/>
                                            <?php
                                        }
                                    ?>
                                    <span class="slider round"></span>
                                </label>
                            </td>
                        </tr>
                        <tr>
                        <td>No Access Page<?php $this->render_tooltip('no_access_page') ?></td>
                        <td>
                            <?php
                            $title = $this->get_title_page_404();
                            if (isset($title) && !empty($title) && $title != NULL) {
                                ?>
                                <div class='no-access-selected-page'>
                                        <span><b class="no-access-selected-page-label">Selected page: </b><span
                                                    class="no-access-selected-page-title"><?php echo $title['title'] ?>
                                                <span></span>
                                        <span class="dashicons dashicons-no remove-no-access-page"></span>
                                </div>
                                <?php
                            } else {
                                echo "<span><b>Default page: </b> 404 page</span>";
                            }
                            ?>
                        </div>
                        <div>
                            <?php wp_nonce_field('internal-linking', '_ajax_linking_nonce', false);
                            ?>
                            <input type="search" id="pda_gold_search" placeholder="Search..."/>
                        </div>
                        <div class="no-access-search-container">
                            <ul id="pda_search_result">
                            </ul>
                            <div class="title_page_404">
                                <input id="pda_search_page_404_input" type="hidden" name="pda_gold_search_result_page_404"
                                    value="<?php _e($title['link'] . ';' . $title['title']); ?>"/> 
                            </div>
                        </div>
                        </td>
                        </tr>
                        <tr>
                            <td>Auto-sync future uploaded files</td>
                            <?php
                                if($auto_synced === true) { ?>
                                <td>
                                    <label class="pda_switch" for="set_auto_synced">
                                        <input type="checkbox" id="set_auto_synced" checked/>
                                        <span class="slider round"></span>
                                    </label>
                                </td>
                            <?php } else { ?>
                                <td>
                                    <label class="pda_switch" for="set_auto_synced">
                                        <input type="checkbox" id="set_auto_synced"/>
                                        <span class="slider round"></span>
                                    </label>
                                </td>
                            <?php } ?>
                        </tr>
                        <tr>
                            <td scope="row">Auto-protect all files synced to Amazon S3</td>
                            <?php if($check === true) { ?>
                                <td>
                                    <label class="pda_switch" for="set_default_public">
                                        <input type="checkbox" id="set_default_public" checked/>
                                        <span class="slider round"></span>
                                    </label>
                                </td>
                            <?php } else { ?>
                                <td>
                                    <label class="pda_switch" for="set_default_public">
                                        <input type="checkbox" id="set_default_public"/>
                                        <span class="slider round"></span>
                                    </label>
                                </td>
                            <?php } ?>
                        </tr>
                        <tr>
                            <td scope="row">Set default expiring time for all private links</td>
                            <td>
                                <?php if($time != '') { ?>
                                    <input required value="<?php echo $time; ?>" class="width-100" type="number" min="1" max="7" id="time_die"/>
                                <?php } else { ?>
                                    <input value="7" class="width-100" type="number" id="time_die"/>
                                <?php } ?>
                            
                                <select id="units_times" class="width-100">
                                    <?php if($units == "days") { ?>
                                        <option selected value="days">Days</option>
                                        <option value="hours">Hours</option>
                                        <option value="minutes">Minutes</option>

                                    <?php } elseif($units == "hours") { ?>
                                        <option value="days">Days</option>
                                        <option selected value="hours">Hours</option>
                                        <option value="minutes">Minutes</option>

                                    <?php } elseif($units =="minutes") { ?>

                                        <option value="days">Days</option>
                                        <option value="hours">Hours</option>
                                        <option selected value="minutes">Minutes</option>

                                    <?php } else { ?>
                                        <option selected value="days">Days</option>
                                        <option value="hours">Hours</option>
                                        <option value="minutes">Minutes</option>
                                    <?php } ?>
                                </select>
                            </td>
                        </tr>
                    </table>
                    <?php
                        submit_button();
                    ?>
                </form>
            </div>
            <?php
        }

        function render_license_tab() {
            $isValidLicense = !empty(Yme_Plugin::getLicenseKey('pda-s3'));
            $error = get_option(PdaS3Constants::LICENSE_ERROR);
            $isError = !empty($error);
            ?>
            <div class="main_container">
                <form id="pda_s3_license_form">
                    <?php
                    if($isValidLicense) {
                        ?> <h3><?php echo PdaS3Constants::LICENSE_ACTIVATED ?></h3> <?php
                    } else {
                        ?> <h3><?php echo PdaS3Constants::LICENSE_NOT_ACTIVATED ?></h3><?php
                    }
                    if($isError) {
                        ?>
                        <div class='notice notice-error is-dismissible'>
                            <p><?php echo $error ?></p>
                        </div>
                        <?php
                    }
                    ?>
                    <input type="hidden" nonce="<?php echo wp_create_nonce('pda_ajax_nonce_s3'); ?>" id="nonce_pda_s3"/>
                    <div>
                        <span>Your license key:</span>
                        <?php
                       if($isValidLicense) {
                           ?>
                           <td><?php echo Yme_Plugin::getLicenseKey('pda-s3') ?></td>
                           <?php
                       } else {
                           ?>
                           <td>
                               <input type="text" value="" id="pda-s3-license">
                               <div class="pda-s3-error" id="pda-s3-error"></div>
                           </td>
                           <?php
                       }
                       ?>
                    </div>
                    <?php
                    if(!$isValidLicense) {
                        submit_button();
                    }
                    ?>
                </form>
            </div>
            <?php
        }

        function get_ip_block($name_ip)
        {
            $ip_block = get_option('pdas3_ip_block');
            if($ip_block) {
                $options = ( !$ip_block ) ? array() : unserialize($ip_block);
                $pdas3_ip = array_key_exists($name_ip, $options) ? $options[$name_ip] : '';
                return $pdas3_ip;
            }
        }

        function render_ipblock_tab()
        {
            $list_ip=$this->get_ip_block('pda_gold_ip_block');
            ?>
            <div class="main_container">
                <form method="post" id="pda_gold_ip_block_form">
                    <input type="hidden" nonce="<?php echo wp_create_nonce('pda_ajax_nonce_ip_block'); ?>"
                        id="nonce_ip_block"/>
                    <div class="ip-block-title">
                        Blacklist these IP addresses: stop the following IP addresses from accessing private download links
                    </div>
                    <input id="pda_gold_ip_block" name="ip_block" value="<?php echo $list_ip; ?>"/><br>
                    <p class="description">Use the asterisk (*) for wildcard matching. E.g: 7.7.7.* will match IP from
                        7.7.7.0 to 7.7.7.255</p>
                    <p>
                        <input type="submit" value="<?php _e('Save IP Black List'); ?>" class="button button-primary"
                            name="btn_ip_lock">
                    </p>
                </form>
            </div>
            <?php
        }

        function render_migrate_tab() {
            if(class_exists('Pda_Gold_Admin')) { 
            ?>
            <div class="main_container">
                <tr>
                    <td>
                        <div class="aws-btn-group" id="wc-name">
                            <p>Migrate data from Prevent Direct Access Gold to Prevent Direct Access S3</p>
                            <input value = "Migration Data PDA Gold" type = "button" id="migratedata" class="button button-default">
                            <input style = "display: none" type="submit" name="submit" id="submitdata" class="button button-primary" value="<?php echo get_home_url();?>/wp-json/wps3/v1/migrate_data/">
                        </div>
                    </td>
                </tr>
            </div>
            <?php } ?>
            <?php
        }

        function render_magic_link_tab() {
            do_action('pda_magic_link_settings');
        }
    }
}