<?php

class PDA_S3_WIZARD {

    private $step = '';

    private $steps = array();

    private $plugin_url = '';

    public function __construct() {
        $this->plugin_url = untrailingslashit( plugins_url( '/', PDA_S3_PLUGIN_FILE ) );
    }


    public function setup() {

        $this->steps = array(
            's3_configuration' => array(
                'name' => __( 'AWS S3', 'prevent-direct-access-by-s3' ),
                'view' => array( $this, 'view_pda_s3_aws' ),
                'handler' => array( $this, 'handler_pda_s3_aws' )
            ),
            'migration' => array(
                    'name' => __('Data Migration', 'prevent-direct-access-by-s3'),
                    'view' => array( $this, 'view_pda_s3_migration' ),
                    'handler' => array( $this, 'handler_pda_s3_migration')
            ),
            'ready' => array(
                'name' => __('Ready', 'prevent-direct-access-by-s3'),
                'view' => array( $this, 'view_pda_s3_ready' ),
                'handler' => array( $this, 'handler_pda_s3_migration')
            )
        );
        $this->step  = isset( $_GET['step'] ) ? sanitize_key( $_GET['step'] ) : current( array_keys( $this->steps ) ); // WPCS: CSRF ok, input var ok.
        wp_enqueue_style( 'pda-s3-setup', $this->plugin_url . '/public/css/pda-s3-wizard-setup.css', array( 'dashicons', 'install' ), PDA_S3_VERSION );
        wp_enqueue_style( 'pda-s3-setup-view', $this->plugin_url . '/admin/css/pda_s3_setting.css', array(), PDA_S3_VERSION );
        wp_enqueue_style( 'toastr', 'https://cdnjs.cloudflare.com/ajax/libs/toastr.js/latest/toastr.min.css', array(), PDA_S3_VERSION );

        wp_register_script( 'select2', 'https://cdnjs.cloudflare.com/ajax/libs/select2/4.0.3/js/select2.min.js' );
        wp_register_script( 'toastr', 'https://cdnjs.cloudflare.com/ajax/libs/toastr.js/latest/js/toastr.min.js' );
        wp_register_script( 'pda-s3-setup-handler', $this->plugin_url . '/admin/js/prevent-direct-access-by-s3-setting-general.js', array( 'jquery', 'select2', 'toastr'), PDA_S3_VERSION );
        wp_localize_script('pda-s3-setup-handler', 'pda_s3_setting_data', array('ajax_url' => admin_url('admin-ajax.php')));

        wp_register_script( 'pda-s3-migration-handler', $this->plugin_url . '/admin/js/pdas3_migrate.js', array( 'jquery', 'select2', 'toastr'), PDA_S3_VERSION );
        wp_localize_script( 'pda-s3-migration-handler', 'migrate_server_data', array ( 'home_url' => home_url()));
        wp_register_script( 'pda-s3-ready', $this->plugin_url . '/public/js/prevent-direct-access-by-s3-wizard-ready.js', array( 'jquery' ), PDA_S3_VERSION );

        if ( ! empty( $_POST['save_step'] ) && isset( $this->steps[ $this->step ]['handler'] ) ) {
            call_user_func( $this->steps[ $this->step ]['handler'], $this );
        }

    }

    public function setup_header() {
        $tracking_link = "https://preventdirectaccess.com/prevent-direct-access-amazon-s3-integration/?utm_source=user-website&utm_medium=plugin_wizard_link&utm_campaign=pda-s3";
        ?>
        <!DOCTYPE html>
    <html <?php language_attributes(); ?>>
        <head>
            <meta name="viewport" content="width=device-width" />
            <meta http-equiv="Content-Type" content="text/html; charset=utf-8" />
            <title><?php esc_html_e( 'PDA S3 &rsaquo; Setup Wizard', 'prevent-direct-access-by-s3' ); ?></title>
            <?php wp_print_scripts( 'pda-s3-setup-handler' ); ?>
            <?php wp_print_scripts( 'pda-s3-migration-handler' ); ?>
            <?php wp_print_scripts( 'pda-s3-ready' ); ?>
            <?php do_action( 'admin_print_styles' ); ?>
            <?php do_action( 'admin_head' ); ?>
        </head>
    <body class="pda-s3-setup wp-core-ui">
    <h1 id="pda-s3-logo"><a href="<?php echo $tracking_link ?>"><img src="<?php echo $this->plugin_url; ?>/assets/images/pda-s3-logo-v2-1.png" alt="prevent-direct-access-by-s3" /></a></h1>
        <?php
    }

    public function setup_steps() {
        $output_steps = $this->steps;
        ?>
        <ol class="pda-s3-setup-steps">
            <?php foreach ( $output_steps as $step_key => $step ) : ?>
                <li class="
					<?php
                if ( $step_key === $this->step ) {
                    echo 'active';
                } elseif ( array_search( $this->step, array_keys( $this->steps ), true ) > array_search( $step_key, array_keys( $this->steps ), true ) ) {
                    echo 'done';
                }
                ?>
				"><?php echo esc_html( $step['name'] ); ?></li>
            <?php endforeach; ?>
        </ol>
        <?php
    }

    public function setup_content() {
        echo '<div class="pda-s3-setup-content">';
        if ( ! empty( $this->steps[ $this->step ]['view'] ) ) {
            call_user_func( $this->steps[ $this->step ]['view'], $this );
        }
        echo '</div>';
    }

    public function setup_wizard_footer() {
        ?>
        <?php if ( 's3_configuration' === $this->step ) : ?>
            <a class="pda-s3-return-to-dashboard" href="<?php echo esc_url( admin_url() ); ?>"><?php esc_html_e( 'Not right now', 'woocommerce' ); ?></a>
        <?php elseif ( 'next_steps' === $this->step ) : ?>
            <a class="pda-s3-return-to-dashboard" href="<?php echo esc_url( admin_url() ); ?>"><?php esc_html_e( 'Return to your dashboard', 'woocommerce' ); ?></a>
        <?php elseif ( 'activate' === $this->step || 'extras' === $this->step ) : ?>
            <a class="pda-s3-return-to-dashboard" href="<?php echo esc_url( $this->get_next_step_link() ); ?>"><?php esc_html_e( 'Skip this step', 'woocommerce' ); ?></a>
        <?php endif; ?>
        </body>
        </html>
        <?php
    }

    public function redirect_to_wizard() {
        if(get_option(PdaS3Constants::WIZARD_REDIRECT_OPTION)) {
            update_option(PdaS3Constants::WIZARD_REDIRECT_OPTION, false);
            $page = PdaS3Constants::WIZARD_PAGES['main_page'];
            $wizard_url = admin_url("index.php?page=$page");
            exit( wp_safe_redirect( esc_url_raw($wizard_url) ) );
        }
    }

    private function get_next_step_link( $step = '' ) {
        if ( ! $step ) {
            $step = $this->step;
        }

        $keys = array_keys( $this->steps );
        if ( end( $keys ) === $step ) {
            return admin_url();
        }

        $step_index = array_search( $step, $keys, true );
        if ( false === $step_index ) {
            return '';
        }

        return add_query_arg( 'step', $keys[ $step_index + 1 ], remove_query_arg( 'activate_error' ) );
    }

    private function view_pda_s3_aws() {
        add_action('pda-s3-setup-actions', array($this, 'render_next_button'));
       require_once ( plugin_dir_path( PDA_S3_PLUGIN_FILE ) . '/includes/views/view-aws-configuration-setting-tab.php');
    }

    public function render_next_button() {
        ?>
        <a class="button button-default" href="<?php echo esc_url_raw( $this->get_next_step_link());?>"> Next </a>
        <?php
    }

    private function handler_pda_s3_aws() {
        wp_safe_redirect( esc_url_raw( $this->get_next_step_link() ) );
        exit();
    }

    private function view_pda_s3_migration() {
        $is_pda_activated = Yme_Plugin_Utils::is_plugin_activated('pda_gold') === -1;
        $enable = $is_pda_activated ? "" : "disabled";
        ?>
        <div class="main-container">
            <h1><?php esc_html_e( 'Data migration', 'prevent-direct-access-by-s3' ); ?></h1>
            <form class="pda-s3-migration-form">
                <ul class="pda-s3-wizard-services featured">
                    <li class="pda-s3-wizard-service-item">
                        <div class="pda-s3-wizard-service-name">
                            <img src="<?php echo $this->plugin_url; ?>/assets/images/logo-prevent-direct-access-red-tick.png" alt="pda-s3" />
                        </div>
                        <div class="pda-s3-wizard-service-description">
                        <span class="pda-s3-wizard-service-settings-description">
                            Protect Your WordPress Files Against Google & Unwanted Users. Recognized as the #1 plugin to protect WordPress files by experts.
                        </span>
                        </div>
                        <div class="pda-s3-wizard-service-enable">
                            <span class="pda-s3-wizard-service-toggle <?php echo $enable ?>">
					            <input
                                    id="pda-s3-wizard-service-pda-gold"
                                    type="checkbox"
                                    name="pda-s3-wizard-service-pda-gold-enabled"
                                    value="yes" />
                            </span>
                        </div>
                    </li>
                </ul>
                <div class="aws-btn-group pda-s3-setup-actions">
                    <?php if($is_pda_activated) {
                      echo "<input type=\"button\" id=\"migratedata\" class=\"button button-default btn-main\" value=\"Migrate\">";
                    } else {
                        echo "<input disabled type=\"button\" id=\"migratedata\" class=\"button button-default btn-main\" value=\"Migrate\">";
                    }
                    ?>
                    <a class="button button-default" href="<?php echo esc_url_raw( $this->get_next_step_link());?>"> Next </a>
                </div>
            </form>
        </div>
        <?php
    }

    private function handler_pda_s3_migration() {

    }

    private function view_pda_s3_ready() {
        $current_user = wp_get_current_user();
        ?>
        <h1><?php esc_html_e( "You're ready to protect your precious assets!", 'prevent-direct-access-by-s3' ); ?></h1>
        <div class="pda-s3-message pda-s3-newsletter">
            <p class="pda-s3-subscribe"><?php esc_html_e( "Be the first to get our latest updates and probably 1-year Gold license for free.", 'pda' ); ?></p>
            <p style="display: none" class="pda-s3-subscribe-ok"><?php esc_html_e( "Thanks for your subscription!", 'pda' ); ?></p>
            <form class="pda-s3-subscribe">
                <div class="pda-s3-newsletter-form-container">
                    <input type="text" id="pda-s3-signup-newsletter-input" class="pda-s3-newsletter-form-email" name="pda-s3-signup-newsletter"
                           placeholder="you@example.com" value="<?php echo $current_user->user_email ?>"/>
                    <p class="pda-s3-setup-actions step pda-s3-newsletter-form-button-container">
                        <input type="button" class="button pda-s3-newsletter-form-button btn-main" id="pda-s3-signup-newsletter"
                               value="<?php echo __('Get Lucky', 'pda') ?>"/>
                    </p>
                    <div id="pda_signup_newsletter_error" style="display: none"
                         class="pda_subscribe_error"><span>Please enter your valid email!</span></div>
                </div>
            </form>
        </div>
        <ul class="pda-s3-wizard-next-steps">
            <li class="pda-s3-wizard-next-step-item">
                <div class="pda-s3-wizard-next-step-description">
                    <p class="next-step-heading"><?php esc_html_e( 'Next step', 'woocommerce' ); ?></p>
                    <h3 class="next-step-description"><?php esc_html_e( 'Protect your first media file now!', 'prevent-direct-access-by-s3' ); ?></h3>
                    <p class="next-step-extra-info"><?php esc_html_e( "Your file is important than your girl friend!", 'prevent-direct-access-by-s3' ); ?></p>
                </div>
                <div class="pda-s3-wizard-next-step-action">
                    <p class="pda-s3-setup-actions step">
                        <a class="button button-large btn-main" href="<?php echo esc_url( admin_url( 'upload.php' ) ); ?>">
                            <?php esc_html_e( 'Protect now', 'prevent-direct-access-by-s3' ); ?>
                        </a>
                    </p>
                </div>
            </li>
        </ul>
        <div class="pda-s3-wizard-next-step-help">
            Visit our <a target="_blank" href="https://preventdirectaccess.com/prevent-direct-access-amazon-s3-integration/?utm_source=user-website&utm_medium=plugin_link&utm_campaign=pda-s3">plugin's official website</a> to learn more about Prevent Direct Access by S3.
        </div>
        <?php
    }
}


