<?php

/**
 * The plugin bootstrap file
 *
 * This file is read by WordPress to generate the plugin information in the plugin
 * admin area. This file also includes all of the dependencies used by the plugin,
 * registers the activation and deactivation functions, and defines a function
 * that starts the plugin.
 *
 * @link              https://preventdirectaccess.com/
 * @since             1.0.0
 * @package           Prevent_Direct_Access_By_S3
 *
 * @wordpress-plugin
 * Plugin Name:       Prevent Direct Access S3
 * Plugin URI:        https://preventdirectaccess.com/prevent-direct-access-amazon-s3-integration/?utm_source=user-website&utm_medium=plugin_link&utm_campaign=pda-s3
 * Description:       Sync and protect unlimited WordPress files and all file types including PDF, DOCS and MP4 using Amazon S3.
 * Version:           1.0.3
 * Author:            BWPS
 * Author URI:        https://preventdirectaccess.com/?utm_source=user-website&utm_medium=author_link&utm_campaign=pda-s3
 * License:           GPL-2.0+
 * License URI:       http://www.gnu.org/licenses/gpl-2.0.txt
 * Text Domain:       prevent-direct-access-by-s3
 * Domain Path:       /languages
 */

// If this file is called directly, abort.
if ( ! defined( 'WPINC' ) ) {
	die;
}

/**
 * Currently plugin version.
 * Start at version 1.0.0 and use SemVer - https://semver.org
 * Rename this for your plugin and update it as you release new versions.
 */
define( 'PDA_S3_VERSION', '1.0.1' );


// Define WC_PLUGIN_FILE.
if ( ! defined( 'PDA_S3_PLUGIN_FILE' ) ) {
    define( 'PDA_S3_PLUGIN_FILE', __FILE__ );
}

/**
 * The code that runs during plugin activation.
 * This action is documented in includes/class-prevent-direct-access-by-s3-activator.php
 */
function activate_prevent_direct_access_by_s3() {
	require_once plugin_dir_path( __FILE__ ) . 'includes/class-prevent-direct-access-by-s3-activator.php';
    Prevent_Direct_Access_By_S3_Activator::activate();
}

//function pdas3_plugin_redirect() {
//    if (get_option('pdas3_plugin_do_activation_redirect', false)) {
//        delete_option('pdas3_plugin_do_activation_redirect');
//        if(!isset($_GET['activate-multi']))
//        {
//            wp_redirect( admin_url( 'index.php?page=pdas3-setup' ) );
//        }
//    }
//}
/**
 * The code that runs during plugin deactivation.
 * This action is documented in includes/class-prevent-direct-access-by-s3-deactivator.php
 */
function deactivate_prevent_direct_access_by_s3() {
	require_once plugin_dir_path( __FILE__ ) . 'includes/class-prevent-direct-access-by-s3-deactivator.php';
    Prevent_Direct_Access_By_S3_Deactivator::deactivate();
}

register_activation_hook( __FILE__, 'activate_prevent_direct_access_by_s3' );
register_deactivation_hook( __FILE__, 'deactivate_prevent_direct_access_by_s3' );

/**
 * The core plugin class that is used to define internationalization,
 * admin-specific hooks, and public-facing site hooks.
 */
require plugin_dir_path( __FILE__ ) . 'includes/class-prevent-direct-access-by-s3.php';
// Include the main WooCommerce class.
/**
 * Begins execution of the plugin.
 *
 * Since everything within the plugin is registered via hooks,
 * then kicking off the plugin from this point in the file does
 * not affect the page life cycle.
 *
 * @since    1.0.0
 */
function run_prevent_direct_access_by_s3() {

    $plugin = new Prevent_Direct_Access_By_S3();
    $plugin->run();

    $licenseKey = Yme_Plugin::getLicenseKey('pda-s3');
    if( !empty($licenseKey) ) {
        Pda_S3_Db::create_table();
    }

    PucFactory::buildUpdateChecker(
        'https://s3-ap-southeast-1.amazonaws.com/bwps.gold.plugins/wp-pda-s3/metadata.json',
        __FILE__,
        'wp-pda-s3'
    );
}
run_prevent_direct_access_by_s3();
